#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: L2Tracking.cgi,v 1.1 2006/03/23 00:46:35 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/L2Tracking.cgi,v $

########################################################################
# L2Tracking.cgi
#
# Provides a method to control the L2 Tracking function of Stinger.
#
# Stinger with Maxwell 5 chips, supports both L2-Engigma (codeless)
# and L2 Civil Signal (L2C).  The system can track 
#   Neither signal (/Loff).       
#   L2E, ignoring L2C.           (l2_mode == enigma)
#   L2C with L2E fallback.
#   L2E AND L2C at the same time.
#
# This is under control of a stinger configuration value 
# "l2_mode". 
# This can be set by using the command
#   put l2_mode [enigma | l2c_or_enigma | l2c_and_enigma | off]
# Likewise "get l2_mode" on the info channel will provide that
# value back to you.  But this all requires opening a socket to
# stinger, which is slow.
#
# To make things quicker, stinger registers its controls with the
# kernel-resident gpsinfo database which can be queried using the
# sysinfo utility program, as with
#   sysinfo -p l2_mode.
# 
#
# This script is called to put up the HTML form for controlling the
# L2 CS tracking function.  No CGI parameters are used.  When the user
# clicks on OK, the script gpsadmin/setL2CSTracking.cgi is called with
# the desired setting to activate the change.  Only the
# setL2Tracking.cgi script has access restrictions.  This script
# (L2Tracking.cgi) is not restricted and anyone can view the form it
# creates.
#
########################################################################


use strict ;
use CGI qw/:standard/;
use Fireball ;

titleWithHelp( 'L2 Tracking Control',
                         '/L2TrackingInfo.html'
                       ) ;

# Obtain current setting from gpsinfo system, using sysinfo.
# Response will be of the forms:
#  put l2mode off
#  put l2mode enigma
#  put l2mode l2c_or_enigma
#  put l2mode l2c_and_enigma
$_ = qx( /usr/local/fireball/sysinfo -p l2_mode ) ;
chomp ;
#print $_, br() ;

my (undef, undef, $L2mode) = split( " ", $_ ) ;
#print $L2mode, br() ;

print startform( -name=>'theForm', 
                 -action=>'/perl-scripts/gpsadmin/setL2Tracking.cgi'
               ) ;


my @Control = radio_group( -name=>'L2Tracking', 
                           -value=>[ 'off', 
                                     'enigma',
                                     'l2c_or_enigma',
                                     'l2c_and_enigma',
                                   ],
                           -nolabels=>1,
                           -default=> $L2mode
                         ) ;

print table( {border=>'1'},
             Tr(
                td( {width=>'30', align=>'center'}, $Control[0] ),
                td( b('Off'),br(),
                    'No L2 Signals will be tracked.'
                  )
               ),
             Tr(
                td( {width=>'30', align=>'center'}, $Control[1] ),
                td( b('L2-Y-code Only'),br(),
                    'Y-code on L2 will be tracked.'
                  )
               ),
             Tr(
                td( {width=>'30', align=>'center'}, $Control[2] ),
                td( b('L2C or L2-Y-code'),
                    br(),
                    'L2C will be tracked if available.',
                    br(),
                   'L2-Y-code will be tracked if L2C is not available.'
                  )
               ),
             Tr(
                td( {align=>'center'}, $Control[3] ),
                td( b('L2C and L2-Y-code'),
                    br(),
                    'Both signals will be tracked simultaneously.'
                  )
               )
           ) ;

print br(),hr() ;


# No action was done. In other words this invocation is just 
# displaying the form for accepting new data.  Put up the buttons.
print button( -name=>'OK', 
              -value=>'OK',
              -onClick=>'parent.submitForm( document.theForm, "" )'
            ) ;
print '&nbsp;' ;
print button( -name=>'Cancel', 
              -value=>'Cancel',
              -onClick=>
            'parent.loadDataFrame( "/perl-scripts/rconfigDisplay.cgi" )'
            ) ;

print endform() ;

 
print end_html() ;

########################################################################
# end of file L2Tracking.cgi
########################################################################
