#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: access.cgi,v 1.8 2004/08/23 17:13:16 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/access.cgi,v $

########################################################################
# access.cgi
#
# Script for displaying a form for modifying the Access Restrictions on
# Fireball.  This script manages an HTML form.  When the user clicks
# OK, the form data is submitted to a sysadmin-secured script in
# sysadmin/setAccess.cgi, which handles changing the system
# configurations.  That script always re-invokes this one, usually
# passing back a message in parameter 'msg' for display on this page.
#
# Access restrictions allow the user to enable or disable the
# restrictions on the three browser function security levels.
# These are embodied in the configuration files
#   /etc/sysconfig/htaccess.sysadm
#   /etc/sysconfig/htaccess.gpsadm
#   /etc/sysconfig/htaccess.fileaccess
# There are symbolic links in the three corresponding directories
# that point to these access control files.  Each of the files looks
# pretty much the same:
#
#       AuthType Basic
#       AuthName Sysadmin
#       AuthUserFile /etc/sysconfig/htpasswd
#       AuthGroupFile /etc/sysconfig/htgroups
#       require group sysadmin
#
# The access control is dictated by the "require" line.  If that
# is present then the restrictions are active.  If not then
# the scripts in that directory are available to anyone.
########################################################################

use strict ;
use CGI qw/:standard/;
use Fireball ;

titleWithHelp( 'Security - Access Controls',
               '/securityAccess.html'
             ) ;

           

# If a message came back from the setAccess.cgi script, display it now.
# The message already contains any needed HTML formatting, and may
# consist of multiple html <h3> titles.
if ( my $msg = param( 'msg' ) )
{
  print $msg ;
}

# Open the htaccess files and determine the current state of the
# access restrictions.

my @levels = ( 'sysadmin', 'gpsadmin', 'fileaccess' ) ;
# That's a list of the restriction types.
#
my %restrictions ;
# That's a hash containing an entry per security level.
# Each level is true (==restricted) or false (==unrestricted).

# Populate the hash from the configuration files.
getGuiSecurity( \%restrictions ) ;

# Create the form for displaying the state of the restriction controls,
# self-documenting the controls, and allowing controls to be changed.
# There will be three sections, each with a title, and two radio
# buttons in a table.  And there will be OK and Cancel buttons.

print startform( -name=>'theForm',
                 -action=>'/perl-scripts/sysadmin/setAccess.cgi',
                 -onSubmit=>'return Okay();'
               ) ;



# Sysadmin control----------

print b('System Controls') ;

# Create two buttons.
my @buttons=radio_group( -name=>'sysadmin',
                      -values=>['Restricted','Open'],
                      -nolabels=>1,
                      -default=>($restrictions{'sysadmin'}
                                 ?'Restricted' : 'Open'
                                ),
                    ) ;

print table( {width=>'100%', border=>1},

             # First button gets a long documentation comment.
             Tr( td( $buttons[0] ), 
                 td( '<b>Restricted</b><BR>
                      Only selected accounts, using the proper
                      passwords, can modify Accounts and Basic Receiver
                      Configuration.' 
                   ) 
               ) ,

             # Second button
             Tr( td( $buttons[1]),
                 td( '<b>Unrestricted</b> ' ) 
               ) 
           ) ;

# GpsAdmin control----------

print br() ;
print b('GPS Controls') ;

# Create two buttons.
@buttons=radio_group( -name=>'gpsadmin',
                      -values=>['Restricted','Open'],
                      -nolabels=>1,
                      -default=>($restrictions{'gpsadmin'}
                                 ?'Restricted' : 'Open'
                                ),
                    ) ;

print table( {width=>"100%", border=>1},

             # First button gets a long documentation comment.
             Tr( td( $buttons[0] ), 
                 td( '<b>Restricted</b><BR>
                         Only selected accounts, using the proper
                         passwords, can modify GPS tracking and Data
                         logging controls.' 
                   ) 
               ) ,

             # Second button
             Tr( td( $buttons[1]),
                 td( '<b>Unrestricted</b> ' )
               ) 
           ) ;



# File Access control----------

print br() ;
print b("File Access") ;

# Create two buttons.
@buttons=radio_group( -name=>'fileaccess',
                      -values=>['Restricted','Open'],
                      -nolabels=>1,
                      -default=>($restrictions{'fileaccess'}
                                 ?'Restricted' : 'Open'
                                ),
                    ) ;

print table( {width=>'100%', border=>1},

             # First button gets a long documentation comment.
             Tr( td( $buttons[0] ), 
                 td( "<b>Restricted</b><BR>
                      Only selected accounts, using the proper
                      passwords, can access logged data files via the
                      browser interface." 
                   )
               ) ,

             # Second button
             Tr( td( $buttons[1]),
                 td( "<b>Unrestricted</b> " )
               ) 
           ) ;


# The OK button calls the Okay() javascript function.
print p(),button( -name=>'OK', 
              -value=>'OK',
              -onClick=>'Okay()'
            ) ;

# Emit the javascript function into the html file.
print <<EOF ;
<script>
  function Okay()
  /* Submit the form's data.  If the user is switching on sysadmin
   * security then we ask for confirmation.  If he doesn't know 
   * the root password then he's locking himself out.
   * The test for this is three OR clauses.
   * Only submit the form If
   *     Sysadmin Unrestricted is checked.
   *  or The Sysadmin Restricted was originally checked.
   *  or the User responds OK to the confirmation request.
   */
  {
    f = document.theForm ;
    if ( f.sysadmin[1].checked  
      || f.sysadmin[0].defaultChecked 
      || confirm( 'This cannot be undone without a Sysadmin password\\n'
                + 'Are you sure?'
                )
       )
    {
      parent.submitForm( f, '' ) ;
    }
    return false ;
  }
</script>
EOF


print '&nbsp;' ;

# The Cancel button takes us out of here to a safe place.
print button( -name=>'Cancel', 
              -value=>'Cancel',
              -onClick=>
           'parent.loadDataFrame( "/perl-scripts/rstatusIdentity.cgi" )'
            ) ;

print endform() ;

print end_html() ;

########################################################################
# end of file access.cgi
########################################################################
