#!/usr/bin/perl -w
# Copyright (c) 2004 Trimble Navigation Limited
# $Id: advertise.cgi,v 1.4 2004/05/17 22:10:28 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/advertise.cgi,v $

########################################################################
# advertise.cgi
#
# Script to allow setting parameters for the Port Advertising feature.
# Called with no CGI parameters. 
# We put up a form waiting for entry of new data.
# When the user hits "OK" we check the entries with javascript and
# submit the new data to the script sysadmin/setAdvertise.cgi.
########################################################################


use CGI qw/:standard/;
use strict ;
use Fireball ;

titleWithHelp( 'Port Advertising',
               '/advertise.html'
             ) ;


# This flag is used to indicate later that a warning should be put on
# the page that the OK button needs to be pressed to make changes 
# take effect.  This is especially important if lines have been added or
# deleted to the target list.
my $needsOK = 0 ;

# How many targets are defined in the current setup.
my $targetCount = 0 ;


if (! param() )
{
  # No parameters were fed into this script.
  # So we generate a fresh page using data from the configuration file.
  # This is done by reading in the current config data, and then
  # manually setting the CGI-parameters needed to draw the form.

  # No changes so far.
  $needsOK = 0 ;

  # A Hash that will be filled with data from the configuration file.
  my %advertiseControls ;

  # Read from the configuration file 
  getAdvertiseControls( \%advertiseControls ) ;

  my @ports = @{$advertiseControls{'ports'}} ;
  my @IPs = @{$advertiseControls{'IPs'}} ;

  $targetCount = scalar( @IPs ) ;

  # Transfer the data from the hash into the parameters used
  # by the Perl-CGI system.  This sets the defaults for all of the GUI
  # controls we will be creating later.

  param( 'enable', $advertiseControls{'enable'} ) ;

  param( 'interval', $advertiseControls{'interval'} ) ;

  # For each target, we'll be creating an IP text field and
  # a port text field.  These will be named 'IPN' and
  # 'portN' with N going from 0 to targetCount-1.
  for ( my $i = 0 ; $i < $targetCount ; ++$i )
  {
    param( 'IP'.$i,   $IPs[$i] ) ;
    param( 'port'.$i, $ports[$i] ) ;
  }
}
else
{
  # Parameters were fed into the script.  This indicates that the user
  # has either added or removed a target definition.  When this happens,
  # the complete form data is fed back to this script, including the
  # changes.  If an addition was made, then the new field was
  # created when the re-post URL was created.  Thus we only need to
  # count the targets and put up the data.  For a Deletion, the
  # parameter 'delete' will be defined as the index of the target to
  # remove.  

  #print( "<pre>Parameters Received<BR>" ) ;
  #foreach my $p (param())
  #{
  #    print  "$p=", param($p), "\n" ;
  #}
  #print "</pre>" ;

  # We want to indicate later that OK must be pressed to
  # make the changes official.
  $needsOK = 1 ;

  # Count how many targets are defined by looking for IPN
  # with N=0,1,2,3,4.  targetcount starts at zero.
  $targetCount = 0 ;
  while ( defined( param( 'IP'.$targetCount ) ) )
  {
    ++$targetCount ;
  }

  my $deleteIndex = param( 'delete' ) ;
  if ( defined( $deleteIndex ) )
  {
    #print "Deleting $deleteIndex", br() ;
    # To implement the Delete, we'll shift the IP and port parameters
    # above the selected index down one slot, and then erase the
    # last set.
    --$targetCount ;
    while ($deleteIndex < $targetCount)
    {
      param( 'IP'.$deleteIndex, 
             param( 'IP'.($deleteIndex+1) )
           ) ;
      param( 'port'.$deleteIndex, 
             param( 'port'.($deleteIndex+1) )
           ) ;
      ++$deleteIndex ;
    }
    CGI::delete( 'IP'.$deleteIndex ) ;
    CGI::delete( 'port'.$deleteIndex ) ;
  }
}

           
# Create the form with fields for key entry.
# A table is used to align labels, text entry fields, and the button on
# a central line.
print startform( -name=>"theForm", 
                 -action=>'/perl-scripts/sysadmin/setAdvertise.cgi'
               ) ;



my @Control = radio_group( -name=>'enable', 
                           -value=>[ 1, 0 ],
                           -nolabels=>1,
                         ) ;




print table( {border=>'1'},
             Tr(
                td( {width=>'30', align=>'center'}, $Control[0] ),
                td( b('Enabled'),
                    br(),
                    'Advertise available ports and other information '.
                    'to specified target systems.'
                  )
               ),
             Tr(
                td( {align=>'center'}, $Control[1] ),
                td( b('Disabled'),
                    br(),
                    "Don't advertise."
                  )
               )
           ) ;


# Widget to select the advertising interval.
my @intervals = ( 1, 5, 60, 300, 600, 3600 ) ;
my %intervalLabels = ( 1    => '1 Second',
                       5    => '5 Seconds',
                       60   => '1 Minute',
                       300  => '5 Minutes',
                       600  => '10 Minutes',
                       3600 => '1 Hour',
                     ) ;
print br(),
      br(),
      b('Interval&nbsp;&nbsp;'),
      popup_menu ( -name=>'interval',
                   -values=>\@intervals,
                   -labels=>\%intervalLabels,
                 ) 
      ;


# Format the Target table
print br(), 
      br(),
      b('Targets'),
      br() 
      ;

if ( $targetCount )
{
  # Create the table of targets.  A Title row, then one row per target.
  print "<table border=1>\n" ;
  print Tr( th('IP Address'), 
            th('&nbsp;UDP Port&nbsp;'), 
            th('&nbsp;Delete&nbsp;')
          ) ;
  my $basecount = 0 ;
  for (my $i = 0 ; $i < $targetCount ; ++$i )
  {
    print Tr( 
              # The target's IP address
              td( textfield( 'IP'.$i,
                             '',
                             20, 
                             15 
                           ) 
                ),

              # The target's port number
              td( {align => 'center'},
                  textfield( 'port'.$i,
                             '',
                             12, 
                             5 
                           ) 
                ),

              # The delete button
              td( {align=>'center'}, 
                  a( {href=>"javascript:deleteTarget($i)"},
                    img( { src=>'/images/redX.gif',
                          border=>'0',
                          alt=>'Delete' 
                         } 
                       )
                   )
                )
            ) ;
  }
  print "</table>\n" ;
}
else
{
  print "<br /><br />No targets specified!<br /><br />" ;
}



# A button to add a new target.  
# Calls the javascript function newTarget()
print br(),
      button( -name=>'AddNewTarget',
              -value=>"Add New Target",
              -onClick=>"newTarget( $targetCount )"
            ) ; 

# If this page has been altered (say by adding or deleting targets)
# then we need to warn the user that OK must be pressed.
if ($needsOK)
{
  print br(),
        br(),
        b( "Additions or Removals need OK to take effect." ) ;
}


# After a horizontal rule, put the OK and Cancel buttons.
print br(), 
      br(),
      hr(), 
      br(), 
      button( -name=>"OK", 
              -value=>"OK",
              -onClick=>'okayClicked()'
            ), 
      button( -name=>'Cancel', 
              -value=>'Cancel',
              -onClick=>
               'parent.loadDataFrame( "/perl-scripts/portSummary.cgi" )'
            )
      ;


print endform() ;


# Javascript functions used by the above forms.
print <<EOF ;
<script>


/**********************************************************************/
function newTarget( newCount )
/**********************************************************************/
/* Called when the New Target button is clicked.
 * This calls the CGI script again, but with updated parameters.
 * All of the existing widgets are encoded in the URL, and 
 * new IP and port parameters are added, based on the newCount.  The
 * whole thing is then submitted to the NetRS which will reformat the
 * page.
 */
{
  f = document.theForm ;

  // Create the URL/CGI-request to submit.
  // We want this to contain all of the CGI parameters for the
  // existing elements.  
  url = '/perl-scripts/advertise.cgi' ;

  // Go through the form's elements one by one and create a
  // CGI parameter for each of them.  Each parameter is prefixed with
  // a '&' except the first one which gets a '?'.
  // We only have to handle TextFields, radio buttons, and the
  // popup menus (== select-one)
  ch = '?' ;
  for (i = 0 ; i < f.elements.length ; ++i )
  {
    e = f.elements[i] ;
    if ( e.type == 'text'
      || e.type == 'radio' && e.checked
       )
    {
      url += ch + e.name + '=' + escape( e.value ) ;
      ch = '&' ;
    }
    else
    if (e.type == 'select-one')
    {
      p = e.options[e.selectedIndex].value ;
      url += ch + e.name + '=' + escape( p ) ;
      ch = '&' ;
    }
  }

  /* And we want the URL to contain parameters to create the a new 
   * element that isn't already on the page.  Default the address
   * portion to the client's address, and the netmask field to 24 
   * which is a class C subnet.  The defaults make it likely that
   * an accidentally created iptables setup will work okay and not
   * cut off communications.
   */
  url += '&IP' + newCount + '=' + '0.0.0.0'
       + '&port' + newCount + '=8000' ;

  // Submit the request, which will display the updated form.
  parent.loadDataFrame( url ) ;

} /* end of newTarget() */



/**********************************************************************/
function deleteTarget( targetIndex )
/**********************************************************************/
/* Called when a Delete target button is clicked.  The index of the
 * target to delete is passed in as the "targetIndex" parameter.
 *
 * This function calls the CGI script again, but with updated
 * parameters.  All of the existing widgets are encoded in the URL.
 * A special 'delete=' parameter is appended to indicate the index
 * of the target we want deleted.
 * The whole thing is then submitted to the NetRS which will
 * reformat the page.
 */
{
  f = document.theForm ;
  
  // Start creating the URL/CGI-request to submit.
  url = '/perl-scripts/advertise.cgi' ;

  // Go through the form's elements one by one and create a
  // CGI parameter for each of them.  Each parameter is prefixed with
  // a '&' except the first one which gets a '?'.
  // We only have to handle TextFields, radio buttons, and the
  // popup menus (== select-one)
  ch = '?' ;
  for (i = 0 ; i < f.elements.length ; ++i )
  {
    e = f.elements[i] ;
    if ( e.type == 'text'
      || e.type == 'radio' && e.checked
       )
    {
      url += ch + e.name + '=' + escape( e.value ) ;
      ch = '&' ;
    }
    else
    if (e.type == 'select-one')
    {
      p = e.options[e.selectedIndex].value ;
      url += ch + e.name + '=' + escape( p ) ;
      ch = '&' ;
    }
  }

  /* Indicate that of those existing fields, we want to delete
   * one of them.  The perl script will figure out how to do that.
   */
  url += '&delete=' + targetIndex ;

  // Submit the request, which will display the updated form.
  parent.loadDataFrame( url ) ;
 
} /* end of deleteTarget() */



/**********************************************************************/
function okayClicked()
/**********************************************************************/
/* Called when the OK button is clicked.
 * This triggers submission of the form data.
 *
 * Before submission, the text entry fields are checked for valid
 * numeric values.  Errors will put up an alert and submissions will
 * be canceled.
 */
{
  f = document.theForm ;

  // Check all of the text entry fields to be certain that they
  // contain valid IP addresses or port numbers.
  for (i = 0 ; i < f.elements.length ; ++i )
  {
    e = f.elements[i] ;
    if ( e.type == 'text' )
    {
      if (e.name.indexOf( 'IP' ) == 0)
      {
        ip = e.value ;
        if (parent.notValidIP( ip ))
        {
          alert( "Invalid IP address " + ip ) ;
          return ;
        }
      }
      else
      if (e.name.indexOf( 'port' ) == 0)
      {
        port = e.value ;
        if ( parent.isNotAnInteger( port, 'UDP Port' ) )
        {
          return ;
        }
        if (port > 65535)
        {
          alert( "UDP Port must be less than 65536" ) ;
          return ;
        }
      }
    }
  }

  parent.submitForm( f, '' ) ;

} /* end of okayClicked() */


</script>
EOF



print end_html() ;

########################################################################
# end of file advertise.cgi
########################################################################
