#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: antenna.cgi,v 1.18 2004/04/23 18:52:31 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/antenna.cgi,v $

########################################################################
# Antenna.cgi
#
# CGI Script to allow selection of an antenna type, serial number,
# height, and the method to be used for measuring height for that
# antenna.  Puts up an HTML form allowing the user to specify those
# values, along with an OK button to accept the values.
#
#   Antenna.cgi
#       No parameters: Initialize the form with current data
#
#   Antenna.cgi?update=antType&...
#       The antenna type has been changed, so we need to update
#       the form data.
#
# When the OK button is clicked, the new selection is fed to another
# script in the secure gpsadmin/ location.  Thus, the user can view
# the settings and choices and is only inhibited from a final change to
# the settings if they lack the proper password.
#       
########################################################################


use strict ;
use CGI qw/:standard/;
use Fireball ;

titleWithHelp( 'Antenna Setup',
               '/antennaInfo.html'
             ) ;


# Read the anteanna database from the file
# /usr/local/fireball/antennaModel.config.
# Parse the results into a giant hash with sub-hashes and sub-arrays.
# See Fireball.pm for specifications.
my %antInfo ;
getAntennaInfo( \%antInfo ) ;


# From that information we need to parse some simpler forms,
# for use by the popup menus, etc.
my @antTypes = keys %antInfo ;

# Create a hash that maps from antenna types to antenna names.
my %antNames ;
for my $type (@antTypes)
{
    $antNames{$type} = $antInfo{$type}{'name'} ;
}

# Sort the list of antenna types by their antNames.
# This makes the entries show up sorted on the popup menu defined later.
# For consistency with other Trimble products, we want Type-0, the
# 'Unknown External' antenna, to show up on the end of the list.
@antTypes = sort( { ($b != 0 and $a != 0)
                    ? $antNames{$a} cmp $antNames{$b} 
                    : ($a == $b) 
                      ? 0
                      : ($a == 0)
                        ? +1
                        : -1
                  }
                  @antTypes 
                ) ;

#print "<pre>" ;
#print "Parameters:\n" ;
#for my $p (param())
#{
#  print "$p = ", param($p), "\n" ;
#}
#print "</pre>" ;


my $type = param('update') ;
if ( defined $type and ($type ne '') )
{
  # User has selected a new antenna type.  Use that value
  # and switch the method to the default for that type.
  param( 'antType', $type ) ;
  param( 'method', $antInfo{$type}{'defMethod'} ) ;
}
else
{
  # No cgi parameters were appended.  Startup the page with 
  # current settings as defaults.

  # Read current settings into a hash.
  my %antSetup ;
  getAntennaSetup( \%antSetup ) ;

  # Default the buttons to match those settings.
  param( 'antType', $antSetup{ 'type' } ) ;
  param( 'method',  $antSetup{ 'method' } ) ;
  param( 'serial',  $antSetup{ 'serial' } ) ;
  param( 'textSerial',  $antSetup{ 'textSerial' } ) ;
  param( 'height',  $antSetup{ 'height' } ) ;
}



print startform( -name=>"theForm",
                 -action=>'/perl-scripts/gpsadmin/setAntenna.cgi' 
               ) ;

print '<table border=0>' ;

print Tr( td( {align=>'right'}, b( "Numeric Serial Number:" ) ),
          td( {align=>'center'}, textfield( 'serial', '', 10, 8 ) )
        ) ;

print Tr( td( {align=>'right'}, b( "Text Serial Number:" ) ),
          td( {align=>'center'}, textfield( 'textSerial', '', 25, 20 ) )
        ) ;

print Tr( td( {align=>'right'}, b("Antenna Name:") ),
          td( popup_menu ( -name=>'antType',
                           -values=>\@antTypes,
                           -labels=>\%antNames,
                           -onChange=>'update() ;',
                           -default=>'Unknown',
                         ),
            )
        ) ;

$type = param( 'antType' ) ;


print hidden( -name=>'antName',
              -value=>$antNames{$type}
            ) ;

print Tr( td( {align=>'right'}, b( "Antenna Type:" ) ),
          td( {align=>'center'}, $type )
        ) ;

print Tr( td( {align=>'right'}, b( "NGS/IGS Name:" ) ),
          td( {align=>'center'}, $antInfo{$type}{'ngs'} )
        ) ;
        
print Tr( td( {align=>'right'}, b( "Character Code:" ) ),
          td( {align=>'center'}, $antInfo{$type}{'code'} )
        ) ;

my $methString ;
my $methCount = $antInfo{$type}{'methodCount'} ;
if ($methCount > 1)
{
  my %methodLabels ;
  for ( my $m = 0 ; $m < $methCount ; ++$m)
  {
    $methodLabels{$m} = $antInfo{$type}{'methods'}[$m] ;
  }
  $methString = popup_menu( -name=>'method',
                            -values=>[0..$methCount-1],
                            -labels=>\%methodLabels ,
                            -default=>$antInfo{$type}{'defMethod'}
                          ) ;
}
else
{
  $methString = $antInfo{$type}{'methods'}[0]
              . hidden( -name=>'method',
                        -value=>'0'
                      ) ;
}

print Tr( td( {align=>'right'}, b( "Height:" ) ),
          td( {align=>'center'}, textfield( 'height', '', 8, 8 ) )
        ) ;

print Tr( td( {align=>'right'}, b("Height Reference Point:") ),
          td( {align=>'center'},$methString )
        ) ;


print '</table>' ;
 
print hr() ;
print button( -name=>"OK", 
              -value=>"OK",
              -onClick=>'Okay()'
            ) ;
print "&nbsp;" ;
print button( -name=>"Cancel", 
              -value=>"Cancel",
              -onClick=>'parent.loadDataFrame("rconfigDisplay.cgi")',
            ) ;

print endform() ;

print <<EOF ;
<script>

/**********************************************************************/
function update()
/**********************************************************************/
/* The user has changed the antenna type.  This requires us to
 * redraw the form, showing popups and other info relevant to the new
 * antenna type.  To do this we construct a url containing the data
 * in the form fields, and the CGI parameter "update=ANTTYPE".  
 * The request is submitted and the new page comes back.  
 * We use loadDataFrame to first blank the page before loading the new
 * URL.  This prevents inadvertent OK presses, multiple submissions,
 * etc.
 */
{
  f = document.theForm ;
  index = f.antType.selectedIndex ;
  antType = f.antType.options[index].value ;
  serial = f.serial.value ;
  textSerial = f.textSerial.value ;
  height = f.height.value ;

  //alert( "Antenna changed " + f.antType.selectedIndex + antType ) ;
  parent.loadDataFrame( 
                    'antenna.cgi?update=' 
                    + escape(antType) 
                    + '&serial='
                    + escape(serial)
                    + '&textSerial='
                    + escape(textSerial)
                    + '&height='
                    + escape(height)
                    )
                    ;

} /* end of update() */


/**********************************************************************/
function Okay()
/**********************************************************************/
/* Invoked when the OK button is pressed.
 * This just submits the entire form with all parameters.
 * Suppressed if we've already done a submission so we won't get 
 * two of the same request.
 */
{
  f = document.theForm ;


  // Verify that the antenna serial number is an integer.
  if ( parent.isNotAnInteger( f.serial.value, 'Serial number' ) )
    return ;

  parent.submitForm( document.theForm, '' ) ;

} /* end of Okay() */


</script>
EOF

print end_html() ;

########################################################################
# End of file antenna.cgi
########################################################################
