#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: autoDelete.cgi,v 1.16 2004/06/15 17:59:46 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/autoDelete.cgi,v $

########################################################################
# autoDelete.cgi
#
# Script for controlling the Auto file deletion mode, and log file pool
# controls.  See the autoDeleteInfo.html help file for a description
# of how this all works.
#
# This script is called without any parameters.  It puts up a form
# that allows the user to control the AutoDelete function.  
# The displayed form data is mostly a table with a row for each "pool"
# of logging files.  The values shown will represent the current
# contents of the configuration file.
#
# If the user uses the form controls to add or delete lines from the
# table, then this script will be re-invoked with a bunch of parameters,
# representing any updates the user may have made.  The page will be
# regenerated from the new data instead of the original stuff.
#
# When the user clicks OK, the updated paramters are fed to the access
# controlled script 'gpsadmin/setAutoDelete.cgi', which will update the
# configuration file with the new data.
#
########################################################################

use strict ;
use CGI qw/:standard/;

use Fireball ;


titleWithHelp( 'Data Logging AutoDelete Mode',
               '/autoDeleteInfo.html'
             ) ;


# This flag is used to indicate later that a warning should be put on
# the page that the OK button needs to be pressed to make changes 
# take effect.  This is especially important if lines have been added or
# deleted to the pool-list.
my $needsOK = 0 ;

if (! param() )
{
  # No parameters were fed into this script.
  # So we generate a fresh page using data from the configuration file.
  # This is done by reading in the current config data, and then
  # manually setting the CGI-parameters needed to draw the form.

  my %autoDeleteState ;

  getAutoDeleteControls( \%autoDeleteState ) ;

  # Transfer the data into the CGI parameters.
  for my $pool (keys %autoDeleteState)
  {
    param( "size_$pool", $autoDeleteState{$pool}{'size'} ) ;
    param( "enable_$pool", $autoDeleteState{$pool}{'enable'} ) ;
  }
}
else
{
  # Parameters were fed into the script.  This indicates that the user
  # has either added or removed a pool definition.  When this happens,
  # the complete form data is fed back to this script, including the
  # changes.  We want to indicate later that OK must be pressed to
  # make the changes official.

  $needsOK = 1 ;
}


print start_form( -name=>'theForm', 
                  -action=>'/perl-scripts/gpsadmin/setAutoDelete.cgi'
                ) ;

print hr() ;
print h3( 'Reserved Space' ) ;
print <<EOF ;
Delete oldest files as needed to maintain the <br />
specified amount of free data logging space.
<br /><br />
EOF
print table( Tr( 
                 td( checkbox( -name=>"enable_Reserved",
                               -value=>1,
                               -label=>''
                             )
                   ),
                 td( 'AutoDelete' ),
                 td( {width=>18}, '&nbsp;' ),
                 # A text field for entering the size of the pool.
                 td( {align=>'center'}, 
                     textfield( -name=>"size_Reserved", 
                                -size=>5,
                                -maxlength=>5,
                                -justification=>'CENTER'
                              )
                   ),
                 td( 'MBytes' ), 
               )
           ) ; 


# The main element is a table with a row for each pool that is defined.
print hr(), h3( 'Session Pools' ) ;

print <<EOF ;
For these Session Identifiers, don't create more than the 
specified amount of data.  
<br />If AutoDelete is enabled and a pool overflows, then delete the
oldest file in that pool.  
<br />If AutoDelete is disabled and a pool overflows, then stop logging
for that session.
<br /><br />
EOF

print '<table border=1>' ;


# Put on a title for each column.
print Tr( th( [ 'Pool',
                '&nbsp;&nbsp;MBytes&nbsp;&nbsp;', 
                'Auto<br />&nbsp;&nbsp;Delete&nbsp;&nbsp;', 
                '&nbsp;&nbsp;Remove&nbsp;&nbsp;'] 
            ) 
        ) ;

# For each possible pool, in a prefered order, see if we need to
# generate a row of data.  The pool is defined if a pool size exists.
# Also make a list containing all of the undefined pools, which will
# be used for the AddPool popup menu.
my @undefinedPools = ( '-Select-' ) ;
my $poolCount = 0 ;

for my $pool ('a'..'z','Default')
{
  if ( param("size_$pool") )
  {
    ++$poolCount ;
    print Tr( 
              # The name of the pool.
              td( {align=>'center'}, 
                  "&nbsp;&nbsp;$pool&nbsp;&nbsp;" 
                ),
              # A text field for entering the size of the pool.
              td( {align=>'center'}, 
                  textfield( -name=>"size_$pool", 
                             -size=>5,
                             -maxlength=>5,
                             -justification=>'CENTER'
                           )
                ),
              # An autoDelete checkbox for the pool.
              td( {align=>'center'}, 
                  checkbox( -name=>"enable_$pool",
                            -value=>1,
                            -label=>'',
                          ), 
                ),
              # A button to undefine the pool.
              td( {align=>'center'}, 
                  a( {href=>"javascript:deletePool('$pool')"},
                    img( { src=>'/images/redX.gif',
                          border=>'0',
                          alt=>'Delete' 
                         } 
                       )
                   )
                )
            ) ;
  }
  else
  {
    # This pool isn't defined.  If not the Reserved pool, add it
    # to the list of undefined pools that will appear on the 
    # AddPool popup.
    push @undefinedPools, $pool ;

  }
}

if ($poolCount == 0)
{
print Tr( td( {colspan=>'4', align=>'center'}, 'None defined.' ) ) ;
  
}

print "</table>" ;
print br() ;

if ($needsOK)
{
  print b( "Additions or Removals need OK to take effect." ), 
        br(), 
        br() ;
}

print "Add New Pool:&nbsp;",
      popup_menu ( -name=>'newPool',
                   -values=>\@undefinedPools,
                   -default=>'',
                   -onChange=>'createNewPool()'
                 ) ;


print br(), hr() ;
print button( -name=>'OK', 
              -value=>'OK',
              -onClick=>'okayClicked()'
            ) ;

print button( -name=>'Cancel', 
              -value=>'Cancel',
              -onClick=>
                  'parent.loadDataFrame( "/perl-scripts/sessions.cgi" )'
            ) ;
print end_form() ;


# Javascript functions used by the above forms.
print <<EOF ;

<script>

/**********************************************************************/
function createNewPool()
/**********************************************************************/
/* Called when the AddNewPool popup selector makes a choice.  The
 * idea is to create a new pool with default values.  To do this, we
 * submit the form back to the original formatting script,
 * autoDelete.cgi, with the existing parameters, AND new parameters
 * showing the defaults for the new pool.  AutoDelete.cgi redraws the
 * page with the parameters.
 */
{
  f = document.theForm ;

  // The select item is an element in the form named 'newPool'.
  e = f.newPool ;

  // Determine the index in the popup choices of the selected item.
  index = e.selectedIndex ;

  // The popup has a dummy entry, '-Select-', as the first option,
  // so if that is somehow selected, don't do anything.
  if (index == 0) return ;

  // Determine the selected pool name.
  name = e.options[index].value ;

  // Start creating the URL/CGI-request to submit.
  url = '/perl-scripts/autoDelete.cgi' ;

  // Add the new pool parameters.
  url += "?enable_" + name + "=1&size_" + name + "=1" ;

  // Determine which other pools are already being displayed by
  // going through the existing form elements.  Any checkboxes or
  // text entry fields will require an addition to the URL.
  for (i = 0 ; i < f.elements.length ; ++i )
  {
    e = f.elements[i] ;
    if ( e.type == 'text' )
    {
      url += '&' + e.name + '=' + escape( e.value ) ;
    }
    else if ( e.type == 'checkbox' && e.checked )
    {
      url += '&' + e.name + '=' + escape( e.value ) ;
    }
  }

  //alert( "Creating New Pool " + index + name + " " + url ) ;

  // Submit the request, which will display the updated form.
  parent.loadDataFrame( url ) ;

} /* end of createNewPool() */



/**********************************************************************/
function deletePool( pool)
/**********************************************************************/
/* Called when a Delete Pool link/button is clicked.  THe name of
 * the pool to delete is passed in as the "pool" parameter.
 * The idea is that we want the pool to disappear from the table of
 * pools.  But since we haven't hit "OK" yet, this is purely a graphical
 * deletion.  The way this works is to refresh the original page, by
 * submitting a URL that leaves out the CGI parameters for the specified
 * pool.  The CGI script will redraw the page without the deleted line.
 */
{
  f = document.theForm ;
  
  // THe URL/CGI-request to send starts out with no parameters.
  url = '/perl-scripts/autoDelete.cgi' ;

  // The first parameter separation character is '?' but this is
  // switched to '&' after the first one.
  c = '?' ;

  // Go through all of the elements of the form to determine which 
  // pools are being displayed.  Any checkboxes or text entry fields
  // will require an addition to the URL.  But we'll drop any
  // elements that match the pool we are deleting.
  for (i = 0 ; i < f.elements.length ; ++i )
  {
    e = f.elements[i] ;
    if ( e.type == 'text' )
    {
      if (e.name != ('size_' + pool) )
      {
        url += c + e.name + '=' + escape( e.value ) ;
        c = '&' ;
      }
    }
    else if ( e.type == 'checkbox' && e.checked )
    {
      if (e.name != ('enable_' + pool) )
      {
        url += c + e.name + '=' + escape( e.value ) ;
        c = '&' ;
      }
    }
  }
  //alert( "Deleting " + pool + ' ' + url ) ;

  // Submit the request, which will display the updated form.
  parent.loadDataFrame( url ) ;
  
} /* end of deletePool() */



/**********************************************************************/
function okayClicked()
/**********************************************************************/
/* Called when the OK button is clicked.
 * This triggers submission of the form data.
 *
 * Before submission, the text entry fields are checked for valid
 * numeric values.  Errors will put up an alert and submissions will
 * be canceled.
 */
{
  f = document.theForm ;

  // Check all of the text entry fields.
  for (i = 0 ; i < f.elements.length ; ++i )
  {
    e = f.elements[i] ;
    if ( e.type == 'text' )
    {
      name = e.name ;
      if ( name == "size_Reserved" )
        name = "Reserved Space size" ;
      else
      {
        name = name.substr(name.length-1) + "-Pool size" ;
      }

      if ( parent.isNotAnInteger( e.value, name ) )
        return ;
    }
  }

  parent.submitForm( f, '' ) ;

} /* end of okayClicked() */


</script>
EOF


print end_html() ;

########################################################################
# end of script autoDelete.cgi
########################################################################
