#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: changePassword.cgi,v 1.18 2003/12/19 01:46:21 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/changePassword.cgi,v $

########################################################################
# changePassword.cgi
#
# Script to allow an individual user to modify their password.
# Can be called one of several ways:
#   o With no CGI parameters - 
#       We put up a form waiting for entry of an account, the
#       old password, the new password, and a confirmation of new
#       password.
#
#   o With CGI parameters specifying an attempt to change the 
#       password.  Parameters are checked to see if a proper account
#       and its correct password are there.  Then the new password
#       is verified and saved in the database.
#       Parameters are:
#         AccountName   : Name of the account to change.
#         OldPassword   : Previous password on that account.
#         Password1     : New password.
#         Password2     : New password verification.
#
#
#
# These passwords are only used for Webpage access authentication.
# The list is maintained in /etc/sysconfig/htpasswd.  The format
# there is:
#   sysadmin:z2Yxdf70dk2kk
#   tom:FB5eywiIw5Gbk
#   fireball:q5n7fxtVhlhFY
#
# This page is also used as the "backdoor" method of retrieving access
# to the system, should the sysadmin password be lost or corrupted.
# If the account is "sysadmin" then a special password is checked in
# addition to the one in htpasswd.  The backdoor pw is based on the
# receiver's mac address, and cannot be changed.  It only allows 
# entry of a new sysadmin password, so that access can be restored.
#
########################################################################


use CGI qw/:standard/;
use strict ;
use Fireball ;

titleWithHelp( 'Security - Change Password',
               '/changePwInfo.html'
             ) ;
           
           
########################################################################
# Process any CGI parameters fed into the script.
#
# This block may detect errors which will halt further parameter
# processing.  If this happens, we set the $error, and use a
# "goto paramDone" to exit the block.  Inelegant, I know, but
# it's the most straightforward method and avoid lots of extraneous 'if'
# statements.
# The error string is saved in $error (which is blank otherwise) so that
# the value can be displayed at the end of the returned page.
########################################################################
my $error = "" ;

if (param())
{
  
  #print 'Parameters:<br />' ;
  #for (param())
  #{
  #  print "$_ = ",param($_),'<br />' ;
  #}

  ######################################################################
  # Open and read the current acount information. 
  # We need this information to:
  #   o Check if the account name already exists.  
  #   o Check if the proper old password was entered.
  #   o Update the password and save the new data.
  # We'll parse the data into the hash %accounts, keyed by the account
  # data.  The encrypted password is in sub-hash 
  # $account{name}{'password'}.
  ######################################################################
  my %accounts ;
  getGuiAccounts( \%accounts ) ;

  ######################################################################
  # Determine the backdoor password.
  # This might be used later if the sysadmin backdoor is accessed.
  ######################################################################
  my $backdoor = qx( /usr/local/fireball/bkdoor ) ;
  chomp $backdoor ;


  # Extract and clean up the incoming parameters.
  my $acct = param('AccountName') ;
  $acct =~ s/^ *// ;    # Strip leading and trailing spaces
  $acct =~ s/ *$// ;

  my $oldpw = param('OldPassword' ) ;
  my $pw1 = param('Password1' ) ;
  my $pw2 = param('Password2' ) ;

  # What do we check for?\
  #   o Account name not blank.
  #   o Account name exists (or is 'sysadmin')
  #   o Old password entered and matches existing one.
  #     or if acct = sysadmin, matches backdoor
  #   o Password1 entered.
  #   o Password2 matches password1.

  if ($acct eq "")
  { 
    $error="An Account Name must be specified!" ; 
    goto paramDone ;
  }

  # The account name must already exist in the password database.
  # In case of problems with that, we will always accept the "sysadmin"
  # account.
  if ( (! exists $accounts{$acct}) and ($acct ne 'sysadmin') )
  {
    $error = "Account $acct doesn't exist!" ;
    goto paramDone ;
  }

  # Check that either the entered "old password" matches what is
  # currently set for this account, or that this is a backdoor
  # access (sysadmin with backdoor password).  If either case,
  # move on.
  my $currentpw = $accounts{$acct}{'password'} ;
  if ( ( $currentpw ne crypt( $oldpw, $currentpw ) )
   and ( ($oldpw ne $backdoor) or ($acct ne 'sysadmin') )
     )
  {
    $error="Current password not correct!" ;
    param("OldPassword","" ) ; # Clear this field
    goto paramDone ;
  }

  if ( $pw1 eq "" )
  { 
    $error="A new Password must be specified!" ; 
    goto paramDone ;
  }

  if ( $pw1 ne $pw2 )
  { 
    $error="New Passwords Do Not match!" ;
    param("Password1","" ) ; # Clear this field
    param("Password2","" ) ; # Clear this field
    goto paramDone ;
  }

  ######################################################################
  # Everything is okay.  Update the password database with the new
  # information.
  ######################################################################

  # Encrypt new password.
  $accounts{$acct}{'password'} = makepw( $pw1 ) ;

  updateGuiAccounts( \%accounts ) ;


  # Display a message indicating acceptance
  print h1("Password Change Accepted.") ;

  # After a timeout, go back to Change password page.
  print <<EOF ;
  <Script>
  parent.loadAfterDelay( "changePassword.cgi", 1000) ;
  </Script>
EOF

  # Done with page.  Skip the rest of this script.  
  # Can't use exit() due to ModPerl constraints.  So just go to the
  # end and run off the edge.
  goto EXIT ;
} 
paramDone:  # This is the exit point if parameter processing was
            # aborted in the preceding block.


########################################################################
# When we get to here, either this is a first invocation of the script,
# or something was wrong with the user's input.  Redraw the forms
# (perhaps with an error message alert) and wait for further input.
########################################################################


# Create the form with fields for account data entry.
# A table is used to align labels, text entry fields, and the button on
# a central line.
print startform( -name=>"theForm" ) ;
print table( 
             Tr( th( {-align=>'right'}, 'Account Name' ),
                 td( {-align=>'left'},
                     textfield( 'AccountName', '', 12, 12 ) )
               ) ,

             Tr( th( {-align=>'right'}, 'Current Password' ),
                 td( password_field( 'OldPassword', '', 12, 12 ) )
               ) ,

             Tr( th( {-align=>'right'}, 'New Password' ),
                 td( password_field( 'Password1', '', 12, 12 ) )
               ) ,

             Tr( th( {-align=>'right', -width=>'50%'}, 
                     'Re-Enter New Password' ),
                 td( {-align=>'left', -width=>'50%'},
                     password_field( 'Password2', '', 12, 12 ) )
               ) ,
             Tr( td( '' ),
                 td( submit( "change", "OK" ) )
               ) ,
           ) ;
print endform() ;

# If an error message was found earlier during CGI processing,
# post an alert message to the user.  Also display the message on the
# bottom of the page, in case the javascript alert system doesn't work.
if ( $error )
{
  print << "EOF" ;
  <Script> 
  alert( "$error" ) ;
  </Script>
EOF

  print "$error" ;
}

EXIT:
print end_html() ;

########################################################################
# end of file changePassword.cgi
########################################################################
