#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: configCreate.cgi,v 1.10 2004/04/21 23:56:22 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/configCreate.cgi,v $

########################################################################
# configCreate.cgi
# Script for Creating Fireball Configuration Files
# 
# Via the Web-GUI interface we can create, delete, download, upload and 
# apply "configuration files".  These are collections of Fireball
# control settings.  The files reside in /etc/sysconfig/configFiles.
#
# This script handles the actual creation of a configfile.  It puts up
# a form allowing the user to specify the Name of the file, along with a
# bunch of checkboxes that control which system configuration
# information blocks will be included in the config file.
# 
# Called with NO CGI parameters.
#
# When the user clicks SAVE, the sysadmin secured script 
# perl-scripts/sysadmin/configSave.cgi is called with the name and
# the desired parameters from the textfield and checkboxes.
#
########################################################################

use strict ;
use CGI qw/:standard/;
use Fireball ;

titleWithHelp( 'Configuration File Creation',
               '/configFilesCreate.html'
             ) ;


my @existingConfigFiles = () ;
if (-d '/etc/sysconfig/configFiles' )
{
  chdir '/etc/sysconfig/configFiles' ;
  @existingConfigFiles = glob '*' ;
}

# We need this javascript for processing the form data.

print "<script>\n" ;

printf "existingFileCount = %d ;\n", scalar(@existingConfigFiles) ;
print "existingFiles = new Object() ;\n" ;
for my $i (0 .. $#existingConfigFiles)
{
  printf "existingFiles[%d] = '%s' ;\n", $i, $existingConfigFiles[$i] ;
}



print <<'EOF';


/********************************************************************/
function Everything()
/********************************************************************/
{
  f = document.theForm ;
  if ( f.everything.checked )
  {
    for (i = 0 ; i < f.elements.length ; ++i )
    {
      if (f.elements[i].type == 'checkbox' )
      {
        f.elements[i].checked = true ;
      }
    }
  }
} /* end of Everything() */

/********************************************************************/
function NotEverything()
/********************************************************************/
{
  f = document.theForm ;
  for (i = 0 ; i < f.elements.length ; ++i )
  {
    if (f.elements[i].type == 'checkbox' )
    {
      if ( ! f.elements[i].checked )
      {
        f.everything.checked = false ;
        return ;
      }
    }
  }
} /* end of NotEverything() */

function saveClicked()
{
  f = document.theForm ;

  newName = f.fname.value ;

  for ( i = 0 ; i < existingFileCount ; ++i )
  {
    if (newName == existingFiles[i])
    {
      if ( ! confirm( "File name " 
                    + newName 
                    + " is already used.\nOverwrite?" 
                    )
         ) return ;
    }
  }

  parent.submitForm( f, "" ) ;

} /* end of saveClicked() */

</script>
EOF


print startform( -name=>"theForm",
                 -action=>'/perl-scripts/sysadmin/configSave.cgi'
               ) ;

print h3( 'Save current configuration...' ) ;

print p(),
      b( 'File Name ' ),
      textfield( -name=>'fname',
                 -default=>'ConfigFile', 
                 -size=>64, 
                 -maxlength=>256 
               ) ;

print p(), 
      'Include configuration information for the following features:' ;

print '<table border="0" width="90%">' ;

print Tr( 
          td( {align=>'center'},
              checkbox( -name=>'IOconfig',
                        -value=>'1', 
                        -label=>'', 
                        -onClick=>'NotEverything();'
                      ) 
            ),
          td( b('I/O configurations'),
              br(),
              'Includes Serial, TCP & UDP Port specifications, as well'
             .' as PPP setups, Met Controls, etc.',
            ), 
        ) ;

print Tr( 
          td( {align=>'center'},
              checkbox( -name=>'refstation',
                        -value=>'1', 
                        -label=>'', 
                        -onClick=>'NotEverything();'
                      ) 
            ),
          td( b('Reference Station and Antenna Setup.'),
            ),
        ) ;

print Tr( 
          td( {align=>'center'},
              checkbox( -name=>'sessions',
                        -value=>'1', 
                        -label=>'', 
                        -onClick=>'NotEverything();'
                      ) 
            ),
          td( b('Data Logging Sessions.'), 
              br(),
              'Includes all defined sessions, the ',
              'AutoDelete function and the Power Saving controls.'
            ),
        ) ;

print Tr( 
          td( {align=>'center'},
              checkbox( -name=>'stinger',
                        -value=>'1', 
                        -label=>'', 
                        -onClick=>'NotEverything();'
                      ) 
            ),
          td( b('Miscellaneous Receiver Controls.'),
              br(),
              'Includes Masks, Clock Steering, Multipath, ',
              'One-Pulse-Per-Second Output, GPS satellite controls, ',
              'and WAAS controls.'
            ),
        ) ;

print Tr( 
          td( {align=>'center'},
              checkbox( -name=>'extref',
                        -value=>'1', 
                        -label=>'',
                        -onClick=>'NotEverything();'
                      ) 
            ),
          td( b('External Frequency Source'),
            ), 
        ) ;

print Tr(
          td( {align=>'center'},
              checkbox( -name=>'ethernet',
                        -value=>'1', 
                        -label=>'', 
                        -onClick=>'NotEverything();'
                      ) 
            ),
          td( b('Ethernet Setup Controls'),
            ),
        ) ;

print Tr(
          td( {align=>'center'},
              checkbox( -name=>'IPfiltering',
                        -value=>'1', 
                        -label=>'', 
                        -onClick=>'NotEverything();'
                      ) 
            ),
          td( b('IP Filtering'), 
            ),
        ) ;

print Tr(
          td( {align=>'center'},
              checkbox( -name=>'http',
                        -value=>'1', 
                        -label=>'', 
                        -onClick=>'NotEverything();'
                      ) 
            ),
          td( b('HTTP Port Controls'), 
            ),
        ) ;

print Tr(
          td( {align=>'center'},
              checkbox( -name=>'ftp',
                        -value=>'1', 
                        -label=>'', 
                        -onClick=>'NotEverything();'
                      ) 
            ),
          td( b('FTP Access Controls'), 
            ),
        ) ;

print Tr(
          td( {align=>'center'},
              checkbox( -name=>'ntp',
                        -value=>'1', 
                        -label=>'', 
                        -onClick=>'NotEverything();'
                      ) 
            ),
          td( b('Network Time Protocol (NTP)'),
            ),
        ) ;

print Tr(
          td( {align=>'center'},
              checkbox( -name=>'security',
                        -value=>'1', 
                        -label=>'', 
                        -onClick=>'NotEverything();'
                      ) 
            ),
          td( b('Security Setup'),
              br(),
              'Includes access controls and http accounts.'
            ),
        ) ;

print Tr( td( '&nbsp;' ) ) ;

print Tr(
          td( {align=>'center'},
              checkbox( -name=>'everything',
                        -value=>'1', 
                        -label=>'', 
                        -onClick=>'Everything();'
                      ) 
            ),
          td( b('Save EVERYTHING'),
              br(),
              'Includes all of the above items and the System Name.'
            ),
        ) ;

print '</table>' ;

print br() ;
print button( -name=>'Save', 
              -value=>'Save',
              -onClick=>'saveClicked()'
            ) ;
print '&nbsp;' ;
print button( -name=>'Cancel', 
              -value=>'Cancel',
              -onClick=>
               'parent.loadDataFrame( "/perl-scripts/configFiles.cgi" )'
            ) ;

print endform() ;

print end_html() ;

########################################################################
# end of file configCreate.cgi
########################################################################
