#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: configFiles.cgi,v 1.25 2004/05/10 22:53:47 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/configFiles.cgi,v $

########################################################################
# configFiles.cgi
# Script for controling Fireball Configuration Files
# 
# Via the Web-GUI interface we can create, delete, download, upload and 
# apply "configuration files".  These are collections of Fireball
# control settings.  The files reside in /etc/sysconfig/configFiles.
#
# This script is the entry point to all of that functionality.  It
# displays the existing configfiles in a table, providing links to
# apply, download and delete them.  It also lets you display the
# contents (which capability may go away), create new files, and
# upload config files.  Beyond displaying the table and the
# contents of a file, this page does nothing but provide links to
# other configFile scripts.
#
#
# CGI parameters.
#
# No parameters: 
#       Put up the default display, showing existing config
#       files, along with some buttons for applying, downloading or
#       deleting them.
# 
# show=fname
#       Display the contents of the named configuration file.
#       This is primarily for diagnostic use and may not be kept.
#       Assumes the file is all Ascii.
########################################################################

use strict ;
use CGI qw/:standard/;
use Fireball ;

titleWithHelp( 'Configuration Files',
               '/configFiles.html'
             ) ;


# Make all links blue with no underlines, even if visited, etc.
# This is a document-level style specification, affecting all
# links in the frame.  This may not work properly on some browsers.
print <<EOF ;
<style type="text/css">
 <!--
       a {color:blue ; text-decoration: none }
 -->
</style>
EOF


#if (param())
#{
#  print "Parameters Received:<br />" ;
#  foreach my $p (param()) 
#  {
#    print "$p=",param($p),br() ;
#  }
#}


# After the title block, we may put a result indicator that may have
# come in from another config file routine.
# To keep things on the page from shifting around, we always send out a
# line, even when there isn't a result.  
if (my $msg = param('message'))
{
  print h4( $msg ) ;
}
else
{
  print h4( '&nbsp;' ) ;
}


# Create the table showing existing configuration files.
chdir '/etc/sysconfig/configFiles' ;
my @allfiles = glob '*' ;
if (@allfiles)
{
  print '<Table border align="center">' ;
  print Tr( th( ['File','Apply','Download','Delete'] ) ) ;

  for my $f (@allfiles)
  {
    my $ef = CGI::escape($f) ;
    $f =~ s/ /&nbsp;/g ;
    print Tr( 
              td( '&nbsp;', 
                  a( {href=>"configFiles.cgi?show=$ef"},
                  $f
                  ),
                  '&nbsp;'
                ),
              td(  {align=>'center'}, 
                   a( {href=>"javascript:Apply('$f')"},
                   img( { src=>'/images/bluedot.gif', 
                          border=>'0',
                          alt=>'Apply'
                        }
                      )
                   )
                ),

              # Notes on the download link supplied below:
              # We give the user a button to click to download a
              # selected configuration file.  This posts a pseudo-URL of
              # the form:
              #
              #   http://netRSXXX/configDownload/configFileName
              #
              # Such a URL will be redirected by our web server due to
              # scriptAliasMatch directives in the httpd configuration.
              # All such requests [starting with /configDownload/] end
              # up calling perl-scripts/sysadmin/configDownload.cgi,
              # which determines the path to the file from the original
              # requesting URL.  Note that this link doesn't actually
              # change the webpage displayed in the browser.  We just
              # want to download the file.
              td(  {align=>'center'}, 
                   a( {href=>"/configDownload/$ef"},
                   img( { src=>'/images/bluedot.gif', 
                          border=>'0',
                          alt=>'Download'
                        } 
                      )
                   )
                ),

              td(  {align=>'center'}, 
                   a( {href=>"javascript:Delete('$f')"},
                   img( { src=>'/images/redX.gif',
                          border=>'0',
                          alt=>'Delete' 
                        } 
                      )
                   )
                ),
            ) ;
  }

  print '</Table>' ;
}
else
{
  print 'There are no Configuration Files available.', hr() ;
}

print <<EOF ;
<script>

/**********************************************************************/
function Apply( filename )
/**********************************************************************/
{
  if (confirm( 'Apply settings in configuration file "' 
             + filename 
             + '"?'
             + "\\nThis will restart your NetRS!" 
             ) 
     ) 
  {
    parent.loadDataFrame( 
                         '/perl-scripts/sysadmin/configApply.cgi?apply='
                       + escape( filename ) 
                        ) ;
  } 
} /* end of Apply() */



/**********************************************************************/
function Delete( filename )
/**********************************************************************/
{
  if (confirm( 'Delete configuration file "' 
             + filename 
             + '"?'
             ) 
     ) 
  {
    parent.loadDataFrame( 
                       '/perl-scripts/sysadmin/configDelete.cgi?delete='
                      + escape( filename )
                        ) ;
  } 
} /* end of Delete() */

</script>
EOF


# Create the two action buttons (hyperlinks, really).
print br(),br() ;
print table(
            {border=>2,align=>'center'},
            Tr( 
              td( {align=>'center', width=>"33%" },
                  a( {href=>"configCreate.cgi?startcreate=yes"} , 
                     "Create New<br />&nbsp;Configuration&nbsp;File&nbsp;"
                   )
                ),
              td( {align=>'center', width=>"34%" },
                  a( {href=>"configUpload.cgi?upload=start"} , 
                     "Upload<br />&nbsp;Configuration&nbsp;File&nbsp;" 
                   )
                ),
              )
           ) ;


if ( my $fname = param('show') )
{
  # Show the contents of the named config file.  This is primarily 
  # for diagnostic use and may not be kept in the final system.

  print hr(), "Configuration File \"$fname\"", p() ;
  open FILE, "/etc/sysconfig/configFiles/$fname" ;
  print '<PRE>' ;
  print <FILE> ;
  print '</PRE>' ;
  close FILE ;
}

print end_html() ;

########################################################################
# end of file configFiles.cgi
########################################################################
