#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: editSession.cgi,v 1.78 2007/03/28 23:34:34 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/editSession.cgi,v $

########################################################################
# editSession.cgi:
#
# Script to put up and manage a form allow creating and editing Data 
# Logging Sessions.  Called in several forms, using CGI parameters:
#
#   editSession.cgi
#       No parameters: Initialize the form for creating a new 
#       sessions.
#
#   ethernetSetup.cgi?restart=Name
#       Start editing the session with sessname==Name.
#
#   ethernetSetup.cgi?(full list of parameters from a form)
#       A normal submit of the form information.
#       The parameter 'submissionType' describes what is wanted.
#       This could be 
#         'Update'
#               The user changed the displayed schedule mode, so
#               we need to update the form to adapt to the new mode.
#         (formerly Delete and Okay were also supported here, but those
#         actions have been moved to gpsadmin/setSession.cgi)
#       The complete list of other parameters is still being defined and
#       will be filled in here when it settles.  
# 
# When the user actually submits the data for creating a new session, or
# for deleting an existing one, the parameters are sent to a separate
# script gpsadmin/setSession.cgi.  Prior to submitting the form's data,
# the field contents are checked with javascript to be sure things look
# okay.
#
# The data for session definitions is kept in /etc/sysconfig/sessions.
# See gpslog/logScheduler.c for a full description of the format of 
# entries in /etc/sysconfig/sessions.
#
########################################################################

use strict ;
use CGI qw/:standard/;
use Time::Local ;

use Fireball ;

titleWithHelp( 'Edit or Create Data Logging Session',
                         '/sessionsInfo.html'
                       ) ;


                
my ( $thisSecond, $thisMinute, $thisHour, $thisMonthDay, 
     $thisMonth, $thisYear, undef) = gmtime() ;
$thisYear += 1900 ;
my $thisMonthString = $monthStrings[$thisMonth] ;


my $theSessionName ;


if (param())
{
  # Display the parameters, if you want, by enabling the next few lines.
  #for my $i (param())
  #{
  #  print "$i =", param($i), "<br />" ;
  #}
}
else
{
  # No parameters came in. 
  # The default for that is to start creating a new session definition.
  param( 'restart', 'New Session' ) ;
}

# Read in the data from the sessions configuration file.
my %sessData ;
getSessionData( \%sessData ) ;


########################################################################
# Determine what type of action is being requested.  
# Okay: process the New Session or changes to an old one.
# Update : recreate the form using the supplied CGI parameters.
# Otherwise we are doing a Restart request, and put up the
# default form for the desired session.
# If we continue after this block, the value $theSessionName will
# be set to the proper value.
########################################################################
if (param('restart'))
{
  # This is a request to put up a fresh form with data from an existing
  # session, or a New session.  

  $theSessionName=param('restart') ;
  #print "SessionName : $theSessionName<br />" ;

  if ($theSessionName ne "New Session")
  {
    my $enable = $sessData{$theSessionName}{'enable'} ;

    my $sched = $sessData{$theSessionName}{'sched'} ;
    $sched = 'continuous' if ($sched eq 'cont') ;

    my $time = $sessData{$theSessionName}{'time'} ;
    my $duration = $sessData{$theSessionName}{'duration'} ;

    my $format =  $sessData{$theSessionName}{'format'} ;
    my $ftype = $sessData{$theSessionName}{'ftype'} ;

    my $sessid = $sessData{$theSessionName}{'sessid'} ;
    my $perday = $sessData{$theSessionName}{'perday'} ;
    my $persessid = $sessData{$theSessionName}{'persessid'} ;


    # This is an existing session.  We want all of the CGI parameters 
    # be set to values dependent on the current definitions.  That
    # causes the GUI widgets below to use these values as their
    # default settings.
    param( 'Enabled', $enable ) ;

    param( 'schedMode', $sched ) ;
    param( 'duration', $duration ) ;


    # Set defaults for Date and Time widgets.  Override these below
    # for specific schedule types.
    param( 'year' , $thisYear ) ;
    param( 'month', $thisMonthString ) ;
    param( 'day'  , $thisMonthDay ) ;
    param( 'hours', $thisHour ) ;
    param( 'minutes', $thisMinute ) ;
 
    # Set the date/time parameters for specific schedule types, based
    # on the "$time" parameter in the sessions database.  
    #   continuous : Value is zero, just use defaults.
    #   daily      : FOr date, use the current date (as defaulted)
    #                But time of day is encoded in $time.
    #   once       :
    #   manual     : These two have a full GPS timestamp in $time.
    #                Decode that into the widgets.  Manual doesn't
    #                show it directly, but use the value as the default
    #                if the user changes schedule mode to once or
    #                daily.
    if ($sched eq 'daily')
    {
      param( 'hours', int( $time / 3600 ) ) ;
      param( 'minutes', int( $time % 3600 / 60 ) ) ;
    }
    elsif ($sched eq 'once' or $sched eq 'manual')
    {
      my (undef,$min,$hour,$day,$month,$year,undef)
        = gmtime( $time + 315964800 ) ;
      $year += 1900 ;
      $month = $monthStrings[$month] ;

      param( 'year' , $year ) ;
      param( 'month', $month ) ;
      param( 'day'  , $day ) ;
      param( 'hours', $hour ) ;
      param( 'minutes', $min ) ;
    }

    param( 'Datatype',  $ftype ) ;

    if ($ftype eq 'T00')
    {
      param( 'measInterval', 
             $sessData{$theSessionName}{'measInterval'} 
           ) ;

      param( 'posInterval', $sessData{$theSessionName}{'posInterval'} );

      param( 'CodeSmoothing', $sessData{$theSessionName}{'option1'} ) ;
      param( 'CarrierSmoothing', 
             $sessData{$theSessionName}{'option2'}
           ) ;
      param( 'RT27', 
             $sessData{$theSessionName}{'RT27'}
           ) ;
      param( 'RawWaas', 
             $sessData{$theSessionName}{'rawWaas'}
           ) ;
    }
    elsif ($ftype eq 'Binex')
    {
      param( 'measInterval', 
             $sessData{$theSessionName}{'measInterval'} 
           ) ;
      param( 'metaInterval', 
             $sessData{$theSessionName}{'metaInterval'} 
           ) ;
      param( 'statusInterval',
             $sessData{$theSessionName}{'statusInterval'} 
           ) ;

      param( 'smoothing', $sessData{$theSessionName}{'smoothing'} ) ;
      param( '7f02', $sessData{$theSessionName}{'7f02'} ) ;
      param( '7f03', $sessData{$theSessionName}{'7f03'} ) ;
      param( 'Rollover',
             $sessData{$theSessionName}{'7f03RxClkOfsRollover'} ) ;
      param( 'Always',
             $sessData{$theSessionName}{'7f03RxClkOfsAlways'} ) ;
      param( '7f03Slips', $sessData{$theSessionName}{'7f03Slips'} ) ;
      param( '7f03Doppler', $sessData{$theSessionName}{'7f03Doppler'} );

      param( 'siteDesc', $sessData{$theSessionName}{'siteDesc'} ) ;
      param( 'firmwareVersion',
             $sessData{$theSessionName}{'firmwareVersion'} 
           ) ;
      param( 'stationID', $sessData{$theSessionName}{'stationID'} ) ;
      param( 'antXYZ', $sessData{$theSessionName}{'antXYZ'} ) ;
      param( 'antNum', $sessData{$theSessionName}{'antNum'} ) ;
      param( 'antOffset', $sessData{$theSessionName}{'antOffset'} ) ;
      param( 'rcvrType', $sessData{$theSessionName}{'rcvrType'} ) ;
      param( 'rcvrNumber', $sessData{$theSessionName}{'rcvrNumber'} ) ;
      param( 'metTilt', $sessData{$theSessionName}{'metTilt'} ) ;
    }

    # Other cases would go here....

    param( 'sid', $sessid ) ;
    param( 'perDay' , $perday ) ;
    param( 'perSID'  , $persessid ) ;
    #param( 'perFtype', $perF ) ;
  }
  else
  {
    # Form for a new session has been requested.
    # In general, the defaults for all of the widgets below will be
    # used.  

    # Datatype has to be explicitly called out, since it is used
    # for more than just setting the default on a widget.
    param( 'Datatype',  'T00' ) ;
    param( 'sid', 'a' ) ;
    param( 'perDay' , '0' ) ;
    param( 'perSID'  , '0' ) ;
    param( 'schedMode', 'continuous' ) ;
  }
}
########################################################################
elsif (param('submissionType') eq 'Update')
{
  $theSessionName=param('sessname') ;
}
########################################################################

# Emit the javascript needed by to process this form.

# First we define an associative array, Sessions, that is used to detect
# conflicts between the newly created or edited session and previously
# created sessions.  The elements of the array are indexed by the name
# of the session.  The content of each element is a string describing
# how file names will be created.  This is used two ways.  First, for
# newly created sessions, the new name can be verified as non-existent.
# Second, any submitted session can be checked to see if it's
# file-naming convention conflicts with existing sessions, allowing us
# to ask the user for confirmation.  The table will end up looking like
# this:
#   Sessions['CORSA'] = 'a10' ;
#   Sessions['CORSB'] = 'b01' ;
#   ...
# Note that if no previous sessions exist, then the array will be 
# completely undefined.
print "\n<Script>\n" ;

print "Sessions = new Object() ;\n" ;
for my $name (keys %sessData)
{
  print "Sessions['$name'] = '",
        $sessData{$name}{'sessid'},
        $sessData{$name}{'persessid'},
        $sessData{$name}{'perday'},
        "' ;\n",
        ;
}
print "</Script>\n" ;

print <<EOF ;
<Script>

/**********************************************************************/
function Update( anchor )
/**********************************************************************/
/* One of the mode-selection radio buttons has been clicked.
 * This may require us to redraw the screen.
 * For now, we are just redrawing it, regardless of whether that
 * is needed.  This could be made smarter, but we'll do that
 * later.
 * When we submit the form data, we set the value of the hidden field
 * 'submissionType' to 'update' so that the CGI script knows that
 * we aren't submitting changes, but rather are updating the form.
 * We also use the Anchor field to get the re-displayed form to scroll
 * down to an appropriate location.
 */
{
  f = document.theForm ;
  f.submissionType.value = 'Update' ;
  parent.submitFormWithTag( f, 
                            '/perl-scripts/editSession.cgi', 
                            anchor 
                          ) ;
} /* end of Update() */


/**********************************************************************/
function restart()
/**********************************************************************/
/* The session to modify is selected by a popupmenu, which is populated
 * with 'New Session' and all of the existing sessions .  When this
 * selector is changed, we re-format the entire screen with that
 * particular session's information.  This is triggered by an onChange
 * condition on the popup, which calls the javascript restart() function
 * defined here.  That function calls the cgi script with the special
 * parameter 'restart=sessionname'.
 */
{
  selectedIndex=document.theForm.sessname.selectedIndex ;
  selectedName=document.theForm.sessname.options[selectedIndex].text ;
  parent.loadDataFrame(   'editSession.cgi?restart=' 
                        + escape(selectedName)
                      ) ;
}

/**********************************************************************/
function strip( s )
/**********************************************************************/
/* Strip leading and trailing spaces from the string s.
 * Returns a copy of the string without the spaces.
 */
{
  while ( (s.length > 1)
       && (s.charAt(0) == " ")
        )
  {
    s = s.substring(1,s.length) ;
  }

  while ( (s.length > 1)
       && (s.charAt(s.length-1) == " ")
        )
  {
    s = s.substring(0,s.length-1) ;
  }



  if ( (s.length == 1)
    && (s.charAt(0) == " ")
     )

    s = "" ;
  //alert( "-" + s + "-" ) ;
  return s ;
}

/**********************************************************************/
function isNotAnInteger( s, label )
/**********************************************************************/
{
  bad = false ;
  s = strip(s) ;
  intval = parseInt(s) ;

  if ( isNaN(intval)) bad = true ;

  for (i = 0 ; i < s.length ; ++i)
  {
    c = s.charAt( i ) ;
    if ( c < '0' || c > '9') bad = true ;
  }

  if (bad)
  {
    alert( label + " must be a positive integer.") ;
    return true ;
  }
  else
    return false ;
} /* end of isNotAnInteger() */




/**********************************************************************/
function Okay()
/**********************************************************************/
/* Invoked when the OK button is pressed.
 * This just submits the entire form with all parameters.
 * The hidden field 'submisionType' is given a value of Okay so
 * that the processing CGI script can tell that we want to accept the
 * data for real, instead of just updating the form.
 */
{
  f = document.theForm ;

  sessname = f.sessname.value ;

  if (f.NewSessionName != null)
  {
     newName=
     f.NewSessionName.value = strip( f.NewSessionName.value ) ;
     if (newName == "")
     { 
       alert( "A session name must be specified" ) ;
       return false ;
     }

     for (i = 0 ; i < newName.length ; ++i)
     {
       c = newName.charAt(i) ;
       if ( ! ( (c >= '0' && c <= '9')
             || (c >= 'A' && c <= 'Z')
             || (c >= 'a' && c <= 'z')
             || (c == '_')
              )
          )
       {
         alert( "Illegal character in Session name." ) ;
         return false ;
       }
     }

     if (Sessions[newName] != null)
     {
       alert( "A session named '"+newName+"' already exists." ) ;
       return false ;
     }

     sessname = newName ;
  }

  if ( f.day != null)
  {
     theDay = f.day.value = strip( f.day.value ) ;
     if (isNotAnInteger(theDay, "Day"))
       return false ;
  }

  if ( f.hours != null)
  {
     f.hours.value = strip( f.hours.value ) ;
     if (isNotAnInteger(f.hours.value, "Hours"))
       return false ;
  }

  if ( f.minutes != null)
  {
     f.minutes.value = strip( f.minutes.value ) ;
     if (isNotAnInteger(f.minutes.value, "Minutes"))
       return false ;
  }

  if ( f.duration != null)
  {
     f.duration.value = strip( f.duration.value ) ;
     if (isNotAnInteger(f.duration.value, "Duration"))
       return false ;
     if (f.duration.value == 0)
     {
       alert( "Duration must be non-zero." ) ;
       return false ;
     }

     /* Daily sessions must be less than 24 hours (1440 minutes)
      * otherwise they can overlap, which the logging system doesn't
      * handle very well (See SPR 447).  So we enforce a <1440
      * limit here.
      */
     if ( (f.duration.value >= 1440)
       && f.schedMode[2].checked      // Daily Schedule type
        )
     {
       alert( "Duration on Daily sessions must be less than 1440 minutes." ) ;
       return false ;
     }
  }

  // Verify that the file naming characteristics of this session
  // don't conflict with those of previously defined sessions, as
  // found in the associative array Sessions[].
  conflict = false ; // false unless we find a conflict

  // This string is the same format as those in the Sessions array.
  naming =  f.sid.value 
          + (f.perSID.checked ? '1' : '0')
          + (f.perDay.checked ? '1' : '0')
          ;

  //alert( sessname + "\\n" + naming) ;

  // For each previously defined session, compare its naming string to
  // this session's naming string.
  for (sname in Sessions)
  { 
    if ( (sessname != sname)
      && (naming == Sessions[sname] )
       )
    {
      //alert( sessname + naming + sname + Sessions[sname] ) ;
      conflict = true ;
    }
  }

  if ( conflict 
    && ! confirm( "File names produced by this session could conflict "
                 +"with those from already defined sessions.  "
                 +"\\nConsider using a different Session Identifier."
                 +"\\n"
                 +"\\nDo you still want to save your session?"
                )
     ) 
  {
    return false ;
  }

  // If Datatype is Binex and smoothing is checked and
  // measInterval values is 0.1 or 0.2 or 0.5 then alert.
  // Note that this code assumes (for ease of coding) a specific
  // order to the measInterval popup elements, as defined in
  // the Perl code definition of array measIntervalValues.  
  if ( ( ( f.Datatype[1].checked // Binex format
        && f.smoothing.checked   // And smoothing
         )
      || ( f.Datatype[0].checked        // T00 format
        && f.CarrierSmoothing.checked   // And carrier smoothing
         )
       )
    && ( f.measInterval.selectedIndex == 1 // 0.1
      || f.measInterval.selectedIndex == 2 // 0.2
      || f.measInterval.selectedIndex == 3 // 0.5
       )
     )
  {
    alert( "Can't smooth carrier on high rate measurements." ) ;
    return false ;
  }
  
  // Passed all of the above tests and verifications.
  // Submit the session definition back to the NetRS.

  f.submissionType.value = 'Okay' ;
  parent.submitForm( f, '/perl-scripts/gpsadmin/setSession.cgi' ) ;
  return false ;

} /* end of Okay() */


/**********************************************************************/
function DeleteSession()
/**********************************************************************/
/* Invoked when the Delete button is pressed.
 * This just submits the entire form with all parameters.
 * The hidden field 'submisionType' is given a value of Delete so
 * that the processing CGI script can tell that we want to delete the
 * specified session.
 */
{

  f = document.theForm ;
  f.submissionType.value = 'Delete' ;
  parent.submitForm( f, '/perl-scripts/gpsadmin/setSession.cgi' ) ;

} /* end of Delete() */


</Script>
EOF


print startform( -name=>'theForm',
                 -action=>'/perl-scripts/editSession.cgi',
                 -onSubmit=>'return Okay()'
               ) ;

# Create the popup menu button containing 'New Session' and
# the previously defined account.  'New Session' has to come first.
# The grep instruction removes 'New Session' from its arbitrary
# location on the list, and splice is used to move it to the top.
my @values = sort( keys %sessData ) ;
splice( @values, 0,0, 'New Session' ) ;


print '<Table ><Tr>', 
      th( {align=>'right'}, 'Select Session ' ),
      td( {align=>'left'},
          popup_menu
          ( -name=>'sessname',
            -values=>\@values,
            -onChange=>'restart() ;',
            -default=>$theSessionName
          ) 
        ) ;

# If this is a New Session form, then we need to 
if ( $theSessionName eq 'New Session' )
{
  print td( {-align=>'left'}, 
            textfield( 'NewSessionName', '', 10, 10 ) 
          ) ;
}

########################################################################
# The Session Enable checkbox
########################################################################

print td( checkbox( -name=>'Enabled',
                        -value=>'1',
                        -checked=>'1',
                        -label=>' Enabled'
                      ) 
        ) ;

print '</Table>' ;





########################################################################
# Schedule control section
########################################################################

print '<a name="schedule">' ;
print hr() ;
print '<Table><Tr>' ;
print td( {colspan=>2}, b('Schedule:')) ;

# put in an empty column to act as a spacer between the buttons and
# The schedule parameter entry area.
print '<td width="25">' ;

print td( "Currently it is $thisYear/$thisMonthString/$thisMonthDay,",
          sprintf( "%02d:%02d UTC", $thisHour,$thisMinute) ) ;

# That's the end of the first row of the table.
# the following rows contain the schedMode selectionbuttons, and the
# schedule parameter entry fields.

# We will need some radio buttons to select the schedule mode.
# These are generated into an array here (as html strings), and then 
# popped into the table rows below as needed.
# When one of these buttons is clicked, the Update() function is called
# to update the form on the page to include the necessary fields for
# the selected schedule type.  The extra parameter is used to make sure
# that the schedule area of the page is visible, by scrolling to the 
# tag named "schedule".
my @schedButtons = radio_group( -name=>'schedMode', 
                            -value=>['manual',
                                     'once',
                                     'daily',
                                     'continuous'],
                            -nolabels=>1,
                            -default=>'continuous',
                            -onClick=>"Update('schedule') ;"
                          ) ;


# Determine the current schedule mode.
my $s = param('schedMode') ;

print '<Tr>',td( [ $schedButtons[0], 'Manual'] ) ;
# Nothing else goes in this row.

print '<Tr>',td( [ $schedButtons[1], 'Once Only'] ) ;
print '<td>' ; # The spacer column
if ( $s eq 'once' )
{
  print '<td>' ;
  print 'Starting Date ' ;

  @values=$thisYear-1..$thisYear+3 ;
  print popup_menu
          ( -name=>'year',
            -values=>\@values,
            -default=>$thisYear 
          ) ;

  print popup_menu
          ( -name=>'month',
            -values=>\@monthStrings,
            -default=>$thisMonthString  
          ) ;
 
  print textfield( 'day', $thisMonthDay, 2, 2 ) ;
}

print '<Tr>',td( [ $schedButtons[2], 'Daily'] ) ;
print '<td>' ; # The spacer column
if ( $s eq 'once' || $s eq 'daily' )
{
  # Show the Time entry fields, one for hours and one for minutes.
  print '<td>Starting Time [GMT] ' ;
  print textfield( 'hours', 
                   $thisHour,
                   2, 2 ) ;

  print "<b>:</b>" ;
  print textfield( 'minutes', 
                   $thisMinute,
                   2, 2 ) ;
}

print '<Tr>',td( [ $schedButtons[3], 'Continuous<br />Logging'] ) ;
print '<td>' ; # The spacer column

if ( $s eq 'continuous' )
{
  print '<td>File Durations ' ;
}
else
{
  print '<td>Session Duration ' ;
}
print textfield( 'duration', 
                 60,    # Default duration
                 4,     # Field Width
                 4      # Maximum characters
               ) ;
print ' minutes.' ;

print '</Table>' ;


########################################################################
# Data Type Controls
# A set of popups that control what data format will be logged.  For
# the moment only T00 is supported.  Yes, it looks odd to have only a 
# single button, but we'll eventually fill this in with other types.
#
# When one of these buttons is clicked, the Update() function is called
# to update the form on the page to include the necessary fields for
# the selected data type.  The extra parameter is used to make sure
# that the format area of the page is visible, by scrolling to the 
# tag named "end".  You would be reasonable if you thought we should
# scroll to the section named "format", but that doesn't work well
# for InternetExplorer, which uses the named targets differently.
# Instead, we move the tag to the end of the page, which has the OK
# button.  This essentially says, give me the bottom half of the page.
########################################################################

my @typeButtons = radio_group( -name=>'Datatype', 
                            -value=>['T00',
                                     'Binex',
                                     #'CMR',
                                     #'RT17',
                                     #'Rinex',
                                     #'RTCM',
                                    ],
                            -default=>'T00',
                            -onClick=>"Update('end') ;"
                          ) ;

# Put an anchor tag so that display updates can use the target for
# auto-scrolling.
print '<a name="format">' ;
print hr() ;

print '<b>Data Format:</b>' ;

print '<Table width="100%">' ;
my $count=@typeButtons ;

print '<Tr>' ;
for (my $i = 0; $i < @typeButtons ; ++$i)
{
  print "<td>$typeButtons[$i]" ;
}

print '</Table>' ;


# Create some popup selectors for use in several types of services.
# These are saved in variables and then dropped into the form
# where needed later on.

my @measIntervalValues = ( 0,
                           0.1 ,    # 0.1 must be index == 1
                           0.2 ,    # 0.2 must be index == 2
                           0.5 ,    # 0.5 must be index == 3
                                    # Modify Okay() function if
                                    # this changes.
                           1   ,
                           2   ,
                           5   ,
                           10  ,
                           15  ,
                           30  ,
                           60  ,
                           300 ,
                           600 ,
                         ) ;

my %measIntervalLabels = ( 0   => 'Off',
                           0.1 => '0.1 Sec', 
                           0.2 => '0.2 Sec', 
                           0.5 => '0.5 Sec', 
                           1   => '1 Sec',
                           2   => '2 Sec',
                           5   => '5 Sec',
                           10  => '10 Sec',
                           15  => '15 Sec',
                           30  => '30 Sec',
                           60  => '1 Min',
                           300 => '5 Min',
                           600 => '10 Min',
                         ) ;

my $measIntervalPopup = popup_menu( -name=>'measInterval',
                                    -values=>\@measIntervalValues,
                                    -labels=>\%measIntervalLabels,
                                    -default=>'15'
                                  ) ;

my $posIntervalPopup = popup_menu( -name=>'posInterval',
                                   -values=>\@measIntervalValues,
                                   -labels=>\%measIntervalLabels,
                                   -default=>'60'
                                 ) ;


my $dtype = param('Datatype') ;

if ($dtype eq 'T00')
{
  print br(), b( "$dtype Options" ) ;
  print table(
          Tr(
            td( {align=>'right'},
                'Measurement Interval', $measIntervalPopup,
            ),
            td( {width=>20} ),
            td(
                checkbox( -name=>'CodeSmoothing',
                          -checked=>'0',
                          -value=>'1',
                          -label=>'',
                        ), 
                "Smooth Code Phase",
              ),
            td(
                checkbox( -name=>'RT27',
                          -checked=>'0',
                          -value=>'1',
                          -label=>'',
                        ), 
                "Use Record Type 27",
              ),
            ),
          Tr(
            td( {align=>'right'},
                'Position Interval',$posIntervalPopup,
            ),
            td(),
            td(
                checkbox( -name=>'CarrierSmoothing',
                          -checked=>'0',
                          -value=>'1',
                          -label=>'',
                ), 
                "Smooth Carrier Phase"
              ),
            td(
                checkbox( -name=>'RawWaas',
                          -checked=>'0',
                          -value=>'1',
                          -label=>'',
                        ), 
                "Include Raw WAAS Data",
              ),
            )
        ) ;
}
elsif ($dtype eq 'Binex')
{
  print br(), b( "$dtype Options" ) ;

  my @metaIntervalValues = ( 0,
                             60,
                             300 ,
                             900 ,
                             1800,
                             3600
                           ) ;

  my %metaIntervalLabels = ( 0   => 'Off',
                             60  => '1 Min',
                             300 => '5 Min',
                             900 => '15 Min',
                             1800 => '30 Min',
                             3600 => '60 Min',
                           ) ;

  print "<br />",
    table( {width=>"100%"},
     Tr( 
        td( {valign=>'top'},
            table( 
                Tr( th( {colspan=>2}, "Measurements" ) ),
                Tr( td( {colspan=>2, align=>'center'},
                        popup_menu ( -name=>'measInterval',
                                     -values=>\@measIntervalValues,
                                     -labels=>\%measIntervalLabels,
                                   ),
                        '&nbsp;&nbsp;Interval'
                      ),
                  ),
                Tr( td( checkbox( -name=>'smoothing',
                                  -value=>'1',
                                  -label=>'',
                                ), 
                      ),
                    td( 'Smooth Carrier Phase' ),
                  ),
                Tr( td( checkbox( -name=>'7f02',
                                  -value=>'1',
                                  -label=>'',
                                ), 
                      ),
                    td( 'Send 7f-02 Measurements' )
                  ),
                Tr( td( checkbox( -name=>'7f03',
                                  -value=>'1',
                                  -label=>'',
                                  -checked=>1
                                 ), 
                      ),
                    td( 'Send 7f-03 Measurements' )
                  ),
                Tr( td(),
                    td( checkbox( -name=>'7f03Doppler',
                                  -value=>'1',
                                  -label=>'',
                                ), 
                        "With L1 Doppler"
                      ),
                  ),
                Tr( td(),
                    td( checkbox( -name=>'7f03Slips',
                                  -value=>'1',
                                  -label=>'',
                                ), 
                        "With Cycle Slip Counters"
                      ),
                  ),
                Tr( td(),
                    td( checkbox( -name=>'Always',
                                  -value=>'1',
                                  -label=>'',
                                  -onClick=>'alwaysClicked();',
                                ), 
                        "With Clock Offsets - Always"
                   ),
                  ),
                Tr( td(),
                    td( checkbox( -name=>'Rollover',
                                  -value=>'1',
                                  -label=>'',
                                  -onClick=>'rolloverClicked();',
                                ), 
                        "With Clock Offsets - On Rollovers"
                      ),
                  ),

                Tr( td( {height=>"5"} ) ), 

                Tr( th( {colspan=>2}, "System Status Record" ) ), 
                Tr( td( {align=>'center', colspan=>2},
                        popup_menu ( -name=>'statusInterval',
                                     -values=>\@metaIntervalValues,
                                     -labels=>\%metaIntervalLabels,
                                   ),
                        "&nbsp;&nbsp;Interval"
                      ),
                  ) 
                 ) # end of table()
          ), # end of td()

        # Second column of options
        td( {valign=>'top'},
            table( 
                   Tr( th( "MetaData" ) ),
                   Tr( td( {align=>'center'},
                            popup_menu ( -name=>'metaInterval',
                                         -values=>\@metaIntervalValues,
                                         -labels=>\%metaIntervalLabels,
                                       ),
                            '&nbsp;&nbsp;Interval'
                          ),
                     ),
                   Tr( td( checkbox( -name=>'siteDesc',
                                     -value=>'1',
                                     -label=>'Site Description',
                                   )
                         ),
                     ),
                   Tr( td( checkbox( -name=>'stationID',
                                     -value=>'1',
                                     -label=>'Station ID',
                                   )
                         ),
                     ),
                   Tr( td( checkbox( -name=>'rcvrType',
                                     -value=>'1',
                                     -label=>'Receiver Type',
                                   )
                         ),
                     ),
                   Tr( td( checkbox( -name=>'rcvrNumber',
                                     -value=>'1',
                                     -label=>'Rinex Receiver Number',
                                   )
                         ),
                     ),
                   Tr( td( checkbox( -name=>'firmwareVersion',
                                     -value=>'1',
                                     -label=>'Firmware Version',
                                   )
                         ),
                     ),
                   Tr( td( checkbox( -name=>'antXYZ',
                                     -value=>'1',
                                     -label=>'Antenna XYZ',
                                   )
                         ),
                     ),
                   Tr( td( checkbox( -name=>'antOffset',
                                     -value=>'1',
                                     -label=>'Antenna Offset',
                                   )
                         ),
                     ),
                   Tr( td( checkbox( -name=>'antNum',
                                     -value=>'1',
                                     -label=>'Rinex Antenna Number',
                                   )
                         ),
                     ),
                   Tr( td( {height=>"5"} ) ), 
                   Tr( th( "Met/Tilt Data" ) ),
                   Tr( td( checkbox( -name=>'metTilt',
                                     -value=>'1',
                                     -label=>'Include in Files',
                                     -checked=>1
                                   )
                         ),
                     ),
                 )
          ) # end of td()
       ) # End of Tr()
     ) ; # End of table()

  # Some javascript functions used in the Binex forms.
  print <<EOF ;
  <script>
  /********************************************************************/
  function alwaysClicked()
  /********************************************************************/
  /* Called when the Binex With Clock Offset Always checkbox is clicked.
   * If the checkbox is active, then we deselect the On Rollovers
   * checkbox.
   */
  {
    f = document.theForm ;
    if (f.Always.checked)
    {
      f.Rollover.checked = false ;
    }
  } // End of alwaysClicked()

  /********************************************************************/
  function rolloverClicked()
  /********************************************************************/
  /* Called when the Binex WIth Clock Offset Rollover checkbox is
   * clicked.  If the checkbox is active, then we deselect the Always
   * checkbox.
   */
  {
    f = document.theForm ;
    if (f.Rollover.checked)
    {
      f.Always.checked = false ;
    }
  } // End of rolloverClicked()

  </script>
EOF



}
else
{
  print "<p>Options for $dtype data format are currently Undefined." ;
}


########################################################################
# File Naming Controls and Directory options
#
# Arrange in a two column table with a sample centered underneath.
# On the left are the description and the Session Identifier popup.
# On the right are the directory options checkboxes.
#
# When one of these elements is clicked, the Update() function is called
# to update the form on the page to change the sample file path to
# match the controls.  The extra parameter is used to make sure
# that the format area of the page is visible, by scrolling to the 
# tag named "end".  You would be reasonable if you thought we should
# scroll to the section named "filenaming", but that doesn't work well
# for InternetExplorer, which uses the named targets differently.
# Instead, we move the tag to the end of the page, which has the OK
# button.  This essentially says, give me the bottom half of the page.
########################################################################
# Put in an anchor tag so that display updates can use the
# target for auto-scrolling.
print '<a name="filenaming">' ;
print hr() ;

# Start the table and put some column headings.
print '<Table width="100%" border="0">' ;


print Tr( td( b('File Naming') ),
          td( b('Directory Options') ),
        ) ;

print '<tr><td>' ;
print 
      'File names will be generated in the form:',
      br(), 
      '<tt><nobr>',
      '&nbsp;SystemNameYYYYMMDDHHmmS.ext',
      '</nobr></tt>' ;


my $stnid = qx( cat /etc/sysconfig/sysname ) ;
chomp $stnid ;

print table( #{border=>'1'},
             Tr(
                 td( {align=>'right'}, 'The SystemName is'),
                 td( $stnid )
               ),
 
             Tr(
                 td( {align=>'right'}, 
                     "'S' (The Session Identifier) is" 
                   ),
                 td( popup_menu ( -name=>'sid',
                                  -values=>['a'..'z'],
                                  -default=>'a',
                                  -onChange=>"Update('end') ;",
                                )
                   )
               )
           ) ;

print '</Td><Td valign="top">' ;
print '<Table>' ;
print Tr( td( checkbox( -name=>'perDay',
                        -checked=>'0',
                        -value=>'1',
                        -label=>'',
                        -onClick=>"Update('end') ;",
                      )
            ),
          td( 'Create Per-Day subdirectories' ) 
        ) ;
print Tr( td( checkbox( -name=>'perSID',
                        -checked=>'0',
                        -value=>'1',
                        -label=>'',
                        -onClick=>"Update('end') ;",
                      )
            ),
          td( 'Create Per-SessionId subdirectories' ) 
        ) ;
print '</Table>' ;

print '</Td></Tr></Table>' ;

# Create the sample file name string.  This shows the 'constant'
# elements in black, with the currently selected variable stuff in 
# blue.
print "<div align=center>Sample:<b><tt>&nbsp;&nbsp;" ;
printf( "/%04d%02d/", 
               $thisYear, $thisMonth+1 ) ;

if (param('perDay'))
{
  printf( '<Font color="blue">%02d/</Font>', $thisMonthDay ) ;
}

my $sid = param('sid') ;
if ( param('perSID') )
{
  print "<Font color=\"blue\">$sid/</Font>" ;
}

my $ext = 't00' ;
$ext = 'T00' if ($dtype eq 'T00') ;
$ext = 'BNX' if ($dtype eq 'Binex') ;

print sprintf( '<Font color="blue">%s</Font>', $stnid ) ; 

print sprintf( '%04d%02d%02d%02d%02d',
               $thisYear, $thisMonth+1, $thisMonthDay, 
               $thisHour, $thisMinute
             ) ;
print sprintf( '<Font color="blue">%s</Font>.%s</tt></b>',
               $sid, $ext
             ) ;
print "</div>" ;


########################################################################
# From control buttons.
########################################################################
print hr() ;

print button( -name=>'OK', 
              -value=>'OK',
              -onClick=>'Okay() ;',
            ) ;
print '&nbsp;' ;
print button( -name=>'Cancel', 
              -value=>'Cancel',
              -onClick=>
                  'parent.loadDataFrame( "/perl-scripts/sessions.cgi" )'
            ) ;
if ( $theSessionName ne 'New Session' )             
{
  print '&nbsp;' ;
  print button( -name=>'Delete', 
                -value=>'Delete',
                -onClick=>'DeleteSession() ;'
              ) ;
}

# This is a 'hidden' field, used during submissions of this
# form to identify various types of actions.  In particular,
# its value can be set to 'Okay', 'Delete' and 'Update' depending on
# why we are submitting the form.
print hidden( -name=>'submissionType',
              -value=>''
            ) ;
print endform() ;

# Put in an anchor tag so that display updates can use the
# target for auto-scrolling.
print '<a name="end">' ;

print end_html() ;

########################################################################
# End of file editSession.cgi
########################################################################
