#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: extRef.cgi,v 1.10 2003/12/19 01:46:21 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/extRef.cgi,v $

########################################################################
# extRef.cgi
#
# Allows control over the External Reference Input.  Puts up an HTML
# form showing the current state of the control and the presence or
# absence of a signal on that input.
#
# No CGI parameters are appended.  When the user clicks OK, the new
# control setting is passed as a CGI parameter to the script
# gpsadmin/setExtRef.cgi, which implements the requested change.
# The single parameter is:
#   ?extref=[Internal|External]
#
# The External Reference configuration is saved in a line in the file 
# /etc/sysconfig/extRefFreq.  This is a script file designed to be
# sourced by init.d/user_stinger.  It will contain either
#
#   REFSOURCE=Internal
# or
#   REFSOURCE=External
#
########################################################################


use strict ;
use CGI qw/:standard/;
use Fireball ;

titleWithHelp( 'Reference Frequency Source',
                         '/extRefInfo.html'
                       ) ;



# Get current state of the frequency reference input.  This has two 
# parts - Obtaining the setting of the user control, and detecting
# whether there is a signal present.  Formerly these were available
# by running a single command, 'extfreq -p' that accesses the control
# state, and returns the detection indicator.  Response should be
# of the forms:
#   disabled,present
#   enabled,not detected
# This command was also used to set the controls.  HOWEVER, this
# method is now in disfavor.  extfreq -p is only useful for reading
# the "present" and 'not detected' fields, and doesn't quickly reflect
# the state of the control setting.  That's now under control of the
# gpsinfo/sysinfo system, based on the sysconfig file.  To avoid
# any conflicts, I'm just going to read the file and call that the
# current state.

# Get the state of the control by reading the config file.
my $setting = qx( cat /etc/sysconfig/extRefFreq ) ;
$setting = ( $setting =~ m/External/ )
           ? 'enabled'
           : 'disabled'
           ;

# Get the state of the extref signal detector from program extfreq.
my (undef, $detected) = split( ',', 
                               qx(/usr/local/fireball/extfreq -p)
                             ) ;
$detected = ($detected =~ m/present/)
            ? 'present'
            : 'not detected'
            ;




print <<EOF ;
<Script>
/**********************************************************************/
function OkButton()
/**********************************************************************/
/* Called when the OK button is pressed.
 * Asks for confirmation and then submits the request to the NetRS.
 * Questions are asked in two cases:  

 * o If the user is requesting the external reference but no external
 *   source is detected.  This warning comes up even if no change is
 *   being made.  If the user says yes, we do what he asks.
 *
 * o We also confirm if we are changing the source, regardless of which
 *   direction or the presence of the external signal.
 */
{

  // If the selection is to use the External signal, but the signal 
  // isn't detected we ask for confirmation.

  if ( "$detected" != 'present' 
    && document.theForm.refsource[1].checked 
     ) 
  {
    if (confirm("External selected but no External source detected!\\n"
            +"Are you sure?" )
       )
    {
      parent.submitForm( document.theForm, "" ) ;
    }

    return ;
  }

  // If we aren't changing the settings, go ahead and submit the
  // request.  
  // But if the setting is being changed, ask for confirmation before
  // submitting the request.

  if ( ( "$setting" == 'disabled' 
      && document.theForm.refsource[0].checked 
       )
    || ( "$setting" == 'enabled' 
      && document.theForm.refsource[1].checked 
       )
    || (confirm( "Reference source is being changed.\\n"
                 +"Are you sure?" 
                )
       )
     )
  {
    parent.submitForm( document.theForm, "" ) ;
  }

} /* end of DefaultsButton() */

</Script>
EOF

print startform(  -name=>'theForm', 
                  -action=>'/perl-scripts/gpsadmin/setExtRef.cgi'
               ) ;


my @Control = radio_group( -name=>'refsource', 
                           -value=>[ 'Internal', 'External' ],
                           -nolabels=>1,
                           -default=> ($setting eq 'disabled')
                                      ? 'Internal' 
                                      : 'External' 
                         ) ;

print table( {border=>'1'},
             Tr(
                td( {width=>'30', align=>'center'}, $Control[0] ),
                td( b('Internal'),br(),
                    'The internal reference oscillator is used for
                    tracking GPS signals.'
                  )
               ),
             Tr(
                td( {align=>'center'}, $Control[1] ),
                td( b('External'),
                    br(),
                    'The 10MHz external reference signal is used for 
                     tracking GPS signals.',
                    br(),
                    'WARNING: This receiver will not track satellites '
                    .'if External is selected and a suitable 10MHz '
                    .'signal is not available.',

                  br(),
                  'An external 10MHz signal ',
                  ($detected eq 'present')
                  ? b('is currently detected.')
                  : b('is NOT currently detected.')
                  )
               )
           ) ;

print br(),hr() ;

print <<EOF ;
Note: Changing the Reference Frequency Source will interrupt GPS
tracking, data logging, data streams, etc.
<br /> <br />
EOF


print button( -name=>'OK', 
              -value=>'OK',
              -onClick=>'OkButton() ;',
            ) ;
print '&nbsp;' ;
print button( -name=>'Cancel', 
              -value=>'Cancel',
              -onClick=>
           'parent.loadDataFrame( "/perl-scripts/rconfigDisplay.cgi" ) '
            ) ;

print endform() ;

 
print end_html() ;

########################################################################
# end of file extRef.cgi
########################################################################
