#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: deleteFile.cgi,v 1.8 2003/12/05 18:12:44 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/fileaccess/deleteFile.cgi,v $

########################################################################
# deleteFile.cgi
#
# Script for deleting a logged data file.
#
# This script is called with two CGI parameters: 
#       
#   ?delete=filepath
#
#   ?sort=[date|size|name]
#
# The Logged Data files are rooted at /var/gps/.  
# This request deletes the named file, and then returns to 
# display the directory that contained that file, including a message
# showing what happened.
#
# If the filepath specifies a directory then the entire directory rooted
# there is deleted.
#
# When this script finishes deleting a file, it immediately re-invokes
# the directory display page at perl-scripts/dataFiles.cgi.  It sends
# that script a ?dir= cgi parameter (derived from the file that has been
# deleted) and a '?sort=' parameter, as was fed into the script.  This
# lets us ping-pong back and forth between the directory page, and
# the delete script without losing track of where we are and how the
# directory display is sorted.
#
########################################################################

use strict ;
use CGI qw/:standard/;

use lib '..' ;
use Fireball ;

startPage( 'File Delete' ) ;

my $Rootdir='/var/gps' ;



#for my $p (param())
#{
#  print $p, '=', param($p) , br() ;
#}



# Determine how the entries should be sorted.  Default is by name.
# This value should come in from 
my $sortBy = param('sortBy') ;
if ( ! $sortBy )
{
  $sortBy = 'name' ;
}


# If this request is to delete a file, handle that.
# Indicate what was deleted.
# Default value is to show that something went wrong.
my $message = 'Missing parameter to deleteFile.cgi' ;

# Time to display the message response.  If everything goes okay,
# don't delay at all.  
my $delay = 4000 ;

# Set this dirpath below to the directory path to display after
# the file(s) are deleted.
my $dirpath ;

if ( my $file = param( 'delete' ) )
{
  # Determine the path of the file to be deleted.
  $dirpath = $file ;
  $dirpath =~ s|/[^/]*$|/| ; # Drop everything after last slash.

  if ($file eq '/')
  {
    $message = "Can't delete '/' directory." ;
  }
  elsif ( $file =~ m|/$| )
  {
    # The filename ends in a '/' so this is really a directory delete
    # (or, perhaps, a symlink pointing to a directory).

    $file =~ s|/$|| ; # Drop the final '/' so -l test below works.
    if ( -l $Rootdir.$file )
    {
      $message = "Symlink $file has been deleted." ;
      unlink( $Rootdir.$file ) ;
      $delay = 0 ;
    }
    else
    {
      # We are deleting a directory.  
      # We are only allowed to do this if the files in the directory
      # (and subdirectories) are not active.  This is detected by 
      # looking for a .A extension.
      # Also, a file might have JUST finished writing, in which case the
      # user might not realize it is no longer a .A file.  To prevent
      # accidental deletions, we save a time mark when the directory
      # page is displayed.  This value is then fed into this delete
      # script, as parameter "time".  Any file modified after that
      # point in time is considered too new to delete.

      # Get the timestamp.  
      my $time = param('time') || 0 ;

      # Create a timestamp file, which will be given a modification
      # time of the saved time.  All regular files we want to delete
      # will be preserved if they are newer than this timestamp file.
      open TS, ">/tmp/timestamp" ;
      print TS "$time\n" ;
      close( TS ) ;
      # Set the modification time on that file to match the timestamp.
      utime $time, $time, "/tmp/timestamp" ;


      # In the specified directory, remove any regular files that aren't
      # named *.A and aren't newer than the timestamp file.
      system( "find $Rootdir$file -type f ! -name '*.A' ! -cnewer /tmp/timestamp -exec rm {} \\;" ) ;

      # Remove the temp file.
      unlink "/tmp/timestamp" ;

      # Attempt to delete all of the directory files in the specified 
      # tree.  This may fail if any of the regular files were
      # un-deleteable.
      system( "find $Rootdir$file -depth -type d -exec rmdir {} \\;" ) ;

      # If the deleted directory is still there, then there were
      # still some real files in the tree.  Give that warning.
      if ( -d "$Rootdir$file" )
      {
        $message = "Attempted to delete directory $file, but "
                  ."active files prevented full deletion." ;
      }
      else
      {
        # Otherwise all is good.  Files are gone.
        $message = "Directory $file has been deleted." ;
        $delay = 0 ;
      }
    }
    $dirpath =~ s|[^/]*/$|| ; # Remove last element/ from path.
  }
  else
  {
    $message = "File $file has been deleted." ;
    unlink $Rootdir.$file ;
    $delay = 0 ;
  }
}

if ($delay)
{
  print h2( $message ) ;
}
# Immediately switch back to the directory page.  
# Pass parameters dir, sort, and msg.
# Since msg can have spaces, have to http encode it. (space to %20,etc.)
my $URL = "/perl-scripts/dataFiles.cgi?dir="
          .CGI::escape( $dirpath )
          ."&sort=$sortBy&msg="
          .CGI::escape( $message ) ;
print <<EOF ;
<script>
  parent.loadAfterDelay( "$URL", $delay ) ;
</script>
EOF

print end_html() ;

########################################################################
# end of script deleteFile.cgi
########################################################################
