#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: firmware.cgi,v 1.32 2005/03/30 01:15:31 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/firmware.cgi,v $

# firmware.cgi
# Script for controling firmware update files.

use strict ;
use CGI qw/:standard/;
use Fireball ;

titleWithHelp( 'Firmware Status',
               '/firmwareStatus.html'
             ) ;


# Make all links blue with no underlines, even if visited, etc.
# This is a document-level style specification, affecting all
# links in the frame.
print <<EOF ;
<style type="text/css">
 <!--
       a {color:blue ; text-decoration: none }
 -->
</style>
EOF


# Emit the javascript functions used by this page.
print <<EOF ;
<script>

function installCheck( filename )
/* Verify that the user really wants to install the selected firmware
 * RPM.  Called either from an image link in the firmware file table, or
 * from the Install button along the bottom of the page.  Puts up a
 * confirmation dialog box and waits for the user's input.  If the user
 * says OK load in the firmwareInstall CGI script.
 */
{
  if ( confirm( "Installing " + filename 
                + "\\nThis can take several minutes?" 
                + "\\nAre you sure?" 
              ) 
     )
  {
    parent.loadDataFrame( 
                   "/perl-scripts/sysadmin/firmwareInstall.cgi?install="
                   + escape( filename )
                        ) ;
  }
}

function deleteCheck( filename )
/* Verify that the user really wants to delete the selected firmware
 * RPM.  Called from an image link in the firmware file table.
 */
{
  if (confirm( "Deleting " + filename + "\\nAre you sure?" ) )
  {
    parent.loadDataFrame( 
                     "/perl-scripts/sysadmin/firmwareDelete.cgi?delete="
                     + escape( filename ) 
                        ) ;
  }
}
</script>
EOF



# warranty -s returns:
#   Serial_number: 5432198765
# Extract the numeric string.
my $serialNumber = qx(/usr/bin/sudo /usr/local/fireball/warranty -s) ;
chomp $serialNumber ;
$serialNumber = (split("[ :]+",$serialNumber))[1] ;

# warranty -w returns:
#   war date :03312003
my $warrantyDate = qx(/usr/bin/sudo /usr/local/fireball/warranty -w) ;
$warrantyDate = (split("[ :]+",$warrantyDate))[2] ;

# Turn the warranty info into something a bit more presentable.
# WarrantyString is displayed to the user
# WarrantyDate is converted to format 2003/06/20 for comparison to the
# firmware date from an RPM package.
my $warrantyString ;
if ($warrantyDate != 0)
{
  my ($month,$day,$year) = ($warrantyDate =~ /(..)(..)(....)/) ;
  $warrantyDate = sprintf( "%04d/%02d/%02d", $year,$month,$day ) ;

  $warrantyString = "$day $monthStrings[ $month-1 ] $year" ;
}
else
{
  $warrantyString = 'none' ;
  $warrantyDate = '2000/01/01' ;
}


print '<table border="0">' ;

my $fwVersion = 'Unknown' ;
my $fwDate    = '01-Jan-2000' ;

if ( -r '/usr/local/fireball/defaults/firmwareVersion' ) 
{
  ($fwVersion, $fwDate) = split( "\n", qx( cat /usr/local/fireball/defaults/firmwareVersion ) ) ;
  
  my ($year,$month,$day) = split( '/', $fwDate ) ;
  $fwDate = "$day $monthStrings[$month-1] $year" ;
}




print table( {align=>'center', border=>0},
             Tr( td( {align=>"right"}, "Firmware Version:" ),
                 td( {align=>"center"}, b( "$fwVersion&nbsp;&nbsp;&nbsp;$fwDate" ) ) ,
               ),
             Tr( td( {align=>'right'}, "Serial Number:" ),
                  th( $serialNumber )
               ),
             Tr( td( {align=>'right'}, "Firmware Warranty Date:" ),
                 th( $warrantyString )
               ),
           ) ;

print hr() ;

print h3( {align=>'center'}, 'Uploaded Firmware Files' ) ;

chdir '/var/gps/incoming' ;
opendir FWDIR, '.' ;
my @allfiles ;
foreach my $f ( readdir(FWDIR) )
{
  if ( ( ! -d $f ) 
    && ( $f !~ /^\./ ) 
     )
  {
    push @allfiles, $f ;
  }
}
closedir FWDIR ;
@allfiles = sort( @allfiles ) ;

if (@allfiles)
{
  print '<Table border align="center">' ;
  print Tr( th( ['File','Type', 'Details', 'Install', 'Delete'] ) ) ;

  for my $f ( @allfiles )
  {
    my $type = 'unknown' ;

    my @fileTest = split( ' ', qx( file $f ) ) ;
    if ( $fileTest[1] eq 'RPM' ) 
    {
      $type = 'RPM package ' ;
    }

    print Tr( 
              td( '&nbsp;', 
                  a({ href => 'firmware.cgi?fwfile='.CGI::escape($f) },
                     $f 
                   ),
                  '&nbsp;'
                ),
              td( $type ),
              td( {align=>'center'},
                  a( { href => 'firmware.cgi?fwfile='.CGI::escape($f) },
                     img( { src=>'/images/bluedot.gif',
                            border=>'0',
                            alt=>'Details' 
                          }
                        )
                  )
                ),
              td( {align=>'center'},
                  a( { href => "javascript:installCheck( '$f' )" },
                     img( { src=>'/images/bluedot.gif',
                            border=>'0',
                            alt=>'Install' 
                          }
                        )
                  )
                ),

              td( {align=>'center'},
                  a( { href => "javascript:deleteCheck( '$f' )" },
                     img( { src=>'/images/redX.gif',
                            border=>'0',
                            alt=>'Delete' 
                          }
                        )
                  )
                )
            ) ;
  }

  print '</Table>' ;

  print br(), hr() ;

  ######################################################################
  # Display the details of the selected file:
  ######################################################################

  my $f = param( 'fwfile' ) ;
  $f = $allfiles[0] if ( !$f ) ;

  print h3( {align=>'center'}, "Firmware file $f" ) ;

  # Check if this file is really an RPM file.  It's first four
  # characters should contain the magic number sequence
  # 0xed 0xab 0xee 0xdb
  if ( ! IsRpmFile( $f ) )
  {
    print h3(  'This does not appear to be an RPM file '
              .'so it cannot be installed.' 
            ) ;
    goto EXIT ;
  }


  # Get the descriptive paragraph info from the RPM package.
  my $description = 
                 qx( /usr/lib/rpm/rpmq -q -p $f --qf '%{DESCRIPTION}') ;

  # Extract the firmware date from the description data.  It will be in
  # a line looking like: "Firmware Date 2003/06/20".  We just want the
  # '2003/06/20' section.
  my $firmwareDate = ($description =~ m/.*Firmware Date\s+(\S+)/ )[0] ;

  if (! $firmwareDate)
  {
    print h3(  'There is no firmware date in this file ' 
              .'so it cannot be installed.' 
            ) ;
    goto EXIT ;
  }

  # Adjust firmwareDate to match the expected format.  This is to
  # compensate for the tendency of people who type in the date to
  # not follow a strict format, especially by dropping leading zeroes
  # on month or day.
  my ($year,$month,$day) = ($firmwareDate =~ m|(.+)/(.+)/(.+)|) ;
  $firmwareDate = sprintf( "%04d/%02d/%02d", $year,$month,$day ) ;

  # Based on a string compare, we decide if this firmware is
  # installable.  This works because we have identical formats and 
  # they have the numeric data with most significant stuff first.
  #print $warrantyDate,br() ;
  #print $firmwareDate,br() ;
  if ( $warrantyDate ge $firmwareDate 
    || $description =~ m/Expected Installation Time.*:.*\./ 
     )
  {
    print "This file <b>is installable</b> under the terms of "
         ."your warranty.\n" ;
  }
  else
  {
    print h3( 'This file is dated after your Warranty Expiration '
             .'date and cannot be installed.' 
            ) ;
  }

  print '<p><b>Details:</b><pre>' ;

  print qx( /usr/lib/rpm/rpmq -q -p $f --qf '\
Name        : %{NAME}
Summary     : %{SUMMARY}
Version     : %{VERSION}-%{RELEASE}
Build Date  : %{BUILDTIME:date}
License     : %{LICENSE}
'
          ) ;

  printf( "Size        : %d bytes\n\n", -s $f ) ;
  print $description ;

  print '</pre>' ;


  print startform( -name=>"theForm" ) ;
  print button( -name=>'Install', 
                -value=>'Install',
                -onClick=> "installCheck( '$f' ) ; "
              ) ;

  print endform() ;
}
else
{
  print 'There are no firmware files available.', hr() ;
  print '<br /><br />' ;
  print table( {align=>'center', border=>2},
            Tr( td( {align=>'center' },
                    a( {href=>
                           '/perl-scripts/sysadmin/firmwareUpload.cgi'},
                       'Upload<br />Firmware File' 
                     )
                  ),
              )
           ) ;
}

EXIT:
print end_html() ;

########################################################################
# end of file firmware.cgi
########################################################################
