#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: doReset.cgi,v 1.12 2006/04/14 21:36:19 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/gpsadmin/doReset.cgi,v $

########################################################################
# doReset.cgi
#
# Script that allows user to restart the system in two ways.
# The script is typically invoked from the /perl-scripts/reset.cgi
# script, when the user selects a type of reset.  Note that although
# that script is not http-secured, this script is.  Requires gpsadmin
# access privileges to run, if security is enabled.
#
# This script is called in one of two ways, depending on the
# cgi parameters:
#       doReset.cgi?clear=1 
#       doReset.cgi?restart=1 
#
# Clear will send a SIGHUP signal to frontpanel_task to trigger
# shutting down Stinger, clearing the almanac/ephemeris data, and
# restart the signal tracking.  This also closes down all data logging
# and I/O streaming.
#
# Restart sends a SIGUSR2 signal to frontpanel_task to trigger a
# complete system restart.  In addition, a delayed "init 6" command
# is issued in case frontpanel_task is unable to respond.
#
# Both methods need to use sudo to send signals, etc., since that msut
# be done as root, while this script is the GUI user, netrx.
#
# A warning message is posted that indicates that the user should
# wait a while before attempting further GUI interaction.  After an
# appropriate timeout, javascript attempts to restart the entire GUI.
#
########################################################################

use strict ;
use CGI qw/:standard/;

use lib '..' ;
use Fireball ;

########################################################################
# Begin formatting the return document.
########################################################################
startPage( 'Do Reset' ) ;


#for my $p (param())
#{
#  print "$p = ",param($p),br() ;
#}

my $frontpanelPid = qx( cat /var/run/frontpanel_task.pid ) ;

if (param('clear'))
{
  print <<EOF ;
<h1>Clearing Ephemeris & Almanac Data</h1>
Restarting signal tracking, data logging and I/O streams
<br />
Wait approximately one minute before attempting further access.
     
<Script>
setTimeout( 'parent.document.location = "/perl-scripts/NetRS.cgi"',
            60000
          ) ;
</Script>
EOF

  # Trigger frontpanel task to clear the eph/alm data.
  system( "/usr/bin/sudo /bin/kill -SIGHUP $frontpanelPid" ) ;
}
elsif ( param('restart') )
{
  print <<EOF ;
<h1>Restarting System</h1>
Wait approximately three minutes before attempting further access.
     
<Script>
setTimeout( 'parent.document.location = "/perl-scripts/NetRS.cgi"',
            180000
          ) ;
</Script>
EOF

  # Send the desired signal to the frontpanel task
  system( "/usr/bin/sudo /bin/kill -SIGUSR2 $frontpanelPid" ) ;

  # Set a failsafe that bypasses the frontpanel_task/Signal mechanism.
  # If we aren't shutdown within 30 seconds, we'll give a direct 
  # 'init 6'
  system( "(sleep 30 ; /usr/bin/sudo /sbin/init 6)&" ) ;
}

print end_html() ;

########################################################################
# end of script doReset.cgi
########################################################################
