#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: portEnableDisable.cgi,v 1.2 2004/04/12 23:29:37 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/gpsadmin/portEnableDisable.cgi,v $

########################################################################
# setPort.cgi:
#
# Script to install new I/O port configurations.  The non-secured
# script /perl-scripts/portSetup.cgi is used to put up an HTML form
# for use in defining I/O ports.  When the user clicks OK, the
# form's parameters are passed into this script, which is gpsadmin
# secured.  Here we verify some items and then install the port by
# updating the gpssd configuration files, etc.
#
# If problems are found with the selected parameters, then an error
# message is posted and the portSetup.cgi script is re-invoked with
# the original parameters.
#
# The interface to the configuartion files is via a large hash
# structure, %allPortData{}.  The main keys are the names of the various
# ports, SerialPortX, TcpPortXXXX or UdpPortXXXX.  Each of those
# indexed elements is a sub-hash with variously named elements, like
# 'service', 'protocol', 'baud', etc.  The process of creating or
# editing a port become a simple matter of reading in the full structure
# (with getPortSetup(), modifying the desired port characteristics,
# and then regenerating the configuration files with saveGpssdConfig().
# This abstraction layer hides the details of the configuration files,
# which is especially useful for PPP ports which have their information
# spread out over multiple config files.
########################################################################

use strict ;
use CGI qw/:standard/;


use lib '..' ;
use Fireball ;

########################################################################
# Begin formatting the return document.
########################################################################
startPage( 'Port Enable/Disable' ) ;


# Read in the data from the configuration files and put in a Hash.
my %allPortData ;
getPortSetup( \%allPortData ) ;

my $enable = 1 ;


# For debugging, display any parameters passed in.
# Disable in shipping code.
#if (param())
#{
#  print '<PRE>' ;
#  for my $i (param())
#  {
#    print "$i = ", param($i),"\n";
#  }
#  print '</PRE>' ;
#}

my $thePort ;
if ($thePort = param('disable'))
{
  $enable = 0 ;
}
elsif ($thePort = param('enable'))
{
  $enable = 1 ;
}

print h2( "$thePort has been ",
          $enable ? "enabled."
                  : "disabled."
        ) ;

# thePort came in with spaces in it, like "TCP Port 1234".  It
# also has all CAPs in TCP and UDP.  But the keys into the %allPortData
# hash have no spaces and use Tcp and Udp.  So we convert:
$thePort =~ s/ //g ;     # No spaces
$thePort =~ s/TCP/Tcp/ ; # Get the case right. 
$thePort =~ s/UDP/Udp/ ;
$thePort =~ s/REP/Rep/ ;

# Set the enable for this port to the new value.
$allPortData{$thePort}{'enable'} = $enable ;

# Regenerate gpssd.config and inform the streaming daemon
# to examine the configuration for changes.
saveGpssdConfig( \%allPortData ) ;


# After a timeout, go back to main Sessions screen.
print <<EOF;
    <Script>
    parent.loadAfterDelay( "portSummary.cgi", 1000) ;
    </Script>
EOF


EXIT:
print end_html() ;

########################################################################
# End of file setPort.cgi
########################################################################
