#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: setAntenna.cgi,v 1.14 2004/07/17 00:46:58 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/gpsadmin/setAntenna.cgi,v $

########################################################################
# setAntenna.cgi:
#
# Script to implement setting the antenna type and related parameters.
# This is typically called by the perl-script/antenna.cgi script,
# when the user clicks the OK button.
#
# Always called with two parameters to specify the desired 
# antenna setup:
#
#   setAntenna.cgi?antType=TheTypeString&method=TheMethodString...
#
# Those strings have to be html-escape sequences.  That is, space
# charcters must be %20, etc.
# When called, this page puts up an acknowledgement page, and
# writes the values into the antenna configuration file.
########################################################################

use strict ;
use CGI qw/:standard/;

use lib '..' ;
use Fireball ;

########################################################################
# Begin formatting the return document.
########################################################################
startPage( 'Set Antenna' ) ;


#print "Parameters:<br />" ;
#for my $p (param())
#{
#  print "$p : ", param($p), "<br />" ;
#}

my $errorMessage = '' ;

my $height = param( 'height' ) ;
my $antType = param( 'antType' ) ;
my $antName = param( 'antName' ) ;
my $serial = param( 'serial' ) ;
my $textSerial = param( 'textSerial' ) ;
$textSerial =~ s/^\s*// ; # Remove leading spaces.
$textSerial =~ s/\s*$// ; # Remove trailing spaces.
my $method = param( 'method' ) ;

if ( $serial eq '' )
{
  $errorMessage = 'Serial number must be specified.' ;
}
elsif ( $height !~ m/^\d*\.\d+$/ # .333
    and $height !~ m/^\d+\.\d*$/ # 333.
    and $height !~ m/^\d+$/     # 333
      )
{
  $errorMessage = 'Illegal height.' ;
}
elsif ( $antType !~ m/^\d+$/ )
{
  $errorMessage = 'Illegal antenna type.' ;
}
elsif ( $antName eq '' )
{
  $errorMessage = 'Illegal antenna name.' ;
}
elsif ( $method !~ m/^\d+$/ )
{
  $errorMessage = 'Illegal method parameter.' ;
}
else
{
  # Have all valid parameters.

  print h2( 'Antenna Changes Accepted' ) ;

  $serial = $serial + 0 ;
  updateAntennaSetup( $serial,
                      $antType,
                      $height,
                      $method,
                      $antName,
                      $textSerial
                    ) ;
  print <<EOF;
    <Script>
    parent.loadAfterDelay( 'rconfigDisplay.cgi', 1000 ) ;
    </Script>
EOF
  goto EXIT ;
}

ERROR:
# Gets to here if something went wrong with the parameter processing 
# above.

# Put up an alert dialog with the error message.
# After a delay, switch back to the original antenna page.
# We want the parameters that came into this script to go
# back to the original script, so we don't lose any data entries
# that have already been done.  So format all of the parameters
# into the URL.
print <<EOF;
    <Script>
    alert( '$errorMessage' ) ;
    parent.loadAfterDelay( 'antenna.cgi?update='
                           + escape('$antType') 
                           + '&serial='
                           + escape('$serial')
                           + '&height='
                           + escape('$height')
                           + '&method='
                           + escape('$method')
                         , 1000) ;
    </Script>
EOF

EXIT:
print end_html() ;
########################################################################
# End of file setAntenna.cgi
########################################################################
