#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: setExtRef.cgi,v 1.7 2003/06/13 16:48:49 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/gpsadmin/setExtRef.cgi,v $

########################################################################
# setExtRef.cgi
#
# Controls the External Reference Input.  This script is called by
# perl-scripts/extRef.cgi to implement the setting of the External
# Reference Frequency Controls.  ExtRef.cgi implements an HTML form
# that the user can fill in.  When the user clicks on OK, the 
# control value is fed as a CGI parameter to this script, which takes
# care of the actual changing of the hardware.
#
# The single parameter is:
#
#   ?extref=[Internal|External]
#
# So a call to this script might use the URL:
#
#   /perl-scripts/gpsadmin/setExtRef.cgi?extref=External
#
# Note that the script is in the gpsadmin section, and thus requires
# gpsadmin security access, if that security level is enabled.
#
# The External Reference configuration is saved in a line in the file 
# /etc/sysconfig/extRefFreq.  This is a script file designed to be
# sourced by init.d/user_stinger.  It will contain either
#
#   REFSOURCE=Internal
# or
#   REFSOURCE=External
#
# To implement a change to this control, the script modifies the config
# file and then performs some sort of run-level change, via a signal
# to the frontpanel task.  This has NOT YET BEEN IMPLEMENTED, so the
# details are not known.
#
# Because changing run level (or rebooting or whatever) is a
# time consuming task, this script checks whether the existing 
# control setting matches what has been requested.  If so, then no
# change is actually made.
#
########################################################################


use strict ;
use CGI qw/:standard/;

use lib '..' ;
use Fireball ;

########################################################################
# Begin formatting the return document.
########################################################################
startPage( 'Set External Reference' ) ;


#print '<PRE>' ;
#foreach my $p (param())
#{
#  print "$p ", param($p), "\n" ;
#}
#print '</PRE>' ;

# Get current state of the frequency reference input.  This has two 
# parts - Obtaining the setting of the user control, and detecting
# whether there is a signal present.  Formerly these were available
# by running a single command, 'extfreq -p' that accesses the control
# state, and returns the detection indicator.  Response should be
# of the forms:
#   disabled,present
#   enabled,not detected
# This command was also used to set the controls.  HOWEVER, this
# method is now in disfavor.  extfreq -p is only useful for reading
# the "present" and 'not detected' fields, and doesn't quickly reflect
# the state of the control setting.  That's now under control of the
# gpsinfo/sysinfo system, based on the sysconfig file.  To avoid
# any conflicts, I'm just going to read the file and call that the
# current state.

# Get the state of the control by reading the config file.
my $setting = qx( cat /etc/sysconfig/extRefFreq ) ;
$setting = ( $setting =~ m/External/ )
           ? 'enabled'
           : 'disabled'
           ;

# Get the state of the extref signal detector from program extfreq.
my (undef, $detected) = split( ',', 
                               qx(/usr/local/fireball/extfreq -p)
                             ) ;
$detected = ($detected =~ m/present/)
            ? 'present'
            : 'not detected'
            ;


my $newsource = param('refsource') ;

if ( $newsource =~ m/External|Internal/ )
{
  # Do the change

  # If the current setting of the External Ref control doesn't match
  # the desired newsource setting, then change change the setting.
  # Otherwise, don't do anything since no change is requested.
  if ( ($newsource eq 'External') xor ($setting eq 'enabled') )
  {
    # Update the extref configuration file to make this change
    # persist across power cycles.
    open( NEWCONFIG, '>/tmp/extRefFreq' ) ;

    print NEWCONFIG "REFSOURCE=", $newsource, "\n" ;

    close NEWCONFIG ;

    system( 'sudo cp /tmp/extRefFreq /etc/sysconfig/extRefFreq' ) ;
    system( 'sudo /usr/local/fireball/sysinfo -m'  ) ;
    print h2("Switching to $newsource source for reference frequency."),
          h3( 'Restarting signal tracking.' ),
          h3( 'It may take several minutes to resume normal operation.')
          ;

    print <<EOF ; 
      <Script>
      parent.loadAfterDelay( '/perl-scripts/rconfigDisplay.cgi', 
                             2000) ;
      </Script>
EOF
  }
  else
  {
    print h2( "No Change Made to External Reference Frequency Source!" ) ;
    print <<EOF ; 
      <Script>
      parent.loadAfterDelay( '/perl-scripts/rconfigDisplay.cgi', 
                             1000) ;
      </Script>
EOF
  }

} # end of parameter processing


print end_html() ;


########################################################################
# end of file setExtRef.cgi
########################################################################
