#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: setRefpos.cgi,v 1.12 2004/08/24 23:09:45 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/gpsadmin/setRefpos.cgi,v $

########################################################################
# setRefpos.cgi
#
# Script to finalize the user's choices for configuration of Reference
# Position and RTCM/CMR setup parameters.  
# The script /perl-scripts/refpos.cgi manages an HTML form.  When the
# user clicks OK there, the form's data is passed via CGI parameters
# into this script.  Here we update the sysconfig/ref_stn.config file
# with the new data, and trigger updating that information in the
# sysinfo/gpsinfo database.
#
########################################################################


use strict ;
use CGI qw/:standard/;

use lib '..' ;
use Fireball ;

########################################################################
# Begin formatting the return document.
########################################################################
startPage( 'Set Reference Position' ) ;


########################################################################
sub strip
########################################################################
# Removes leading and trailing spaces from the parameter string $_[0]
# Returns the altered string.
{
  $_[0] =~ s/^\s+// ;  # Drop any leading spaces
  $_[0] =~ s/\s+$// ;  # Drop any trailing spaces
  return $_[0] ;
}

my $errorMessage = '' ; # blank until an error is detected

#print( "Parameters Received<br />" ) ;
#foreach my $p (param())
#{
#    print  "$p=", param($p), " " ;
#}

# This is a submission of new information.
# Verify that the parameters are acceptable.

my ( $X, $Y, $Z ) ;
my ( $latDeg, $latMin, $latSec, $NS, 
     $lonDeg, $lonMin, $lonSec, $EW, 
     $height 
   ) ;

# This is a perl-regular expression that allows the following
# formats:  
#   Possible leading + or -.
#   ddd.ddd     .ddd    ddd.    ddd
# where ddd is one or more decimal digits.
my $fixedPointPattern = '^[+-]?(\d*\.\d+|\d+\.\d*|\d+)$' ;

# This is the same pattern, not allowing for a leading + or -
my $posFixedPointPattern = '^(\d*\.\d+|\d+\.\d*|\d+)$' ;

my $lat ;
my $lon ;

# Grab the current position from the sysinfo facility.
# This is used later to determine if the new reference position is
# quite far from the current position.
open( SYSINFO, '/usr/local/fireball/sysinfo -p pos_info |' );
# First line of the form:
# 246021.8, 1225 : 37.3891238781 -122.0368426604 -4.921
# time      week   lat           lon             height
my (undef, undef, undef, 
    $currentLat, $currentLon, $currentHeight) 
                                            = split( ' ', <SYSINFO>) ;
close( SYSINFO ) ;

# For each of the two modes below, we'll calculate the approximate
# difference in meters between current position and the entered
# position.
my $diff ; 

if (param('Mode') eq 'lla')
{
  $latDeg = strip( param( 'latDeg' ) ) ;
  $latMin = strip( param( 'latMin' ) ) ;
  $latSec = strip( param( 'latSec' ) ) ;
  $NS     = param( 'NS' ) ;
  $lonDeg = strip( param( 'lonDeg' ) ) ;
  $lonMin = strip( param( 'lonMin' ) ) ;
  $lonSec = strip( param( 'lonSec' ) ) ;
  $EW     = param( 'EW' ) ;
  $height = strip( param( 'Height' ) ) ;

  if ( $latDeg !~ m/^\d+$/
    or $latMin !~ m/^\d+$/
    or $latSec !~ m/$posFixedPointPattern/
    or $lonDeg !~ m/^\d+$/
    or $lonMin !~ m/^\d+$/
    or $lonSec !~ m/$posFixedPointPattern/
    or $height !~ m/$fixedPointPattern/
    or $latMin >= 60
    or $latSec >= 60.0
    or ($latDeg+$latMin/60.0+$latSec/3600.0) > 90.0
    or $lonMin >= 60
    or $lonSec >= 60.0
    or ($lonDeg+$lonMin/60.0+$lonSec/3600.0) > 180.0
     )
  {
    $errorMessage = "Lat/Lon/Alt field(s) are not all valid." ;
    goto PARAMDONE ;
  }

  $lat = $latDeg + $latMin/60 + $latSec/3600 ;
  $lat = -$lat if ($NS eq 'S') ;

  $lon = $lonDeg + $lonMin/60 + $lonSec/3600 ;
  $lon = -$lon if ($EW eq 'W') ;

  #printf( "<pre>Lat: %3d&deg;%02d'%07.4f\"%s = %.10f\n".
  #             "Lon: %3d&deg;%02d'%07.4f\"%s = %.10f\n".
  #             "Hgt: %.4f\n".
  #        "</pre>",
  #        $latDeg, $latMin, $latSec, $NS,
  #        $lat,
  #        $lonDeg, $lonMin, $lonSec, $EW,
  #        $lon,
  #        $height,
  #      ) ;

  # Determine if the lat/lon/hgt is extremely far from the current 
  # GPS-determined position.  
  # Determine the differences in Height,Lat and Lon, all in meters.

  my $heightDiff = ($currentHeight - $height) ;

  # A degree of latitude is 60 minutes, which is 60 nautical miles.
  # There are 1852 meters in a nautical mile.
  my $latDiff = ($currentLat - $lat ) * 60.0 * 1852.0 ;

  # Longitude lines squeeze together at the poles, so we have to
  # scale them by cos(lat) [lat converted to radians).
  # Then we convert to meters as for the latDiff.
  my $lonDiff = ($currentLon - $lon ) 
              * cos( $lat / 360.0 * 2.0 * 3.1415926535 )
              * 60.0 * 1852.0 ;

  $diff = sqrt( $heightDiff * $heightDiff
              + $latDiff * $latDiff
              + $lonDiff * $lonDiff
              ) ;

} # Mode eq lla
else
{
  # ecef/XYZ mode
  $X = strip( param( 'X' ) ) ;
  $Y = strip( param( 'Y' ) ) ;
  $Z = strip( param( 'Z' ) ) ;
  if ( $X !~ m/$fixedPointPattern/ 
    || $Y !~ m/$fixedPointPattern/ 
    || $Z !~ m/$fixedPointPattern/ 
     )
  {
    $errorMessage = "XYZ values must all be numeric." ;
    goto PARAMDONE ;
  }
  #printf( "<pre>X: %12.3f\nY: %12.3f\nZ: %12.3f\n</pre>",
  #        $X, $Y, $Z
  #      ) ;

  my ($currentX, $currentY, $currentZ ) 
     = split( ' ',
              qx( /usr/local/fireball/ecefConvert lla $currentLat $currentLon $currentHeight ) 
            ) ;
  #print "$currentX, $currentY, $currentZ <br />" ;
  my $deltaX = $currentX - $X ;
  my $deltaY = $currentY - $Y ;
  my $deltaZ = $currentZ - $Z ;

  $diff = sqrt( $deltaX * $deltaX
              + $deltaY * $deltaY
              + $deltaZ * $deltaZ
              ) ;
} # Mode = ecef


# The $diff value determined above is the approximate error between
# the entered reference position and the current GPS-based position.
# If that's above a threshold, then we need to warn the user.  The
# primary reason for this is that some reference station formatters
# (RTCM in particular) will not produce any corrections if the error
# is above some limit, probably 200m for RTCM.  Here we use a somewhat
# arbitrary threshold of 100meters.  This is mainly intended to catch
# the situation where a user enters something catstrophically wrong,
# like a slipped degrees digit, or the wrong N/S/E/W identifier.
# 
# Note that we accept the values as given, without the user's
# confirmation.  We just put up an alert box to warn him.
if ($diff > 100)
{
  $diff = int( $diff ) ;
  print <<EOF ;
  <script>
  alert( 'New Reference Position is quite different from '
        +'Current GPS position. ($diff m)' 
        +'\\nYou might want to verify it.'
       ) ;
  </script>
EOF
}



# Update the reference station configuration file with the new data.

my %refStationData ;

# Add the updated information lines to the end of the new file.
if (param('Mode') eq 'lla')
{
  $refStationData{'lat'} = $lat ;
  $refStationData{'lon'} = $lon ;
  $refStationData{'height'} = $height ;
}
else
{
  $refStationData{'x'} = $X ;
  $refStationData{'y'} = $Y ;
  $refStationData{'z'} = $Z ;
}

my $id = param('rtcmID') ;
if ( $id !~ /^ *\d+ *$/
  or $id > 1023
   )
{
  $errorMessage = "Invalid RTCM Station ID" ;
  goto PARAMDONE ;
}
$refStationData{'rtcmid'} = $id ;


$id = param('cmrID') ;
if ( $id !~ /^ *\d+ *$/
  or $id > 31
   )
{
  $errorMessage = "Invalid CMR Station ID" ;
  goto PARAMDONE ;
}
$refStationData{'cmrid' } = $id ;


$refStationData{'binexid' } = param('binexID') ;
$refStationData{'cmrStationName' } = param('cmrStationName') ;
$refStationData{'rtcmdesc' } = param('rtcmDesc') ;

# Write that structure back into the configuration file.
updateRefStationData( \%refStationData, 
                      (param('Mode') ne 'lla') 
                    ) ;

print h2( 'New Reference Station parameters accepted' ) ;

PARAMDONE:

# If an error message was found earlier during CGI processing,
# post an alert message to the user.  Also display the message on the
# bottom of the page, in case the javascript alert system doesn't work.
if ( $errorMessage )
{
  print h3( $errorMessage ) ;

  # Since there was an error, we would like to go back to the
  # HTML form and let the user correct it.  So we call that URL and
  # pass back all of the CGI parameters that came into this script.

  # Switch back to the original portSetup.cgi page.  In order to do this
  # we need to reconstruct the original URL, containing all of the
  # parameters that came into this script.  PortSetup.cgi will redraw 
  # the page, using the supplied parameters.  The list of parameters
  # must be of the form:
  #   portSetup.cgi ? parm1 = value & parm2 = value & parm3 = ...
  # In that, I put in spaces for clarity, but they aren't there in the
  # real thing.  Note that the parameter separators start with ? and
  # then switch to '&'.  The parameter values must be CGI::escaped to
  # convert spaces, etc., into URL acceptable sequences like %20.

  my $returnURL = '/perl-scripts/refpos.cgi' ;

  my $startC = '?' ;
  for my $p (param())
  {
    $returnURL .= $startC ;
    $returnURL .= $p.'='.CGI::escape( param($p) ) ;
    $startC = '&' ;
  }
  $returnURL .= '&msg='.CGI::escape($errorMessage) ;

  # Emit some javascript to switch back to that port after a delay.
  # And whhile we are at it, post an error message dialog box.
  print <<EOF ;
  <Script>
    alert( "$errorMessage" ) ;
    parent.loadAfterDelay( "$returnURL", 1000) ;
  </Script>

EOF
} # if have error message
else
{
  # No problem detected.  After a timeout, go to the I/O menu default
  # screen.
  print <<EOF;
    <Script>
    parent.loadAfterDelay( "portSummary.cgi", 1500) ;
    </Script>
EOF
} # no error message


print end_html() ;

########################################################################
# End of file setRefpos.cgi
########################################################################
