#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: setSession.cgi,v 1.20 2007/03/28 23:34:34 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/gpsadmin/setSession.cgi,v $

########################################################################
# setSession.cgi:
#
# Script to allow creation and deletion of Data Logging Sessions.
# This script is typically called from /perl-scripts/editSessions.cgi
# when the user clicks OK or DELETE.  It is fed the parameters from
# that form, specifying the session name and characteristics.
# setSession.cgi is gpsadmin secured, while the editSessions.cgi script
# isn't.  Only when we try to submit the data or delete a session do
# we restrict the user.
#
#   setSession.cgi?(full list of parameters from a form)
#       A normal submit of the form information.
#       The parameter 'submissionType' describes what is wanted.
#       This could be 
#         'Okay'
#               The Okay button was pressed.  Accept the new data.
#         'Delete'
#               The user wants to delete the current session.
#       The complete list of other parameters is still being defined and
#       will be filled in here when it settles.  
#
# When Delete is requested, the supplied session name is removed from
# the session data in /etc/sysconfig/sessions.
#
# When Okay is requested, the new parameters are formatted into a new
# configuration line in /etc/sysconfig/sessions.  
#
# See gpslog/logScheduler.c for a full description of the format of 
# entries in /etc/sysconfig/sessions.
#
########################################################################

use strict ;
use CGI qw/:standard/;
use Time::Local ;

use lib '..' ;
use Fireball ;

########################################################################
# Begin formatting the return document.
########################################################################
startPage( 'Set Session' ) ;


# Get the time of day in GPS seconds.  Includes a 13 second offset for
# the UTC to GPS leapseconds.
my $gpsTime = time() - 315964800 + 13 ;


my $theSessionName ;


# Display the parameters, if you want, by enabling the next few lines.
#for my $i (param())
#{
#  print "$i =", param($i), "<Br>" ;
#}


# Read in the data from the sessions configuration file.
my %sessData ;
getSessionData( \%sessData ) ;



########################################################################
# Determine what type of action is being requested from submissionType.
#  Okay: process the New Session or changes to an old one.
#  Delete: remove the named session.
########################################################################
if (param('submissionType') eq 'Okay')
{
  # User has indicated he likes his settings and wants them stored
  # away.  This is either 
  #
  #   editing an existing session 
  #         : take the name from the popup
  #   Creating a new session
  #         : take the name from the NewSessionName text field
  #
  # If this is a manual session, we have to detect if we are just 
  # starting the session, so that we can set the start time to the
  # current time, thus triggering the session to begin.
  # This gets a bit complicated.


  # The following cgi parameters may come in from the form.
  # sessname
  # NewSessionName : Only if this is a new session.
  # Enabled
  # schedMode
  # year
  # month
  # day
  # hours
  # minutes
  # duration
  # Datatype
  # sid

  #print( 'Parameters Received<BR>' ) ;
  #foreach my $p (param())
  #{
  #    print  "$p=", param($p), br() ;
  #}
  #print br() ;

  my $sessname = param('sessname') ;

  if ( $sessname eq 'New Session' )
  {
    $sessname = param('NewSessionName') ;
  }

  # Record the old session information (if it exists) by copying
  # the hash associated with the $name.  This is used later when
  # we want to detect changes when potentially restarting a manual
  # session.  If the old session doesn't exist, we'll give this a value
  # that always conflicts with a valid session.
  my %oldSessData ;

  if (! exists( $sessData{$sessname} ) )
  {
    # No session exists with this name.  We are creating a new one. 
    # Since all of the entries will be filled in below, we really 
    # don't need to create a default template.  However, if this will
    # be an enabled-manual session, we'll need to trigger it to start.
    # That's done later by detecting a change between old and new
    # settings.  To get that to work, we'll define 'enable'.
    # 'enable' will either be 1 or 0, so 2 will never match.
    $oldSessData{'enable'} = 2 ;
  }
  else
  {
    # Otherwise the session we are modifying already exists.
    # Copy all of its settings into %oldSessData.
    %oldSessData = %{$sessData{$sessname}} ;
  }

  # Update the information in the structure for the new/modified session

  # Append the new session configuration data to the new file.
  $sessData{$sessname}{'enable'} = param('Enabled') ? '1' : '0' ;

  my $sched = param('schedMode') ;
  $sched = 'cont' if ($sched eq 'continuous') ;
  $sessData{$sessname}{'sched'} = $sched ;

  $sessData{$sessname}{'duration'} = param('duration') ;

  if ($sched eq 'daily')
  {
    $sessData{$sessname}{'time'} = param('hours' ) * 3600 
                                 + param('minutes') * 60 ;
  }
  elsif ($sched eq 'once')
  {
    my %StringToMonth = ( 'Jan' => 0,
                          'Feb' => 1,
                          'Mar' => 2,
                          'Apr' => 3,
                          'May' => 4,
                          'Jun' => 5,
                          'Jul' => 6,
                          'Aug' => 7,
                          'Sep' => 8,
                          'Oct' => 9,
                          'Nov' => 10,
                          'Dec' => 11
                        ) ;
    # Convert the sec..year values into a UTC time since 01Jan1970.
    # Then add the offset between that and the start of GPS time
    # at 06Jan1980.  This gives us the absolute GPS time for the
    # start of the session.
    $sessData{$sessname}{'time'} = timegm( 0, # Seconds
                                         param('minutes'),
                                         param('hours'),
                                         param('day'),
                                         $StringToMonth{param('month')},
                                         param('year')
                                       ) - 315964800 ;
  }
  elsif ($sched eq 'cont')
  {
    $sessData{$sessname}{'time'} = 0 ;
  }
  else
  {
    # Manual session.  Leave its time as found.  This may be
    # updated later if the session change detector finds a change.

    # There is an odd case below for a new manual session, that isn't
    # enabled.  The 'time' element doesn't get set.  So we give it
    # a default here.
    $sessData{$sessname}{'time'} = 0 ;
  }

  my $dtype = param( 'Datatype' ) ;
  if ( $dtype eq 'T00' )
  {
    $sessData{$sessname}{'ftype'} = 'T00' ;

    my $measInterval = param('measInterval' ) ;
    #$measInterval = 0 if ($measInterval eq 'Off') ;
    #$measInterval =~ s/ .*$// ; # Drop " Sec" from values.
    $sessData{$sessname}{'measInterval'} = $measInterval ;


    my $posInterval = param('posInterval' ) ;
    #$posInterval = 0 if ($posInterval eq 'Off') ;
    #$posInterval =~ s/ .*$// ; # Drop " Sec" from values.
    $sessData{$sessname}{'posInterval'} = $posInterval ;

    $sessData{$sessname}{'option1'} = ( param( 'CodeSmoothing' ) 
                                        ? 1 : 0 
                                      ) ;
    $sessData{$sessname}{'option2'} = ( param( 'CarrierSmoothing' ) 
                                        ? 1 : 0
                                      ) ;
    $sessData{$sessname}{'RT27'} = ( param( 'RT27' ) 
                                     ? 1 : 0
                                   ) ;
    $sessData{$sessname}{'rawWaas'} = ( param( 'RawWaas' ) 
                                        ? 1 : 0
                                      ) ;
  }
  elsif ( $dtype eq 'Binex' )
  {
    $sessData{$sessname}{'ftype'} = 'Binex' ;

    $sessData{$sessname}{'measInterval'} = param( 'measInterval' ) ;
    $sessData{$sessname}{'metaInterval'} = param( 'metaInterval' ) ;
    $sessData{$sessname}{'statusInterval'} = param( 'statusInterval' ) ;
    $sessData{$sessname}{'metTilt'} = param( 'metTilt' ) ;

    $sessData{$sessname}{'smoothing'} = param( 'smoothing' ) || 0 ;
    $sessData{$sessname}{'7f02'} = param( '7f02' ) || 0 ;
    $sessData{$sessname}{'7f03'} = param( '7f03' ) || 0 ;
    $sessData{$sessname}{'7f03Doppler'} = param( '7f03Doppler' ) || 0 ;
    $sessData{$sessname}{'7f03Slips'} = param( '7f03Slips' ) || 0 ;
    $sessData{$sessname}{'7f03RxClkOfsAlways'} 
                                              = param( 'Always' ) || 0 ;
    $sessData{$sessname}{'7f03RxClkOfsRollover'} 
                                            = param( 'Rollover' ) || 0 ;
    
    $sessData{$sessname}{'siteDesc'} = param('siteDesc') || 0 ;
    $sessData{$sessname}{'firmwareVersion'} 
                                       = param('firmwareVersion') || 0 ;
    $sessData{$sessname}{'stationID'} 
                                       = param('stationID') || 0 ;
    $sessData{$sessname}{'antXYZ'} 
                                       = param('antXYZ') || 0 ;
    $sessData{$sessname}{'antNum'} 
                                       = param('antNum') || 0 ;
    $sessData{$sessname}{'rcvrType'} 
                                       = param('rcvrType') || 0 ;
    $sessData{$sessname}{'antOffset'} 
                                       = param('antOffset') || 0 ;
    $sessData{$sessname}{'rcvrNumber'} 
                                       = param('rcvrNumber') || 0 ;

  }
  else
  {
    # Unknown file format type.
    $sessData{$sessname}{'ftype'} = 'unknown' ;
    $sessData{$sessname}{'measInterval'} = 0 ;
    $sessData{$sessname}{'posInterval'} = 0 ;
    $sessData{$sessname}{'option1'} = 0 ;
    $sessData{$sessname}{'option2'} = 0 ;
  }
 
  # Determine the file template portion of the configuration.
  $sessData{$sessname}{'sessid'} = param( 'sid' ) ;

  $sessData{$sessname}{'perday'} = param( 'perDay' ) ? 1 : 0 ;

  $sessData{$sessname}{'persessid'} = param( 'perSID'  ) ? 1 : 0 ;

  ######################################################################
  # That concludes updating the structure from the cgi parameters.
  ######################################################################

  # Check if a manual session restart is needed.
  if (  $sessData{$sessname}{'sched'} eq 'manual'
    and $sessData{$sessname}{'enable'}
     )
  {
    # We need to restart this session if any of its characteristics
    # have changed, most especially, if the session schedule type
    # has just changed to manual.
    for my $k (keys %oldSessData)
    {
      if ($oldSessData{$k} ne $sessData{$sessname}{$k})
      {
        # This is an enabled manual start session and the session is 
        # being modified in some way.  This requires us to restart
        # the session, and hence we have to reset the start time to
        # NOW.  A little margin is added so that the session can start
        # cleanly and get its full duration.

        print h2( "Restarting manual session" ) ;

        # Get the time of day in GPS seconds.  Includes a 13 second
        # offset for the UTC to GPS leapseconds.  It would be nice to
        # have the 13 second value be obtained from a system utility,
        # but that's not available yet.  But even if the value changes
        # by a few seconds, it won't matter.  All that we want is for
        # the time to start the session to be in the future a little
        # bit - about 10 seconds.  Won't matter if that's 9 or 8 or 7.
        my $gpsTime = time() - 315964800 + 13 ;

        $sessData{$sessname}{'time'} = $gpsTime + 10 ;
        last ;
      }
    }
  }

  # Update the session configuration file with the new data, and
  # trigger the log scheduler to act on the change, if any.
  updateSessionData( \%sessData ) ;

  # After a timeout, go back to main Sessions screen.
  print <<EOF;
      <h2>Data for Session "$sessname" Accepted.</h2>
      Switching to Data Logging Sessions Display.
      <Script>
      parent.loadAfterDelay( "/perl-scripts/sessions.cgi", 1 ) ;
      </Script>
EOF
} # end of OKAY section
########################################################################
elsif (param('submissionType') eq 'Delete')
{
  # Delete the session named by sessname.
  my $sessname=param('sessname') ;
  
  # Remove this sessions element from the hash.
  delete( $sessData{$sessname} ) ;

  # Update the session configuration file with the new data, and
  # trigger the log scheduler to act on the change, if any.
  updateSessionData( \%sessData ) ;

  # After a timeout, go back to main Sessions screen.
  print <<EOF;
      <h2>Session "$sessname" has been deleted</h2>
      Switching to Data Logging Sessions Display.
      <Script>
      setTimeout( 'document.location = "/perl-scripts/sessions.cgi"',1000) ;
      </Script>
EOF
} # end of delete section
########################################################################

print end_html() ;

########################################################################
# End of file setSession.cgi
########################################################################
