#!/usr/bin/perl -w
# Copyright (c) 2004 Trimble Navigation Limited
# $Id: iptables.cgi,v 1.7 2004/05/13 00:30:52 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/iptables.cgi,v $

########################################################################
# perl-scripts/iptables.cgi
#
# This script displays the current state of the Iptables security
# system and the controls that allow the user to modify the settings.
#
# An "OK" button accepts changes to the form.
########################################################################

use strict ;
use CGI qw/:standard/;
use Fireball ;

titleWithHelp( 'IP Filtering Configuration',
               '/iptablesInfo.html'
             ) ;

# Determine the I.P. address of the client computer that has 
# accessed this script.  This is fed into all CGI scripts as 
# an environment variable REMOTE_ADDR, in the '192.168.142.7' format.
# This value will be displayed to the user.  It is also used in
# the javascript functions that this script sends to the browser,
# for checking that the client address is in one of the defined ranges
# and warning user that they will lose communications when filtering is
# enabled.
my $clientAddress = $ENV{'REMOTE_ADDR'} ;

#for my $k (sort keys %ENV)
#{
#  print "$k = $ENV{$k}<br>" ;
#}



# This flag is used to indicate later that a warning should be put on
# the page that the OK button needs to be pressed to make changes take
# effect.  This is triggered when lines have been added or deleted to
# the range-list.
my $needsOK = 0 ;


# How many address ranges are defined in the current setup.
my $rangeCount = 0 ;


if (! param() )
{
  # No parameters were fed into this script.
  # So we generate a fresh page using data from the configuration file.
  # This is done by reading in the current config data, and then
  # manually setting the CGI-parameters needed to draw the form.

  # A Hash that will be filled with either the iptables data from
  # the configuration file.
  my %iptablesSetup ;
  getIptablesSetup( \%iptablesSetup ) ;


  # Transfer the data from the hash into the parameters used
  # by the Perl-CGI system.  This sets the defaults for all of the GUI
  # controls we will be creating later.

  param( 'filterEnable', $iptablesSetup{'enable'} ) ;

  # For each address range, we'll be creating a text field and
  # a popup menu.  These will be named 'baseFieldN' and 'bitsFieldN'
  # with N going from 0 to rangeCount-1.
  $rangeCount = $iptablesSetup{'rangeCount'} ;
  for ( my $i = 0 ; $i < $rangeCount ; ++$i )
  {
    param( 'baseField'.$i, $iptablesSetup{'rangeBases'}[$i] ) ;
    param( 'bitsField'.$i, $iptablesSetup{'rangeBits'}[$i] ) ;
  }
}
else
{
  # Parameters were fed into the script.  This indicates that the user
  # has either added or removed an iptables range.  When this happens,
  # the complete form data is fed back to this script, including the
  # changes.  If an addition was made, then the new field was
  # created when the re-post URL was created.  Thus we only need to
  # count the fields and put up the data.  For a Deletion, the
  # parameter 'delete' will be defined as the index of the field to
  # remove.  

  #print( "<pre>Parameters Received<BR>" ) ;
  #foreach my $p (param())
  #{
  #    print  "$p=", param($p), "\n" ;
  #}
  #print "</pre>" ;

  # We want to indicate later that OK must be pressed to
  # make the changes official.
  $needsOK = 1 ;

  # Count how many ranges are defined by looking for baseFieldN
  # with N=0,1,2,3,4.  Rangecount starts at zero.
  while ( defined( param( 'baseField'.$rangeCount ) ) )
  {
    ++$rangeCount ;
  }

  my $deleteIndex = param( 'delete' ) ;
  if ( defined( $deleteIndex ) )
  {
    #print "Deleting $deleteIndex", br() ;
    # To implement the Delete, we'll shift the baseField and bitsField
    # above the selected index down one slot, and then erase the
    # last set.
    --$rangeCount ;
    while ($deleteIndex < $rangeCount)
    {
      param( 'baseField'.$deleteIndex, 
             param( 'baseField'.($deleteIndex+1) )
           ) ;
      param( 'bitsField'.$deleteIndex, 
             param( 'bitsField'.($deleteIndex+1) )
           ) ;
      ++$deleteIndex ;
    }
    CGI::delete( 'baseField'.$deleteIndex ) ;
    CGI::delete( 'bitsField'.$deleteIndex ) ;
  }
}

#print "Count: $rangeCount <br />" ;

# The popup menus (for selecting the size of the address ranges)
# produces values between 8 and 32, and the items are labeled as
# shown below.
my @bitsPopupValues = (8..32) ;
my %bitsPopupLabels = ( 8 =>  '8 : 255.0.0.0',
                        9 =>  '9 : 255.128.0.0',
                       10 => '10 : 255.192.0.0',
                       11 => '11 : 255.224.0.0',
                       12 => '12 : 255.240.0.0',
                       13 => '13 : 255.248.0.0',
                       14 => '14 : 255.252.0.0',
                       15 => '15 : 255.254.0.0',
                       16 => '16 : 255.255.0.0',
                       17 => '17 : 255.255.128.0',
                       18 => '18 : 255.255.192.0',
                       19 => '19 : 255.255.224.0',
                       20 => '20 : 255.255.240.0',
                       21 => '21 : 255.255.248.0',
                       22 => '22 : 255.255.252.0',
                       23 => '23 : 255.255.254.0',
                       24 => '24 : 255.255.255.0',
                       25 => '25 : 255.255.255.128',
                       26 => '26 : 255.255.255.192',
                       27 => '27 : 255.255.255.224',
                       28 => '28 : 255.255.255.240',
                       29 => '29 : 255.255.255.248',
                       30 => '30 : 255.255.255.252',
                       31 => '31 : 255.255.255.254',
                       32 => '32 : 255.255.255.255',
                      ) ;


print startform( -name=>'theForm', 
                 -action=>'/perl-scripts/sysadmin/setIptables.cgi'
               ) ;

# Create the Enable and Disable buttons.
my @buttons = radio_group( -name=>'filterEnable',
                           -values=>[ 1, 0 ],
                           -nolabels=>1,
                         ) ;

# Put those buttons in a table with the commentary on each choice.
print b('IP Filtering') ;
print table( {border=>1, width=>'70%'},
        Tr( td( $buttons[0]),
            td( b('Enabled'),
                br(),
                'Reject IP packets from all IP addresses except those '
               .'ranges listed below.'
              )
          ),
        Tr( td( $buttons[1]),
            td( b('Disabled'), 
                br(), 
                'Accept IP packets from any IP addresses.  Do not '
               .'filter based on the ranges listed below.'
              )
          )
      ) ;



# Format the Address Range section.
#
print br(), 
      br(),
      b('Address Ranges'),
      br(),
      'If IP Filtering is enabled, only respond to packets '
      .'from these IP addresses.',
      br(),
      "Your current client address is $clientAddress" ;

if ( $rangeCount )
{
  # Creat the table of ranges.  A Title row, then one row per range.
  print "<table border=1>\n" ;
  print Tr( th('Base Address'), th('Bits : Netmask'), th('Delete') ) ;
  my $basecount = 0 ;
  for (my $i = 0 ; $i < $rangeCount ; ++$i )
  {
    print Tr( 
              # The range's base address
              td( textfield( 'baseField'.$i,
                             '',
                             20, 
                             20 
                           ) 
                ),

              # The range size popup menu button.
              td( popup_menu( -name=>'bitsField'.$i,
                              -values=>\@bitsPopupValues,
                              -labels=>\%bitsPopupLabels
                            )
                ),

              # The delete button
              td( {align=>'center'}, 
                  a( {href=>"javascript:deleteRange($i)"},
                    img( { src=>'/images/redX.gif',
                          border=>'0',
                          alt=>'Delete' 
                         } 
                       )
                   )
                )
            ) ;
  }
  print "</table>\n" ;
}
else
{
  print "<br /><br />No ranges specified!<br /><br />" ;
}



# A button to add a new range.  
# Calls the javascript function newRange()
print br(),
      button( -name=>'AddNewRange',
              -value=>"Add New Range",
              -onClick=>"newRange( $rangeCount )"
            ) ; 

# If this page has been altered (say by adding or deleting ranges)
# then we need to warn the user that OK must be pressed.
if ($needsOK)
{
  print br(),
        br(),
        b( "Additions or Removals need OK to take effect." ) ;
}


# After a horizontal rule, put the OK and Cancel buttons.
print br(), 
      br(),
      hr(), 
      br(), 
      button( -name=>"OK", 
              -value=>"OK",
              -onClick=>'okayClicked()'
            ), 
      button( -name=>'Cancel', 
              -value=>'Cancel',
              -onClick=>
          'parent.loadDataFrame( "/perl-scripts/internetEthernet.cgi" )'
            ),
      endform() ;

# Javascript functions used by the above forms.
print <<EOF ;
<script>


/**********************************************************************/
function newRange( newCount )
/**********************************************************************/
/* Called when the New Range button is clicked.
 * This calls the CGI script again, but with updated parameters.
 * All of the existing widgets are encoded in the URL, and 
 * new baseField and bitsField parameters are added, based on the
 * newCount.  The whole thing is then submitted to the NetRS which
 * will reformat the page.
 */
{
  f = document.theForm ;

  // Create the URL/CGI-request to submit.
  // We want this to contain all of the CGI parameters for the
  // existing elements.  
  url = '/perl-scripts/iptables.cgi' ;

  // Go through the form's elements one by one and create a
  // CGI parameter for each of them.  Each parameter is prefixed with
  // a '&' except the first one which gets a '?'.
  // We only have to handle TextFields, radio buttons, and the
  // popup menus (== select-one)
  ch = '?' ;
  for (i = 0 ; i < f.elements.length ; ++i )
  {
    e = f.elements[i] ;
    if ( e.type == 'text'
      || e.type == 'radio' && e.checked
       )
    {
      url += ch + e.name + '=' + escape( e.value ) ;
      ch = '&' ;
    }
    else
    if (e.type == 'select-one')
    {
      p = e.options[e.selectedIndex].value ;
      url += ch + e.name + '=' + escape( p ) ;
      ch = '&' ;
    }
  }

  /* And we want the URL to contain parameters to create the a new 
   * element that isn't already on the page.  Default the address
   * portion to the client's address, and the netmask field to 24 
   * which is a class C subnet.  The defaults make it likely that
   * an accidentally created iptables setup will work okay and not
   * cut off communications.
   */
  url += '&baseField' + newCount + '=' + '$clientAddress'
       + '&bitsField' + newCount + '=24' ;

  // Submit the request, which will display the updated form.
  parent.loadDataFrame( url ) ;

} /* end of newRange() */



/**********************************************************************/
function deleteRange( rangeIndex )
/**********************************************************************/
/* Called when a Delete range button is clicked.  The index of the range
 * to delete is passed in as the "rangeIndex" parameter.
 *
 * This function calls the CGI script again, but with updated
 * parameters.  All of the existing widgets are encoded in the URL.
 * A special 'delete=' parameter is appended to indicate the index
 * of the address range we want deleted.
 * The whole thing is then submitted to the NetRS which will
 * reformat the page.
 */
{
  f = document.theForm ;
  
  // Start creating the URL/CGI-request to submit.
  url = '/perl-scripts/iptables.cgi' ;

  // Go through the form's elements one by one and create a
  // CGI parameter for each of them.  Each parameter is prefixed with
  // a '&' except the first one which gets a '?'.
  // We only have to handle TextFields, radio buttons, and the
  // popup menus (== select-one)
  ch = '?' ;
  for (i = 0 ; i < f.elements.length ; ++i )
  {
    e = f.elements[i] ;
    if ( e.type == 'text'
      || e.type == 'radio' && e.checked
       )
    {
      url += ch + e.name + '=' + escape( e.value ) ;
      ch = '&' ;
    }
    else
    if (e.type == 'select-one')
    {
      p = e.options[e.selectedIndex].value ;
      url += ch + e.name + '=' + escape( p ) ;
      ch = '&' ;
    }
  }

  /* Indicate that of those existing fields, we want to delete
   * one of them.  The perl script will figure out how to do that.
   */
  url += '&delete=' + rangeIndex ;

  // Submit the request, which will display the updated form.
  parent.loadDataFrame( url ) ;
 
} /* end of deleteRange() */


/**********************************************************************/
function ipField( ip, n )
/**********************************************************************/
/* ip : an IP address string of the form 192.168.142.7
 * n  : a number from 1 to 4
 * Returns the specified numeric field from the ip address.
 * For example, field 3 of 192.168.142.7 would be 142.
 */
{
  var i ;
  var j ;
  i = 0 ; // start of field.
  ip = ip + '.' ; // Terminate final field with a '.' 

  while (n > 1)
  {
    i = ip.indexOf( '.', i ) + 1 ;
    --n ;
  }

  j = ip.indexOf( '.', i ) ;

  return ip.substring( i, j ) ;

} /* end of ipField() */



/**********************************************************************/
function ipInRange( ip, rangeName )
/**********************************************************************/
/* Check if a given IP address falls in the range specified by
 * rangeName.
 * ip        : An Ip address of the form "192.168.142.7"
 * rangeName : The name of a text entry field that contains the
 *             base address of an address range.  This will be of the
 *             form 'baseField3'.  Associated with that will be a
 *             popup/select field named 'bitsField3'.
 * The idea is to determine if 'ip' falls inside the range specified
 * by baseFieldN and bitsFieldN.
 */
{
  var f ;
  f = document.theForm ;

  // Get the IP string from the baseField.
  var rangeIP ;
  rangeIP = f[ rangeName ].value ;

  // Determine the size field associated with the rangeName,
  // and extract the bitmask from that.
  // The sizename is of the form bitsFieldN.  That contains an
  // index specifier, and an options list.  The options will
  // have text fields of the form:
  //    31 : 255.255.255.254  << Note the spaces around ':'.
  // We want to find the selected text and extract the netmask field.

  var sizeName, sizeIndex, sizeLabel, sizeIP ;
  sizeName = 'bitsField' + rangeName.substring( 9 ) ;
  sizeIndex = f[sizeName].selectedIndex ;
  sizeLabel = f[sizeName].options[sizeIndex].text ;
  sizeIP = sizeLabel.substring( sizeLabel.lastIndexOf( ' ' ) + 1 ) ;


  var result = true ;
  var i ;
  for (i = 1 ; i <= 4 ; ++i)
  {
    var b1, b2, mask ;
    b1 = ipField( ip, i ) ;
    b2 = ipField( rangeIP, i ) ;
    mask = ipField( sizeIP, i ) ;

    // Compare the masked octet from the ip and the rangeBase.
    // If they don't match, then we set the result false.
    // we return false.
    if ( (b1 & mask) != (b2 & mask) ) 
      result = false ;
  }

  /*
  alert( "checking " + ip 
       + " in " + rangeName +' : ' + sizeName
       + "\\n"
       + rangeIP
       + "\\n"
       + sizeIP 
       + "\\n"
       + result
       ) ;
  */

  return result ;

} /* end of ipInRange() */



/**********************************************************************/
function okayClicked()
/**********************************************************************/
/* Called when the OK button is clicked.
 * This triggers submission of the form data.
 *
 * Before submission, the text entry fields are checked for valid
 * numeric values.  Errors will put up an alert and submissions will
 * be canceled.
 */
{
  f = document.theForm ;

  // Check all of the text entry fields to be certain that they
  // contain valid IP addresses.
  currentClientFound = false ;
  for (i = 0 ; i < f.elements.length ; ++i )
  {
    e = f.elements[i] ;
    if ( e.type == 'text' )
    {
      ip = e.value ;
      if (parent.notValidIP( ip ))
      {
        alert( "Base addresses must all be legal IP address "
             + "of the form 192.168.10.1" ) ;
        return ;
      }

      // Check if this address range includes the current 
      // client address.
      if ( ipInRange( "$clientAddress", e.name ) )
        currentClientFound = true ;
    }

  }


  if (f.filterEnable[0].checked)
  {
    // If Filtering is enabled, be sure that at least one ACCEPT
    // line is defined.  Otherwise we would lock out all IP access.
    if (f.baseField0 == null)
    {
      alert( "Can't enable IP Filtering without at least "
            +"one defined address range."
           ) ;
      return ;
    }

    // If Filtering is enabled, and the current client wasn't found
    // in the address ranges, warn the user and give them a chance to
    // cancel the operation.
    if ( ! currentClientFound
      && ! confirm( "Current client address ($clientAddress) not "
                 + "included in address ranges.\\n"
                 + "You won't be able to communicate after this change?"
                 + "\\nDo you wish to continue?"
                  )
       )
      return ;
  
    // Make the user confirm that he wants IP filtering and give
    // a warning that this is dangerous.
    if (! confirm( "Accept New IP filtering Configuration?"
                  +"\\nWarning: An error here could disable "
                  +"communications with this NetRS." 
                 )
       )
      return ;
  }
  
  parent.submitForm( f, '' ) ;

} /* end of okayClicked() */


</script>
EOF


print end_html() ;

########################################################################
# End of script iptables.cgi
########################################################################
