#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: portSummary.cgi,v 1.26 2004/06/01 18:13:00 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/portSummary.cgi,v $

########################################################################
# portSummary.cgi
#
# Script for displaying the summary of Input/Output port configuration.
# This describes the Serial port configurations and any TCP/UDP/REP 
# ports that have been set up for streaming.
########################################################################

use strict ;
use CGI qw/:standard/;
use Fireball ;

titleWithHelp( 'Input/Output Summary',
               '/portInfo.html'
             ) ;


########################################################################
# Make all links blue with no underlines, even if visited, etc.
# This is a document-level style specification, affecting all
# links in this frame.
########################################################################
print <<EOF ;
<style type="text/css">
 <!--
       a {color:blue ; text-decoration: none }
 -->
</style>
EOF



# Read the complete port configuration into a hash structure.
my %allPortData ;
getPortSetup( \%allPortData ) ;


########################################################################
# Create the list of items for the table.
# Sort to put the Serial Ports first, followed by REP,TCP,UDP.
########################################################################
my @portList = sort keys %allPortData ;
@portList = ( (grep /Serial/, @portList),
              (grep !/Serial/, @portList) 
            ) ;

########################################################################
# Create the table containing the configuration data.
########################################################################
print '<Table align="center" border=1>' ;

print Tr( th( ['Port',
               '&nbsp;Service&nbsp;',
               '&nbsp;Enable&nbsp;'
              ] 
            )
        );

print startform( -name=>'theForm' ) ;

foreach my $p (@portList)
{
  next if ($p eq 'AuthKey') ;
  next if ($p eq 'MetControls') ;

  my $service = $allPortData{$p}{'service'} ;
  my $enable = $allPortData{$p}{'enable'} ;

  next if ($service eq 'internal4') ;
  next if ($service eq 'internal3') ;

  $service = 'PPP' if ($service eq 'ppp') ;
  $service = 'PPP-Dialout' if ($service eq 'ppp-dialout') ;
  $service = 'RTCM' if ($service eq 'Rtcm') ;
  $service = 'SV Status' if ($service eq 'SvStatus') ;

  my $name = $p ;
  $name =~ s/Port/ Port / ;
  $name =~ s/tcp/TCP/i ;
  $name =~ s/udp/UDP/i ;
  $name =~ s/rep/REP/i ;

  my $target= 'portEnableDisable( "'
              . $name
              . '", "'
              . ($enable ? 'disable")' : 'enable")')
              ;

  print Tr( {align=>'center'},
          td( '&nbsp;',
              a( {href=>'portSetup.cgi?restart='.CGI::escape($p)}, 
                 $name ) ,
              '&nbsp;'
            ),
          td( '&nbsp;'.$service.'&nbsp;', ),
          td( checkbox( -name=>"Enable$name",
                        -checked=>$enable,
                        -value=>'1',
                        -label=>'',
                        -onClick=>$target
                      )
            )
          );
}

print '</Table>' ;

print endform() ;

# Emit the javascript function used to submit the Enable/Disable
# requests above.
print <<EOF ;
<script>

/**********************************************************************/
function portEnableDisable( portname, state )
/**********************************************************************/
/* Enable or disable the named port.
 * portname will be something like "serial port 4" or "TCP Port 5123"
 * state will be "enable" or "disable" which is the desired new state.
 */
{
  url = '/perl-scripts/gpsadmin/portEnableDisable.cgi?'
      + state  // enable or disable
      + '=' 
      + escape(portname) ;

  // If the user is disabling a port, ask for confirmation.
  // If he aborts, then re-check the checkbox he just unchecked.
  if (state == "disable")
  {
    if (! confirm( "Disable " + portname ) )
    {
      f = document.theForm ;
      f["Enable"+portname].checked = 1 ;
      return ;
    }
  }

  // Submit the request to enable or disable the port.
  parent.loadDataFrame( url ) ;
}

</script>
EOF



########################################################################
# Some action link/buttons at the bottom of the page.
########################################################################
print '<br /><br />' ;
print table( {align=>'center', width=>'60%', border=>2},
            Tr( td( {align=>'center', width=>'25%' },
                    a( {href=>'portSetup.cgi'} , 
                       'Edit<br />Port Setups' 
                     )
                  ),
                td( {align=>'center', width=>'25%'},
                    a( {href=>'portSetup.cgi?restart='.
                              CGI::escape( 'New TCP Port') },
                       'Create New<br />TCP Port'
                     )
                  ),
                td( {align=>'center', width=>'25%'},
                    a( {href=>'portSetup.cgi?restart='.
                              CGI::escape( 'New UDP Port') },
                       'Create New<br />UDP Port'
                     )
                  ),
                td( {align=>'center', width=>'25%'},
                    a( {href=>'portSetup.cgi?restart='.
                              CGI::escape( 'New REP Port') },
                       'Create New<br />REP Port'
                     )
                  ),
              )
           ) ;


print end_html() ;

########################################################################
# end of script portSummary.cgi
########################################################################
