#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: pps.cgi,v 1.8 2004/04/21 16:00:02 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/pps.cgi,v $

########################################################################
# pps.cgi
#
# Puts up a form that allows user to view and enter the controls for the
# the Pulse-Per-Second feature.
# Can be used to enable or disable the pulses and to set the polarity of
# the leading edge.
#
# No CGI parameters are expected.  We just display the current state
# of the controls in an HTML form.  When the suer clicks OK, we submit
# the data to a separate gpsadmin-secured script setPps.cgi.
#
# The pps configuration is saved in a line from 
# /etc/sysconfig/stinger.cfg.  The line is of the forms: 
#
#   put pps_mode on pos
#   put pps_mode off neg
#
# That same format is used for direct control of Stinger using the
# Stinger Info connections.
#
# The sysinfo system is used to query the current setting of the
# information using 'sysinfo -p pps_mode', which returns a 
# "put pps_mode command" string showing the current settings.
#
# NOTE ON SLOPE...  The hardware driver in NetRS (and some other
# products) is unable to generate a negative-going pulse.  Thus, NetRS
# can only support a positive slope pulse.  We'll preserve the Stinger
# fiction that we can do both, but the GUI and Programmatic interface
# will no longer provide an option for negative slope/negative pulse.
#
########################################################################


use strict ;
use CGI qw/:standard/;
use Fireball ;

titleWithHelp( 'PPS (Pulse-Per-Second) Output',
                         '/ppsInfo.html'
                       ) ;


# Determine the current setting of the pps controls.
# This is done by querying the gpsinfo database via 'sysinfo', which
# must be run as root via sudo.
# The response will be of the form:
#   put pps_mode [on|off] [pos|neg]

my (undef, undef, $pps_enable, $pps_slope) 
   = split( " ",  
            qx(  /usr/bin/sudo /usr/local/fireball/sysinfo -p pps_mode )
          ) ;

param( 'PpsEnable', ($pps_enable eq 'on' ? 'On' : 'Off' ) ) ;

print startform( -name=>'theForm', 
                 -action=>'/perl-scripts/gpsadmin/setPps.cgi'
               ) ;


my @PPSControl = radio_group( -name=>'PpsEnable', 
                              -value=>[ 'On', 'Off' ],
                              -nolabels=>1,
                              -default=>'On',
                            ) ;

print table( {border=>'1'},
             Tr(
                td( {width=>'30', align=>'center'}, $PPSControl[0] ),
                td( b('Enabled'),br(),
                    'A one pulse per second signal will be generated',
                    br(),
                    'with its positive edge aligned to UTC.'
                  )
               ),
             Tr(
                td( {align=>'center'}, $PPSControl[1] ),
                td( b('Disabled'),
                    br(),
                    'No PPS signal will be generated.'
                  )
               )
           ) ;

print br(), hr() ;

# Put up the buttons.
print button( -name=>'OK', 
              -value=>'OK',
              -onClick=>'parent.submitForm( document.theForm, "" ) '
            ) ;
print '&nbsp;' ;
print button( -name=>'Cancel', 
              -value=>'Cancel',
              -onClick=>'parent.loadDataFrame( 
                           "/perl-scripts/rconfigDisplay.cgi" 
                                             )'
            ) ;


print endform() ;

 
print end_html() ;

########################################################################
# end of file pps.cgi
########################################################################
