#!/usr/bin/perl -w
# Copyright (c) 2002 Trimble Navigation Limited
# $Id: progTest.cgi,v 1.61 2006/03/23 19:55:23 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/progTest.cgi,v $

########################################################################
# progtest.cgi
#
# Provides a GUI interface to allow testing the programmatic 
# interface.
#
# The fireball interface window will show a bunch of program commands
# that can be issued.  When these are clicked or submitted, javascript
# will submit those CGI requests to the fireball, directing the output
# to a separate window.  The requests can be entered directly in a text
# widget.
#
########################################################################

use strict ;
use CGI qw/:standard/;
use Fireball ;

if (! param() )
{
  Fireball::FBheader() ;

  print <<EOF ;
  <html xmlns="http://www.w3.org/1999/xhtml" lang="en-US">
  <head>
  <title>Programmatic Test Frames</title> 
  <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1"/>
  <meta name="copyright" content="Copyright(c) 2003 Trimble Navigation Ltd." />
  </head>

  <frameset rows="*,30%" >
    <frame src="/perl-scripts/progTest.cgi?group=Status" name=top frameborder=no>
    <frameset rows="30,*">
    <frame src="/blankpage.html" name=command scrolling=no >
    <frame src="/blankpage.html" name=results>
    </frameset>
  </frameset>

  </html>
EOF

  goto EXIT ;
}

titleWithHelp( 'Programmatic Interface',
               '/progInfo.html'
             ) ;

# Make all links blue with no underlines, even if visited, etc.
# This is a document-level style specification, affecting all
# links in the frame.
print <<EOF ;
<style type="text/css">
 <!--
       a {color:blue ; text-decoration: none }
 -->
</style>
EOF


print startform( -name=>"theForm",
                 -action=>"javascript:submitRequest()"
               ) ;

print button( -name=>'send', 
                   -value=>'Send',
                   -onClick=> 'submitRequest()'
                 ) ;
print '&nbsp;&nbsp;&nbsp;' ;

print textfield( 'Command', '', 60, 200 ) ;

print '&nbsp;&nbsp;&nbsp;' ;

print checkbox( -name=>"autosubmit", 
                -label=>'AutoSubmit', 
                -checked=>1 
              ) ; 


########################################################################
sub makeGroupButton
########################################################################
# Makes a button which when clicked switches this page to a different
# "group" of programmatic commands.
# $1 = group name, to be used as a Label.
########################################################################
{
  my $link = "/perl-scripts/progTest.cgi?group=$_[0]" ;
  return td( a( {href =>$link},
                img( {src=>'/images/reddot.gif', border=>0} )
              ), 
             a( {href =>$link}, $_[0] ) ,
           ) ;
}

print "<table><tr>" ;
print makeGroupButton( 'Status' ) ;
print makeGroupButton( 'Sessions' ) ;
print makeGroupButton( 'Configuration1' ) ;
print makeGroupButton( 'ConfigFiles' ) ;
print makeGroupButton( 'I/O' ) ;
print makeGroupButton( 'Security' ) ;
print "</tr>" ;
print "<tr>" ;
print makeGroupButton( 'Satellites' ) ;
print makeGroupButton( 'DataFiles' ) ;
print makeGroupButton( 'Configuration2' ) ;
print makeGroupButton( 'Internet' ) ;
print makeGroupButton( 'Firmware' ) ;
print "</tr></table>" ;


########################################################################
sub makeButton
########################################################################
# Generate and return the text needed to make a labeled action button
# for inclusion as the cell data in a table.  
# $_[0] is the text to use as the label.
# $_[1] is the value submitted to javascript function 'doCommand()' 
# when the button is pressed.
{
  my $value = td( a( {href =>"javascript:doCommand( \"$_[1]\" )"},
                     img( {src=>'/images/bluedot.gif', border=>0} ),"\n"
                   ), 
                  a( {href =>"javascript:doCommand( \"$_[1]\" )"}, 
                     $_[0],"\n" 
                   ) 
                ) ;
  $value .= td({width=>10}).td( $_[2] ) if ($_[2]) ;
  return Tr($value );
}



print <<EOF ;

<script src="/progInfo.js">
</script>


<script>





////////////////////////////////////////////////////////////////////////
function makeButton( 
                     label, 
                     cmd, 
                     desc
                   ) 
////////////////////////////////////////////////////////////////////////
{
  baseCommand = cmd ;

  // Have to do string replacements on the inputs, but the
  // regular expression functions in Javascript don't work on
  // Internet Explorer!!!!  Have to do these the old-fashioned way.

  //baseCommand = cmd.replace( /&.*/, "" ) ;
  index = baseCommand.indexOf( '&' ) ;
  if (index > 0)
  {
    baseCommand = baseCommand.substring(0,index);
  }

  //baseCommand = baseCommand.replace( /?/, " " ) ;
  while ( (index = baseCommand.indexOf( '?' )) >= 0 )
  {
    baseCommand = baseCommand.substring(0,index) 
                + ' ' 
                + baseCommand.substring(index+1)
  }

  //baseCommand = baseCommand.replace( /^\*/, "" ) ;
  if (baseCommand.substring(0,1) == '*')
  {
    baseCommand = baseCommand.substring(1) ; 
  }

  //anchor = baseCommand.replace( / /g, "" ) ;
  anchor = baseCommand ;
  while ( (index = anchor.indexOf( ' ' )) != -1)
  {
    anchor = anchor.substring(0,index)
           + anchor.substring(index+1) ;
  }

  // Find the baseCommand in the CommandList.  Set the index to 
  // the proper index.
  index = 0 ;
  for ( index = 0 ; index < CommandCount ; ++index )
  {
    if (CommandList[index] == baseCommand)
    {
      break ;
    }
  }


  if (desc == '')
  {
    desc = CommandDesc[index] ;
  }

  d=document ;
  d.writeln( "<tr><td>",
              "<a href=\\"javascript:doCommand( '",
              cmd,
              "')\\">",
              '<img src="/images/bluedot.gif" border=0 \>',
              label,
              '</a>',
             '</td><td width=10>',
             '<a href=\\"javascript:top.showHelp(\\'',
             CommandPage[index],
             '#', 
             anchor,
             '\\')"><Img src="/images/littlehelp.gif" border=0/></a>',
             '</td><td>',
             desc,
             '</td></tr>\\n' 
           ) ;
}
</script>
EOF



sub jsButton
{
  my $desc = $_[2] || '' ;
  return "<script>makeButton( '$_[0]',\"$_[1]\",'$desc' )</script>\n" ;
}


my $group = param('group') ;

if ($group eq 'Status')
{
  print b("Status Commands") ;
  print table(
        jsButton( 'Show SystemName', 
                  'Show?SystemName'
                ),
        jsButton( '*Set SystemName', 
                  '*Set?SystemName&name=SYSNAME'
                ),
        jsButton( 'Show SerialNumber', 
                  'Show?SerialNumber'
                ),
        jsButton( 'Show UtcTime', 
                  'Show?UtcTime'
                ),
        jsButton( 'Show GpsTime', 
                  'Show?GpsTime'
                ),
        jsButton( 'Show Position', 
                  'Show?Position', 
                ),

        jsButton( 'Show Voltage (input=1)', 
                  'Show?Voltage&input=1' ,
                  'Show the input voltage on power input 1.'
                ),
        jsButton( 'Show Voltage (input=2)', 
                  'Show?Voltage&input=2' ,
                  'Show the input voltage on power input 2.'
                ),
        jsButton( 'Show Temperature', 
                  'Show?Temperature' ,
                ),
    ) ;
}
elsif ( $group eq 'Firmware')
{
  print b("Firmware Commands") ;
  print table(
        jsButton( 'Show FirmwareVersion', 
                  'Show?FirmwareVersion',
                ),
        jsButton( 'Show FirmwareWarranty', 
                  'Show?FirmwareWarranty',
                ),
        jsButton( 'Set FirmwareWarranty', 
                  'Set?FirmwareWarranty&code=b957c46872ea70c7',
                ),
        jsButton( 'Upload FirmwareFile', 
                  'Upload?FirmwareFile',
                ) ,
        jsButton( 'Show FirmwareFiles', 
                  'Show?FirmwareFiles',
                ),
        jsButton( 'Show FirmwareFile', 
                  'Show?FirmwareFile&name=test.rpm',
                ),
        jsButton( 'Verify FirmwareFile', 
                  'Verify?FirmwareFile&name=test.rpm',
                ) ,
        jsButton( '*Install FirmwareFile',
                  '*Install?FirmwareFile&name=test.rpm',
                ) ,
        jsButton( '*Delete FirmwareFile', 
                  '*Delete?FirmwareFile&name=test.rpm',
                ) ,
        jsButton( '*Delete All FirmwareFiles', 
                  '*Delete?FirmwareFile&all=yes',
                  'Delete ALL uploaded firmware files.',
                ) ,
    ) ;
}
elsif ( $group eq 'Satellites')
{
  print b("Satellite Commands") ;
  print table(
    jsButton( 'Show TrackingStatus', 
              'Show?TrackingStatus',
            ) ,
    jsButton( 'Show GpsHealth', 
              'Show?GpsHealth',
            ) ,
    jsButton( 'Show GpsSatControls', 
              'Show?GpsSatControls', 
            ) ,
    jsButton( 'Set GpsSatControls', 
             'Set?GpsSatControls&Enable=All&disable=5,6&ignorehealth=7',
            ) ,
    jsButton( 'Show GpsEphemeris', 
              'Show?GpsEphemeris&prn=1',
            ) ,
    jsButton( 'Show GpsAlmanac', 
              'Show?GpsAlmanac&prn=1',
            ) ,
    jsButton( 'Show GpsUtcData', 
              'Show?GpsUtcData',
            ) ,
    jsButton( 'Show GpsIonoData', 
              'Show?GpsIonoData',
            ) ,
    jsButton( 'Reset GpsData', 
              'Reset?GpsData',
            ),
    jsButton( 'Show WaasTracking', 
              'Show?WaasTracking',
            ) ,
    jsButton( 'Set WaasTracking', 
              'Set?WaasTracking&enable=yes',
            ) ,
    jsButton( 'Show WaasSatControls', 
              'Show?WaasSatControls',
            ) ,
    jsButton( 'Set WaasSatControls', 
         'Set?WaasSatControls&Disable=All&enable=134&ignorehealth=none',
            ) ,
    ) ;
}
elsif ( $group eq 'Configuration2')
{
  print b("Configuration Commands (part 2)") ;
  print table(
    jsButton( 'Show MultipathReject', 
              'Show?MultipathReject',
              'Display the current Multipath Rejection control setting.'
            ) ,
    jsButton( 'Set MultipathReject', 
              'Set?MultipathReject&enable=yes',
              'Modify the Multipath Rejection control setting.'
            ) ,
    jsButton( 'Show L2Tracking', 
              'Show?L2Tracking',
            ) ,
    jsButton( 'Set L2Tracking', 
              'Set?L2Tracking&mode=CorY',
            ) ,
    jsButton( 'Show PPS', 
              'Show?PPS',
              'Display the current settings of the '
             .'Pulse-Per-Second controls.'
            ) ,
    jsButton( 'Set PPS', 
              'Set?PPS&enable=yes&slope=positive',
              'Modify the settings of the Pulse-Per-Second controls.'
            ) ,
    jsButton( 'Show AntennaTypes',
              'Show?AntennaTypes',
              'Display a list of supported antenna types.'
            ) ,
    jsButton( 'Show Antenna',
              'Show?Antenna',
              'Display the current Antenna specifications.'
            ) ,
    jsButton( 'Set Antenna',
              'Set?Antenna&type=27&serialNumber=12345&height=1.234'
             .'&refpoint=APC',
             'Modify the Antenna specifications.'
            ) ,
    ) ;
}
elsif ( $group eq 'Configuration1')
{
  print b("Configuration Commands (part 1)") ;
  print table(
    jsButton( '*Reset System', 
              '*Reset?System',
            ) ,
    jsButton( 'Show ReferenceFrequency', 
              'Show?ReferenceFrequency',
            ) ,
    jsButton( '*Set ReferenceFrequency', 
              '*Set?ReferenceFrequency&source=external',
            ) ,
    jsButton( 'Show ElevationMask', 
              'Show?ElevationMask',
              'Display the current Elevation Mask control setting.'
            ) ,
    jsButton( 'Set ElevationMask', 
              'Set?ElevationMask&mask=0',
              'Modify the Elevation Mask control setting.'
            ) ,
    jsButton( 'Show PdopMask', 
              'Show?PdopMask',
              'Display the current PDOP Mask control setting.'
            ) ,
    jsButton( 'Set PdopMask', 
              'Set?PdopMask&mask=7',
              'Modify the PDOP Mask control setting.'
            ) ,
    jsButton( 'Show ClockSteering', 
              'Show?ClockSteering',
              'Display the current Clock Steering control setting.'
            ) ,
    jsButton( 'Set ClockSteering', 
              'Set?ClockSteering&enable=yes',
              'Modify the current Clock Steering control setting.'
            ) ,
    jsButton( 'Show ShutdownVoltage', 
              'Show?ShutdownVoltage',
            ) ,
    jsButton( 'Set ShutdownVoltage', 
              'Set?ShutdownVoltage&volts=10.68',
            ) ,
    ) ;
}
elsif ( $group eq 'ConfigFiles')
{
  print b("ConfigFile Commands") ;
  print table(
    jsButton( 'Show ConfigFiles', 
              'Show?ConfigFiles',
            ) ,
    jsButton( 'Show ConfigFile',
              'Show?ConfigFile&name=Testing',
            ) ,
    jsButton( 'Create ConfigFile',
              'Create?ConfigFile&name=Testing&'
             .'functions=Sessions,FTP',
            ) ,
    jsButton( 'Delete ConfigFile',
              'Delete?ConfigFile&name=Testing',
            ) ,
    jsButton( 'Upload ConfigFile',
              'Upload?ConfigFile&name=Testing',
            ) ,
    jsButton( '*Install ConfigFile',
              '*Install?ConfigFile&name=Testing',
            ) ,
    ) ;
}
elsif ( $group eq 'Internet')
{
  print b("Internet Commands") ;
  print table(
    jsButton( 'Show Ethernet',
              'Show?Ethernet',
            ),
    jsButton( '*Set Ethernet',
              '*Set?Ethernet&ipmode=static',
            ),
    jsButton( 'Show FtpSetup',
              'Show?FtpSetup',
            ),
    jsButton( 'Set FTP (clearPW)',
              'Set?FtpSetup&namedClearPW=netrs&adminClearPW=admin',
              'Modify FTP passwords, using cleartext passwords.'
            ),
    jsButton( 'Set FTP (EncryptedPW)',
              'Set?FtpSetup&namedPassword=mPOjPVn6bCMIw'
             .'&adminPassword=YRNcvOGsvrIBo',
              'Modify FTP passwords, using encrypted passwords.'
            ),
    jsButton( 'Set FTP (all enabled)',
              'Set?FtpSetup&anon=enabledWithDelete'
             .'&named=enabledWithDelete&admin=Enabled',
              'Modify settings to completely enable FTP.'
            ),
    jsButton( 'Set FTP (all disabled)',
              'Set?FtpSetup&anon=disabled&named=disabled'
             .'&admin=disabled&namedPassword=NotSet'
             .'&adminPassword=NotSet',
              'Modify settings to completely disable FTP.'
            ),
    jsButton( 'Show NtpClient', 
              'Show?NtpClient',
            ), 
    jsButton( 'Set NtpClient', 
              'Set?NtpClient&enable=yes&servers=192.168.142.1', 
            ), 
    jsButton( 'Show HttpPorts', 
              'Show?HttpPorts',
            ), 
    jsButton( 'Set HttpPorts', 
              'Set?HttpPorts&httpPort80=yes&httpsPort443=yes',
            ), 
    jsButton( 'Show IpFiltering', 
              'Show?IpFiltering',
            ), 
    jsButton( 'Set IpFiltering', 
              'Set?IpFiltering&enable=no&range1=192.168.142.0/24',
            ), 
    ) ;
}
elsif ( $group eq 'I/O')
{
  print b("I/O (Input/Output) Commands") ;
  print table(
    jsButton( 'Show IOstatus', 
              'Show?IOstatus',
            ),
    jsButton( 'Show IOports', 
              'Show?IOports',
            ),
    jsButton( 'Show IOport',
              'Show?IOport&port=SerialPort1',
            ),
    jsButton( 'Set IOport (ephemerides on TCP)',
              'Set?IOport&port=TcpPort9999&service=ephemerides',
            ),
    jsButton( 'Set IOport (configure PPP)',
              'Set?IOport&port=serialport2&service=ppp&baud=57600'
             .'&flow=none&connection=modem&reset=ReseT'
             .'&init1=InitString&localip=10.1.1.1&remoteip=10.2.2.2',
            ),
    jsButton( 'Set IOport (configure RT17 on TCP)',
              'Set?IOport&port=TcpPort9999&service=rt17'
             .'&interval=1&measurements=yes&concise=yes'
             .'&enable=yes&clientAuth=no',
            ),
    jsButton( 'Delete IOport', 
              'Delete?IOport&port=TcpPort9999',
            ),
    jsButton( '*Delete IOport (all)', 
              '*Delete?IOport&All=yes',
            ),
    jsButton( 'Show RefStation', 
              'Show?RefStation',
            ),
    jsButton( 'Set RefStation', 
              'Set?RefStation&cmrId=5&rtcmId=10&cmrStationName=RtcmName'
             ."&description=\\\\'A Descriptive Phrase\\\\'",
            ),
    jsButton( 'Show ClientAuthKey', 
              'Show?ClientAuthKey',
            ),
    jsButton( '*Set ClientAuthKey', 
              '*Set?ClientAuthKey&clearKey=theKey!',
            ),
    jsButton( 'Show Port Advertising', 
              'Show?PortAdvertising',
            ),
    jsButton( 'Set Port Advertising', 
              'Set?PortAdvertising&enable=yes&interval=5'
             .'&target=192.168.142.7:8000',
            ),
    jsButton( 'Show PPP Dialout', 
              'Show?PppDialout'
            ),
    jsButton( 'Set PPP Dialout', 
              'Set?PppDialout'
            ),
    ) ;
}
elsif ( $group eq 'Security')
{
  print b("Security Commands") ;
  print table(
    jsButton( 'Show GuiSecurity',
                'Show?GuiSecurity',
                'Display the current HTTP access restrictions.'
              ) ,
    jsButton( 'Set GuiSecurity',
                'Set?GuiSecurity&system=unrestricted'
                                 .'&gps=unrestricted'
                               .'&files=unrestricted',
                'Modify the HTTP access restrictions.'
              ) ,
    jsButton( 'Show GuiAccounts',
                'Show?GuiAccounts',
                'Display the currently defined HTTP access accounts.'
              ) ,
    jsButton( 'Show GuiAccount',
                'Show?GuiAccount&name=test',
                'Show the settings for a specific HTTP access account.'
              ) ,
    jsButton( 'Set GuiAccount',
                'Set?GuiAccount&name=test&clearPassword=abcdef'
               .'&system=yes&gps=yes&files=yes',
                'Create or modify an HTTP access account.'
              ) ,
    jsButton( 'Delete GuiAccount',
                'Delete?GuiAccount&name=test',
                'Remove an HTTP access account.'
              ) ,
    ) ;
}
elsif ( $group eq 'Sessions')
{
  print b("Logging Session Commands") ;
  print table(
        jsButton( 'Show LoggingStatus', 
                  'Show?LoggingStatus',
                ),
        jsButton( 'Show Sessions',
                  'Show?Sessions',
                ),
        jsButton( 'Show Session', 
                  'Show?Session&name=testing',
                ),
        jsButton( 'Enable Session', 
                  'Enable?Session&name=testing',
                ),
        jsButton( 'Disable Session', 
                  'Disable?Session&name=testing',
                ),
        jsButton( 'Set Session', 
                'Set?Session&name=testing&enable=no&schedule=continuous'
               .'&duration=5&format=T00&measinterval=15&posinterval=60'
               .'&smoothcode=no&smoothcarrier=no&sessid=t&daySubdir=no'
               .'&sessidSubdir=no',
                ),
        jsButton( 'Delete Session', 
                  'Delete?Session&name=testing',
                ),
        jsButton( '*Delete ALL Sessions', 
                  '*Delete?Session&all=yes',
                  'Delete ALL data logging sessions.'
                ),
        jsButton( 'Show AutoDelete', 
                  'Show?AutoDelete',
                ) ,
        jsButton( 'Set AutoDelete', 
                  'Set?AutoDelete&Reserved=1,yes&a=50,no&default=50,yes'
                ) ,
        jsButton( 'Show PowerSaveMode', 
                  'Show?PowerSaveMode',
                ),
        jsButton( '*Set PowerSaveMode', 
                  '*Set?PowerSaveMode&enable=yes',
                ),
        jsButton( 'Show FailsafeWakeup', 
                  'Show?FailsafeWakeup',
                ),
        jsButton( 'Set FailsafeWakeup', 
                  'Set?FailsafeWakeup&enable=yes&interval=4&duration=10'
                ),
    ) ;
}
elsif ( $group eq 'DataFiles')
{
  print b("Data File Commands") ;
  print table(
        jsButton( 'Show LoggedFileStats', 
                  'Show?LoggedFileStats&directory=/',
                ),
        jsButton( 'Show LoggedFilePools', 
                  'Show?LoggedFilePools',
                ),
        jsButton( 'Show LoggedFiles', 
                  'Show?LoggedFiles&directory=/',
                ),
        jsButton( 'Show LoggedFile', 
                  'Show?LoggedFile&path='
                  .'/200310/TomsProto200310281800c.T00',
                ),
        jsButton( 'Download LoggedFile', 
                  'Download?LoggedFile&path='
                  .'/200310/TomsProto200310281800c.T00',
                ),
        jsButton( '*Delete LoggedFiles', 
                  '*Delete?LoggedFiles&directory=/200310/02/c',
                ),
        jsButton( '*Delete LoggedFile', 
                  '*Delete?LoggedFile&path='
                  .'/200310/02/a/TomsProto200310020000a.T00',
                ),
    ) ;
}
else
{
  print b("Unknown Group of commands") ;
}

print <<EOF ;
  <script>

  function submitRequest() 
  {
    w = parent.results ;
    d = w.document ;
    d.open() ;
    d.writeln( 'Waiting for results' ) ;
    d.close() ;
    d.bgColor='fff5e4' ;

    f = document.theForm ;
    url = '/prog/' + f.Command.value ;
    d = parent.command.document ;
    d.open() ;
    //d.body.style.marginLeft = '72px' ;
    d.write( '<style type="text/css">' ) ;
    d.write( 'body {margin-left: 3px; margin-top:3px}' ) ;
    d.write( '</style>' ) ;
    escapedUrl = url ;
    while ( ( i = escapedUrl.indexOf( ' ' ) ) != -1 )
    {
      escapedUrl = escapedUrl.substr(0,i) 
                 + '%20' 
                 + escapedUrl.substr(i+1) ;
    }
    d.write( escapedUrl ) ;
    d.close() ;
    d.bgColor='fff5e4' ;

    w.document.location = url ;
  }

  function doCommand( command ) 
  /* This is called by each of the predefined buttons, to
   * activate the function defined for that button.
   * In general this means copying the command into the text entry
   * box (for later editing and sending) and then triggering the
   * submit action of the form.
   *
   * If the AutoSubmit checkbox isn't clicked, then we don't do the
   * automatic submission.  We just fill in the text entry box.
   *
   * If the command starts with the special character '*', that
   * character is removed.  Then a confirm dialog box is posted
   * asking if the user really wants the command to be executed.
   */
  {
    f = document.theForm ;

    if (command.charAt( 0 ) == "*")
    {
      command = command.substr(1) ;
      f.Command.value = command ;
      if ( f.autosubmit.checked
        && confirm( 'Okay to execute "' + command + '"?' ) 
         )
      {
        submitRequest() ;
      }
    }
    else
    {
      f.Command.value = command ;
      if (f.autosubmit.checked)
        submitRequest() ;
    }

  }

  </script>
EOF
print endform() ;


print end_html() ;

EXIT:

########################################################################
# End of progTest.html
########################################################################

