#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: rconfigDisplay.cgi,v 1.57 2006/03/23 19:09:02 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/rconfigDisplay.cgi,v $

# rconfigDisplay.cgi
# Display the configuration settings of the receiver.
# Has the facilities to show this information in a separate window
# suitable for printing.


use strict ;
use CGI qw/:standard/;
use Fireball ;

titleWithHelp( 'Receiver Configuration',
               '' # No Help Button
             ) ;


# Read in the stinger configuration file, putting each line into array
# @stringerCfg.  This can be searched for individual control settings.
my @stingerCfg = split( "\n", qx( cat /etc/sysconfig/stinger.cfg ) ) ;

print '<table border="0">' ;

# Get the system identifier string from a config file.
my $identityString= qx|cat /etc/sysconfig/sysname| ;

# Determine a serial number by querying the "warranty" command.
# warranty -s returns:
#   Serial_number: 5432198765
# Extract the numeric string.
my $serialNumber = qx(/usr/bin/sudo /usr/local/fireball/warranty -s) ;
chomp $serialNumber ;   # Drop linefeed
$serialNumber = (split("[ :]+",$serialNumber))[1] ;



print Tr( td( {align=>'right', valign=>'top'}, b( 'Identity ' ) ),
          td( $identityString
             .'<br />SN'
             .$serialNumber ) 
        ) ;


my $fwVersion = 'Unknown' ;
my $fwDate    = '01-Jan-2000' ;

if ( -r '/usr/local/fireball/defaults/firmwareVersion' ) 
{
  ($fwVersion, $fwDate) = split( "\n", qx( cat /usr/local/fireball/defaults/firmwareVersion ) ) ;
  
  my ($year,$month,$day) = split( '/', $fwDate ) ;
  $fwDate = "$day $monthStrings[$month-1] $year" ;
}

print Tr( td( {align=>'right'}, b( 'Firmware ' ) ),
          td( "$fwVersion&nbsp;&nbsp;$fwDate" ) 
        ) ;



########################################################################
# Antenna information.
########################################################################
my %antInfo ;
getAntennaInfo( \%antInfo ) ;
my %antSetup ;
getAntennaSetup( \%antSetup ) ;

my $type = $antSetup{'type'} ;
print Tr( td( {align=>'right', valign=>'top'}, b( 'Antenna ' ) ),
          td( "Type: $type = $antInfo{$type}{'name'}"
             ."<br />SN$antSetup{'serial'} "
             .', "'.$antSetup{'textSerial'}.'"'
             ."<br />Height: $antSetup{'height'} meters"
             ."<br />Height Method: "
             ."$antInfo{$type}{'methods'}[$antSetup{'method'}]"
            )
        ) ;

########################################################################

my $clockSteering = (grep( m/clock_steering/, @stingerCfg ))[0] ;
print Tr( td( {align=>'right'}, b( 'Clock Steering ' ) ), 
          td( ($clockSteering =~ /off/) ? 'Disabled' : 'Enabled' ) 
        ) ;

########################################################################

my $multipath = (grep( m/everest/, @stingerCfg ))[0] ;
print Tr( td( {align=>'right'}, b( 'Multipath Reduction ' ) ), 
          td( ($multipath =~ /off/) ? 'Disabled' : 'Enabled' ) 
        ) ;

########################################################################

my $l2mode = (grep( m/l2_mode/, @stingerCfg ))[0] || 'off' ;
print Tr( td( {align=>'right'}, b( 'L2 Tracking ' ) ), 
          td( ($l2mode =~ /off/) 
              ? 'Off' 
              : ($l2mode =~ /l2c_or_enigma/)  
                ? 'L2C or L2-Y-code' 
                : ($l2mode =~ /l2c_and_enigma/)  
                  ? 'L2C and L2-Y-code' 
                  : 'L2-Y-code' 
            ) 
        ) ;

########################################################################

my $elev_mask = (grep( m/elev_mask/, @stingerCfg ))[0] ;
$elev_mask =~ s/.* // ;
print Tr( td( {align=>'right'}, b( 'Elevation Mask ' ) ), 
          td( $elev_mask.'&deg;' ) 
        ) ;

########################################################################

my $pdop_mask = (grep( m/pdop_mask/, @stingerCfg ))[0] ;
$pdop_mask =~ s/.* // ;
print Tr( td( {align=>'right'}, b( 'PDOP Mask ' ) ), 
          td( $pdop_mask ) 
        ) ;

########################################################################

my $pps = (grep( m/pps_mode/, @stingerCfg ))[0] ;
print Tr( td( {align=>'right', valign=>'top'}, 
              b( 'Pulse Per Second Output ' ) 
            ), 
          td( ($pps =~ ' on ')
              ? 'Enabled, positive-slope timing edge.' 
              : 'Disabled'
            ) 
        ) ;

########################################################################

my $extRef = qx( cat /etc/sysconfig/extRefFreq ) ;
# Get the state of the extref signal detector from program extfreq.
my $extRefDetected = qx(/usr/local/fireball/extfreq -p) ;

print Tr( td( {align=>'right', valign=>'top'}, 
              b( 'Reference Frequency ' ) 
            ), 
          td( ( ($extRef =~ /External/)
                ? 'Using External Source' 
                : 'Using Internal Source' 
              )
             .br()
             .( ($extRefDetected =~ /present/)
                ? 'External source detected'
                : 'External source NOT detected'
              )
            ) 
        ) ;


########################################################################
# GPS Satellite controls
########################################################################
my $masks = (grep( m/ sv_masks /, @stingerCfg ))[0] ;
#  put sv_masks E E E D E .... up to 32 of them.
my @state = split( " ", $masks ) ;
splice( @state, 0, 2 ) ; # Drop 'put sv_masks'

my @disables = ();
my @ignores = ()  ;
for ( my $prn = 1 ; $prn <= 32 ; ++$prn )
{
  if ( $state[$prn-1] =~ 'D' )
  {
    push( @disables, $prn ) ;
  }
  if ( $state[$prn-1] =~ 'I' )
  {
    push( @ignores, $prn ) ;
  }
}

print Tr( td( {align=>'right', valign=>'top'}, b('GPS Satellites ') ) ,
          td( 
             'Disabled: '
             .( (@disables)
                ? join( ', ', @disables )
                : 'None'
              )
             .'<br />IgnoreHealth: '
             .( (@ignores)
                ? join( ', ', @ignores )
                : 'None'
              )
            )
        ) ;

########################################################################
# WAAS
########################################################################
my $waasMode = (grep( m/waas_mode/, @stingerCfg ))[0] ;
print '<tr>',
      td( {align=>'right', valign=>'top'}, b( 'WAAS ' ) ),
      '<td>',
      ($waasMode =~ / on/)
      ? 'Enabled for Tracking and Corrections' 
      : 'Disabled' 
    ;

$masks = (grep( m/waas_sv_masks/, @stingerCfg ))[0] ;
#  put waas_sv_masks E E E D E I .... up to 19 of them.
@state = split( " ", $masks) ;
splice( @state, 0, 2 ) ; # Drop 'put waas_sv_masks'

# Figure out which satellites are enabled or ignoreHealth.
# Make a list of each with the satellite names if we know them.
my @enables = () ;
@ignores = () ;
for ( my $prn = 120 ; $prn <= 138 ; ++$prn )
{
  my $name = $waasStrings{ $prn } ;
  push( @enables, $name ) if ( $state[$prn-120] eq 'E' ) ;
  push( @ignores, $name ) if ( $state[$prn-120] eq 'I' ) ;
}

# Print the list of enabled satellites.  This list can be really long,
# so it can easily force a line-wrap on the browser.  Although this
# works okay, there is a tendency for the long lines to mess up the
# formatting in the first column of the table.  To solve this, we
# attempt to break up the list into multiple lines of about 40
# characters each.
print br() ;
my $string = 'Enabled Satellites: ' ;

my $needComma = 0 ;

if (!@enables)
{
  print $string, 'None' ;
}
else
{
  for my $sv (@enables)
  {
    $string .= ',' if ( $needComma ) ;
    $needComma = 1 ;
    if (length($string) > 40)
    {
      # String has gotten long enough to force a linewrap
      print $string,br() ;
      $string = '' ;
    }
    $string .= " $sv" ;
  }
  print $string if (length($string)) ;
}

# Do the same job of formatting the IgnoreHealth list if that exists.
if (@ignores)
{
  print br() ;
  $string = 'Ignore Health Satellites: ' ;
  $needComma = 0 ;

  for my $sv (@ignores)
  {
    $string .= ',' if ( $needComma ) ;
    $needComma = 1 ;
    if (length($string) > 40)
    {
      print $string,br() ;
      $string = '' ;
    }
    $string .= " $sv" ;
  }
  print $string if (length($string)) ;
}
print '</td></tr>' ;


########################################################################
# Reference Station information
########################################################################

# Read the configuration values from the sysinfo system.
my %refStationData ;
getRefStationData( \%refStationData ) ;

print Tr( td( {align=>'right', 
              valign=>'top'}, b( 'Reference Station ' ) ),
          td( "RTCM Station ID: $refStationData{'rtcmid'}",
              "<br />Binex Station ID: $refStationData{'binexid'}",
              "<br />CMR Station ID: $refStationData{'cmrid'}",
            "<br />CMR Station Name: $refStationData{'cmrStationName'}",
              "<br />Description: $refStationData{'rtcmdesc'}",
              "<br />Position:",
              table( {border=>'0', cellspacing=>'0', cellpadding=>'0'},
              Tr( td( {align=>'right'}, ('Lat: ') ),
                  td( {align=>'right'},
                      sprintf( 
                         '<code>&nbsp;%02d&deg;%02d\'%08.5f" %s</code>',
                               $refStationData{'latdeg'},
                               $refStationData{'latmin'},
                               $refStationData{'latsec'},
                               $refStationData{'latns'},
                             )
                    ),
                  td( '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' ), 
                  td( {align=>'right'}, ('X: ') ),
                  td( {align=>'right'}, 
                      sprintf( '<code>%12.3f</code> m', 
                               $refStationData{'x'}
                             ) 
                    ) 
                ),
              Tr( td( {align=>'right'}, ('Lon: ') ),
                  td( {align=>'right'},
                      sprintf( '<code>%03d&deg;%02d\'%08.5f" %s</code>',
                               $refStationData{'londeg'},
                               $refStationData{'lonmin'},
                               $refStationData{'lonsec'},
                               $refStationData{'lonew'},
                             )
                    ),
                  td( '&nbsp;' ) ,
                  td( {align=>'right'}, ('Y: ') ),
                  td( {align=>'right'}, 
                      sprintf( '<code>%12.3f</code> m', 
                               $refStationData{'y'}
                             ) 
                    ) 
                ),
              Tr( td( {align=>'right'}, ('Height: ') ),
                  td( {align=>'right'},
                     sprintf( '<code>%.3f</code> meters', 
                              $refStationData{'height'} 
                            ),
                    ),
                  td( '&nbsp;' ) ,
                  td( {align=>'right'}, ('Z: ') ),
                  td( {align=>'right'}, 
                      sprintf( '<code>%12.3f</code> m', 
                               $refStationData{'z'}
                             ) 
                    ) 
                ),

              ),
            )
        ) ;

########################################################################
# Data Logging Info
########################################################################


print '<tr>',
      td( {align=>'right', valign=>'top'}, b( 'Data Logging ' ) ),
      '<td>' ;
      
print 'Enabled Sessions:' ;
# Read in the data from the sessions configuration file.
my %sessData ;
getSessionData( \%sessData ) ;

# Get the time of day in GPS seconds.  Includes a 13 second offset for
# the UTC to GPS leapseconds.  315964800 converts from Unix time to
# gps time (01Jan1970 to 06Jan1980).
my $gpsTime = time() - 315964800 + 13 ;

my $count = 0 ;
print '<table border="0" cellspacing="0" cellpadding="0"' ;
for my $sess (sort keys %sessData)
{
  
  if ($sessData{$sess}{'enable'})
  {
    my $type = $sessData{$sess}{'sched'} ;
    my $duration = $sessData{$sess}{'duration'} ;
    my $time = $sessData{$sess}{'time'} ;
    my $sched ;
    if ($type eq 'cont')
    {
      $sched = "Continuous $duration minute sessions." 
    }
    elsif ($type eq 'manual')
    {
      next if ( $time + $duration*60 < $gpsTime ) ;
      $sched = "Manual $duration minute session." 
    }
    elsif ($type eq 'daily')
    {
      $sched = sprintf( 'Daily at %02d:%02d for %d minutes.', 
                       int( $time / 3600 ),
                       int( $time % 3600 / 60 ),
                       $duration 
                      ) ;
    }
    elsif ($type eq 'once')
    {
      next if ( $time + $duration*60 < $gpsTime ) ;
      $sched = "Once only, $duration minute session." 
    }
    ++$count ;
    print Tr( td( {align=>'right'}, '&nbsp;&nbsp;&nbsp;',$sess.':' ), 
              td( {align=>'left'}, '&nbsp;&nbsp;&nbsp;', $sched ) ) ;
  }
}


print Tr( td( ['&nbsp;&nbsp;&nbsp;','None'] ) ) if ($count == 0) ;
print '</table>' ;


my @sleepmode = split( ' ', qx( cat /etc/sysconfig/sleepmode ) ) ;
print 'Power Saving Mode: ',
      ( ( $sleepmode[0] eq 'yes')
        ? ('Enabled.'
           .'<br />Failsafe Wakeup: '
           .( ($sleepmode[1] eq 'no')
              ? 'Disabled.'
              : "$sleepmode[3] minutes every $sleepmode[2] hours."
            )
          )
        : 'Disabled.' 
      )
      ;


my %autoDeleteControls ;
getAutoDeleteControls( \%autoDeleteControls ) ;
print '<br />Reserved Space:&nbsp;&nbsp;&nbsp;&nbsp;' ;
if ( $autoDeleteControls{'Reserved'}{'enable'} )
{
  print 'AutoDeleting to ', 
        sprintf( "%d Mbytes.", 
                 $autoDeleteControls{'Reserved'}{'size'} 
               ) ;
}
else
{
  print 'Not AutoDeleting.' ;
}

print '<br />AutoDelete Pools:' ;
if ( scalar( keys %autoDeleteControls ) > 1 )
{
  print '<table border="0" cellspacing="0" cellpadding="0"' ;
  for my $pool ('a'..'z','Default')
  {
    if (exists $autoDeleteControls{$pool})
    {
      print Tr( 
                td( {width=>10}, '&nbsp;' ),
                td( {align=>'right'}, 
                    "$pool-pool" 
                  ),
                td( {width=>10}, '&nbsp;' ),
                td( {align=>'right'}, 
                    sprintf( "%d Mbytes",
                             $autoDeleteControls{$pool}{'size'}
                           )
                  ),
                td( {width=>10}, '&nbsp;' ),
                td( {align=>'right'}, 
                    $autoDeleteControls{$pool}{'enable'} 
                    ? 'AutoDeleting.' : 'Not AutoDeleting.'
                  )
              ) ;
    }
  }
  print '</table>' ;
  }
else
{
  print '&nbsp;&nbsp;&nbsp;None Defined.' ;
}
       


print '</td></tr>' ;

########################################################################
# Ethernet setup
########################################################################

# Get the 12 character Hardware MAC address.  
# Insert colons every two characters.
my $macAddress = qx( /usr/local/fireball/getmac ) ;
substr( $macAddress, 10, 0 ) = ':' ;
substr( $macAddress,  8, 0 ) = ':' ;
substr( $macAddress,  6, 0 ) = ':' ;
substr( $macAddress,  4, 0 ) = ':' ;
substr( $macAddress,  2, 0 ) = ':' ;

# Read configuration from /etc/sysconf/interfaces
# The line in question will look like one of these:
#   iface eth0 inet dhcp 
#   iface eth0 inet static
# We want the fourth field.
my $EthMode = 
     (split( ' ', 
             qx| grep '^ *iface  *eth0' /etc/sysconfig/interfaces| 
           ) 
     )[3] ;

# Determine the current Ethernet IP and Gateway.
# If the network setup is okay, this will pop out of 'ifconfig' in the
# form:
#   inet addr:192.168.142.141  Bcast:192.168.142.255  Mask:255.255.255.0
# The information is parsed from fields 2 and 4.
# If the current network config has a problem, then ifconfig might not
# be functional.  If we were in static mode, then we can read the 
# ip and gateway from /etc/sysconfig/interfaces file. 
# If all else fails, just default the IP to 0.0.0.0 and mask to
# 255.255.255.0.
my $ipAddress = '0.0.0.0' ;
my $netMask = '255.255.255.0' ;
my @results=split( ' ', qx( /sbin/ifconfig eth0 | grep 'inet addr:' ) );
if (@results)
{
  $ipAddress  = (split( ':', $results[1] ))[1] ;
  $netMask  = (split( ':', $results[3] ))[1] ;
}
elsif ($EthMode eq 'static')
{
  $ipAddress = 
         (split( ' ', qx(grep address /etc/sysconfig/interfaces) ))[1] ;
  $netMask  =  
         (split( ' ', qx(grep netmask /etc/sysconfig/interfaces) ))[1] ;
}




print Tr( td( {align=>'right', valign=>'top'}, b( 'Ethernet ' ) ), 
          td( table( {border=>'0', cellspacing=>'0', cellpadding=>'0'},
                Tr( td( {align=>'right'}, 'MAC Address:'), 
                    td( '&nbsp;&nbsp;&nbsp;' ),
                    td( $macAddress ) 
                  ),
                Tr( td( {align=>'right'}, 'IP Address:' ),
                    td( '' ),
                    td( "$ipAddress ($EthMode)" )
                  ),
                Tr( td( {align=>'right'}, 'Netmask:' ),
                    td( '' ),
                    td( $netMask ),
                  )
              )
            ) 
        ) ;

########################################################################
# IP Filtering
########################################################################
my %iptablesSetup ;
getIptablesSetup( \%iptablesSetup ) ;
print '<tr>' ;
print td( {align=>'right', valign=>'top'}, b( 'IP Filtering' ) ) ;


print '<td><table border=0 cellspacing=0 cellpadding=0>' ;
print Tr( td( $iptablesSetup{'enable'} ? 'Enabled' : 'Disabled' ) ) ;
if ($iptablesSetup{'rangeCount'} )        
{
  for (my $i = 0 ; $i < $iptablesSetup{'rangeCount'} ; ++$i )
  {
    print Tr( td( 'Range:'),
              td( {align=>'right'}, $iptablesSetup{'rangeBases'}[$i] ),
              td( '&nbsp;&nbsp;/&nbsp;&nbsp;' ),
              td( {align=>'right'}, $iptablesSetup{'rangeBits'}[$i] )
            ) ;
  }
}
print '</table></td></tr>' ;


########################################################################
# Shutdown Voltage
########################################################################
#
# Get the software shutdown voltage threshold (Vsleep).
my $shutdownVoltage = getVsleep();
print Tr( td( {align=>'right', valign=>'top'}, 
              b( 'Shutdown Voltage ' ) 
            ), 
          td( sprintf( "%.2f", $shutdownVoltage ), 'Volts' ) 
        ) ;

########################################################################


########################################################################
print '</table>' ;

print end_html() ;

########################################################################
# end of file rconfigDisplay.cgi
########################################################################
