#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: refpos.cgi,v 1.26 2004/08/24 23:09:45 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/refpos.cgi,v $

########################################################################
# refpos.cgi:
#
# Script to allow entry of Reference Position and RTCM setup parameters.
# Position can be entered as either Lat/Lon/Hgt or ECEF-XYZ.  We also
# control RTCM-ID, RTCM-Name and RTCM-Description.
#
# This script puts up and manages an HTML form.  When the user hits
# OK, the data is submitted to the script gpsadmin/setRefpos.cgi which
# handles the actual installtion of the new data.  If that script
# finds any problems, it passes control back to this script.
#
#
# Called in several forms, using CGI parameters:
#
#   refposSetup.cgi
#       No parameters: Initialize the form with current data
#
#   refposSetup.cgi?Mode=lla&...
#       With parameters, this is either an update submission (where
#       the lla/xyz mode has changed) or a re-submission due to
#       a detected error in the setRefpos.cgi script.
#       In either case, we just redraw the screen using whatever
#       values have been posted in the cgi parameters.
# 
# There is an invisible widget/parameter "Here" which is set only by
# pressing the "Here" button in the Lat/Lon section.  If this is
# detected, the user has requested that the currently determined
# position be used for setting the reference position.  We grab the
# current position and fill in the form's entry widgets with the data.
#
########################################################################


use strict ;
use CGI qw/:standard/;

use Fireball ;

titleWithHelp( 'Reference Station Setup',
                         '/refposInfo.html'
                       ) ;

          
          
########################################################################
sub strip
########################################################################
# Removes leading and trailing spaces from the parameter string $_[0]
# Returns the altered string.
{
  $_[0] =~ s/^\s+// ;  # Drop any leading spaces
  $_[0] =~ s/\s+$// ;  # Drop any trailing spaces
  return $_[0] ;
}


my ($latitude, $longitude, $height ) = ( 0,0,0) ;
my ($ecefX, $ecefY, $ecefZ) = (0,0,0) ;
my ($rtcmID, $cmrID, $binexID, $rtcmDesc, $cmrStationName ) ;
my $mode = 'lla' ;

my %refStationData ;
getRefStationData( \%refStationData ) ;


if (!param())
{
  $latitude = $refStationData{'lat'} ;
  $longitude = $refStationData{'lon'} ;
  $height = $refStationData{'height'} ;
  $ecefX = $refStationData{'x'} ;
  $ecefY = $refStationData{'y'} ;
  $ecefZ = $refStationData{'z'} ;
  $rtcmID = $refStationData{'rtcmid'} ;
  $cmrID = $refStationData{'cmrid'} ;
  $binexID = $refStationData{'binexid'} ;
  $cmrStationName = $refStationData{'cmrStationName'} ;
  $rtcmDesc = $refStationData{'rtcmdesc'} ;


  # Determine which mode the entry system should start in, LatLonAlt
  # or ECEF.  This can only be determined by reading the actual
  # configuration file.  We presume the mode is lla unless we find
  # an "X:" configuration line.
  if (open( CONF, '/etc/sysconfig/ref_stn.config' ))
  {
    while( <CONF> )
    {
      # Each line is either a comment  or a 'tag: value'
      if ( my ($tag, $value) = m/^\s*(\S*)\s*:\s*(.*)\s*$/ )
      {
        if ($tag eq "X" )
        {
          $mode = 'ecef' ;
          last ;
        }
      }
    }
    close( CONF ) ;
  }
  param( 'Mode', $mode ) ;

} # No parameters
else
{ 
  # Parameters found
  $mode = param( 'Mode' ) ;

  #print "<pre>\n" ;
  #for my $p (param())
  #{
  #  print "$p = ", param($p), "\n" ;
  #}
  #print "</pre>\n" ;

  my ($latDeg, $latMin, $latSec, $NS ) ;
  my ($lonDeg, $lonMin, $lonSec, $EW ) ;

  if (param( 'X'))
  {
    $ecefX = strip( param( 'X' ) ) ;
    $ecefY = strip( param( 'Y' ) ) ;
    $ecefZ = strip( param( 'Z' ) ) ;
  }
  else
  {
    $latDeg = strip( param( 'latDeg' ) ) ;
    $latMin = strip( param( 'latMin' ) ) ;
    $latSec = strip( param( 'latSec' ) ) ;
    $NS     = param( 'NS' ) ;
    $lonDeg = strip( param( 'lonDeg' ) ) ;
    $lonMin = strip( param( 'lonMin' ) ) ;
    $lonSec = strip( param( 'lonSec' ) ) ;
    $EW     = param( 'EW' ) ;
    $height = strip( param( 'Height' ) ) ;
  }


  if (param('SetHere'))
  {
    # User clicked the Here button, expecting the current location to
    # be entered into the form.  In response, we grab the current
    # position from sysinfo, and use it to set Lat/Lon/Height.

    # Delete the SetHere parameter, so it only takes effect once.
    CGI::delete( 'SetHere' ) ;


    open( SYSINFO, '/usr/local/fireball/sysinfo -p pos_info |' );
    # First line of the form:
    # 246021.8, 1225 : 37.3891238781 -122.0368426604 -4.921
    # time      week   lat           lon             height
    (undef, undef, undef, $latitude, $longitude, $height) 
                                              = split( ' ', <SYSINFO>) ;
    close( SYSINFO ) ;

    # The current Lat/Lon/Alt may need to be converted to XYZ 
    # Do it now, just in case.
    ($ecefX, $ecefY, $ecefZ ) 
       = split( ' ',
                qx( /usr/local/fireball/ecefConvert lla $latitude $longitude $height ) 
              ) ;

    if ( $mode eq 'lla' )
    {
      # Mode is lat/lon/alt.  The current Lat/Lon/Alt determined above
      # will be forced into the form's text fields.
      
      # Delete these pre-existing values of CGI parameters.  This
      # allows the GUI widgets to display their default values, which
      # will be based on the new position we just determined.
      CGI::delete( 'latDeg' ) ;
      CGI::delete( 'latMin' ) ;
      CGI::delete( 'latSec' ) ;
      CGI::delete( 'NS' ) ;
      CGI::delete( 'lonDeg' ) ;
      CGI::delete( 'lonMin' ) ;
      CGI::delete( 'lonSec' ) ;
      CGI::delete( 'EW' ) ;
      CGI::delete( 'Height' ) ;
    }
    else
    {
      # Mode is Ecef-XYZ.  
      # Delete these pre-existing values of these CGI parameters.  This
      # allows the GUI widgets to display their default values, which
      # will be based on the new position we just determined.
      CGI::delete( 'X' ) ;
      CGI::delete( 'Y' ) ;
      CGI::delete( 'Z' ) ;
    }
  } # SetHere
  elsif (param('ModeChanged'))
  {
    # Because the position mode was changed, we have to preserve the
    # position the user may have entered, by converting it from one
    # form to the other.  This requires completely parsing the
    # position entry fields, with error detection, and then converting
    # between the two formats.  Yuck.  Lots of work and code for very
    # little payback.  But that's what the SPR requested.

    # Delete the ModeChanged parameter, so it only takes effect once.
    CGI::delete( 'ModeChanged' ) ;

    # This is a perl-regular expression that allows the following
    # formats:  
    #   Possible leading + or -.
    #   ddd.ddd     .ddd    ddd.    ddd
    # where ddd is one or more decimal digits.
    my $fixedPointPattern = '^[+-]?(\d*\.\d+|\d+\.\d*|\d+)$' ;

    # This is the same pattern, not allowing for a leading + or -
    my $posFixedPointPattern = '^(\d*\.\d+|\d+\.\d*|\d+)$' ;


    if ( $mode eq 'lla' )
    {
      # New Mode is lat/lon/alt.  There should be XYZ parameters
      # available for converting to Lat/Lon.
      if ( $ecefX !~ m/$fixedPointPattern/ 
        || $ecefY !~ m/$fixedPointPattern/ 
        || $ecefZ !~ m/$fixedPointPattern/ 
         )
      {
        # Problem detected.  Can't do much but warn the user and put
        # up the default for Lat/Lon/Alt.
        print <<EOF ;
        <script>
        alert( "Couldn't convert XYZ to Lat/Lon/Alt." ) ;
        </script>
EOF
      }
      else
      {
        # Parameters okay.
        # Use ecefConvert to convert XYZ to LLA.
        # Returns a single line with "Lat Lon Alt" values.
        # That Lat/Lon/Alt will be forced into the form's text fields.
        ($latitude, $longitude, $height ) 
          = split( ' ',
         qx( /usr/local/fireball/ecefConvert ecef $ecefX $ecefY $ecefZ )
                 );
        # Those values will be used below to format the entry widgets.
      }
    } # lla mode
    else
    { 
      # New Mode is Ecef-XYZ.  The current Lat/Lon/Alt will be converted
      # to XYZ and the form textfields updated with those values.

      if ( $latDeg !~ m/^\d+$/
        or $latMin !~ m/^\d+$/
        or $latSec !~ m/$posFixedPointPattern/
        or $lonDeg !~ m/^\d+$/
        or $lonMin !~ m/^\d+$/
        or $lonSec !~ m/$posFixedPointPattern/
        or $height !~ m/$fixedPointPattern/
        or $latMin >= 60
        or $latSec >= 60.0
        or ($latDeg+$latMin/60.0+$latSec/3600.0) > 90.0
        or $lonMin >= 60
        or $lonSec >= 60.0
        or ($lonDeg+$lonMin/60.0+$lonSec/3600.0) > 180.0
         )
      {
        print <<EOF ;
        <script>
        alert( "Couldn't convert Lat/Lon/Alt to XYZ." ) ;
        </script>
EOF
      }
      else
      {
        # Nothing wrong with the parameters.  Combine them all
        # into composite values.  Then use ecefConvert to convert
        # lla to xyz.
        GETLATLON:
        $latitude = $latDeg + $latMin/60 + $latSec/3600 ;
        $latitude = -$latitude if ($NS eq 'S') ;

        $longitude = $lonDeg + $lonMin/60 + $lonSec/3600 ;
        $longitude = -$longitude if ($EW eq 'W') ;

        # ecefConvert returns a single line with "X Y Z" values.
        ($ecefX, $ecefY, $ecefZ ) 
         = split( ' ',
                  qx( /usr/local/fireball/ecefConvert lla $latitude $longitude $height ) 
                ) ;
        # Those values will be used below to format the entry widgets.
      }
    } # xyz mode
  } # ModeChanged
  else
  {
    # Neither a mode change or a SetHere operation.  
    # This must be a resubmission due to an error message.
    # The only thing we have to do special is to calculate the
    # current ECEF of the refpos, so we can later calculate the
    # refpos offset.
    if ( $mode eq 'lla' )
    {
      goto GETLATLON ;
    } 
    else
    { # mode eq 'ecef'
      # Just use the values of ECEF much earlier
    } 
  }

} # Parameters found


# In case we are dealing with lat/lon/height, we need to split
# those values into separate fields...
my ($latDeg, $latMin, $latSec, $NS ) = (0,0,0,'N');
my ($lonDeg, $lonMin, $lonSec, $EW ) = (0,0,0,'E');

if ($latitude < 0.0) 
{
  $latitude = - $latitude ;
  $NS = 'S' ;
}

# Round the value to nearest 0.00001 second.  This works
# in conjunction with the 'seconds = int(seconds) line at little
# bit below here, which truncates to nearest 0.00001 second.
$latitude += (0.000005 / 3600) ;

$latDeg   = int( $latitude ) ;
$latitude = ($latitude - $latDeg) * 60 ;
$latMin   = int( $latitude ) ;
$latSec   = ($latitude - $latMin) * 60 ;
$latSec   = int($latSec * 100000.0)/100000.0 ;


if ($longitude < 0.0) 
{
  $longitude = - $longitude ;
  $EW = 'W' ;
}

# Round the value to nearest 0.00001 second.  This works
# in conjunction with the 'seconds = int(seconds) line at little
# bit below here, which truncates to nearest 0.00001 second.
$longitude += (0.000005 / 3600) ;

$lonDeg = int( $longitude ) ;
$longitude = ($longitude - $lonDeg) * 60 ;
$lonMin   = int( $longitude ) ;
$lonSec   = ($longitude - $lonMin) * 60 ;
$lonSec   = int($lonSec * 100000.0)/100000.0 ;

########################################################################
# Determine the error between the current location and the
# value we are about to display.  If that's over 100 meters we
# display a warning.
########################################################################

# Grab the current position from the sysinfo facility.
open( SYSINFO, '/usr/local/fireball/sysinfo -p pos_info |' );
# First line of the form:
# 246021.8, 1225 : 37.3891238781 -122.0368426604 -4.921
# time      week   lat           lon             height
my (undef, undef, undef, 
    $currentLat, $currentLon, $currentHeight) 
                                            = split( ' ', <SYSINFO>) ;
close( SYSINFO ) ;

#print "$currentLat, $currentLon, $currentHeight <br />" ;

my ($currentX, $currentY, $currentZ ) 
   = split( ' ',
            qx( /usr/local/fireball/ecefConvert lla $currentLat $currentLon $currentHeight ) 
          ) ;

#print "$currentX, $currentY, $currentZ <br />" ;
#print "$ecefX, $ecefY, $ecefZ <br />" ;
my $deltaX = $currentX - $ecefX ;
my $deltaY = $currentY - $ecefY ;
my $deltaZ = $currentZ - $ecefZ ;

my $diff = sqrt( $deltaX * $deltaX
               + $deltaY * $deltaY
               + $deltaZ * $deltaZ
               ) ;
# The $diff value determined above is the approximate error between
# the entered reference position and the current GPS-based position.
# If that's above a threshold, then we need to warn the user.  The
# primary reason for this is that some reference station formatters
# (RTCM in particular) will not produce any corrections if the error
# is above some limit, probably 200m for RTCM.  Here we use a somewhat
# arbitrary threshold of 100meters.  This is mainly intended to catch
# the situation where a user enters something catstrophically wrong,
# like a slipped degrees digit, or the wrong N/S/E/W identifier.
if ($diff > 100)
{
  printf( '<br /><h3>Reference position is different from '
          .'measured position by %.0d meters!</h3>', 
          $diff  
        ) ;
}


########################################################################
# Create the HTML data entry form.  We give it a name for later
# reference in javascripting.  The Action when submitted is to post the
# data to the URL specified.
########################################################################
print startform( -name=>'theForm',
                 -action=>'/perl-scripts/gpsadmin/setRefpos.cgi',
                 -onSubmit=>'return Okay()'
               ) ;


# The data entry widgets are arranged on the page in a table, so 
# that their left edges all line up.  This puts them into the
# second columns of the table.  The first column is reserved for
# right justified labels for each entry row.  Also we have some
# rows that are section titles, and these span both columns.

print '<table border=0>' ;

# We will need two radio buttons to select the position entry mode.
# These are generated into an array here (as html strings), and then 
# popped into the table below as needed.  When either button is
# clicked, the form contents are redrawn.  This is done by 
# redirecting the form's action URL back onto this original script,
# and then submitting the data.
my @modeButtons = radio_group( -name=>'Mode', 
                            -value=>['lla','ecef'],
                            -nolabels=>1,
                            -default=>'lla',
                            -onClick=>'ModeChange()',
                          ) ;

print Tr( td( {-colspan=>'2'}, b('Position:') ) ) ; 

print Tr( td( "&nbsp;" ),
          td( {-colspan=>'2'},
              $modeButtons[0], 
              " WGS-84 : Lat/Lon/Hgt&nbsp;&nbsp;&nbsp;", 
              $modeButtons[1], " ECEF : X/Y/Z",
            )
        ) ;


if ($mode eq 'lla')
{
  my @NSbuttons = radio_group( -name=>'NS', 
                            -value=>['N','S'],
                            #-nolabels=>1,
                            -default=>$NS,
                          ) ;
  print Tr( td( {-align=>'right'}, b('Lat:')),
            td( textfield( 'latDeg', 
                           $latDeg, 
                           3, 
                           3 
                         ),
                b('&deg;'),
                textfield( 'latMin', 
                           sprintf( "%02d", $latMin ), 
                           2, 
                           2 
                         ),
                b("'"),
                textfield( 'latSec', 
                           sprintf( "%07.5f", $latSec ),
                           8, 
                           8
                         ),
                b('"'),
                $NSbuttons[0],
                $NSbuttons[1],
              )
          ) ;

  my @EWbuttons = radio_group( -name=>'EW', 
                            -value=>['E','W'],
                            -default=>$EW,
                          ) ;
  print Tr( td( {-align=>'right'}, b('Lon:')),
            td( textfield( 'lonDeg', 
                           sprintf( "%03d", $lonDeg ), 
                           3, 
                           3 
                         ),
                b('&deg;'),
                textfield( 'lonMin', 
                           sprintf( "%02d", $lonMin ), 
                           2, 
                           2 
                         ),
                b("'"),
                textfield( 'lonSec', 
                           sprintf( "%07.5f", $lonSec),
                           8, 
                           8 
                         ),
                b('"'),
                $EWbuttons[0],
                $EWbuttons[1],
              )
          ) ;

  print Tr( td( {-align=>'right'}, b('Height:')),
            td( textfield( 'Height', 
                           sprintf( "%.4f", $height ), 
                           10, 
                           10 
                         ),
                'meters',
              )
          ) ;
  print Tr( td( '&nbsp;' ),
            td( button( -name=>'Here', 
                        -onClick=>'HereClicked()'
                      ),
                '&nbsp;&nbsp;Set Lat, Lon & Height to the current location.'
              ), 
            
          ) ;
}
else
{
  print Tr( td( {-align=>'right'}, b('X:')),
            td( textfield( 'X', 
                           sprintf( "%13.4f", $ecefX ), 
                           13, 
                           13 
                         ),
                'meters',
              )
          ) ;

  print Tr( td( {-align=>'right'}, b('Y:')),
            td( textfield( 'Y', 
                           sprintf( "%13.4f", $ecefY ), 
                           13, 
                           13 
                         ),
                'meters',
              )
          ) ;

  print Tr( td( {-align=>'right'}, b('Z:')),
            td( textfield( 'Z',
                           sprintf( "%13.4f", $ecefZ ), 
                           13, 
                           13 
                         ),
                'meters',
              )
          ) ;
  print Tr( td( '&nbsp;' ),
            td( button( -name=>'Here', 
                        -onClick=>'HereClicked()'
                      ),
                '&nbsp;&nbsp;Set X, Y and  Z to the current location.'
              ), 
            
          ) ;
}


print Tr( td( {-colspan=>'2'}, "&nbsp;")) ;

print Tr( td( {-colspan=>'2'}, b('RTCM, CMR & BINEX Parameters:') ) ) ; 
print Tr( td( '&nbsp;' ) ) ;


print Tr( td( {-align=>'right'}, 
              b('RTCM Station ID:'),
              br(),
              '<font size=-2>Range 0 to 1023</font>&nbsp;&nbsp;'
            ),
          td( {-valign=>'top'},
              textfield( "rtcmID", $rtcmID, 7, 4 ) 
            ), 
        ) ;

print Tr( td( {-align=>'right'}, 
              b('Binex Station ID:'),
              br(),
              '<font size=-2>Four Characters</font>&nbsp;&nbsp;'
            ),
          td( {-valign=>'top'},
              textfield( "binexID", $binexID, 7, 4 ) 
            ), 
        ) ;

print Tr( td( {-align=>'right'}, 
              b('CMR Station ID:'),
              br(),
              '<font size=-2>Range 0 to 31</font>&nbsp;&nbsp;'
            ),
          td( {-valign=>'top'},
              textfield( "cmrID", $cmrID, 7, 2 ) 
            ), 
        ) ;

print Tr( td( {-align=>'right'}, 
              b('CMR Station Name:'),
              br(),
              '<font size=-2>Eight Characters</font>&nbsp;&nbsp;'
            ),
          td( {-valign=>'top'},
              textfield( "cmrStationName", $cmrStationName, 15, 8 ) 
            ),
        ) ;

print Tr( td( {-align=>'right'}, 
              b('Description:'),
              br(),
              '<font size=-2>Fifty Characters</font>&nbsp;&nbsp;'
            ),
          td( {-valign=>'top'},
              textfield( "rtcmDesc", $rtcmDesc, 50, 50 ) 
            ),
        ) ;


print '<tr><td></td><td>' ;
print button( -name=>"OK", 
              -value=>"OK",
              -onClick=>'Okay() ;'
            ) ;
print "&nbsp;" ;

# On Cancel, go back to the toplevel page of the I/O menu = portSummary.
print button( -name=>'Cancel', 
              -value=>'Cancel',
              -onClick=>'parent.loadDataFrame( "portSummary.cgi" )'
            ) ;

print '</td></tr></table>' ;

# These are some 'hidden' fields, used during submissions of this
# form to identify various types of action requests.  The values are
# defaulted to zero and set to one only if the action is requested.
print hidden( -name=>'SetHere',
              -value=>'0'
            ) ;
print hidden( -name=>'ModeChanged',
              -value=>'0'
            ) ;
print endform() ;

print <<EOF ;
<script>

/**********************************************************************/
function ModeChange()
/**********************************************************************/
/* The user has changed the Mode between LLA and ECEF modes.  We need
 * to redraw the page keeping some of the parameters, but altering
 * others.  This is done by submitting the page back to this original
 * URL, instead of the default action-target of the form.
 * Of course, if we detect no change, then the user has clicked the
 * choice that was already selected, so we ignore the action.
 */
{

  f = document.theForm ;

  // Import the current mode from Perl.
  mode = "$mode" ;

  // Compare the state of the checkboxes versus the current mode.
  // If there was a change, submit the form.
  if ( f.Mode[0].checked && (mode != 'lla')
    || f.Mode[1].checked && (mode != 'ecef')
     )
  {
    // Indicate that this is a mode-change request.
    f.ModeChanged.value = '1' ;
    
    parent.submitForm( f, '/perl-scripts/refpos.cgi' ) ;
  }

} /* end of ModeChange() */


/**********************************************************************/
function HereClicked()
/**********************************************************************/
/* The user has changed the Mode between LLA and ECEF modes.  We need
 * to redraw the page keeping some of the parameters, but altering
 * others.  This is done by submitting the page back to this original
 * URL, instead of the default action-target of the form.
 */
{

  f = document.theForm ;
  f.SetHere.value = '1' ;
  
  parent.submitForm( f, '/perl-scripts/refpos.cgi' ) ;

} /* end of Update() */






/**********************************************************************/
function Okay()
/**********************************************************************/
/* Invoked when the OK button is pressed.
 * This just submits the entire form with all parameters.
 */
{
  parent.submitForm( document.theForm, '' ) ;
  return false ;

} /* end of Okay() */

</script>

EOF



# If an error message was found during CGI processing in setRefpos.cgi,
# it will already be displayed in a dialog box from that script.  But
# we'd like to show it here as well.  It is passed in as parameter
# 'msg'.  
if ( my $errorMessage = param('msg') )
{
  print b( $errorMessage ) ;
}

print end_html() ;

########################################################################
# End of file refpos.cgi
########################################################################
