#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: rstatusActivity.cgi,v 1.39 2006/03/23 19:09:02 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/rstatusActivity.cgi,v $

########################################################################
# File rstatusActivity.cgi
# 
# Perl Script to format a page showing current "activities" in a 
# Fireball.  This includes a summary of how many satellites are being
# tracked, what data logging is going on, what output streams are
# active, etc.
#
# Currently this is just a mockup of the page with some samples of
# what might be included.  None of the data is real except for the
# timestamp, and the satellites being tracked.
#
########################################################################

use strict ;

use CGI qw/:standard/;
use IO::Socket;

use Fireball ;

titleWithHelp( 'Receiver Status - Activity',
                         '/activityInfo.html'
                       ) ;


########################################################################
# List satellites being tracked.
########################################################################

# Determine which satellites are being tracked.  Ask the sysinfo
# utility for the tracking status report.
open( SYSINFO, 
      '/usr/bin/sudo /usr/local/fireball/sysinfo -p tracking_status |');

# Response is twelve lines, one per channel of the form
#   0 :  15 51.7 93.3 D2FF   8001 50.8 41.8 35 2.8
#  Chn  PRN Elev Azm  l1Stat     
# Or (if channel is unused)
#   0 : idle
# Scan the list of Sv information, and extract a list
# of satellite's being tracked.  The definition of tracked is
# that the L1 status indicates measurements are being made <=> the
# 64-bit in the L1 status is set.
my @svList ;
for ( my $i = 0 ; $i < 12 ; $i++ )
{
  if ( ($_ = <SYSINFO>) !~ /idle/ )
  {
    #print $_, "<br />" ;
    ( undef, undef, my $prn, undef, undef, 
      my $l1stat, my $l2stat, undef       ) = split( ' ' ) ; 

    if ( hex($l1stat) & 64)
    {
      push @svList, $prn ;
    }
  }
  else
  {
    #print $_, "<br />" ;
  }
}

close( SYSINFO ) ;

my $indent='&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' ;

print "<b>Satellites Tracked:</b>" ;

# Emit sorted list of PRN numbers. 
# Sort the SV list numerically, replace the WAAS prns with their
# textual names, and output the list, separated by spaces.  
@svList = sort( {$a <=> $b} @svList ) ;
for my $prn (@svList)
{
  if ($prn >= 120 and $prn <= 138)
  {
    $prn = $waasStrings{ $prn } ;
  }
}

$svList[0] = 'None' if (! @svList) ;
print table( Tr( td( "&nbsp;&nbsp;&nbsp;" ),
                 td( code( join( ' ', @svList ) ) )
               )
           ) ;


########################################################################
# List active data logging sessions.
########################################################################
print "<p><b>Data Logging:</b><br />";
chdir '/var/run' ;
my @sessions = glob( "gpslog.*" ) ;
if (@sessions)
{
  print '<Table>' ;
  for my $sess (@sessions)
  {
    my ( undef, undef, $sessname, undef, undef, undef, undef, undef,
         $file, undef
       ) = split( " ", qx( cat $sess ) ) ;
    $file =~ s|^/var/gps|| ;
    print "<Tr><td>&nbsp;&nbsp;&nbsp;</td><td>Session</td><td>",
          $sessname,
          "</td><td>logging to ",
          $file,
          "</td></tr>"
          ;
  }
  print "</Table>" ;
}
else
{
  print "${indent}No Sessions Active<br />" ;
}


########################################################################
# List active I/O ports
########################################################################
print '<br /><b>Input/Output:</b><table>' ;
my @clients = getIoStatus() ;
# Access an element like this:
#   $clients[1]{'name'} 

if (@clients)
{
  for my $i (0..$#clients)
  {
    my $status ;
    my $name = $clients[$i]{'name'} ;
    my $port = $clients[$i]{'port'} ;
    my $protocol = $clients[$i]{'protocol'} ;
    $protocol = 'TCP' if $protocol eq 'tcp' ;
    $protocol = 'UDP' if $protocol eq 'udp' ;
    $protocol = 'REP' if $protocol eq 'rep' ;
    my $service = $clients[$i]{'service'} ;
    $service = 'CMR' if $service eq 'cmr' ;
    $service = 'RTCM' if $service eq 'rtcm' ;
    $service = 'Rt17' if $service eq 'rt17' ;
    $service = 'Binex' if $service eq 'binex' ;
    my $ip = $clients[$i]{'ip'} ;
    $status = "--$name $port $protocol $service $ip" ;

    if ($service eq 'met')
    {
      $status = "Met/Tilt interface active on Serial Port $port" ;
    }
    elsif ($protocol eq 'serial')
    {
      $status = "Streaming $service over Serial Port $port" ;
    }
    else
    {
      $status = "Streaming $service to $ip over $protocol Port $port" ;
    }


    print Tr( td( '&nbsp;&nbsp;&nbsp;' ),
              td( $status ) 
            ) ;
  }
}
else
{
  print Tr( td( '&nbsp;&nbsp;&nbsp;' ),
            td( 'There are no currently active I/O Streams.' )
          ) ;
}
print '</table>' ;


my ( $primaryV, $secondaryV, $temperature ) = getVoltages() ;

print '<br /><b>Power Supply Info:</b>' ;

print table( Tr( td( '&nbsp;&nbsp;&nbsp;' ),
                 td( 'Primary Voltage:' ),
                 td( sprintf( "%.3f Volts", $primaryV ) ) 
               ),
             Tr( td( '&nbsp;&nbsp;&nbsp;' ),
                 td( 'Secondary Voltage:' ),
                 td( sprintf( "%.3f Volts",  $secondaryV ) ) 
               ),
             Tr( td( '&nbsp;&nbsp;&nbsp;' ),
                 td( 'Temperature:' ),
                 td( "$temperature&deg; C" ) 
               )
           ) ;

print '<br /><b>Run Time:</b>' ;
open( UPTIME, "/proc/uptime" ) ;
$_ = <UPTIME> ;
close( UPTIME ) ;
my ( $uptime, undef ) = split ;
my $days = int( $uptime / 86400 ) ;
my $hours = int( $uptime / 3600 ) % 24 ;
my $minutes = int( $uptime / 60 ) % 60 ;

print br(),
      $indent, 
      sprintf( "System has been running for "
               ."%d day%s %d hour%s %d minute%s",
               $days,
               ($days != 1) ? "s" : "",
               $hours,
               ($hours != 1) ? "s" : "",
               $minutes,
               ($minutes != 1) ? "s" : "",
             ),
      br() ;

if (-e "/var/run/dialout.pid")
{
  # A PPP Dialout session is active.  We'd like to determine if
  # it has successfully created a ppp network connection.
  # PPP Dialout runs a pppd process and records its pid in
  # /var/run/dialout.pid.  If that succeeds in making a connection,
  # then it will set up a network device ppp[0123] whose ifconfig
  # entry will show the network addresses that are in use.
  print '<br /><b>PPP Dialout:</b><br />' ;

  # Get the process id of the pppd dialout process.
  my $dialoutPid = qx( cat /var/run/dialout.pid ) ;
  chomp $dialoutPid ;
  #print "${indent}dialoutPid : $dialoutPid", br()  ;

  # If that process is successful, it will have made a file like
  # /var/run/ppp0.pid whose contents match the dialout.pid.
  my @pidFiles = glob( "/var/run/ppp*.pid" ) ;
  #print "${indent}Pid files: ",join( ", ", @pidFiles ), br() ;


  my $device = '' ;
  for my $pidFile (@pidFiles)
  {
    my $thisPid = qx(cat $pidFile) ;
    chomp $thisPid ;
    #print "${indent}$pidFile : $thisPid", br() ;
    if ($dialoutPid == $thisPid)
    {
      $device = $pidFile ;
      $device =~ s/\.pid// ; # Drop extension to get device
      $device =~ s|/var/run/|| ; # Drop path portion
    }
  }

  #print "${indent}device = $device", br() ;
  if ( ! $device)
  {
    print "${indent}Connecting.....", br() ;
  }
  else
  {
    # Run the ifconfig command to determine the currently assigned
    # IP addresses.  The results should be:
    #
    # ppp0  Link encap:Point-to-Point Protocol
    # inet addr:68.240.198.195  P-t-P:192.168.13.31 Mask:255.255.255.255
    # UP POINTOPOINT RUNNING NOARP MULTICAST MTU:1500  Metric:1
    # RX packets:157 errors:4 dropped:0 overruns:0 frame:0
    # TX packets:157 errors:0 dropped:0 overruns:0 carrier:0
    # collisions:0 txqueuelen:3
    #
    # We only need the inet addr value from the second line.

    my $ifconfig = qx( /sbin/ifconfig $device ) ;
    #print ${indent},$ifconfig, br() ;
    $ifconfig =~ m/inet addr:(\S*)/ ; # $1 is now the address.

    print "${indent}Connected.&nbsp;&nbsp;Assigned IP address is $1" ;
  }
}


########################################################################
# Create Timestamp:
# 2001 Sep 6 - Thu of GPS Week 1130 - 17:46:52 GMT 
print hr() ;
print timestamp() ;

# Delay a bit and then refresh this page.  We try to make this happen on
# exact 5 second boundaries.  If this time was an exact 5 second time,
# then we wait 4000 msec to allow a little bit of time for
# transmission and processing.  The range of delays is from 2000 to 
# 6000.
my $delay = 6000 - ( (time()+2) % 5 ) * 1000 ;
print "
<script>
  // Reload this page every five seconds
  parent.loadAfterDelay( window.location.href, $delay) ;
</script>
" ;


print end_html() ;

########################################################################
# end of file rstatusActivity.cgi
########################################################################
