#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: rstatusIdentity.cgi,v 1.36 2007/01/30 00:35:10 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/rstatusIdentity.cgi,v $

########################################################################
# rstatusIdentity.cgi
#
# Cgi script that display basic receiver identification inforamtion.
# This includes system name, serial number, Ethernet IP address, etc.
#
# Also includes a link-button to start the process of setting a new
# system name.
#
# Also does a check to see if the power-saving mode is active.  If so
# then an alert message is opened warning that the system could power
# down at any time.
########################################################################

use strict ;
#push( @INC, '/usr/local/apache/htdocs/fireball/perl-scripts/') ;
use CGI qw/:standard/;
use Fireball ;


# Get the system identifier string from a config file.
my $identityString= qx|cat /etc/sysconfig/sysname| ;

# Determine a serial number by querying the "warranty" command.
# warranty -s returns:
#   Serial_number: 5432198765
# Extract the numeric string.
my $serialNumber = qx(/usr/bin/sudo /usr/local/fireball/warranty -s) ;
chomp $serialNumber ;   # Drop linefeed
$serialNumber = (split("[ :]+",$serialNumber))[1] ;


titleWithHelp( 'Receiver Status - Identity',
                         '/identityInfo.html'
                       ) ;

print <<EOF ;
<style type="text/css">
 <!--
       a {color:blue ; text-decoration: none }
 -->
</style>
EOF


# Determine the ethernet IP address by runnin ifconfig.  This
# parsing of the output depends on the undocumented functioning of the
# ifconfig command, which could change.  A better implementation would
# be to create a specialized command (like getmac) that used standard
# libraries to obtain the value.  Maybe some day there will be time.
my $etherIP = 
 (split( ':', 
        (split( ' ', 
               qx( /sbin/ifconfig eth0 | grep 'inet addr:' ) ))[1] 
      ))[1] ;


# Determine the network name (if any) associated with the IP address
# as determined by the DNS system.  We use nslookkup to attempt a
# lookup, and parse the results.  This is dependent on the exact
# formatting of nslookup's output, which is dependent on which version
# we use.  This one is written around the busybox implementation.
# Ideally the output will look like:
#   Server:     carolyn.fireball.trimble.com
#   Address:    192.168.142.7
#   <blank line>
#   Name:       fbtc.fireball.trimble.com
#   Address:    192.168.142.150
# There can be several failures of this process.
#   No Name server in /etc/sysconfig/resolv.conf.
#       We check for lines with 'nameserver xxx.xxx.xxx.xxx.
#       Indicate No Name Server if 
#   Name server doesn't respond.
#       (We check by opening a socket to port 53 on each address until
#       we find a server that is live.)
#   Name not defined on the server.
# 

# This will be the looked up name.  The default assumes that no name 
# server is defined.  This will be overwritten if we find a server
# or there is a successful lookup.
my $dnsname = "[No Name Server Defined]" ;


if (open( RESOLVE, "/etc/sysconfig/resolv.conf" ) )
{
  # Scan through all lines in the resolv.conf file, looking for those
  # that start with 'nameserver'.  Try each of them in turn until
  # we find a live server.
  for my $serverIP (grep(/^nameserver/, <RESOLVE>))
  {
    $serverIP =~ s/^nameserver\s+// ;

    # Since we have at least one defined server, change the default 
    # result.  This will be overwritten if we find an active server.
    $dnsname = "[Name Server not responding]" ;

    # Attempt a socket connection to that server's DNS port 53.  
    # If this succeeds then we know the DNS server is alive.
    # We use the IO::Socket::INET library module because that has
    # a timeout feature, while the basic perl socket() call does not.
    # Without this timeout, problems have been noticed, especially
    # when the name server is hidden by iptables filters.
    use IO::Socket::INET ;
    my $s = IO::Socket::INET->new( PeerHost=>$serverIP,
                                   PeerPort=>53,
                                   Timeout=>4,
                                   Proto=>'tcp'
                                 ) ;
    if ( $s ) 
    {
      # Socket opened okay, so it appears that we can connect to
      # a name server.  Close the socket and then use nslookup to
      # attempt the name lookup.
      close( $s ) ;

      my @dnsresults = split( "\n", 
                              qx( /usr/bin/nslookup $etherIP $serverIP )
                            ) ;
      # A good response looks like:
      #   Server:     carolyn.fireball.trimble.com
      #   Address:    192.168.142.7
      #   <blank line>
      #   Name:       tomsproto.fireball.trimble.com
      #   Address:    192.168.142.174                                                                                                      
      if ( (scalar(@dnsresults) == 5) 
        && ($dnsresults[3] =~ /^Name:/)
         )
      {
        # The fourth line shows the name we want in this format:
        #         Name:  name.domain.dom
        # Grab the second field.
        $dnsname = ( split( ' ', $dnsresults[3] ) )[1] ;
      }
      else
      {
        $dnsname = "[Unknown]" ;
      }

      last ; # Exit the for-loop.
    }
  }

  close( RESOLVE ) ;
}


my $fwVersion = 'Unknown' ;
my $fwDate    = '01-Jan-2000' ;

if ( -r '/usr/local/fireball/defaults/firmwareVersion' ) 
{
  ($fwVersion, $fwDate) = split( "\n", 
                  qx( cat /usr/local/fireball/defaults/firmwareVersion )
                               ) ;
  
  my ($year,$month,$day) = split( '/', $fwDate ) ;
  $fwDate = "$day $monthStrings[$month-1] $year" ;
}


# Make a table with three columns.  The idea is to center the middle
# column in the middle of the page.  The left column contains
# right-justified labels for the middle column values.  The right
# column is blank, and only there to balance out the left column.

print table( {border=>0, align=>'center'},
             Tr( td( {align=>'right', width=>'25%'}, 'System Name: ' ),
                 td( {align=>'center'}, 
                     b($identityString) 
                   ),
                 td( {width=>'25%'},  '&nbsp;' )
               ),
             Tr( td( {align=>'right'}, 'Serial Number: ' ),
                 td( {align=>'center'}, b($serialNumber) ) 
               ),
             Tr( td( {align=>'right'}, 'Ethernet IP: ' ),
                 td( {align=>'center'}, b($etherIP) ) 
               ),
             Tr( td( {align=>'right'}, 'DNS Resolved Name: ' ),
                 td( {align=>'center'}, b($dnsname) ) 
               ),
             Tr( td( {align=>'right'}, 'Firmware Version: ' ),
                 td( {align=>'center'}, 
                     b( "$fwVersion&nbsp;&nbsp;&nbsp;$fwDate" ) 
                   ) ,
               ),
             # Next row contains a pseudo-button in the middle.  We use
             # an embedded table with the border decoration to 
             # implement this.  The button loads in another script over
             # this page.
             Tr( td( '&nbsp;' ),
                 td( {align=>'center'},
                     table( {border=>2,align=>'center',cellpadding=>5},
                            Tr( 
                                td( {align=>'center'},
                                    a( {href=>
            "javascript:parent.loadDataFrame('sysadmin/setSysname.cgi')"
                               }, 
                               'Set System Name' 
                                     )
                                  ),
                              )
                          ) 
                   )
               )
           ) ;


print '<p>&nbsp;' x 3 ;
print <<EOF ;
&copy; 2003-2007, Trimble Navigation Limited. All rights reserved.
Trimble is a trademark of Trimble Navigation Limited registered in the
United States Patent and Trademark Office. The Globe &amp; Triangle logo
is a trademark of Trimble Navigation Limited.  All other trademarks are
the property of their respective owners.
EOF


# Check if the power-saving mode is active, and put up an alert if so.
# THe first line of the sleepmode configuration file contains 4 fields,
# like:
#     yes yes 3 5
# The first field [yes or no] controls the power saving mode.

if ( open( SLEEP, '/etc/sysconfig/sleepmode' ) )
{
  $_ = <SLEEP> ;
  close SLEEP ;
  if ( m/^yes/) 
  {
    print <<EOF ;
<hr />
<h2>Power Saving Mode Is Active!</h2>
This system will attempt to save battery power by shutting down between
data logging sessions.  This could happen at any time, depending on the
currently defined data logging schedule.
<p>
If you don't want to risk the system turning off during your browser
session, go to the 
<a href="/perl-scripts/sleepMode.cgi">Power&nbsp;Saving&nbsp;control&nbsp;page</a> and
turn off the Power Saving mode.
<hr />
<script>
alert( 'Power Saving Mode is active.\\n' +
       'Please read the warning on the Identity page.' ) ;
</script>
EOF

  } 
}

print end_html() ;

########################################################################
# end of file rstatusIdentity.cgi
########################################################################
