#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: rstatusPosition.cgi,v 1.34 2004/08/19 21:56:13 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/rstatusPosition.cgi,v $

use strict ;
use CGI qw/:standard/;

use Fireball ;

titleWithHelp( 'Receiver Status - Position',
               '/positionInfo.html'
             ) ;

########################################################################
sub printll
########################################################################
# Function printll : 
#   Prints a latitude or longitude in Degrees:minutes:seconds format,
#   with a leading label field.
#   First argument is a string label of three characters, either "Lat"
#   or "Lon".
#   Second argument is the lat or lon in degrees.
#   Outputs to standard output a string of the form:
#     Lat  37*22'59.9999" N
#   (where * = the HTML degrees symbol).
########################################################################
{
 

  # Parse parameters into local variables.
  my $label = $_[0] ;
  my $ll = $_[1] ;
  my $negative = ($ll < 0.0) ;
  my $units ;
  if ($negative) {$ll = -$ll} ;

  #my $pi=3.14159265358979 ;
  #$ll = $ll*180/$pi ; # Use this if argument were radians

  my $degrees = int($ll) ;
  $ll = ($ll - $degrees) * 60.0 ; 
  my $minutes = int($ll) ;
  my $seconds = ($ll - $minutes) * 60.0 ;

  if ($label =~ "Lat")
  {
    $units = ($negative ? "S" : "N" ) ;
  }
  elsif ($label =~ "Lon")
  {
    $units = ($negative ? "W" : "E" ) ;
  }

  print sprintf( "<b>%s</b> %3d&deg;%02d'%07.4f\" %s\n", 
                 $label, $degrees, $minutes, $seconds,$units ) ;
} #end of printll() 


# Open a channel to the sysinfo program so we can read the
# pos_info information.
open( SYSINFO, '/usr/local/fireball/sysinfo -p pos_info |' ) ;

# Response is of the form
#   167368.0, 1166:  37.3891508323 -122.034234235 -3.456
#    [timetag   week#  Lat-degrees   Lon-degrees    Height-meters]
#   3 14 9 11 18 25 23 31
#    [Satellites used]
#   1.5 0.9 1.3 0.8
#    [Pdop,hdop,vdop,tdop]
#   0.03 -0.05 0.02
#    [Velocities E,N,Up]
#   0.000000e+00 -2.054596e-02
#    [clock offset, clock drift]
#   3D autonomous
#    [postype (2D 3D or OLD) and corrections]
#   OK

my ($latitude, $longitude, $altitude ) = (0,0,0) ;
my @svsUsed = "None" ;
my ($pdop, $hdop, $vdop, $tdop) = (0,0,0,0) ;
my ($velE, $velN, $velUp) = (0,0,0) ;
my ($clockOffset, $clockDrift) = (0,0) ;
my ($posType, $posCorrections) = ("old", "Autonomous") ;
my $warning = '' ;

my $line = <SYSINFO> ;
chomp $line ;
if ($line !~ /^OK/)
{
  my ($timetag, $position) = split(':', $line) ;
  ($latitude, $longitude, $altitude) = split(' ',$position) ;

  @svsUsed = split( ' ', <SYSINFO> ) ;

  ($pdop, $hdop, $vdop, $tdop) = split( ' ', <SYSINFO> ) ;

  ($velE, $velN, $velUp) = split( ' ', <SYSINFO> ) ;
  ($clockOffset, $clockDrift) = split( ' ', <SYSINFO> ) ;

  ($posType, $posCorrections) = split( ' ', <SYSINFO> ) ;
  if ($posType eq 'old' ) 
  {
    print h3( 'This is old data.  There is no current position fix.' ) ;
    $posType = '<b><font color=red>OLD</font></b>' ;
    @svsUsed = () ;
  }
  $posCorrections = 'Autonomous' if ($posCorrections eq 'autonomous') ;
  $posCorrections =~ s/_/-/ ;
}
close(SYSINFO) ;


# Done with interaction with Stinger.  
# Format the data into the HTML page.

print '<Table width="100%" border=0>
<tr><td valign="top" width="50%">' ;

print "<b>Position:</b>" ;

print "<pre>" ;
printll( "Lat:", $latitude ) ;
printll( "Lon:", $longitude ) ;
printf( "<b>Hgt:</b> %.3f meters\n", $altitude ) ;
print   "<b>    </b> WGS84, $posType, $posCorrections" ;
print "</pre>" ;

print '<p><b>Velocity:</b>
<pre>' ;

printf( "<b>  North:</b> %6.2f m/sec\n", $velN ) ;
printf( "<b>   East:</b> %6.2f m/sec\n", $velE ) ;
printf( "<b>     Up:</b> %6.2f m/sec\n", $velUp ) ;

# Emit composite horizontal speed in m/s, mph, and km/h
print "\n";
my $speed = sqrt($velN*$velN + $velE*$velE) ;
printf( "<b>  Horiz:</b> %6.2f m/sec\n", $speed ) ;
printf( "<b>        </b> %6.2f mph\n", 
        $speed * 2.22369363 # Converts m/s to miles/hour
      ) ;
printf( "<b>        </b> %6.2f km/h\n", 
        $speed * 3.6 # Converts m/s to km/hour
      ) ;

my $heading = atan2( $velE, $velN ) / 3.1415926 * 180.0 ;
$heading += 360.0 if ($heading < 0 ) ;

printf( "<b>Heading:</b> %5.0f&deg; True North\n", $heading ) ;

print '</pre>' ;



print '
</td>

<td valign="top" width="50%">
' ;

print '<b>Satellites used:</b><pre>';
# Sort the SV list numerically, and output the list, separated
# by spaces.
print join( ' ', sort( {$a <=> $b} @svsUsed) ) ;

print ' 

</pre>' ;
print '<b>Receiver Clock:</b><br />' ;


# Obtain current clock steering setting from gpsinfo system, using
# sysinfo.  Response will be of the forms:
#  put clock_steering on 
#  put clock_steering off
my $clockSteering = qx( /usr/local/fireball/sysinfo -p clock_steering );
chomp $clockSteering ;
$clockSteering =~ s/put clock_steering // ;   # Now 'on' or 'off'
if ( $clockSteering eq 'yes' )
{
  $clockOffset = 0.0 ;
  $clockDrift  = 0.0 ;
}

print '<code>' ;
printf( "<br /><b>Offset:</b> %10.6f msec\n", 
        $clockOffset/2.99792458e5 ) ;

#printf( "<b> Drift:</b> %10.1f meters/sec\n", $clockDrift ) ;
# Convert drift to parts-per-million by dividing by speed of light
# (2.99e8, and then multiplying by a million.
printf( "<br /><b>&nbsp;Drift:</b> %10.3f ppm\n", $clockDrift/2.99792458e2 ) ;

print '</code>' ;

print "<br />Clock steering is $clockSteering." ;




print '<br /><br /><br /><b>Dilutions of Precision:</b> <pre>' ;
print( "\n<b>PDOP:</b>  ", $pdop ) ;
print( "\n<b>HDOP:</b>  ", $hdop ) ;
print( "\n<b>VDOP:</b>  ", $vdop ) ;
print( "\n<b>TDOP:</b>  ", $tdop ) ;

print '
</pre>
</td>
</tr>
</table>
' ;

print hr() ;

# Create Timestamp:
# 2001 Sep 6 - Thu of GPS Week 1130 - 17:46:52 GMT 
print timestamp() ;

# Refresh the page every two seconds.  Figure out the delay so that the
# times usually end up on an even 2 second boundary.
my $delay = 1700 - (time() % 2) * 1000 ;
print <<EOF ;
<script>
parent.loadAfterDelay( window.location.href, $delay) ;
</script>
EOF


print end_html() ;

########################################################################
# end of rstatusPosition.cgi
########################################################################
