#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: satData.cgi,v 1.25 2003/06/12 20:25:38 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/satData.cgi,v $

########################################################################
# satData.cgi
# Control the display of formatted data from satellites.
#
# These CGI parameters may be present:
#
#   dataType : 
#       one of several strings indicating what type of data is desired.
#
#   sv : A qualifier on the dataType.
#
########################################################################


use strict ;
use CGI qw/:standard/ ;
use Fireball ;

titleWithHelp( 'Satellites - Broadcast Data',
                         '/satDataInfo.html'
                       ) ;

#print( '<BR>Parameters Received:<BR>' ) ;
#foreach $p (param())
#{
#    print( "$p=", param($p), '<BR>' ) ;
#}
#print '<BR>' ;


# Determine if this is in printer friendly window or not.
# $ps will be an empty string (false) if not.

# Determine which satellite number has been requested.
# May be "All".  May be unspecified, in which case we set it to 1.
# Otherwise, the parameter is of the form "SV 17" and we need to strip
# off the leading characters.
my $sv = param( 'sv' ) ;
if ( (!$sv) 
  || ($sv eq 'Select Satellite')
   )
{
  $sv = 'All' ;
}
elsif ($sv ne 'All')
{
  (undef, $sv) = split( ' ', $sv ) ;
}



########################################################################
# Javascript Functions
print <<EOF ;
<Script>

/**********************************************************************/
function respondToSelection()
/**********************************************************************/
/* When a choice is made via the popup menus, this function is called.
 * A new URL is loaded of the states of the popups, to request the
 * specified data.
 */
{
  index = document.theForm.dataType.selectedIndex  ;
  option = document.theForm.dataType.options[index].value  ;
  if (option == 'Select Desired Data') return ;

  parent.submitForm( document.theForm, '/perl-scripts/satData.cgi') ;

} /* end of respondToSelection() */

</Script>
EOF
# End of Javascript Functions
########################################################################



print startform( -name=>'theForm' ) ;

# Create a list of items to put in the popup menu that is coming later.
my @items=('Ephemeris Data',
           'Almanac Data',
           'UTC Data',
           'Ionospheric Model') ;

# If this is the initial display of this page (with no parameters) then
# we label the default on the dataType popup with a "fake" label to
# instruct the user what to do.
if (!param())
{
  splice( @items, 0, 0, ('Select Desired Data') ) ;
}

print popup_menu( -name=>'dataType', 
                  -values=>\@items,
                  -onChange=>'respondToSelection()' ) ;


# Create the SV selection popup if needed.
my $dt=param('dataType') ;

if ( defined( $dt )
  && ( ($dt eq 'Ephemeris Data')
    || ($dt eq 'Almanac Data')
     )
   )
{
  # 'SV 1', 'SV 2', etc.
  @items =('All') ;
  for (1..32)
  {
    push @items, "SV $_" ;
  }

  print popup_menu( -name=>'sv', 
                    -values=>\@items,
                    -default=>'All',
                    -onChange=>'respondToSelection()' ) ;
}


print br() ;

# If any parameters came in on this invocation, then we have
# some real data to display.  Figure out what type is requested,
# and then produce it.  
if (param())
{
  my $type=param('dataType') ;

  if ($type eq 'UTC Data')
  {
    print '<h3>UTC (Universal Coordinated Time) Data</h3><BR>' ;

    print '<pre>' ;

    open( SYSINFO, 
       "/usr/bin/sudo /usr/local/fireball/sysinfo -p utc_data |"
        ) ;
    # Response is of the form
    #   **** UTC data ****
    #   A_0(s):     -4.645555323423E-09
    #   ...
    #   OK

    # Print the returned data  until a line with "OK" comes back.
    while ( ($_ = <SYSINFO>) !~ '^OK' )
    {
      print $_ ;
    } 
    close( SYSINFO ) ;

    print '</pre>' ;
  }

  elsif ($type eq 'Ionospheric Model')
  {
    print '<h3>Ionospheric Model Data</h3><BR>' ;
    print '<pre>' ;

    open( SYSINFO, 
       "/usr/bin/sudo /usr/local/fireball/sysinfo -p iono_data |"
        ) ;
    # Response is of the form
    #   **** IONO data ****
    #   ALPHA_0(sec):     -4.645555323423E-09
    #   ...
    #   OK
    # Print the returned data  until a line with "OK" comes back.
    while ( ($_ = <SYSINFO>) !~ '^OK' )
    {
      print $_ ;
    } 
    close( SYSINFO ) ;

    print '</pre>' ;

  }

  elsif ($type eq 'Ephemeris Data')
  {
    print '<h3>Satellite Ephemeris Data</h3><BR>' ;
    if ($sv eq 'All')
    {
      print '<pre>' ;

      for ($sv = 1 ; $sv <=32 ; ++$sv)
      {
        open( SYSINFO, 
           "/usr/bin/sudo /usr/local/fireball/sysinfo -p eph_data $sv |"
            ) ;
        # Response is of the form
        #   ...
        #   OK
        # Print the returned data  until a line with "OK" comes back.
        while ( ($_ = <SYSINFO>) !~ '^OK' )
        {
          print $_ ;
        } 
        close( SYSINFO ) ;

        print "\n" ;
      }

      print '</pre>' ;
    }
    elsif ($sv != 0)
    {
      my $search = sprintf( 'SV-%2d', $sv ) ;

      print '<pre>' ;

      open( SYSINFO, 
           "/usr/bin/sudo /usr/local/fireball/sysinfo -p eph_data $sv |"
          ) ;
      # Response is of the form
      #   ...
      #   OK
      # Print the returned data  until a line with "OK" comes back.
      while ( ($_ = <SYSINFO>) !~ '^OK' )
      {
        print $_ ;
      } 
      close( SYSINFO ) ;
      print '</pre>' ;
    }
  }

  elsif ($type eq 'Almanac Data')
  {
    print '<h3>Satellite Almanac Data</h3><BR>' ;

    if ($sv eq 'All')
    {
      print '<pre>' ;

      for ($sv = 1 ; $sv <=32 ; ++$sv)
      {
        open( SYSINFO, 
           "/usr/bin/sudo /usr/local/fireball/sysinfo -p alm_data $sv |"
            ) ;
        # Response is of the form
        #   ...
        #   OK
        # Print the returned data  until a line with "OK" comes back.
        while ( ($_ = <SYSINFO>) !~ '^OK' )
        {
          print $_ ;
        } 

        print "\n" ;
      }
      close( SYSINFO ) ;

      print '</pre>' ;
    }
    elsif ($sv != 0)
    {
      print '<pre>' ;

      open( SYSINFO, 
         "/usr/bin/sudo /usr/local/fireball/sysinfo -p alm_data $sv |"
          ) ;
      # Response is of the form
      #   ...
      #   OK
      # Print the returned data  until a line with "OK" comes back.
      while ( ($_ = <SYSINFO>) !~ '^OK' )
      {
        print $_ ;
      } 
      close( SYSINFO ) ;
      print '</pre>' ;
    }

  }
}

print endform() ;

print hr() ; # Separator bar

print timestamp() ;

print end_html() ;

# end of script satData.cgi
