#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: satEnaDis.cgi,v 1.7 2003/12/19 01:46:21 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/satEnaDis.cgi,v $

########################################################################
# satEnaDis.cgi
#
# Displays the enabled/disabled state of all satellites, and
# allow the user to change that state for each SV to Enabled, Disabled,
# or IgnoreHealth.
#
# This script is called in one of two forms.  
#
# If no CGI parameters are appended, then we display the current state
# of the enable controls on all satellites.
#
# If called with any parameters, it is assumed that a full set of
# Enables/Disables/Ignores is being set.  The parameters that actually
# come in are based on the check boxes that are checked.  If an Enable
# checkbox is set then there will be a parameter like "Enable12" where
# 12 is a prn number.  A similar parameter will come in for every
# IgnoreHealth checkbox, named like "Ignore23".
# The cgi request, thus would look like this:
#     satEnabDis.cgi?Enable1=1&Enable2=1&Ignore2=1&....
# If there are no Enable or Ignore parameters for a particular PRN, then
# that prn is disabled.
#
# The display of the information is a table of PRNs and checkboxes.  
# At the bottom is an OK button which will accept all of the changes in
# a single operation.  An earlier version made multiple requests, using
# a popup selector and buttons, but this has been abandoned.
# Also along the bottom is a "Cancel" button to exit without making an
# changes, and an "Enable All" button which sets the controls to the
# default settings (but an OK is still needed).
#
########################################################################


use strict ;
use CGI qw/:standard/;
use Fireball ;

titleWithHelp( 'Satellites - GPS Enable/Disable',
                         '/satEnaDisInfo.html'
                       ) ;


# Ask Sysinfo system what the current SV masks are...
$_ = qx( /usr/bin/sudo /usr/local/fireball/sysinfo -p sv_masks ) ;
# Return string should be of the form
#  put sv_masks E I E D E .... 32 of them.
my @state = split( " ", $_ ) ;
# Drop the "put sv_masks" portion.
splice( @state, 0, 2 ) ;


#print @state ;
#print "<P>" ;


# Set all of the CGI parameters that control the states of the
# buttons.  These are set in advance of actually defining the buttons so
# that the defaults of the buttons will be controlled by the current
# state of the stinger controls.
for ( my $i = 1 ; $i < 33 ; ++$i )
{
  param( 'Enable'.$i, 
         ($state[$i-1] ne 'D') ? 1 : 0 
       ) ;

  param( 'Ignore'.$i, 
         ($state[$i-1] eq 'I') ? 1 : 0 
       ) ;
}



print startform( -name=>'theForm', 
                 -action=>'/perl-scripts/gpsadmin/setEnaDis.cgi'
               ) ;


# The array of satellite names and enable/ignoreHealth checkboxes will
# be formatted in a table with two major columns, one for the lower
# numbered satellites and one for the upper numbers.  Actually, this is
# done with multiple columns to keep the checkboxes and everything
# nicely aligned.

print '<table border="0">' ;

# Print headers above the columns.
print th( "PRN" ),
      th( "&nbsp;&nbsp;Enable&nbsp;&nbsp;" ),   
      th( "Ignore<br />Health" ),

      # A spacer column between the two sections
      th( "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" ),

      th( "PRN" ),
      th( "&nbsp;&nbsp;Enable&nbsp;&nbsp;" ),
      th( "Ignore<br />Health" ),

      # A spacer column between the two sections
      th( "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" ),

      th( "PRN" ),
      th( "&nbsp;&nbsp;Enable&nbsp;&nbsp;" ),
      th( "Ignore<br />Health" ),
      ;

# A loop that runs for each row of the table.
for ( my $i = 0 ; $i < 11 ; ++$i )
{
  print "<tr>" ;

  # Within each row we put two satellite.
  for my $prn ( $i+1, $i+12, $i+23 )
  {
    last if ($prn > 32) ;

    print td( {align=>'center'},$prn )  ;
    
    # The next two columns have the Enable and IgnoreHelath
    # checkboxes.  Each of these is given a unique name base
    # on the PRN.  THose names become the parameters fed
    # back to the cgi script when OKAY is pressed.
    # Each button, when clicked, calls a javascript function
    # with the prn as a parameter.  The functions implement
    # the linkage between the two buttons.  See the script
    # definitions below for details.
    print td( {align=>'center'}, 
              checkbox( -name=>"Enable$prn", 
                        -value=>'1', 
                        -label=>'', 
                        -onClick=>"enClick($prn)" 
                      ) 
            ) ;
    print td( {align=>'center'}, 
              checkbox( -name=>"Ignore$prn", 
                        -value=>'1', 
                        -label=>'',
                        -onClick=>"igClick($prn)" 
                      ) 
            ) ;
    print td() ;
  }
  print "</tr>" ;
}

print '</table>' ;

print hr() ;


# Put up the buttons.
# Okay submits the data.  
# Cancel goes back to the satellite status page.
# Enable All resets all the checkboxes.
print button( -name=>'OK', 
              -value=>'OK',
              -onClick=>'parent.submitForm( document.theForm, "" )'
            ) ;
print '&nbsp;' ;
print button( -name=>'Cancel', 
              -value=>'Cancel',
              -onClick=>'parent.loadDataFrame( 
                                "/perl-scripts/satGeneral.cgi" ) '
            ) ;
print '&nbsp;' ;
print button( -name=>'EnableAll', 
              -value=>'Enable All',
              -onClick=>"enableAll();"
            ) ;

print endform() ;

# Emit some javascript functions that are called above when one of
# several GUI elements are clicked.
print <<EOF ;
<Script>

/**********************************************************************/
function igClick( prn )
/**********************************************************************/
/* Called when an ignore checkbox is clicked, with the prn number as
 * a parameter.  If the Ignore button is being enabled, we automatically
 * click the matching Enabled button as well.
 */
{
  en = document.theForm[ "Enable" + prn ] ;
  ig = document.theForm[ "Ignore" + prn ] ;

  if (ig.checked)
    en.checked = true ;
} /* end of Update() */


/**********************************************************************/
function enClick( prn )
/**********************************************************************/
/* Called when an Enable checkbox is clicked, with the prn number as
 * a parameter.  If the Enable button is being disabled, we automatically
 * uncheck the matching IgnoreHealth button as well.
 */
{
  en = document.theForm[ "Enable" + prn ] ;
  ig = document.theForm[ "Ignore" + prn ] ;

  if (! en.checked)
    ig.checked = false ;
} /* end of Update() */


/**********************************************************************/
function enableAll()
/**********************************************************************/
/* Called when the Enable All button is clicked.
 * Modifies all of the checkboxes to the Enabled state.
 */
{
  for (prn = 1 ; prn < 33 ; ++prn )
  {
    document.theForm[ "Enable" + prn ].checked = true ;
  }
} /* end of Update() */



</Script>
EOF
 
print end_html() ;

########################################################################
# end of file satEnaDis.cgi
########################################################################
