#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: satTracking.cgi,v 1.36 2006/03/23 23:50:38 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/satTracking.cgi,v $

########################################################################
# File satTracking.cgi
#
# Perl script to generate the Fireball GUI page showing the 
# tracking status of all channels.  This is essentially a big table
# showing what PRNs are being tracked, plus a bunch of information about
# those satellites.
#
# This page is auto-refreshed every few seconds.
#
# A single CGI parameter can be fed to this script:
#   sortby : prn | chan | elev | L1 | L2
#            Allows sorting the table elements by the selected
#            column.  Default without this parameter is by-prn.
#
########################################################################

use strict ;
use CGI qw/:standard/;

use Fireball ;

titleWithHelp( 'Satellites - Tracking Information',
                         '/satTrackingInfo.html'
                       ) ;


# Determine how to sort the tracking information.  Either we
# use the default, or we get a parameter telling us what is wanted.
my $sortby = 'prn' ; # The default sorting order
if (param('sortby'))
{
  $sortby = param('sortby') ;
}

# Make all links blue with no underlines, even if visited, etc.
# This is a document-level style specification, affecting all
# links in this frame.  (Doesn't work very well with early Netscape
# browsers.)
print <<EOF ;
<style type="text/css">
 <!--
       a {color:blue ; text-decoration: none }
 -->
</style>
EOF


open( SYSINFO, 
      '/usr/bin/sudo /usr/local/fireball/sysinfo -p tracking_status |');
# Response is twelve lines, one per channel of the form
#   0 :  15 51.7 93.3 D2FF   8001 50.8  41.8  35   2.8
#  Chn  PRN Elev Azm  L1Stat L2St L1SNR L2SNR IODE URA  
# Or (if channel is unused)
#   0 : idle

# Scan the list of Sv information, and extract a list
# of satellite's being tracked.  These are the [2] element on
# each of the non-idle tracking lines.
my $i ;
my @svData ;

for ( $i = 0 ; $i < 12 ; $i++ )
{
  if ( ($_ = <SYSINFO>) !~ /idle/ )
  {
    #print $_, "<br />" ;
    push @svData, $_ ;
  }
  else
  {
    #print $_, "<br />" ;
  }
}

$_ = <SYSINFO> ; # drop OK
close( SYSINFO ) ;


$_ = qx( /usr/local/fireball/sysinfo -p gps_hlth ) ;
# Comes back with "0 0 0 0 ... -1 0  63 0 0 0 0 0 -1\nOK\n"
my @allHealth = split() ;


$_ = qx( /usr/bin/sudo /usr/local/fireball/sysinfo -p waas_hlth ) ;
# Comes back with '1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1\nOK\n'
my @waasHealth = split( ' ', $_ ) ;




# Sort the lines in @svData in one of several ways.  The sort() function
# takes a simple expression on $a and $b, two items to be compared.
# Then it produces a list of the elements in that order.
if ($sortby eq 'prn')
{
  # The PRN sorting key is the #2 field in each line.
  # Sort in ascending numerical order.
  @svData = sort( { (split( ' ', $a))[2] <=> (split( ' ', $b))[2] } 
                  @svData 
                ) ;
}
elsif ($sortby eq 'chan')
{
  # The channel number  sorting key is the #0 field in each line.
  # Sort in ascending numerical order.
  @svData = sort( { (split( ' ', $a))[0] <=> (split( ' ', $b))[0] } 
                  @svData 
                ) ;
}
elsif ($sortby eq 'elev')
{
  # The elevation sorting key is the #3 field in each line.
  # Sort in DESCENDING numerical order, highest satellites first.
  @svData = sort( { (split( ' ', $b))[3] <=> (split( ' ', $a))[3] } 
                  @svData 
                ) ;
}
elsif ($sortby eq 'azm')
{
  # The azimuth sorting key is the #4 field in each line.
  # Sort in DESCENDING numerical order, highest satellites first.
  @svData = sort( { (split( ' ', $a))[4] <=> (split( ' ', $b))[4] } 
                  @svData 
                ) ;
}


########################################################################
# Create the main tracking data table
########################################################################
print '<Table width="100%" border=1 >' ;

# Create titles for the columns
print Tr( {align=>'center'},
        th( a( {href=>'satTracking.cgi?sortby=prn'} ,'PRN') ),
        th( a( {href=>'satTracking.cgi?sortby=elev'} ,'Elv') ),
        th( a( {href=>'satTracking.cgi?sortby=azm'} ,'Azm') ),
        th( 'L1<br />SNR' ),
        th( 'L2<br />SNR' ),
        th( 'IODE' ),
        th( 'URA' ),
        th( 'Health' ),
        th( a( {href=>'satTracking.cgi?sortby=chan'} ,'Chan') ),
        ) ;

foreach (@svData)
{
  my ($chan, undef, $prn, $elev, $azm, undef, $L2STAT, 
      $L1SNR, $L2SNR, $IODE, $URA, $L2CSNR) = split( ' ' ) ; 

  my $health ;
  my $l2cs ;

  my $l2 ;
  if ($L2SNR ne '0.0')
  {
    if ($L2CSNR ne '0.0')
    {
      $l2 = $L2SNR . ' / ' . 'C' . $L2CSNR ;
    }
    else
    {
      $l2 = $L2SNR ;
    }
  }
  else
  {
    if ($L2CSNR ne '0.0')
    {
      $l2 = 'C' . $L2CSNR ;
    }
    else
    {
      $l2 = '-' ;
    }
  }
  
  if ($prn <=32)
  {
    $health = $allHealth[$prn-1] ;

    if ($health == -1)
    {
      $health = 'unknown' ;
    }
    elsif ($health == 0)
    {
      $health = 'healthy' ;
    }
  }
  else
  {
    $health = ($waasHealth[$prn-120] == 1)
              ? "healthy"
              : "unhealthy" ;
  }

  if ($elev == -90)
  {
    $elev = $azm = "-" ;
  }

  # Translate WAAS/EGNOS prn's into strings.
  if ($prn >= 120 and $prn <= 138)
  {
    my $name = $waasStrings{ $prn } ;
    if ( $name !~ /^prn/ ) 
    {
      $name =~ s/^(WAAS|EGNOS|ESTB)-/$1<br \/>/ ;
      $prn = $name ;
    }
  }

  print Tr(
            td( {-align=>"center"},
                [
                  $prn,
                  $elev,
                  $azm,
                  ($L1SNR != 0) ? $L1SNR : "-",
                  $l2,
                  ($IODE != -1) ? $IODE : "-",
                  ($URA != -1) ? $URA : "-",
                  $health,
                  $chan+1,
                ]
              )
          ) ;
}

print "</Table><br />" ;


# Describe how many unused channels there are.
# Assume 12 channels, and subtract from that the number of non-idle
# elements listed in @svData.
my $unusedChannels = 12 - scalar( @svData ) ;   

if ($unusedChannels == 0)
{
  print "There are no unused channels." ;
}
elsif ($unusedChannels == 1)
{
  print 'There is one unused channel.' ;
}
else
{
  print "There are $unusedChannels unused channels." ;
}


print hr() ; # Separator bar

# Create Timestamp:
# 2001 Sep 6 - Thu of GPS Week 1130 - 17:46:52 
print timestamp() ;

# Delay a bit and then refresh this page.  We try to make this happen on
# exact 5 second boundaries.  If this time was an exact 5 second time,
# then we wait 4000 msec to allow a little bit of time for
# transmission and processing.  The range of delays is from 2000 to 
# 6000.
my $delay = 6000 - ( (time()+2) % 5 ) * 1000 ;
print <<EOF ;
<script>
  parent.loadAfterDelay( window.location.href, $delay) ;
</script>
EOF

print end_html() ;

########################################################################
# end of script satTracking.cgi
########################################################################
