#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: sleepMode.cgi,v 1.20 2004/07/29 20:52:40 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/sleepMode.cgi,v $

########################################################################
# sleepmode.cgi
#     Script for controlling the Sleepmode power saving mode.
# This script is called with no parameters and puts up the form
# with elements for controlling the Power-Saving mode.  It uses
# the values from the sysconfig/sleepmode configuration file as the
# defaults.
#
# When the user clicks OK, some basic checks are done on the choices,
# with a possible confirmation dialog box, and then the data is
# submitted to a separate cgi script (gpsadmin/setSleepMode.cgi) to
# make the changes to the config file.
#
# These controls are saved in a configuration file -
# /etc/sysconfig/sleepmode, which contains a single line with the four
# CGI fields in the order given above.  For example it might show:
#
#       yes no 24 10
# Fields are:
#   1. PowerSaving Mode Enable
#   2. Failsafe Wake Enable
#   3. Failsafe wakeup interval, in hours.
#   4. Failsafe wakeup duration, in minutes.
# 
########################################################################

use strict ;
use CGI qw/:standard/;

use Fireball ;

titleWithHelp( 'Data Logging Power Saving Mode',
                         '/powerSavingInfo.html'
                       ) ;


# Obtain the default values from the config file.
my ($Enable,$Periodic,$Hours,$Minutes) ;
if ( open( SLEEP, '/etc/sysconfig/sleepmode' ) )
{
  my $line=<SLEEP> ;
  close SLEEP ;
  chomp $line ;
  ($Enable,$Periodic,$Hours,$Minutes) = split( ' ', $line ) ;
}
else
{
  # Defaults if no file exists.
  ($Enable,$Periodic,$Hours,$Minutes) = ('no','yes',3, 5 ) ;
}


# Create the form.  Set the action to the URL of the cgi script that
# will actually make the change.  This is kept in the gpsadmin/
# directory so that we can put http security on it.
print start_form( -name=>'theForm',
                  -action=>'/perl-scripts/gpsadmin/setSleepMode.cgi',
                  -onSubmit=>'return Okay()'
                ) ;

print hr() ;
print b('Power Down Between Sessions Mode'), br(), br() ;
print 'Shut down receiver between data logging sessions?' ;

my @buttons = radio_group( -name=>'Enable', 
                        -value=>['yes','no'],
                        -nolabels=>1,
                        -default=>$Enable,
                        #-onClick=>'Update() ;'
                      ) ;

print table(
             Tr( td( $buttons[0] ),
                 td( 'Yes, conserve power between sessions.',
                     '<br />Note that communications will be impossible
                     while power is off.'
                   )
               ),
             Tr( td( $buttons[1] ),
                 td( 'No.  Keep power on between sessions.' )
               ),
           ) ;

print hr() ;
print b('Failsafe Wakeup Alarm'), br(), br() ;
print 'Wake up periodically?', br() ;

@buttons = radio_group( -name=>'Periodic', 
                        -value=>['yes','no'],
                        -nolabels=>1,
                        -default=>$Periodic,
                        #-onClick=>'Update() ;'
                      ) ;

print table(
             Tr( td( $buttons[0] ),
                 td( 'Yes, for ',
                     scrolling_list( -name=>'Minutes',
                                     -values=>['5','10','15','30','60'],
                                     -size=>1,
                                     -default=>$Minutes,
                                   ),
                     'minutes, every',
                     scrolling_list( -name=>'Hours',
                                     -values=>['1','2','3','4',
                                               '6','8','12','24'],
                                     -size=>1,
                                     -default=>$Hours,
                                   ),
                     'hours, starting at 00:00 UTC.',
                   )
               ),
             Tr( td( $buttons[1] ),
                 td( 'No.' )
               ),
           ) ;

print <<EOF ;
<b><font color="red">WARNING:</font></b> 
Turning off the Failsafe Wakeup Alarm could result in loss of
communications with a remote receiver.
<p>
Periodic wakeups are always aligned to 00:00/24:00 hours UTC.
<br />For example, setting the period to 10 minutes every 8 hours would 
force the system to be awake three times every day, from 00:00 to 00:10,
and from 08:00 to 08:10, and from 16:00 to 16:10 (all times in UTC).
EOF


print <<EOF;
<Script>

/**********************************************************************/
function Okay()
/**********************************************************************/
/* Called when the OK button is pressed.
 * Submits the contents of the form to the web server.
 * If the failsafe wakeup alarm is disabled and Power Saving mode is
 * Enabled, posts a warning to the user asking for confirmation.
 */
{
  f = document.theForm ;
  if ( f.Periodic[0].checked // Failsafe is on
    || f.Enable[1].checked   // Power Saving is off
    || confirm( 'The Failsafe Wakeup Alarm is being disabled.\\n'
       + 'This can result in loss of communications to remote systems!\\n'
       + 'Are you sure you want to do this?') 
     )
  {
    parent.submitForm( f, '' ) ;
  }
  return false ;
} /* end of Okay() */


</Script>
EOF


print hr() ;

print button( -name=>'OK', 
              -value=>'OK',
              -onClick=>'Okay()'
            ) ;

print button( -name=>'Cancel', 
              -value=>'Cancel',
              -onClick=>'parent.loadDataFrame( "sessions.cgi" )'
            ) ;

print end_form() ;


print end_html() ;

########################################################################
# end of script sleepMode.CGI
########################################################################
