#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: accounts.cgi,v 1.17 2004/08/23 17:13:16 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/sysadmin/accounts.cgi,v $

# accounts.cgi
# Script for displaying the current account data in Fireball.
# Reads the data from multiple sources.
#   /etc/sysconfig/htpasswd : list of accounts and their encrypted 
#                             passwords.
#                             Format: one user per line,
#                               username:passwd
#
#   /etc/sysconfig/htgroups : list of capabilities groups and which 
#                             users have those capabilities.
#                             Format: one group per line.
#                               groupname: user user ...

use strict ;
use CGI qw/:standard/;
use Fireball ;

titleWithHelp( 'Security - Accounts',
               '/securityAccounts.html'
             ) ;


# Read the current GUI account state into the local hash %accounts.
# Keys are the account names.
# Each key defines a sub-hash containing 
#   'password'   : encrypted password
#   'sysadmin'   : boolean indicating System Controls capabilities
#   'gpsadmin'   : boolean indicating GPS Controls capabilities
#   'fileaccess' : boolean indicating File Access capabilities
my %accounts ;
getGuiAccounts( \%accounts ) ;


# Make all links blue with no underlines, even if visited, etc.
# This is a document-level style specification, affecting all
# links in this frame.
print <<EOF ;
<style type="text/css">
 <!--
       a {color:blue ; text-decoration: none }
 -->
</style>
EOF


print 'The following accounts exist:' ;

# Start a table.  Fill the whole frame.  Put an outline grid on the
# whole thing.
print '<Table width="100%" border> ' ;

# Put in the column titles.  Use embedded <BR> to get a linefeed within
# the titles.
print Tr( th( ['Account<BR>Name',
              'System<BR>Controls',
              'GPS<BR>Controls',
              'File<BR>Access',
              ]
            )
        ) ;

# Create a row for each account, showing the information.
foreach $a (sort( keys( %accounts ) ) ) 
{
  # Clicking on the account name leads to the screen to edit that
  # account.
  my $link= a( {href =>
                 "/perl-scripts/sysadmin/editAccount.cgi?restart=$a"
               },
               $a, 
             ) ;

  print Tr( td( {align=>'center', width=>'25%'},
                [
                  $link,
                  $accounts{$a}{'sysadmin'} ? 'Yes' : 'No' ,
                  $accounts{$a}{'gpsadmin'} ? 'Yes' : 'No' ,
                  $accounts{$a}{'fileaccess'} ? 'Yes' : 'No' ,
                ]
              )
          ) ;
}

print '</Table>';

print br();


# Put two link-buttons on the bottom for creating a new account and
# for editing existing accounts.  These both lead to the same page,
# only with different parameters.  Create gives the "New Account"
# selection (which is editAccount.cgi's default.  For Edit account, we
# always show the sysadmin account first, and then allow the user to
# select a different one with a popup.

print table( {border=>2,align=>'center',width=>'50%'},
            Tr( td( {align=>'center', width=>'50%'},
                    a( {href=>'/perl-scripts/sysadmin/editAccount.cgi'},
                       'Create a NEW<BR>Account' 
                     )
                  ),
                td( {align=>'center', width=>'50%'},
                    a( {href=>
               '/perl-scripts/sysadmin/editAccount.cgi?restart=sysadmin'
                       } , 
                       'Edit<BR>Accounts' 
                     )
                  ),
              )
           ) ;

print end_html() ;

########################################################################
# end of file accounts.cgi
########################################################################
