#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: commandHandler.cgi,v 1.66 2006/03/23 19:55:23 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/sysadmin/commandHandler.cgi,v $

########################################################################
# commandHandler.cgi
#
# Programmatic interface script that parses and dispatches requests
# from the programmatic interface URLs.
# 
########################################################################

use strict ;
no strict 'refs' ;
use CGI qw/:standard/;
use IO::Socket;
use Time::Local ;

use lib '..' ;
use Fireball ;
use Commands ;

# All commands begin with a Verb from this list.
# Those are followed by and object and possibly a list of parameters.
my @verbs = qw(
Show
Set
Enable
Disable
Verify
Install
Delete
Reset
Upload
Create
Download
) ;

# This is a list of all valid Verb-Object pairs.
# Each must correspond to a identically named function.
my @commands = qw(
ShowPps
SetPps
ShowAutodelete
SetAutodelete
ShowClockSteering
SetClockSteering
ShowMultipathReject
SetMultipathReject
ShowElevationMask
SetElevationMask
ShowPdopMask
SetPdopMask
ShowSerialNumber
ShowSystemName
SetSystemName
ShowTemperature
ShowVoltage
ShowFirmwareVersion
ShowFirmwareWarranty
ShowFirmwareFiles
ShowFirmwareFile
VerifyFirmwareFile
InstallFirmwareFile
UploadFirmwareFile
DeleteFirmwareFile
ShowSessions
ShowSession
SetSession
EnableSession
DisableSession
DeleteSession
ShowPowerSaveMode
SetPowerSaveMode
ShowFailsafeWakeup
SetFailsafeWakeup
ShowGpsHealth
ShowGpsSatControls
SetGpsSatControls
ShowTrackingStatus
ShowWaasSatControls
SetWaasSatControls
ShowWaasTracking
SetWaasTracking
ShowReferenceFrequency
SetReferenceFrequency
ShowLoggedFiles
ShowLoggedFile
DeleteLoggedFiles
DeleteLoggedFile
CreateConfigFile
DeleteConfigFile
UploadConfigFile
ShowConfigFile
ShowConfigFiles
InstallConfigFile
ResetSystem
ResetGpsData
ShowFtpSetup
SetFtpSetup
ShowGuiSecurity
SetGuiSecurity
ShowGuiAccount
ShowGuiAccounts
SetGuiAccount
DeleteGuiAccount
ShowAntennaTypes
ShowAntenna
SetAntenna
ShowEthernet
SetEthernet
ShowIOports
ShowIOport
SetIOport
ShowClientAuthKey
SetClientAuthKey
DeleteIOport
ShowUtcTime
ShowGpsTime
ShowLoggedFileStats
ShowLoggedFilePools
SetFirmwareWarranty
ShowPosition
ShowLoggingStatus
DownloadLoggedFile
ShowGpsEphemeris
ShowGpsAlmanac
ShowGpsUtcData
ShowGpsIonoData
ShowRefStation
SetRefStation
ShowNtpClient
SetNtpClient
ShowIoStatus
ShowL2Tracking
SetL2Tracking
ShowL2C
SetL2C
ShowHttpPorts
SetHttpPorts
ShowIpFiltering
SetIpFiltering
ShowPortAdvertising
SetPortAdvertising
ShowPppDialout
SetPppDialout
ShowShutdownVoltage
SetShutdownVoltage
) ;


########################################################################
sub ShowShutdownVoltage
########################################################################
# Displays the Vsleep, firmware shutdown threshold voltage.
# Response is of the form:
#       ShutdownVoltage volts=10.680
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;
  my $vsleep = getVsleep() ;

  printf "ShutdownVoltage volts=%.3f\n", $vsleep ;

} # end of ShowShutdownVoltage



########################################################################
sub SetShutdownVoltage
########################################################################
# Accept a new value for the ShutdownVoltage threshold, vsleep.
# Parameter should be of the form 'volts=10.68'.  For the voltage
# we'll take   dd   or dd.  or  dd.d etc.  Won't take .dd
# Responds by setting the vsleep voltage in the configuration file
# and then showing "OK:" followed by the ShowShutdownVoltage response.
{
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  # Parameter should be 
  if ( $_[0] !~ m/volts=(\d+(.\d*)?)/ )
  { 
    print "ERROR: invalid parameter $_[0]\n" ;
    return ;
  }
  
  my $vsleep = $1 ;
  if ( $vsleep < 10.68
    || $vsleep > 11.4
     )
  {
    print "ERROR: voltage $vsleep outside of range 10.68 to 11.4\n" ;
    return ;
  }

  saveVsleep( $vsleep ) ;

  print "OK: " ;
  ShowShutdownVoltage() ;

} # end of SetShutdownVoltage




########################################################################
sub ShowDummy
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;
  print "dummy\n" ;

} # end of ShowDummy



########################################################################
sub SetDummy
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  print "OK: " ;
  dummy() ;

} # end of SetDummy



########################################################################
# Done with subroutines.  Begin real operation of the command handler.
########################################################################


# Determine the desired command.  
# All command URLs look like
#   /prog/verb?object&params
# Even though we redirect ALL /prog/ requests to this commandHandler,
# the apache CGI processer still reports the original URL information
# as if we were calling a script located at '/prog/verb'.  It splits 
# the URL into the script name, and the query portion, and passes these
# in via the environment variables SCRIPT_NAME and QUERY_STRING.

my $command = $ENV{'SCRIPT_NAME'} ;
my $query=$ENV{'QUERY_STRING'} ;

$command .= " $query" if ($query) ;

# Drop the /prog/ leader from the URL
$command =~ s|/prog/|| ;

# Replace '&' with space.  These will be used to break up the
# command into sections.
$command =~ s|&| |g ;

# Replace any %xx hexadecimal coded values with their characters.
while ($command =~ /(%[0-9a-f]{2})/i)
{
  my $c = chr( hex(substr($1, 1 ) ) ) ;
  $command =~ s//$c/ ;
}

# Extract the verb and object which are the first words in the command.
my $verb;
my $object ;
my $parms ;
($verb, $object, $parms ) = 
                              ( $command =~ m/^(\S+)\s+(\S+)\s*(.*)$/) ;

#plainTextHeader() ;
#print "verb=$verb object=$object  parms=$parms\n" ;

# The splitting done above will have a fit on totally invalid strings,
# leaving the $verb unset.  Flag such results as just plain bad.
if (! $verb)
{
  plainTextHeader() ;
  print "ERROR: Invalid command format '$command'\n" ;
  goto EXIT ;
}

# The rest of the fields are a list of parameters.
# Sadly, these can contain quoted substrings, and this requires us to
# play around a bit.  First we have to scan the entire string processing
# those regions that are enclosed in '' separately from those that
# aren't, and dropping the quotes.  Any space characters inside the
# quoted regions are replaced with backspaces.  This allows us to 
# later break up the entire string into parameters, using space
# separators.
# Note that single quotes can be escaped with backslashes, \` to avoid
# their special meaning.  Before we do the work below, we replace 
# \' with a control character (\f=formfeed) and then reverse the
# substitution afterward.
#
# We'll be transfering the characters from $parms to $parameters,
# one-by-one.
my $parameters = '' ;

# If we are inside quotes, this will be set.
my $inQuotes = 0 ;

# Replace backslashed single quotes with formfeeds.
$parms =~ s/\\'/\f/g ;

# Replace blanks inside quoted strings with backspace.
for (my $i = 0 ; $i < length( $parms ) ; ++$i )
{
  my $c = substr( $parms, $i, 1 ) ;
  if (! $inQuotes)
  {
    # Detect start of quote string.
    $inQuotes = ($c eq "'" ) ;

    # Add this character to the output string, unless it's a quote.
    $parameters .= $c if ( $c ne "'" ) ;
  }
  else
  {
    # Transform spaces inside quotes to backspaces.
    $c = "\b" if ($c eq ' ') ;

    # Terminate quote string if trailing quote found.
    $inQuotes = ($c ne "'" ) ;

    # Add this character to the output string, unless it's a quote.
    $parameters .= $c if ($c ne "'") ;
  }
}

# Restore any backslashed single quotes from formfeeds back to single
# quotes.
$parameters =~ s/\f/'/g ;

# Split the string on whitespace boundaries.
my (@parameters) = ( $parameters =~ m/(\S+)/g ) ;

# Restore backspaces to spaces in all parameters.
for my $p (@parameters)
{
  $p =~ s/[\b]/ /g ;
}

#plainTextHeader() ;
#print "Verb = $verb\n" ;
#print "Object = $object\n" ;
#print "$parameters\n" ;
#print "Parameters = ", join( ',', @parameters), "\n" ;

# Determine the Verb in the command, which is the first word.
# Find its canonical form in the table of verbs.
# If not found, then give an appropriate error message.
my ($foundVerb) = grep m/^$verb$/i, @verbs ;
if (! $foundVerb )
{
  plainTextHeader() ;
  print "ERROR: Invalid verb '$verb'\n" ;
  goto EXIT ;
}

# Determine the VerbObject setting and search for it in the
# list of valid commands.

my ($foundCommand) = grep m/^$verb$object$/i, @commands ;
if (! $foundCommand )
{
  plainTextHeader() ;
  print "ERROR: Invalid verb/object combination '$verb $object'\n" ;
  goto EXIT ;
}

if (exists &$foundCommand)
{
  # All commands except 'download' commands return plaintext responses.
  # To simplify those commands, we set that http header type here,
  # rather than duplicating this code in every handler.
  # The Download programs send binary data or plain-text error messages,
  # so they have to deal with the header on their own.
  if (lc($verb) ne 'download')
  {
    plainTextHeader() ;
  }

  &$foundCommand( @parameters ) ;
}
else
{
  plainTextHeader() ;
  print "ERROR: No subroutine $foundCommand.  Perhaps not yet implemented.\n" ;
}

EXIT:
########################################################################
# end of script commandHandler.cgi
########################################################################
