#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: configApply.cgi,v 1.11 2006/05/23 20:49:34 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/sysadmin/configApply.cgi,v $

########################################################################
# configApply.cgi
#
# Script for applying (installing) Fireball Configuration Files
# 
# Via the Web-GUI interface we can create, delete, download, upload and 
# apply "configuration files".  These are collections of Fireball
# control settings.  The files reside in /etc/sysconfig/configFiles.
#
# This script deals with what is needed to apply the configuration
# settings in an existing configfile.
#
# CGI parameters.
#
# apply=fname
#       A request to apply the configfile named fname.
#
########################################################################

use strict ;
use CGI qw/:standard/;

use lib '..' ;
use Fireball ;

########################################################################
# Begin formatting the return document.
########################################################################
startPage( 'Config Apply' ) ;


#if (param())
#{
#  print "Parameters Received:<br />" ;
#  foreach my $p (param()) 
#  {
#    print "$p=",param($p),br() ;
#  }
#}

# After the title block, we may put a result indicator showing what
# happened as a result of a previous operations.  To keep things on the
# page from shifting around, we always send out a line, even when there
# isn't a result.  This provides default string which will be emitted
# later.  Overwrite the string if a real result is needed.
my $result = "&nbsp;" ;

my $filename = param('apply') ;
my $pathname = "/etc/sysconfig/configFiles/$filename" ;

if ( -f $pathname )
{
  chdir '/etc/sysconfig/configFiles' ;

  # Verify that the chosen configfile is okay to install.
  my $status = system( "/usr/local/fireball/clone -t $filename " ) >> 8;

  if ($status != 0)
  {
    $result = "Configuration File $filename is corrupted!" ;
    goto ERROR ;
  }

  # At this point, the selected file has been verified and we are 
  # allowed to apply it.  We need a method of passing the file to
  # the ramdisk installer.  The method must survive a reboot into
  # a different operating system and file system layout.  So we put 
  # a symbolic link into /etc/sysconfig/configFiles and point it at the
  # desired file.
  my $linkName = 'InstallThisConfig' ;

  # Get rid of this link if it already exists.
  if ( -r $linkName ) 
  {
    system( "sudo rm $linkName" ) ;
  }

  # Create the desired hard link.
  system( "/usr/bin/sudo /bin/ln -s $filename $linkName" ) ;


  print h2("Configuration file \"$filename\" is now being applied.") ;
  print b( 'This requires a system restart.  It will take about four ',
           'minutes to complete the process and restore normal ',
           'operations.'
         ) ;

  print <<EOF ;
  <script>
    setTimeout( \'parent.document.location = "../NetRS.cgi"\',
                240000
              ) ;
  </script>
EOF
  print end_html() ;

  # Trigger the actual reconfiguration 
  #   - set the bootstatus flags
  #     We set flag zero to 252 as a signal to the boot system to
  #     load in the ramdisk system, and perform a clone operation.
  #   - do a sleep and reboot
  #     The sleep gives the GUI displays time to get out the chute
  #     before the reboot.  Reboot is implemented by sending a SIGUSR2
  #     signal to the frontpanel task.

  system( "/usr/bin/sudo /sbin/boot_status 0 252 f" ) ;

  # Obtain the process id of the frontpanel_task.  Launch a background
  # process that sends a signal to frontpanel_task.
  my $spid = qx( cat /var/run/frontpanel_task.pid ) ;
  system( "(/usr/bin/sudo /bin/kill -SIGUSR2 $spid)&" ) ;

  goto EXIT ;

}
else
{
  $result = "Configuration File $filename does not exist!" ;
}

ERROR:

print h1( $result ) ;

my $returnURL = '/perl-scripts/configFiles.cgi'
              . '?message='.CGI::escape( $result )
              ;

print <<EOF ;
<Script>
  parent.loadAfterDelay( "$returnURL", 1000) ;
</Script>
EOF


print end_html() ;

EXIT:
########################################################################
# end of file configApply.cgi
########################################################################
