#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: configUpload.cgi,v 1.6 2003/12/19 01:46:22 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/sysadmin/configUpload.cgi,v $

########################################################################
# configUpload.cgi
#
# Script for uploading Fireball Configuration Files
# 
# Via the Web-GUI interface we can create, delete, download, upload and 
# apply "configuration files".  These are collections of Fireball
# control settings.  The files reside in /etc/sysconfig/configFiles.
#
# This script handles accepting an uploaded configuration file.
# It is passed a parameter that is the suggested name of the file,
# and can find the already uploaded file.
# 
# The CGI parameter is:
# configFile=fname
#       Indicates that an upload form was posted.  The CGI library has
#       already handled uploading the file to a temporary location, and
#       we have to move it to its final destination.
########################################################################

use strict ;
use CGI qw/:standard/;


use lib '..' ;
use Fireball ;

########################################################################
# Begin formatting the return document.
########################################################################
startPage( 'Config Upload' ) ;


my $result = 'configUpload Error' ;

if (my $fname=param('configFile'))
{
  # If a file was uploaded, the parameter configFile will be set to
  # its name or path from the sourcing system.

  #print "Uploading $fname<br />" ;

  my $fname = upload( 'configFile' ) ;

  # The file has already been uploaded and is in a temporary location,
  # whose path is accessed here:
  my $tmpfilename = tmpFileName($fname) ;

  # If the file successfully uploaded, it will have a non-zero size.
  my $size=(-s $tmpfilename) ;
  #print $tmpfilename, " size=", (-s $tmpfilename), "<br />";

  # Strip any path information off of the suggested filename.
  my $newFname = $fname ;
  $newFname =~ s/^.*\\// ; # Remove path that might use DOS backslashes.
  $newFname =~ s/^.*\/// ; # Remove path that might use Unix slashes.

  # Substitute underscore for any spaces in the name.
  $newFname =~ s/ /_/g   ;
  
  if ($size)
  { 
    # The upload succeeded and the file is in $tmpfilename
    # There is chance that somebody fooled around with the line
    # termination characters so we have to filter the file to clean
    # it up.

    open TMP, $tmpfilename ;
    open FIXED, ">/tmp/$newFname" ;
    while (<TMP>)
    {
      # Filter each line.  Have to handle any kind of line-terminations
      # DOS is CRLF. Convert the pairs to a single LF
      s/\015\012/\012/ ;

      # An old Macintosh might have just CRs and no LFs.  
      # Convert them to LF.
      s/\015/\012/ ;

      print FIXED ;
    }
    close( FIXED );
    close( TMP ) ;

    # Move the file to the final resting place.
    system( "sudo cp /tmp/$newFname /etc/sysconfig/configFiles/$newFname" ) ;
    system( "sudo chmod 444 /etc/sysconfig/configFiles/$newFname" ) ;

    $result = 
            "Configuration File \"$fname\" was successfully uploaded." ;
  }
  else
  {
    print "There was a problem uploading file '$fname'<br />" ;
  }
  close( $fname ) ;
} # End of file upload processing.


print h3( $result ) ;

my $returnURL = '/perl-scripts/configFiles.cgi'
              . '?message='.CGI::escape( $result )
              ;

print <<EOF ;
<Script>
  parent.loadAfterDelay( "$returnURL", 1000) ;
</Script>
EOF


print end_html() ;

########################################################################
# end of file configUpload.cgi
########################################################################
