#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: ethernetSetup.cgi,v 1.16 2004/09/23 21:30:10 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/sysadmin/ethernetSetup.cgi,v $

########################################################################
# Script to enable setting Ethernet setup parameters.
# Called in several forms, using CGI parameters:
#
#   ethernetSetup.cgi
#       No parameters: Initialize the form with current data
#
#   ethernetSetup.cgi?Mode=dhcp
#   ethernetSetup.cgi?Mode=static
#       These calls indicate that a change has been made to the
#       ethernet mode, requiring an updating of the displayed form.
#
#   ethernetSetup.cgi?(full list of parameters from a form)
#       A normal submit of the information.
#       
#   Disabling the ethernet is currently not functional.
########################################################################


use strict ;
use CGI qw/:standard/;

use lib '..' ;
use Fireball ;

titleWithHelp( 'Ethernet Configuration',
               '/internetEthernetInfo.html'
             ) ;

# Read configuration data into a hash for local reference.
my %ethernetSetup ;
getEthernetSetup( \%ethernetSetup ) ;

my $EthMode = $ethernetSetup{ 'ipmode' } ;

my $macAddress = $ethernetSetup{ 'mac' } ;

my $ipAddress = $ethernetSetup{ 'ip' } ;
my $netMask = $ethernetSetup{ 'netmask' } ;
my $gateway = $ethernetSetup{ 'gateway' } ;
my $mtu     = $ethernetSetup{ 'mtu'     } ;


# Determine the Name server setup.
# This is controlled in the file /etc/resolv.conf.  
#
# The NameServers are defined on lines starting with 'nameserver '.  The
# values will always be numeric IPs.  There can be several.  We allow up
# to three.  The grep below both detects lines starting with 'nameserver
# ' AND removes those strings from the beginnings of the lines.
#
# The domain is defined with a line starting with "domain ".
# The search path is defined with 

# Read in the resolv.conf file, one line per element of an array.  
# If the file doesn't exist, we'll get an empty array.
my @results = qx| cat /etc/resolv.conf | ;
#print join( '<BR>', @results ), '<BR>' ;

my @DnsServers = split( ',', $ethernetSetup{ 'nameServers' } ) ;

my $domain = $ethernetSetup{ 'dnsDomain' } ;

my $searchPath = $ethernetSetup{ 'dnsSearch' } ;


# If this is a CGI request then some parameters will be passed into the
# script.  We'll need to parse those and set the state(s) as requested.
my $mode ;
if (!param())
{
  # No cgi parameters were appended.
  $mode=$EthMode ;
}
else
{
  #print( "Parameters Received<BR>" ) ;
  #foreach my $p (param())
  #{
  #    print  "$p=", param($p), " " ;
  #}
  my $NewMode = param( 'Mode' ) ;
  my $network ;
  my $broadcast ;
  my $NewIP ;
  my $NewNetmask ;
  my $NewGateway ;
  my $NewMTU ;

  if (param("submissionType") eq "Okay")
  {
    # This is a submission of new control state.
    # Update the configuration with the new data.
    # We post a "Changes accepted" message and switch back to the
    # Ethernet configuration screen

    # If the new mode is static then we need to determine the
    # network base address and the broadcast address for later
    # inclusion in the interfaces file.
    $ethernetSetup{'ipmode'} = $NewMode ;

    if ($NewMode eq 'static')
    {

      $NewIP = param( 'IP' ) ;
      # Strip off any leading zeroes from the octets.  The networking
      # scripts might interpret these as Octal values.
      $NewIP = '.'.$NewIP ;
      $NewIP =~ s/\.0+(\d+)/.$1/g ;
      $NewIP =~ s/^\.// ;
      $ethernetSetup{'ip'} = $NewIP ;


      $NewNetmask = param( 'Netmask' ) ;
      # Strip off any leading zeroes from the octets.  The networking
      # scripts might interpret these as Octal values.
      $NewNetmask = '.'.$NewNetmask ;
      $NewNetmask =~ s/\.0+(\d+)/.$1/g ;
      $NewNetmask =~ s/^\.// ;
      $ethernetSetup{'netmask'} = $NewNetmask ;

      $NewGateway = param( 'Gateway' ) ;
      # Strip off any leading zeroes from the octets.  The networking
      # scripts might interpret these as Octal values.
      $NewGateway = '.'.$NewGateway ;
      $NewGateway =~ s/\.0+(\d+)/.$1/g ;
      $NewGateway =~ s/^\.// ;
      $ethernetSetup{'gateway'} = $NewGateway ;

    }

    $ethernetSetup{ 'mtu' } = param( 'mtu' ) ;

    $ethernetSetup{ 'dnsDomain' } = param( 'Domain' ) ;

    my $path = param( 'SearchPath' ) ;
    $path =~ s/ +/,/ ;
    $ethernetSetup{ 'dnsSearch' } = $path ;

    my @servers = () ;
    if (my $ns = param( 'Nameserver1' ))
    {
      # Strip leading zeroes on octets
      $ns = '.'.$ns ;
      $ns =~ s/\.0+(\d+)/.$1/g ;
      $ns =~ s/^\.// ;
      push( @servers, $ns ) ;
    }

    if (my $ns = param( 'Nameserver2' ))
    {
      # Strip leading zeroes on octets
      $ns = '.'.$ns ;
      $ns =~ s/\.0+(\d+)/.$1/g ;
      $ns =~ s/^\.// ;
      push( @servers, $ns ) ;
    }

    if (my $ns = param( 'Nameserver3' ))
    {
      # Strip leading zeroes on octets
      $ns = '.'.$ns ;
      $ns =~ s/\.0+(\d+)/.$1/g ;
      $ns =~ s/^\.// ;
      push( @servers, $ns ) ;
    }
    $ethernetSetup{ 'nameServers' } = join( ',', @servers ) ;


    # All of the configuration is now in the hash %ethernetSetup.
    # Push that setup into the config files and restart networking.
    # Note that the restart occurs in background five seconds after
    # this call, which gives us time to put up the response page,
    # and exit.
    updateEthernetSetup( \%ethernetSetup ) ;


    # After a timeout, go back to main Ethernet screen.
    print <<EOF ;

<h2>Ethernet Configuration Accepted.</h2>
The Ethernet port is now being reconfigured.  This may take 
about 15 seconds.
<BR><h3>Warning</h3>
The IP address of the target system may have been changed.  This web 
interface may require redirection to a new URL.

<Script>
  alert( "Warning\\nA change to the Ethernet configuration may have "+
         "disabled the web interface." ) ;

  parent.loadAfterDelay( '/perl-scripts/internetEthernet.cgi', 15000) ;
</Script>

EOF

    # Done with page.  
    goto EXIT ;

  }
}



print <<EOF ;
<Script>


/**********************************************************************/
function modeChange()
/**********************************************************************/
/* One of the mode-selection radio buttons has been clicked.
 * This may require us to redraw the screen, but only if the mode has
 * been changed in specific ways.
 * If mode is changing away from or to static, then an update is
 * necessary, because the static IP portions of the form must be
 * erased or drawn.  Otherwise, we don't have to change anything.
 * This is detected by the javascript variable staticStuffDisplayed,
 * which is set true only if the static IP data was sent.  If the
 * state of the Static IP checkbox no longer matches the displayed
 * form, we reload the data.
 * When we submit the form data, we set the value of the hidden field
 * "submissionType" to "modeChange" so that the CGI script knows that
 * we aren't submitting changes, but rather are updating the form.
 */
{
  f=document.theForm ;
  if ( f.Mode[1].checked != (staticStuffDisplayed) )
  { 
    f.submissionType.value = "modeChange" ;
    parent.submitForm( f, "/perl-scripts/sysadmin/ethernetSetup.cgi" ) ;
  }
} /* end of modeChange() */


/**********************************************************************/
function Okay()
/**********************************************************************/
/* Invoked when the OK button is pressed.
 * This just submits the entire form with all parameters.
 * The hidden field "submisionType" is given a value of Okay so
 * that the processing CGI script can tell that we want to accept the
 * data for real, instead of just updating the form.
 */
{
  df=document.theForm ;

  if (df.IP != null)
  {
    // Remove leading & trailing spaces from the string.
    value = df.IP.value = parent.strip(df.IP.value) ;

    if ( parent.notValidIP( value ) )
    { 
      alert( "Invalid IP Address : " + value ) ;
      return ;
    }
  }

  if (df.Netmask != null)
  {
    // Remove leading & trailing spaces from the string.
    value = df.Netmask.value = parent.strip(df.Netmask.value) ;

    if ( parent.notValidIP( value ) )
    { 
      alert( "Invalid Netmask : " + value ) ;
      return ;
    }
  }

  if (df.Gateway != null)
  {
    // Remove leading & trailing spaces from the string.
    value = df.Gateway.value = parent.strip(df.Gateway.value) ;

    if ( (value != "") && parent.notValidIP( value ) )
    { 
      alert( "Invalid Gateway : " + value ) ;
      return ;
    }
  }

  if (df.Nameserver1 != null)
  {
    // Remove leading & trailing spaces from the string.
    value = df.Nameserver1.value = parent.strip(df.Nameserver1.value) ;

    if ( (value != "") && parent.notValidIP( value ) )
    { 
      alert( "Invalid Nameserver : " + value ) ;
      return ;
    }
  }

  if (df.Nameserver2 != null)
  {
    // Remove leading & trailing spaces from the string.
    value = df.Nameserver2.value = parent.strip(df.Nameserver2.value) ;

    if ( (value != "") && parent.notValidIP( value ) )
    { 
      alert( "Invalid Nameserver2 : " + value ) ;
      return ;
    }
  }

  if (df.Nameserver3 != null)
  {
    // Remove leading & trailing spaces from the string.
    value = df.Nameserver3.value = parent.strip(df.Nameserver3.value) ;

    if ( (value != "") && parent.notValidIP( value ) )
    { 
      alert( "Invalid Nameserver3 : " + value ) ;
      return ;
    }
  }

  if (df.mtu != null)
  {
    // Remove leading & trailing spaces from the string.
    value = df.mtu.value = parent.strip(df.mtu.value) ;

    if ( parent.isNotAnInteger( value, "MTU" ) )
    { 
      return ;
    }

    if ( (value < 600) || (value > 1500) )
    {
      alert( "MTU out of range 600 to 1500" ) ;
      return ;
    }
  }

  df.submissionType.value = "Okay" ;
  parent.submitForm( df, "/perl-scripts/sysadmin/ethernetSetup.cgi" ) ;

} /* end of Okay() */
</script>
EOF



print startform( -name=>"theForm",
                 -onSubmit=>'return Okay()'
               ) ;

# We will need three radio buttons to select the mode.
# These re generated into an array here (as html strings), and then 
# popped into the tables below as needed.
my @modeButtons = radio_group( -name=>'Mode', 
                            -value=>['dhcp','static','disable'],
                            -nolabels=>1,
                            -default=>$EthMode,
                            -onClick=>"modeChange() ;"
                          ) ;

print p(), $modeButtons[0], " DHCP" ;
print p(), $modeButtons[1], " Static IP" ;
#print p(), $modeButtons[2], " Disable" ;

if ( param("Mode") 
     ? (param("Mode") eq "static")
     : ($EthMode eq "static")
   ) 
{
  print table(

  Tr( td( {align=>"right"}, "IP Address" ), 
            td( textfield( 'IP', $ipAddress, 15, 15 ) ) ) ,

  Tr( td( {align=>"right"}, "MTU" ), 
      td( textfield( 'mtu', $mtu, 15, 4 ),
          '<font size="-2">600 to 1500</font>' 
        )
    ),

  Tr( td( {align=>"right"}, "Netmask" ), 
            td( textfield( 'Netmask', $netMask, 15, 15 ) ) ) ,
  
  Tr( td( {align=>"right"}, "Gateway" ), 
            td( textfield( 'Gateway', $gateway, 15, 15 ) ) ) ,

  Tr( td( {align=>"right"}, "Name Server" ), 
            td( textfield( 'Nameserver1', $DnsServers[0], 15, 15 ))),

  Tr( td( {align=>"right"}, "Name Server2" ), 
            td( textfield( 'Nameserver2', $DnsServers[1], 15, 15 ))),

  Tr( td( {align=>"right"}, "Name Server3" ), 
            td( textfield( 'Nameserver3', $DnsServers[2], 15, 15 ))),

  Tr( td( {align=>"right"}, "DNS Domain Name" ), 
            td( textfield( 'Domain', $domain, 30, 60 ))),

  Tr( td( {align=>"right"}, "DNS Search Path" ), 
            td( textfield( 'SearchPath', $searchPath, 30, 60 ))),
  ) ;

  print "<Script>staticStuffDisplayed = true ; </Script>"
}
else
{
  print "<Script>staticStuffDisplayed = false ; </Script>",
        br(),
        br(),
        table(
               Tr( td( {align=>"right"}, "MTU" ), 
                   td( textfield( 'mtu', $mtu, 15, 4 ),
                       '<font size="-2">600 to 1500</font>' 
                     )
                 ),
             ) ;
}

print p(), b("MAC Address: "), $macAddress, br() ;

print p(),button( -name=>"OK", 
              -value=>"OK",
              -onClick=>"Okay() ;",
            ) ;
print "&nbsp;" ;
print button( -name=>"Cancel", 
              -value=>"Cancel",
              -onClick=>
          'parent.loadDataFrame( "/perl-scripts/internetEthernet.cgi" )'
            ) ;

# This is a "hidden" field, used during submissions of this
# form to identify various types of actions.  In particular,
# its value can be set to "Okay" and "modeChange" depending on
# why we are submitting the form.
print hidden( -name=>"submissionType",
              -value=>""
            ) ;
print endform() ;

EXIT:
print end_html() ;

########################################################################
# End of file ethernetSetup.cgi
########################################################################
