#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: firmwareInstall.cgi,v 1.16 2003/12/19 01:46:22 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/sysadmin/firmwareInstall.cgi,v $

########################################################################
# firmwareInstall.cgi:
#
# Script to implement installing firmware files.  
# Called with a parameter to specify the file.  
#
#   firmwareInstall.cgi?install=name
#
# This script re-invokes itself in several stages as progress is made to
# check the warranty date, verify the package, and begin installation.
# This is controlled via a parameter "phase".
#   phase is undefined - startup.
#   phase is 1 - 
#
########################################################################

use strict ;
use CGI qw/:standard/;

use lib '..' ;
use Fireball ;

titleWithHelp( 'Firmware - Install',
               '' # No help button
             ) ;

my $errorMessage ;
my $testResults ;

# Process the install request.
my $basename = param('install') ;
my $fname='/var/gps/incoming/'.$basename ;


if ( param('install') )
{
  print b( "Installing file $basename" ) ;
}
else
{
  $errorMessage = 'No Firmware file to install.' ;
  goto ERROR ;
}


# Check if this file is really an RPM file.  It's first four
# characters should contain the magic number sequence
# 0xed 0xab 0xee 0xdb
if ( ! IsRpmFile( $fname ) )
{
  $errorMessage = 'This does not appear to be an RPM file '
                  .'so it cannot be installed.' ;
  goto ERROR ;
}

print p(),b( 'File is an RPM file' ) ;


print p(),b( 'Checking Warranty' ), br() ;


if (!param('phase'))
{
  print <<EOF ;
  <script>
    URL = '/perl-scripts/sysadmin/firmwareInstall.cgi?install='
        + escape( "$basename" ) 
        + '&phase=1' ;
    parent.loadAfterDelay( URL, 1000 ) ;
  </script>
EOF
goto EXIT ;
}


# Determine if this firmware file is installable under the current
# warranty date.  While we are at it, obtain the description field
# from the RPM file, and return it via a reference to this function.
my $description ;
if ( underWarranty( $fname, 1, \$description ) )
{
  print 'This firmware file is installable under the terms of your '
       .'warranty.<p>' ;
}
else
{
  $errorMessage = 'This file is dated after your Warranty Expiration '
                 .'date and cannot be installed.' ;
  goto ERROR ;
}

print '<p><b>Verifying the RPM file</b><br />' ;

if (param('phase') eq '1' )
{

  print <<EOF ;
  <script>
    URL = '/perl-scripts/sysadmin/firmwareInstall.cgi?install='
        + escape( "$basename" ) 
        + '&phase=2' ;
    parent.loadAfterDelay( URL, 1000 ) ;
  </script>
EOF
goto EXIT ;
}


# Verify that the selected RPM file is uncorrupted.
$testResults  = RpmVerify( $fname ) ;

if ($testResults ne '')
{
  $errorMessage = "RPM file is corrupted!" ;
  goto ERROR ;
}
else
{
  print 'Package verifies okay.<br />' ;
}

$testResults = '' ;

# Extract some information from the rpm file description.
# We are trying to extract values from these lines:
#
#   Patch Base   0.1-0
#   Expected Installation Time : 10 seconds
# These values are put into the RPM from the .spec file used in
# the RPM generation process.
# PatchBase may not exist, and defaults to blank in that case.
# For patch installations it says what version the current firemware
# must be before we'll allow the installation.
#
# InstallationTime should be there, but we default it to 200 seconds
# in case something goes wrong with the RPM generation process.
# It is used to give an indication of how long the installation should
# take.  This is fed into the javascript installation monitoring
# pages below.

my $patchBase = '' ;
my $installationTime = '200' ;

$installationTime = $1 
        if ($description =~ m/Expected Installation Time\s*:\s*(\d+)/ );

$patchBase = $1 if ($description =~ m/Patch Base\s*(\S*)/ );

print "Installation time: $installationTime seconds.<br />" ;

if ($patchBase)
{
  my( $thisVersion, $thisDate ) = split( "\n", 
                  qx( cat /usr/local/fireball/defaults/firmwareVersion )
                                       ) ;

  # Display the versions.
  print 'This is a <b>PATCH upgrade</b>.<br />' ;
  print table( 
               Tr( td( 'Required Patch Base:'),
                   td( b($patchBase) )
                 ),
               Tr( td( 'Installed Version:'),
                   td( b($thisVersion) )
                 ),
             ) ;

  # Compare the patch base to the existing installed version.
  if ( $patchBase ne $thisVersion )
  {
    $errorMessage = 
         "Installed firmware version doesn't match patch requirement!" ;
    goto ERROR ;
  }
}
else
{
  print 'This is a FULL upgrade.<br />' ;
}

print '<p><b>Beginning Installation.</b><br />' ;

# At this point, the selected file has been verified and we are 
# allowed to install it.  We need a method of passing the file to
# the ramdisk installer.  The method must survive a reboot into
# a different operating system and file system layout.  The only
# thing we are guaranteed is that we can see the files in 
# /var/incoming.  We could put the name of the selected file into
# another file with a well defined name.  Or we can just make a 
# link to that file with a well defined name.  We'll try the latter.
# Link the the selected file to /var/gps/incoming/TheRPMtoInstall.
# Use a hard link to make sure that there are no symlink oddities
my $linkName = '/var/gps/incoming/TheRPMtoInstall' ;

# Get rid of this link if it already exists.
if ( -r $linkName ) 
{
  system( "sudo rm $linkName" ) ;
}

# Create the desired hard link.
link( $fname, $linkName ) ;


# Tell the browser to display the installation monitor page.  This
# is a real HTML page (not a cgi script) that uses javascript to
# show a summary of what is happening, implement a seconds counter,
# and, after a delay, put up a "reconnect" button.  Note that this
# is not a real installation monitor, since that can't be done easily 
# or reliably using html/http/javascript.
# Two javascript variables are passed into the page by defining them
# in the toplevel frame of the Fireball GUI.  The installation page
# has to reference them as, for example, "parent.Filename".
print <<EOF ;
  <Script>
    parent.Filename = "$basename" ;
    parent.expectedInstallTime = $installationTime ;
    parent.loadAfterDelay( '/installFrame.html', 1000) ;
  </Script>
EOF

# Trigger the actual installation 
#   - set the bootstatus flags
#     We set flag zero to 255 as a signal to the boot system to
#     load in the ramdisk system, and perform an installation.
#   - do a sleep and reboot
#     The sleep gives the GUI script time to get started before the
#     reboot.  Reboot is implemented by sending a SIGUSR2 signal to the
#     frontpanel task.

system( "/usr/bin/sudo /sbin/boot_status 0 255 f" ) ;

# Obtain the process id of the frontpanel_task.  Launch a background
# process that waits a bit then sends a signal to frontpanel_task.
my $spid = qx( cat /var/run/frontpanel_task.pid ) ;
system( "(sleep 5 ; /usr/bin/sudo /bin/kill -SIGUSR2 $spid)&" ) ;

goto EXIT ;


ERROR:
########################################################################
# If an error was detected during CGI processing, we trigger a
# javascript alert box and put up a similar warning on the page.
########################################################################
if ($errorMessage)
{
  print <<EOF ;
    <Script>
    alert( "$errorMessage" ) ;
    </Script>
EOF

  print "<p><b>$errorMessage</b>" ;
  if ($testResults)
  {
    print "<br />RPM test results:<pre>$testResults</pre>" ;
  }

  print startform( -name=>"theForm" ) ;
  print button( -name=>'Done', 
                -value=>'Done',
                -onClick=> 
                  'parent.loadDataFrame( "/perl-scripts/firmware.cgi" )'
              ) ;

  print endform() ;

}


EXIT:
print end_html() ;

########################################################################
# End of file firmwareInstall.cgi
########################################################################
