#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: firmwareUpload.cgi,v 1.13 2004/04/14 20:56:03 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/sysadmin/firmwareUpload.cgi,v $


########################################################################
# firmwareUpload.cgi
#
# Script to allow transfering a firmware file from the GUI user's
# computer to the firmware area on a Fireball.  Puts up a form allowing
# the user to browse/type-in the file to upload.  Accepts the resulting
# multi-part form containing the firmware file data.  The new file's
# name is extracted and used to move the CGI library's temporary file
# into /var/gps/incoming.  Status is indicated to show the upload is
# complete, and the firmwareVerify.cgi script is called to check the
# file.
########################################################################

use strict ;
use CGI qw/:standard/;

use lib '..' ;
use Fireball ;

titleWithHelp( 'Firmware Upload',
               '/firmwareUpload.html'
             ) ;
           
           

########################################################################

# Define the location where the firmware files are stored, switch
# the current directory to there and set the CGI scripts to upload
# data directly into that directory.

my $firmwareDir = '/var/gps' ;
chdir $firmwareDir ;



# Javascript Functions
########################################################################
print <<EOF ;
<Script>

/**********************************************************************/
function upload()
/**********************************************************************/
{
  //alert( "submit" ) ;
  document.theForm.submit() ;
} /* end of Update() */

</Script>
EOF

my $errorMessage = '' ;
my $testResults  = '' ;

########################################################################
# CGI parameter processing
########################################################################
if (param())
{
  # In this script, the only possible parameter is the filename that
  # was passed in.  So we assume an upload was attempted.  This uses
  # some rather obscure CGI library stuff.

  #for $p (param())
  #{
  #  print "$p = ", param($p), "<br />" ;
  #}

  # Get the name of the file as specified by the user
  my $fname = upload( 'firmwareFile' ) ;

  # The file has already been uploaded and is in a temporary location,
  # whose path is accessed here:
  my $tmpfilename = tmpFileName($fname) ;

  # If the file successfully uploaded, it will have a non-zero size.
  my $size = (-s $tmpfilename) ;
  #print $tmpfilename, " size=", (-s $tmpfilename), "<br />";

  if ($size)
  { 
    # Adjust the received filename to point to our desired path and
    # file naming conventions.

    # Strip any path information off of the filename.
    my $newFname = $fname ;
    
    # Remove path that might use DOS backslashes.
    $newFname =~ s/^.*\\// ; 

    # Remove path that might use Unix slashes.
    $newFname =~ s/^.*\/// ; 

    # prefix the raw name with the directory we use for firmware file
    # uploads.
    my $newPath = '/var/gps/incoming/'.$newFname ;
  
    if ( ! -d '/var/gps/incoming' )
    {
      mkdir '/var/gps/incoming' ;
    }

    if ( ! -w '/var/gps/incoming' )
    {
      system( '/usr/bin/sudo chown netrx:netrx /var/gps/incoming' ) ;
      system( '/usr/bin/sudo chmod a+rw /var/gps/incoming' ) ;
    }

    # Move the file to the final resting place.
    rename $tmpfilename, $newPath ;
    system( "chmod a+rx $newPath" ) ;
    close( $fname ) ;

    
    print '<br /><b>Upload complete.</b><br />',
          '<b>Verifying RPM package.</b>' ;

    # After a timeout, go back to main Sessions screen.
    my $url = '/perl-scripts/sysadmin/firmwareVerify.cgi?fname='
            . CGI::escape( $newFname ) ;

    print <<EOF;
        <Script>
        parent.loadAfterDelay( "$url", 1000) ;
        </Script>
EOF
    print end_html() ;


    goto EXIT ;
  }
  else
  {
    $errorMessage = "There was a problem uploading file '$fname'" ;
  }
}

########################################################################
# Create the html form with the file selection field and a few
# buttons.
########################################################################
print start_multipart_form( -name=>'theForm' ) ;

print "File to Upload ",
      filefield( -name=>'firmwareFile',
                 -size=>60
               ) ;

print hr() ;

print button( -name=>'Upload', 
              -value=>'Upload',
              -onClick=>'upload() ;',
            ) ;
print '&nbsp;' ;
print button( -name=>'Cancel', 
              -value=>'Cancel',
              -onClick=>
                  'parent.loadDataFrame( "/perl-scripts/firmware.cgi" )'
            ) ;

print end_form() ;

my($availSpace) = ( split " ", qx( df /var/gps ) )[10] ;

print <<EOF ;
Firmware files are uploaded to the target system's data logging area.
There must be sufficient empty space in that area to contain the entire
firmware file.  Please verify that the size of the file you are about to
upload is less than the available space listed below.  If not, then you
will need to delete one or more logged data files before starting the
upload.  You might also be able to reclaim space by deleting any old
firmware files that you no longer need.
<br /><br />
<b>Space available for uploaded firmware files:</b>
EOF

printf( "%.3f Mbytes<br />", $availSpace/1024 ) ;


########################################################################
# If an error was detected during CGI processing, we trigger a
# javascript alert box and put up a similar warning on the page.
########################################################################
if ($errorMessage)
{
  print <<EOF ;
    <Script>
    alert( "$errorMessage" ) ;
    </Script>
EOF

  print "<br /><b>$errorMessage</b>" ;
  if ($testResults)
  {
    print "<br />RPM test results:<pre>$testResults</pre>" ;
  }

}

EXIT:
print end_html() ;
########################################################################
# end of file firmwareUpload.cgi
########################################################################
