#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: ftpConfig.cgi,v 1.23 2004/07/28 20:40:56 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/sysadmin/ftpConfig.cgi,v $

########################################################################
# ftpConfig.cgi
#
# Script to enable configuration of the NTP subsystem.
# Called in several forms, using CGI parameters:
#
#   ftpConfig.cgi
#       No parameters: Initialize the form with current data
#
#   ftpConfig.cgi?(full list of parameters from a form)
#       A normal submit of the information.
#
# FTP configuration is controlled by two files:
#   /etc/ftpaccess 
#       (Actually a link to /etc/sysconfig/ftpaccess)
#       This file contains the basic configuration data.  See that
#       file for extensive documentation.  We only adjust a few
#       lines in it to enable or disable specific forms of FTP.
#
#   /etc/sysconfig/ftppasswd
#       This file contains the named accounts used by FTP for 
#       password access accounts.  The file is kept separate from the
#       generic /etc/passwd file so that it can be kept as part of the
#       system configuration, and not wiped out during firmware
#       upgrades.  The file msut be merged with /etc/passwd, by using
#       the script /etc/init.d/mergeftp.  This is done every time we
#       power up (but only if needed) and by the GUI when a change is
#       made to the passwords.
########################################################################


use strict ;
use CGI qw/:standard/;

use lib '..' ;
use Fireball ;

titleWithHelp( 'FTP Setup',
               '/ftpInfo.html'
             ) ;


# Read the current FTP controls into a hash.
# There will be elements for 
#   standardPort = [ yes, no ]
#   alternatePort = [1024...65535]
#   anon  = ['disabled' 'enabled' 'enabledDelete']
#   named = ['disabled' 'enabled' 'enabledDelete']
#   admin = ['disabled' 'enabled']
#   namedPassword = Encrypted password for user netrsFTP
#   adminPassword = Encrypted password for user adminFTP
my %ftpSetup ;
getFtpSetup( \%ftpSetup ) ;


#for my $p (sort keys %ftpSetup)
#{
#  print "$p = $ftpSetup{$p}<br />" ;
#}


# If this is a CGI request then some parameters will be passed into the
# script.  We'll need to parse those and set the state(s) as requested.
# If there is a problem, then this error message might be set and
# we'll display it later.
my $errorMessage = '' ; # blank until an error is detected

if (!param())
{
  # No parameters.  Read the current setup and make that the defaults
  # for all widgets.

  # Set the cgi parameters used by the GUI widgets to match what's
  # just been determined.
  param( 'anon', $ftpSetup{'anon'} ) ;

  param( 'netrs', $ftpSetup{'named'} ) ;

  param( 'admin', $ftpSetup{'admin'} ) ;

  param( 'alternatePort', $ftpSetup{'alternatePort'} ) ;

  param( 'ftpPort', 
         ($ftpSetup{'ftpEnabled'} eq 'no')
         ? 'disabled'
         : ($ftpSetup{'standardPort'} eq 'yes')
           ? 'standard'
           : 'alternate'
       ) ;
  {
  }

}
else
{
  # This is a submission of a new FTP setup.
  # Verify the parameters are okay and then install them.

  #print( "<pre>Parameters Received<br />" ) ;
  #foreach my $p (param())
  #{
  #    print  "$p=", param($p), "\n" ;
  #}
  #print "</pre>" ;

  # These are the new states of the non-password controls.
  $ftpSetup{'anon'}  = param( 'anon' ) ;
  $ftpSetup{'named'} = param( 'netrs' ) ;
  $ftpSetup{'admin'} = param( 'admin' ) ;

  my $ftpPort = param('ftpPort') ;
  if ($ftpPort eq 'standard')
  {
    $ftpSetup{'ftpEnabled'} = 'yes' ;
    $ftpSetup{'standardPort'} = 'yes' ;
  }
  elsif ($ftpPort eq 'alternate')
  {
    $ftpSetup{'ftpEnabled'} = 'yes' ;
    $ftpSetup{'standardPort'} = 'no' ;
  }
  else
  {
    $ftpSetup{'ftpEnabled'} = 'no' ;
    $ftpSetup{'standardPort'} = 'yes' ;
  }

  my $port = param( 'alternatePort' ) ;
  if ( $ftpSetup{'standardPort'} eq 'no'
   and ( $port !~ /^\d+$/
      or $port < 1024
      or $port > 65535
       )
     )
  {
    $errorMessage = 'Alternate port must be 1024 to 65535' ;
    goto PARAMDONE ;
  }

  $ftpSetup{'alternatePort'} = $port + 0 ;

  my $pw1 ;
  my $pw2 ;

  $pw1 = param( 'netrspw1' ) || '' ;
  $pw2 = param( 'netrspw2' ) || '' ;

  if ( $pw1 or $pw2 )
  {
    if ($pw1 ne $pw2)
    {
      $errorMessage = 'netrsFTP Passwords must match.' ;
      goto PARAMDONE ;
    }
    $ftpSetup{'namedPassword'} = makepw( $pw1 ) ;
  }
  
 
  $pw1 = param( 'adminpw1' ) || '' ;
  $pw2 = param( 'adminpw2' ) || '' ;

  if ( $pw1 or $pw2 )
  {
    if ($pw1 ne $pw2)
    {
      $errorMessage = 'adminFTP Passwords must match' ;
      goto PARAMDONE ;
    }
    $ftpSetup{'adminPassword'} = makepw( $pw1 ) ;
  }
  

  # Check if the passwords aren't defined where they are needed.
  if ( ($ftpSetup{'named'} ne 'disabled') 
   and !$ftpSetup{'namedPassword'}
     ) 
  {
    $errorMessage = 'netrsFTP needs a password.' ;
    goto PARAMDONE ;
  }

  if ( ($ftpSetup{'admin'} ne 'disabled')
   and !$ftpSetup{'adminPassword'}
     )
  {
    $errorMessage = 'adminFTP needs a password.' ;
    goto PARAMDONE ;
  }
 
  # Update the stored ftp configuration and trigger the
  # FTP system to restart.
  saveFtpSetup( \%ftpSetup ) ;


  print h3( 'New FTP setup accepted.' ) ;
  print h3( ($ftpSetup{'ftpEnabled'} eq 'yes') 
            ? 'Restarting FTP Server.' 
            : 'FTP Server is disabled.' 
          ) ;

  print <<EOF ;
  <Script>
    parent.loadAfterDelay( "/perl-scripts/ftp.cgi", 2000) ;
  </Script>
EOF

  goto EXIT ;

}
PARAMDONE:

# Done processing parameters (if any) and nobody forced an exit.
# So we aren't done and have to put up the HTML form to request
# user input.

# If an error message was found earlier during CGI processing,
# post an alert message to the user.  Also display the message on the
# top of the page, in case the javascript alert system doesn't work.
if ( $errorMessage )
{
  print << "EOF" ;
  <Script> 
  alert( "$errorMessage" ) ;
  </Script>
EOF

  print b( $errorMessage ) ;
}

print startform( -name=>"theForm" ) ;


print '<table border=0>' ;


my @buttons ;

########################################################################
# ANONYMOUS FTP CONTROLS
########################################################################
print Tr( td( br(), b("Anonymous FTP") )  ) ;

# Create three buttons.
@buttons=radio_group( -name=>'anon',
                      -values=>[ 'enabledDelete',
                                 'enabled', 
                                 'disabled'
                               ],
                      -nolabels=>1,
                      -default=>"enabled",
                    ) ;

print Tr( td( table( {width=>"100%", border=>1},
                     Tr( td( $buttons[0] ), 
                         td( "<b>Enabled  with Delete</b><br />"
                            ." Anyone using the names 'anonymous' or "
                            ."'ftp' can use FTP to access logged data "
                            ."files.  No password is required."
                            ."<br />Anonymous user <em>can</em> delete "
                            ."files."
                           ) 
                       ) ,

                     Tr( td( $buttons[1] ), 
                         td( "<b>Enabled without Delete</b><br />"
                            ."Anyone using the names 'anonymous' or "
                            ."'ftp' can use FTP to access logged data "
                            ."files.  No password is required.<br />"
                            ."Anonymous user <em>cannot</em> delete "
                            ."files."
                           ) 
                       ) ,

                     Tr( td( $buttons[2]),
                         td( "<b>Disabled</b><br />
                              Anonymous FTP is not available." ) 
                       ) 
                   ) 
            )
        );


########################################################################
# Named FTP controls
########################################################################

print Tr( td( br(), b("Named FTP") )  ) ;

print '<tr><td>' ;

# Create three buttons.
@buttons=radio_group( -name=>'netrs',
                      -values=>[ 'enabledDelete', 
                                 'enabled', 
                                 'disabled'
                               ],
                      -nolabels=>1,
                      -default=>"enabled",
                    ) ;


print table( {width=>"100%", border=>1},

             Tr( td( $buttons[0] ), 
                 td( "<b>Enabled  with Delete</b><br />"
                    ."User 'netrsFTP' can access logged data files "
                    ."using FTP and the proper password.<br />"
                    ."User 'netrsFTP' <em>can</em> delete files."
                   ) 
               ) ,

             Tr( td( $buttons[1] ), 
                 td( "<b>Enabled without Delete</b><br />"
                    ."User 'netrsFTP' can access logged data files "
                    ."using FTP and the proper password.<br />"
                    ."User 'netrsFTP' <em>cannot</em> delete files."
                   ) 
               ) ,

             Tr( td( $buttons[2]),
                 td( "<b>Disabled</b><br />"
                    ."User 'netrsFTP' cannot use FTP." 
                   ) 
               ) 
           ) ;

print '</td><td width="1"></td><td valign="top">' ;


print table( {border=>'0'},
             Tr( th( {-align=>'right'}, 'netrsFTP Password' ),
                 td( password_field( 'netrspw1', '', 12, 12 ) )
               ),

             Tr( th( {-align=>'right'}, 'Re-enter Password' ),
                 td( password_field( 'netrspw2', '', 12, 12 ) )
               ),

           ) ;

print <<EOF ;
<div align="right">
<font size="-1"'>
If both fields are left blank, the
<br /> password will not be changed.</font></div></td>
</tr>
<tr>
<td>
EOF



########################################################################
# adminFTP controls
########################################################################

print Tr( td( br(), b("Administration FTP") )  ) ;
print '<tr><td valign="top">' ;

# Create three buttons.
 @buttons=radio_group( -name=>'admin',
                       -values=>[ 'enabled', 'disabled' ],
                       -nolabels=>1,
                       -default=>"enabled",
                     ) ;



print table( {width=>"100%", border=>1},

             Tr( td( $buttons[0] ), 
                 td( "<b>Enabled</b><br />
                      User 'adminFTP' can upload or delete firmware 
                      files, using FTP and the proper password. 
                     "
                   ) 
               ) ,

             Tr( td( $buttons[1] ), 
                 td( "<b>Disabled</b><br />
                      User 'adminFTP' cannot use FTP.
                     "
                   ) 
               ) ,
           ) ;

print '</td><td></td><td valign="top">' ;

print table( 
             #Tr( th( '&nbsp;' ),
             #  ) ,

             Tr( th( {-align=>'right'}, 'adminFTP Password' ),
                 td( password_field( 'adminpw1', '', 12, 12 ) )
               ) ,

             Tr( th( {-align=>'right'}, 'Re-enter Password' ),
                 td( password_field( 'adminpw2', '', 12, 12 ) )
               ) 
           ) ;

print <<EOF ;
<div align="right">
<font size="-1"'>
If both fields are left blank, the
<br /> password will not be changed.</font></div></td>
</tr>
EOF


########################################################################
# Port Selection Controls.
########################################################################
print Tr( td( br(), b("FTP Server Port") )  ) ;

@buttons=radio_group( -name=>'ftpPort',
                   -values=>[ 'standard', 'alternate', 'disabled' ],
                   -nolabels=>1,
                 ) ;

print Tr( td( table( {width=>"100%", border=>1},

                     Tr( td( $buttons[0] ), 
                         td( "<b>Use Standard Port</b><br />"
                            ."Clients must connect to Port 21"
                           ) 
                       ) ,

                     Tr( td( $buttons[1] ), 
                         td( "<b>Use Alternate Port</b>",
                             textfield( 'alternatePort', 
                                        '',
                                        10,
                                        5
                                      )
                           ) 
                       ) ,

                     Tr( td( $buttons[2] ), 
                         td( "<b>Disable FTP</b>"
                           ) 
                       ) ,
                   )
            )
        ) ;


print "</table>" ;



print p(),button( -name=>'OK', 
              -value=>'OK',
              -onClick=>'parent.submitForm( document.theForm, 
                              "/perl-scripts/sysadmin/ftpConfig.cgi")'
            ) ;
print "&nbsp;" ;
print button( -name=>'Cancel', 
              -value=>'Cancel',
              -onClick=>
                       'parent.loadDataFrame( "/perl-scripts/ftp.cgi" )'
            ) ;


print endform() ;

EXIT:
print end_html() ;

########################################################################
# End of file ftpConfig.cgi
########################################################################
