#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: pppDialout.cgi,v 1.10 2004/07/29 23:27:10 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/sysadmin/pppDialout.cgi,v $

########################################################################
# pppDialout.cgi
#
# Script to allow configuration of the PPP Dialout function.
#
#   pppDialout.cgi
#       No parameters: Initialize the form with current data
#
#   ethernetSetup.cgi?
#
########################################################################


use strict ;
use CGI qw/:standard/;

use lib '..' ;
use Fireball ;


titleWithHelp( 'PPP Dialout Configuration',
               '/pppDialoutInfo.html'
             ) ;

my %pppDialoutControls ;

getPppDialoutControls( \%pppDialoutControls ) ;

# If this is a CGI request then some parameters will be passed into the
# script.  We'll need to parse those and set the state(s) as requested.
# If there is a problem, then this error message might be set and
# we'll display it later.
my $errorMessage = '' ; # blank until an error is detected


# If some parameters were fed to the script, then this is a submission
# of data.
if (param())
{
  #print "<pre>" ;
  #for my $p (param())
  #{
  #  print "$p : ", param( $p ), br() ;
  #}
  #print "</pre>" ;

  my $pw1;
  my $pw2;

  $pw1 = param( 'PapPw1' ) || '' ;
  $pw2 = param( 'PapPw2' ) || '' ;
  if ( $pw1 eq '' && $pw2 eq '' )
  {
    $pppDialoutControls{'papPw'} = '' ;
  }
  else
  {
    if ($pw1 ne $pw2)
    {
      $errorMessage = 'PAP Passwords must match' ;
      goto PARAMDONE ;
    }

    if ( $pw1 ne 'FaKe PW' )
    {
      $pw1 =~ s/^ +// ;
      $pw1 =~ s/ +$// ;
      if ($pw1 =~ /\s/)
      {
        $errorMessage = 'No spaces allowed in passwords.' ;
        goto PARAMDONE ;
      }
      $pw1 = qx( /usr/local/fireball/makeKey $pw1 ) ;
      chomp $pw1 ;
      $pppDialoutControls{'papPw'} = $pw1 ;
    }
  }

  $pw1 = param( 'ChapPw1' ) || '' ;
  $pw2 = param( 'ChapPw2' ) || '' ;
  if ( $pw1 eq '' && $pw2 eq '' )
  {
    $pppDialoutControls{'chapPw'} = '' ;
  }
  else
  {
    if ($pw1 ne $pw2)
    {
      $errorMessage = 'Chap Passwords must match' ;
      goto PARAMDONE ;
    }

    if ( $pw1 ne 'FaKe PW' )
    {
      $pw1 =~ s/^ +// ;
      $pw1 =~ s/ +$// ;
      if ($pw1 =~ /\s/)
      {
        $errorMessage = 'No spaces allowed in passwords.' ;
        goto PARAMDONE ;
      }
      $pw1 = qx( /usr/local/fireball/makeKey $pw1 ) ;
      chomp $pw1 ;
      $pppDialoutControls{'chapPw'} = $pw1 ;
    }
  }


  # Process the new options data from the form.  
  # HTML forms always return line terminations as "\r\n", but internally
  # we use only '\n'.  SO get rid of all of the '\r's.
  my $text = param( 'options' ) ;
  $text =~ s/\r//g ;
  $pppDialoutControls{'options'} = $text ;

  # Process the new chatScript data from the form.  
  # HTML forms always return line terminations as "\r\n", but internally
  # we use only '\n'.  SO get rid of all of the '\r's.
  $text = param( 'chatScript' ) ;
  $text =~ s/\r//g ;
  $pppDialoutControls{'chatScript'} = $text ;

  savePppDialoutControls( \%pppDialoutControls ) ;

  #print "<pre>" ;
  #print "\n\n", qx(cat /etc/sysconfig/dialout) ;
  #print "</pre>" ;
  print h2( "New Configuration Accepted." ) ;

  print <<EOF ;
  <Script>
    parent.loadAfterDelay( "/perl-scripts/portSummary.cgi", 2000) ;
  </Script>
EOF

  goto EXIT ;
}
else
{
  # NO parameters.  This must be the first invocation of this
  # script.  Read in the existing values of the pppDialoutControls.
  param( 'options', $pppDialoutControls{'options'} ) ;
  param( 'chatScript', $pppDialoutControls{'chatScript'} ) ;
}
PARAMDONE:



# If an error message was found earlier during CGI processing,
# post an alert message to the user.  Also display the message on the
# top of the page, in case the javascript alert system doesn't work.
if ( $errorMessage )
{
  print b( $errorMessage ),br(),br() ;

  print << "EOF" ;
  <Script> 
  alert( "$errorMessage" ) ;
  </Script>
EOF

}


print startform( -name=>"theForm" ) ;

print '<table>' ;

my $default = ( $pppDialoutControls{'papPw'} )
              ? 'FaKe PW'
              : '' ;
print Tr( td( {align=>'right'}, 'PAP password' ),
          td( password_field( 'PapPw1', $default, 12, 28 ),
              '<font size="-1">&nbsp;&nbsp;Verify</font>',
              password_field( 'PapPw2', $default, 12, 28 ),
            ),
        ) ;

$default = ( $pppDialoutControls{'chapPw'} )
           ? 'FaKe PW'
           : '' ;
print Tr( td( {align=>'right'}, 'CHAP password' ),
          td( password_field( 'ChapPw1', $default, 12, 28 ),
              '<font size="-1">&nbsp;&nbsp;Verify</font>',
              password_field( 'ChapPw2', $default, 12, 28 ),
            )
        ) ;
print Tr( td( {valign=>'top', align=>'right'}, 'PPP Options' ),
          td(  textarea( -name=>'options',
                         -rows=>5,
                         -columns=>30,
                         -wrap=>'off',
                       )
            )
        ) ;
print Tr( td( {valign=>'top', align=>'right'}, 'Chat Script' ),
          td( textarea( -name=>'chatScript',
                        -rows=>10,
                        -columns=>30,
                        -wrap=>'off',
                      )
            )
        ) ;


print '</table>' ;



print 
      "Accepting these parameters with <b>OK</b> will "
     ."restart any existing PPP Dialout session.",
      br(),
      "The Serial Port and baudrate to use for PPP Dialout "
     ."must be set from the Port Configuration pages."
     ;

print hr() ;
print p(),button( -name=>"OK", 
              -value=>"OK",
              -onClick=>"submit() ;",
            ) ;
print "&nbsp;" ;
print button( -name=>"Cancel", 
              -value=>"Cancel",
              -onClick=>
          'parent.loadDataFrame( "/perl-scripts/portSummary.cgi" )'
            ) ;

print endform() ;

EXIT:
print end_html() ;

########################################################################
# End of file pppDialout.cgi
########################################################################
