#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: setAccess.cgi,v 1.6 2003/10/09 20:10:31 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/sysadmin/setAccess.cgi,v $

########################################################################
# setAccess.cgi
# Script for accepting changes to the Access Restrictions on Fireball.  
# The script /perl-scripts/access.cgi manages an HTML form that allows
# the user to control Access Restrictions to the web pages.  When the
# user clicks OK on that page, this script is invoked with the form data
# encoded in CGI parameters.  In particular, the parameters 'sysadmin',
# 'gpsadmin' and 'fileaccess' can come in as 'Open' or 'Restricted'.
# This script parses those and updates the htaccess files to match.
# Once done, it switches control back to the original access.cgi page,
# usually with some sort of message to be displayed.
#
# Access restrictions allow the user to enable or disable the
# restrictions on the three browser function security levels.
# These are embodied in the configuration files
#   /etc/sysconfig/htaccess.sysadm
#   /etc/sysconfig/htaccess.gpsadm
#   /etc/sysconfig/htaccess.fileaccess
# There are symbolic links in the three corresponding directories
# that point to these access control files.  Each of the files looks
# pretty much the same:
#
#       AuthType Basic
#       AuthName Sysadmin
#       AuthUserFile /etc/sysconfig/htpasswd
#       AuthGroupFile /etc/sysconfig/htgroups
#       require group sysadmin
#
# The access control is dictated by the "require" line.  If that
# is present then the restrictions are active.  If not then
# the scripts in that directory are available to anyone.
########################################################################

use strict ;
use CGI qw/:standard/;

use lib '..' ;
use Fireball ;

########################################################################
# Begin formatting the return document.
########################################################################
startPage( 'Set Access' ) ;


# This variable is used to collect and return the setAccess.cgi status
# after each OK commit.  It is fed back to perl-scripts/access.cgi via
# a 'msg' CGI parameter.
my $message = '' ;

my @levels = ( 'sysadmin', 'gpsadmin', 'fileaccess' ) ;


my %oldRestrictions ;
# populate that hash with the current access control states.
getGuiSecurity( \%oldRestrictions ) ;

# This hash will be filled in with the newly requested security modes.
my %newRestrictions ;

#print( "Parameters Received<BR>" ) ;
#foreach my $p (param())
#{
#  print  "$p=", param($p), " " ;
#}


foreach my $r (@levels)
{
  my $p = param( $r ) ;

  $p = $newRestrictions{ $r } = (param( $r ) eq 'Restricted') ;

  # A change is happening,  Indicate the change in the returned
  # $message CGI parameter.
  if ( $p != $oldRestrictions{$r} )
  {
    $message .= h4( 'Access is now ',
                    $p ? 'Restricted' : 'Unrestricted',
                    ' for ',
                    ($r eq 'sysadmin') 
                    ? 'System Controls'
                    : ($r eq 'gpsadmin') 
                      ? 'GPS Controls'
                      : 'File Access' ,
                    ' pages.',
                  ) ;
  }
}

# Adjust all of the htaccess files, as needed.
updateGuiSecurity( \%newRestrictions ) ;

if (! $message ) 
{
  $message = h4('No changes were made.') ;
}

# Immediately switch back to the original HTML form page, to show
# the message and allow further changes.
print <<EOF ;
<script>
  document.location = '/perl-scripts/access.cgi?msg='
                    + escape( '$message' ) ;
</script>
EOF

print end_html() ;

########################################################################
# end of file setAccess.cgi
########################################################################
