#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: setNtp.cgi,v 1.8 2003/10/29 19:30:31 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/sysadmin/setNtp.cgi,v $

########################################################################
# setNtp.cgi
#
# Script to enable activating new configuration of the NTP subsystem.
# Called by /perl-scripts/ntp.cgi, feeding in CGI parameters of the
# form:
#   ?server1=abc.com&server2=def.org&server3=xyz.net
#
# The parameters are parsed and saved away in the NTP configuration
# files.  And then the NTP daemon is killed.  It is supposed to be
# automatically restarted by the frontpanel_task after a short time
# (perhaps 30 seconds).
#
# Note that the parameters are checked for valid character set, but
# that's about all.  If the user enters garbage, it might not be caught.
# If bad parameters are detected, then a warning is put up and the
# perl-scripts/ntp.cgi script is called, with the same parameters, and
# a 'msg=' parameter so that the user can fix the errors.
#
########################################################################


use strict ;
use CGI qw/:standard/;

use lib '..' ;
use Fireball ;

########################################################################
# Begin formatting the return document.
########################################################################
startPage( 'Set NTP' ) ;

my $errorMessage ;


#print( "<pre>Parameters Received<BR>" ) ;
#foreach my $p (param())
#{
#    print  "$p=", param($p), "\n" ;
#}
#print "</pre>" ;


# This is a submission of a new setup.
# Verify the parameters are okay...

my @servers = () ;

# For each of the external server text fields, read the value and
# drop leading/trailing spaces.
# If the string is non-blank, check it for valid characters.
# If okay, add to the list of @servers for later insertion into the
# config file.
for (my $i = 0 ; $i < 3 ; ++$i)
{
  my $server = param( 'server'.$i ) ;
  $server =~ s/^\s*// ;
  $server =~ s/\s*$// ;
  next if ($server eq "") ; # blank string.
  if ($server =~ m/^[\-a-zA-Z0-9.]*$/)
  {
    push @servers, ($server) ;
  }
  else
  {
    # Bad characters.  Put up a warning.
    $errorMessage = 'Illegal Characters in External Time Servers' ;
    goto ERROR ;
  }
}

print h2( 'New NTP setup accepted.' ) ;
print h2( 'Restarting NTP daemon.' ) ;

saveNtpServers( @servers ) ;

# After a timeout, switch to another display.
print <<EOF ;
<Script>
  parent.loadAfterDelay( "/perl-scripts/rstatusIdentity.cgi", 2000) ;
</Script>

EOF
goto EXIT ;


ERROR:
# An error was found during CGI parameter processing.  We need to
# display the error message, and then switch back to the HTML form
# page that got us here.  In order to preserve what the user had
# already entered, we need to post all of the CGI parameters back to
# the original script.

print h3( $errorMessage ) ;

my $returnURL = '/perl-scripts/ntp.cgi'
              . '?server0='.CGI::escape( param('server0') )
              . '&server1='.CGI::escape( param('server1') )
              . '&server2='.CGI::escape( param('server2') )
              ;

if ( defined( param('ntpDisable') ) )
{
  $returnURL .= '&ntpDisable='.CGI::escape( param('ntpDisable') ) ;
}
$returnURL .= '&msg='.CGI::escape( $errorMessage ) ;

print <<EOF ;
<Script>
  parent.loadAfterDelay( "$returnURL", 4000) ;
</Script>
EOF


EXIT:
print end_html() ;

########################################################################
# End of file setNtp.cgi
########################################################################
