#!/usr/bin/perl -w
# Copyright (c) 2002 Trimble Navigation Limited
# $Id: setSysname.cgi,v 1.11 2003/06/12 22:45:40 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/sysadmin/setSysname.cgi,v $

########################################################################
# setSysname.cgi
#
# Script for modifying the system name of this Fireball.
#
# Can be called one of several ways:
#   o With no CGI parameters - 
#     We put up the default form showing the current system name.
#
#   o With CGI parameters requesting an account deletion.
#       ?Sysname=string
#     Attempt to set the system name to the new string.
#
# The current system name is obtained using 
#   cat /etc/sysconfig/sysname
# That configuration file contains the name and nothing else.
# We haven't figured out yet where during system configuration the
# default value for this will be generated.
#
# The value of sysname is NOT directly related to the system's name as
# obtained via any DNS lookup, although having them match is a good
# idea.  
#
# When a new system name is requested the file /etc/sysconfig/sysname is
# rewritten.  Furthermore, the title bar of the GUI interface must be
# redrawn since it shows the system name.  That is done by reloading the
# entire browser window.
#
########################################################################



use strict ;
use CGI qw/:standard/;

use lib ".." ;
use Fireball ;

titleWithHelp( 'Set System Name', '/rxstatSysName.html' ) ;

# If this is a CGI request then some parameters will be passed into the
# script.  We'll need to parse those and do what is requested.  This
# process may find errors, the first of which will be saved in $error.
my $error="" ;

if (param())
{
  # cgi parameters were appended.
  
  #print( "Parameters Received<BR>" ) ;
  #foreach (param())
  #{
  #    print( "$_=", param($_), "<BR>" ) ;
  #}

  my $newname = param( "Sysname" ) ;
  if (! $newname )
  {
    $error="A System Name must be specified!" ;
    goto paramDone ;
  }

  # Spaces aren't allowed in sysnames, but to make it easy on the
  # customer we just benignly substitute underscores.
  $newname =~ s/ /_/g ;

  # Certain other characters are not allowed, but it seems best to
  # inform the user instead of performing a substitution.
  if ( $newname =~ /[^A-Za-z0-9_-]/ )
  {
    # In the error message, we put a specific \n sequence.  This
    # has to be in single quotes to avoid translation to a true
    # newline character.  Otherwise, the call to Javascript alert
    # later on can't handle it.
    $error='Invalid Characters in System Name!\n'.
           'Use only A to Z, a to z, 0 to 9, - and _' ;
    goto paramDone ;
  }


  # New name is okay to use.  Put it into the configuration file.
  open SYSNAME, '>/tmp/sysname' ;
  print SYSNAME "$newname\n" ;
  close SYSNAME ;

  # Copy the new file on top of the old one using sudo.  This
  # gives us permission to do the copy, and preserves all of the 
  # original file characteristics.
  system( 'sudo cp /tmp/sysname /etc/sysconfig/sysname' ) ;


  system( "sudo /bin/hostname $newname" ) ;

  # Display a message indicating acceptance
  print h1("System Name Change Accepted.") ;

  # After a timeout, restart the whole Fireball interface.  This is
  # needed because the system identity if shown in the top title frame,
  # which isn't drawn except at Fireball GUI startup.
  print <<EOF ;
    Switching to Identity Information display.
     
    <script>
    parent.updateSysname( "$newname" ) ;
    parent.loadAfterDelay( "/perl-scripts/rstatusIdentity.cgi", 1000) ;
    </script>
EOF

  # Done with page.  Skip the rest of this script.  
  print end_html() ;
  goto EXIT ;

} # End of "if cgi parameters came in".

paramDone:  # jump to here if parameter processing terminates early.

########################################################################
# When we get to here, either this is a first invocation of the script,
# or something was wrong with the user's input.  Redraw the forms
# (perhaps with an error message alert) and wait for further input.

# Create the form with fields for account data entry.
print startform( {name=>"theForm"} ) ;


my $sysname = qx( cat /etc/sysconfig/sysname ) ;

# The basic form we are after is a two-column table, split down the
# middle of the panel.  On the left is the label.   
# On the right is the text entry widget.
# The last row in the table is used for the action buttons for Okay
# and Cancel.
print '<Table align=center width=100% >' ;

print Tr( th( {-align=>'right'}, 'System Name' ),
          td( textfield( 'Sysname', $sysname, 12, 12 ) )
        ) ;

# Create the action buttons along the bottom of the form.  

print Tr( 
        # Left cell of row gets the submit/okay button.
        td( {align=>'right'},
            button( -name=>"OK", 
                    -value=>"OK", 
                     -onClick=>
                             'parent.submitForm( document.theForm, 
                              "/perl-scripts/sysadmin/setSysname.cgi" )'
                  ) 
          ),

        # Right cell of row gets the Cancel button.
        td( button( -name=>"cancel", -value=>"Cancel", 
                     -onClick=>
           'parent.loadDataFrame( "/perl-scripts/rstatusIdentity.cgi" )'
                  ) 
          )
        ) ;

# End of table.  End of form.
print "\n</Table>\n</Form>\n" ;


# If an error message was found earlier during CGI processing,
# post an alert message to the user.
if ( length $error )
{
  print "<Script> 
  alert( \"$error\" ) ;
  </Script>
  " ;

  # Also display the message on the bottom of the page, in case
  # the javascript alert system doesn't work.  Note that we have to 
  # substitute an HTML break for the string '\n' to get the formatting
  # right.
  $error =~ s/\\n/<BR>/g ;
  print "$error" ;
}

print end_html() ;

EXIT:
########################################################################
# end of script setSysname.cgi
########################################################################
