#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: syslogs.cgi,v 1.23 2005/03/24 22:28:47 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/syslogs.cgi,v $

########################################################################
# syslogs.cgi
#     Script for displaying and managing system log files.
#
# This script is called in one of several ways.  
#
#   no CGI parameters: 
#       Initial display at top level directory.
#       
#   ?dir=path
#       Show that directory path.
#
#   ?view=filepath
#       Show the file after the directory listing.
#
#   ?erase=filepath
#
#   ?sort=name
#   ?sort=time
#   ?sort=timer
#   ?sort=size
#   ?sort=sizer
#       These request the directory list be sorted by the value.
#       time : earliest to latest
#       timer: latest to earliest
#       size : biggest to smallest
#       sizer: smallest to biggest
#
# 
# The parameters are commonly used in combination as with:
#   syslogs.cgi?sort=size&dir=/abc/
#
########################################################################

use strict ;
use CGI qw/:standard/;

use Fireball ;

titleWithHelp( 'System Log Files',
               #'/syslogsInfo.html'
               '' # No Help
             ) ;


# The root directory of all of the log files.  All file references are
# assumed rooted here.
my $Rootdir='/var/log' ;

# If CGI parameter specifies a directory, use it.  Otherwise, start
# at the top of the tree with '/'.  
# Always end the directory with a '/'.  
# Note that $dirpath is a path relative to the log directory root as
# specified in $Rootdir.
my $dirpath = param('dir') ;
if ( !$dirpath) {$dirpath = '/'} ;

if ( $dirpath !~ '/$' )
{
  $dirpath .= '/' ;
}


# Determine how the entries should be sorted.  Default is by name.
my $sortBy = param('sort') ;
if ( (! $sortBy)
    || ( $sortBy ne 'size' 
      && $sortBy ne 'time' 
      && $sortBy ne 'timer' 
      && $sortBy ne 'sizer' 
       )
   )
{
  $sortBy = 'name' ;
}



# Make all links blue with no underlines, even if visited, etc.
# This is a document-level style specification, affecting all
# links in this frame.
print <<EOF ;
<style type="text/css">
 <!--
       a {color:blue ; text-decoration: none }
 -->
</style>
EOF



# Show the directory path we are displaying.
print b( "Directory path $Rootdir$dirpath" ) ;


# If this request is to erase a file, handle that before displaying
# the directory again.  Indicate what was erased.
if ( my $file = param( 'erase' ) )
{
  # Implement the reset by creating a zero-length file and copying
  # it over the original.  That preserves other attributes such as
  # owner, permissions, etc.
  open F, ">/tmp/emptyFile" ;
  close( F );

  # As root copy the empty file over the one to be erased.
  system( "/usr/bin/sudo cp /tmp/emptyFile $Rootdir$file" ) ;

  print "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;", 
        b("File $Rootdir$file has been erased." ) ;
}


# Start the table showing the files.
print p() ;
print '<Table border="0">' ;

# Headers for each column.
# Name, Modified and Size are made "bold" depending on which sorting
# option is chosen.
print Tr( td( "" ), # Nothing over the file icons

          td( a( {href=>"syslogs.cgi?sort=name&dir=$dirpath" },
                 (($sortBy eq 'name') ? '<b>Name</b>' : 'Name')
               )
            ),

          td( {width=>"20"} ),  # Spacer Column

          td( {align=>'center'},
              a( {href=>( ($sortBy ne 'time')
                          ? "syslogs.cgi?sort=time&dir=$dirpath"
                          : "syslogs.cgi?sort=timer&dir=$dirpath"
                        ) 
                 },
                 (($sortBy eq 'time' || $sortBy eq 'timer') 
                  ? '<b>Modified</b>' 
                  : 'Modified')
               )
            ),

          td( {width=>"20"} ),  # Spacer Column

          td( {align=>'center'},
              a( {href=>( ($sortBy ne 'size')
                          ? "syslogs.cgi?sort=size&dir=$dirpath" 
                          : "syslogs.cgi?sort=sizer&dir=$dirpath" 
                        )
                 },
                 (($sortBy eq 'size' || $sortBy eq 'sizer') 
                  ? '<b>Size</b>' 
                  : 'Size')
               )
            ),

          td( {width=>"20"} ),  # Spacer Column

          td("Erase"),
        ) ;

# Print a horizontal bar that spans all of the columns.
print Tr( td( {colspan=>8}, hr() ) ) ;


if (! chdir $Rootdir.$dirpath )
{
  print "</Table>" ;
  print "This directory does not exist, or cannot be read." ;

  print end_html() ;

  goto EXIT ;   # ModPerl won't let us use exit().
}


# Later on in this script we need to determine file types for all of the
# files in this directory, especially when displaying an ascii icon
# versus a binary icon.  This can't be done using the builtin Perl test
# functions, since the log files might not be readable by the Apache
# user.  This was initially solved by running a "/usr/bin/file"
# operation using sudo, for each file in the list.  However that took a
# lot of time - several seconds.  To save that time, we run a single
# sudo on all of the files and read the results into a hash for each
# file.  That allows us to test for ascii, empty, binary, directory,
# etc.  The %types hash can then be used with simple reg-exp tests for
# various purposes.  

my %types = () ; # For example, $types{'messages'} eq 'ASCII text' ;

# Pipe the output of the sudo-file command into a readable stream.
open( TYPES, '-|', '/usr/bin/sudo /usr/bin/file * ..' ) ;

# Read and process the lines from the stream.
while (<TYPES>)
{
  chomp ;   # Drop newline

  # Each line looks like
  #   fname:     File Type
  # Split on ': *' to split at the : and drop any spaces after it.
  # Only split into two fields.
  my ($name, $type ) = split ': *', $_, 2 ;

  $type = 'wtmp' if ($name =~ /^wtmp/) ;

  $types{$name} = $type ;

}
close( TYPES ) ;



# Go through the entire list of files in the current directory, and
# extract those that are directories.
# Also create a list of the regular, non-directory, files.  
#
# glob produces stuff in name order, which is preserved in both
# sections.  Alternate sorting within sections may be done later.  

my @dirfiles = () ;
my @regfiles = () ;

for my $f (glob '*')
{
  if ($types{$f} =~ 'directory') 
  {
    push @dirfiles, $f ;
  } 
  else
  {
    push @regfiles, $f ;
  }
}

# We only sort the directories if "time" sort is selected.  If so, we
# sort by oldest files first.
# The sort function is fed the Modification Age (-M) of the files.
# That value is age in Days (including fractional days) of each file.
if ($sortBy eq 'time')
{
  @dirfiles = sort {(-M $b) <=> (-M $a)} @dirfiles
}


# Sort the regular files if something besides Name order is required.
if ($sortBy eq 'size')
{
  @regfiles = sort {(-s $b) <=> (-s $a)} @regfiles
}
elsif ($sortBy eq 'sizer')
{
  @regfiles = sort {(-s $a) <=> (-s $b)} @regfiles
}
elsif ($sortBy eq 'time')
{
  @regfiles = sort {(-M $b) <=> (-M $a)} @regfiles
}
elsif ($sortBy eq 'timer')
{
  @regfiles = sort {(-M $a) <=> (-M $b)} @regfiles
}


# If this isn't the top of the tree, insert "..", the parent directory,
# in the front of the list.
if ($dirpath ne "/")
{
  splice @dirfiles, 0, 0, ( '..' ) ;
}

# Go through each of the files/dirs and generate its line in the table.
# Each file gets an Icon.  A target is determined, which is used if the
# name or icon is clicked.
for my $f (@dirfiles, @regfiles)
{
  my $icon ;
  my $isRegularFile = 1 ;
  my $target ;
  my $erase ;

  if ($f eq '..')
  {
    $icon = '/icons/back.gif' ;
    $isRegularFile = 0 ;
    # Target is the directory level just above this one.
    $target = $dirpath ;
    $target =~ s|[^/]*/$|| ; # Remove last element/ from path.
    $target="syslogs.cgi?sort=$sortBy&dir=".$target ;
  }
  elsif ($types{$f} =~ 'directory')
  {
    # A directory.
    $icon = '/icons/folder.gif' ;
    $isRegularFile = 0 ;
    # Target is the subdirectory.
    $target="syslogs.cgi?sort=$sortBy&dir=".$dirpath.$f."/" ;
  }
  elsif ( $types{$f} =~ '(text)|(empty)|(wtmp)' )
  {
    # A text file.
    $icon = '/icons/text.gif' ;
    # As the target, just view the file.  That will display the
    # contents below the directory listings.
    $target = "syslogs.cgi?dir=$dirpath&sort=$sortBy&view=$dirpath$f";
  }
  else
  {
    # A binary file.
    $icon = '/icons/binary.gif' ;

    # The target for a binary file is a separate script that knows
    # how to force-download a file through the browser. The URL
    # will look like:
    #   http://sysname/syslogDownload/path/to/file
    # where 'path/to/file' is a relative path from /var/log.  For
    # example, the URL might look like:
    #   http://sysname/syslogDownload/messages.1.gz
    # The URL uses an alias (in /etc/sysconfig/httpd.conf) to actually
    # call the script perl-scripts/syslogDownload.cgi, which then
    # extracts the path from the full URL.
    # Note that this doesn't actually change the webpage displayed in
    # the browser.
    $target = "/syslogDownload".$dirpath.$f ;
    $target = "syslogs.cgi?dir=$dirpath&sort=$sortBy&view=$dirpath$f";
  }
    
  # Determine the status information on the file.  We only need
  # the size and modification timestamp.
  my (undef, undef,undef, undef, undef, undef, undef, 
      $size, undef, $mtime, undef ) = stat( $Rootdir.$dirpath.$f ) ;

  # Convert modification time to date and time of day.
  my (undef,$min,$hour,$day,$month,$year) = gmtime($mtime) ;
  $month=("Jan","Feb","Mar","Apr","May","Jun",
          "Jul","Aug","Sep","Oct","Nov","Dec")[$month] ;

  # Format nicely with a fixed-length field.  Put leading zeroes on all
  # variable length numbers.  Should look like: "11-JUL-2002 19:32".
  my $date = sprintf( "%02d-%s-%04d&nbsp;%02d:%02d", 
                     $day,
                     $month,
                     $year+1900,
                     $hour,
                     $min ) ;

  # If this is ".." then it is the parent directory.  Rename it to
  # make that clearer.
  $f = 'Parent Directory' if ($f eq '..') ;


  # Format the row of data associated with this file.
  # We use the "code()" function to force the strings on the web-page
  # to be in a mono-spaced font.
  # NOTE the following code is written in slighly obscure fashion in
  # order to avoid excessive html function calls that took too much 
  # time.
  print "<TR>\n" ;
  print '<td> <a href ="',
        $target,
        '"> <img border="0" src="',
        $icon,
        '"> </a> </td> <td> <a href ="',
        $target,
        '"> <code>',
        $f,
        '</code> </a> </td> <td></td> <td> <code>',
        $date,
        '</code> </td>', 
        ;

  # That's all if this is a directory.  Otherwise we want to show
  # filesize, and an erase button.
  # NOTE the following code is written in slighly obscure fashion in
  # order to avoid excessive html function calls that took too much 
  # time.
  if ( $isRegularFile )
  {
    print '<td></td> <td align="right"> <code>', 
          $size,
          '</code> </td> <td></td> <td align="center"> <a href="syslogs.cgi?sort=', 
          $sortBy, 
          '&erase=',
          $dirpath,$f,
          '"> <img border="0" alt="erase" src="/images/redX.gif"> </a> </td>' ;
  }
 
  # End of the Row.
  print "</TR>\n" ;
}

# End of the Table.
print "</Table>\n", hr() ;



# If this is a request to view a specific ascii log file, then display
# it.  But by default we show the /var/log/error_log file.
my $file = param('view') || '/error_log' ;

# - A title.
# - A button that scrolls us to the end of the page.  Sadly, this isn't
#   a clean implementation.  No good method (that isn't browser
#   specific) to find the true end of the document.  So we just scroll
#   down ten million pixels and hope.  Also, this method fails if the
#   whole document isn't yet loaded in.  Oh well.  It's just a test
#   function.

my $target = "/syslogDownload".$file ;
print startform(),
      table( {width=>"100%"},
           Tr( td( b( ($file =~ /\.gz$/) 
                        ? "Gunzipped "
                        : "",
                      "Contents of ", 
                      a( {href=>$target}, 
                         $Rootdir.$file
                       ),
                      " :" 
                    ) 
                 ),
               td(  {align=>'right'},
                    button( -value=>'To End',
                            -onClick=> "scrollTo( 0,10000000) "
                          ),
                 )
             )
           ),
      endform() ;

# Use mono-space font and preserve formatting of file.
print "<pre>" ; 

if ($file =~ /^\/wtmp/)
{
  open THEFILE,  "-|", "/usr/bin/last -f $Rootdir$file"  ;
}
elsif($file =~ /\.gz$/)
{
  open THEFILE,  "-|", "/bin/gunzip -c $Rootdir$file"  ;
}
else
{
  # Open and print the file one line at a time.  Since the Apache user
  # can't necessarily read the file, we have to use a sudo-cat to gain
  # access.  Sudo is run as a separate process and piped to this script
  # for reading. 
  open THEFILE,  "-|", "/usr/bin/sudo /bin/cat $Rootdir$file"  ;
}

print while <THEFILE> ;
close THEFILE ;

print "</pre>" ;

# Emit some end of page stuff.  This is especially useful if the file
# we just displayed is very long and painful to scroll around in.
# - Indicator of what file we jsut displayed.
# - A button that will refresh the data in this log file, and 
#   automatically scroll down to the end of it.
# - A button to scroll back to the top of the page.  (Complement of the
#   "To End" button at the top.)
print b( "End of Contents of $Rootdir$file" ),
      startform(),
      table( {width=>"100%"},
          Tr( td( button( -value=>'Refresh',
                          -onClick=> "parent.loadDataFrame( "
                                      ."'/perl-scripts/syslogs.cgi"
                                      ."?dir=$dirpath&view=$file#end' )"
                        )
                ),
              td( {align=>'right'},
                  button( -value=>'To Top',
                          -onClick=> "scrollTo( 0,0) "
                        )
                )
            ),
           ),
      endform(),
      a( {name=>"end"} ) ;

print end_html() ;

EXIT:
# end of script syslogs.cgi
