#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: waas.cgi,v 1.9 2006/03/23 19:09:02 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/waas.cgi,v $

########################################################################
# waas.cgi
#
# Shows controls for tracking and usage of WAAS satellites.  
# Can be used to completely disable WAAS, and to enable or disable
# individual WAAS satellites.
#
# This unsecured script is called with no parameters.  It just puts
# up the form showing the current state of the controls.  When the
# user clicks "OK" the data is submitted to a secured script in
# gpsadmin/setWaas.cgi.
#
# Each satellite's state can be either Disabled, Enabled, or 
# IgnoreHealth.  The checkbuttons on the page appear to be independent
# but are actually linked.  Ignorehealth also implies Enabled.  And
# disabled implies not ignore health.  We enforce this a bit on the
# GUI with a bit of javascript.  
#
# The checkbox names are based on the satellite numbering scheme used
# by Stinger.  PRNs 120 through 138 are Waas/Egnos style satellites.
# These are mapped to labels like 'WAAS POR' on the displayed page.  IF
# WE PUBLISH THIS CGI INTERFACE, we'll need to rethink that naming.
# It's too implementation dependent.
#
# The Ignore health controls might not be needed in the long run.  See
# Stuart Riley for details on that.  For the moment I've left
# IgnoreHealth installed pending further discussions.
#
# The WAAS controls are available from the gpsinfo system, vis the
# program sysinfo.  
#   sysinfo -p waas_mode
#      =>  put waas_mode [on|off]
#
#   sysinfo -p waas_sv_masks
#      =>  waas_sv_masks D D D D D D D D D D D D D D E D D D D
########################################################################


use strict ;
use CGI qw/:standard/;

use Fireball ;

titleWithHelp( 'Satellites - WAAS Controls',
                         '/satWAASEnaDisInfo.html'
                       ) ;


# Use the sysinfo utility to obtain the current settings for the
# WAAS controls.

$_ = qx( /usr/bin/sudo /usr/local/fireball/sysinfo -p waas_sv_masks ) ;
# Return string should be of the form
#   put waas_sv_masks D D D D D D D D D D D D D D E D D D D

my @state = split( " ", $_ ) ;
# Drop the "put sv_masks" portion.
splice( @state, 0, 2 ) ;

my $waasCount = scalar( @state ) ; # Should be 19!
#print join( ',', @state), $waasCount, br() ;


$_ = qx( /usr/bin/sudo /usr/local/fireball/sysinfo -p waas_mode ) ;
# Response will be
#  put waas_mode on         (or off)

my $WaasEnabled = ( (split)[2] eq 'on') ? 'On' : 'Off' ;
#print "Waas Enabled: $WaasEnabled", br() ;


# Set all of the CGI parameters that control the states of the
# buttons.  These are set in advance of actually defining the buttons so
# that the defaults of the buttons will be controlled by the current
# state of the stinger controls.
for ( my $i = 0 ; $i < $waasCount ; ++$i )
{
  param( 'Enable'.($i+120), 
         ($state[$i] ne 'D') ? 1 : 0 
       ) ;

  param( 'Ignore'.($i+120), 
         ($state[$i] eq 'I') ? 1 : 0 
       ) ;
}

# Create an html form.  Submit the data to the setWaas.cgi script.
print startform( -name=>'theForm', 
                 -action=>'/perl-scripts/gpsadmin/setWaas.cgi'
               ) ;

my @WaasControl = radio_group( -name=>'WaasControl', 
                         -value=>[ 'On', 'Off' ],
                         -default=>$WaasEnabled,
                        ) ;
print table( 
             Tr( th( {colspan=>'2'}, 'WAAS Tracking:' ) ), 
             Tr( {align=>'center'},td( $WaasControl[0] ), td( $WaasControl[1] ) ),
           ) ;

print hr() ;

# The array of satellite names and enable/ignoreHealth checkboxes will
# be formatted in a table with two major columns, one for the lower
# numbered satellites and one for the upper numbers.  Actually, this is
# done with multiple columns to keep the checkboxes and everything
# nicely aligned.

print '<table border="0">' ;

# Print headers above the columns.
print th( "Satellite" ),
      th( "&nbsp;&nbsp;Enable&nbsp;&nbsp;" ),   
      th( "Ignore<br />Health" ),

      # A spacer column between the two sections
      th( "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" ),

      th( "Satellite" ),
      th( "&nbsp;&nbsp;Enable&nbsp;&nbsp;" ),
      th( "Ignore<br />Health" ),
      ;

# A loop that runs for each row of the table.
for ( my $i = 0 ; $i < 10 ; ++$i )
{
  print "<tr>" ;

  # Within each row we put two satellite.
  for my $prn ( $i+120, $i+130 )
  {
    # If this prn is beyond the valid range, exit.
    # This will probably leave a blank spot in the table.
    last if ($prn >= 120+$waasCount ) ;

    # Determine the name to show.  If this satellite prn is known to
    # represent a real satellite, put its real name here.
    my $name = $waasStrings{$prn} ;
    print td( {align=>'center'},$name )  ;
    
    # The next two columns have the Enable and IgnoreHelath
    # checkboxes.  Each of these is given a unique name base
    # on the PRN.  THose names become the parameters fed
    # back to the cgi script when OKAY is pressed.
    # Each button, when clicked, calls a javascript function
    # with the prn as a parameter.  The functions implement
    # the linkage between the two buttons.  See the script
    # definitions below for details.
    print td( {align=>'center'}, 
              checkbox( -name=>"Enable$prn", 
                        -value=>'1', 
                        -label=>'', 
                        -onClick=>"enClick($prn)" 
                      ) 
            ) ;
    print td( {align=>'center'}, 
              checkbox( -name=>"Ignore$prn", 
                        -value=>'1', 
                        -label=>'',
                        -onClick=>"igClick($prn)" 
                      ) 
            ) ;
    print td() ;
  }
  print "</tr>" ;
}

print '</table>' ;

print hr() ;

# No action was done. In other words this invocation is just 
# displaying the form for accepting new data.  Put up the buttons.
# Okay submits the data.  Cancel goes back to the receiver
# configuration page.
print button( -name=>'OK', 
              -value=>'OK',
              -onClick=>'parent.submitForm( document.theForm, "" )',
            ) ;
print '&nbsp;' ;
print button( -name=>'Cancel', 
              -value=>'Cancel',
              -onClick=>
             'parent.loadDataFrame( "/perl-scripts/rconfigDisplay.cgi")'
            ) ;

print endform() ;

# Emit two javascript functions which are called when any of the
# checkboxes are clicked.
print <<EOF ;
<Script>

/**********************************************************************/
function igClick( prn )
/**********************************************************************/
/* Called when an ignore checkbox is clicked, with the prn number as
 * a parameter.  If the Ignore button is being enabled, we automatically
 * click the matching Enabled button as well.
 */
{
  en = document.theForm[ "Enable" + prn ] ;
  ig = document.theForm[ "Ignore" + prn ] ;

  if (ig.checked)
    en.checked = true ;
} /* end of Update() */


/**********************************************************************/
function enClick( prn )
/**********************************************************************/
/* Called when an Enable checkbox is clicked, with the prn number as
 * a parameter.  If the Enable button is being disabled, we automatically
 * uncheck the matching IgnoreHealth button as well.
 */
{
  en = document.theForm[ "Enable" + prn ] ;
  ig = document.theForm[ "Ignore" + prn ] ;

  if (! en.checked)
    ig.checked = false ;
} /* end of Update() */



</Script>
EOF
 
print end_html() ;

########################################################################
# end of file waas.cgi
########################################################################
