#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: warranty.cgi,v 1.15 2005/02/16 22:25:07 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/warranty.cgi,v $

########################################################################
# warranty.cgi
#
# Script to allow managing the firmware warranty license, and 
# perhaps the receiver serial number.  For extra details (if you are 
# lucky) see fireball src/apps/warranty/README.  But until that's
# generated, this will have to do:
#
# The receiver has a serial number available via 'warranty -s'.  This is
# a decimal ten digit number.
#
# Can be called one of several ways:
#   o With no CGI parameters - 
#       We put up a form waiting for entry of data.
#
#   o With CGI parameters specifying an attempt to change the 
#       warranty or serial number.  
#
########################################################################


use CGI qw/:standard/;
use strict ;
use Fireball ;

titleWithHelp( 'Firmware Warranty Date',
               '/warranty.html'
             ) ;


########################################################################
# Process any CGI parameters fed into the script.
#
# This block may detect errors which will halt further parameter
# processing.  If this happens, we set the $error, and use a
# "goto paramDone" to exit the block.  Inelegant, I know, but
# it's the most straightforward method and avoid lots of extraneous 'if'
# statements.
# The error string is saved in $error (which is blank otherwise) so that
# the value can be displayed at the end of the returned page.
########################################################################
my $error = "" ;
my $action = "" ;

if (param())
{
  
  #print 'Parameters:<br />' ;
  #for (param())
  #{
  #  print "$_ = ",param($_),'<br />' ;
  #}

  if ( my $newSerialNumber = param("serialNumber") )
  {
    if ($newSerialNumber !~ /^\d{10}$/ )
    {
      $error = "Serial number must have 10 digits" ;
      goto paramDone ;
    }
    $action = "New Serial Number is: $newSerialNumber<br />" ;

    system( "/usr/bin/sudo /usr/local/fireball/warranty -p"
            . $newSerialNumber ) ;

    # If the current system name was based on an unknown serial number,
    # we'd like to change it to match the new serial number.
    # This is indicated by the sysname 'RS0000000000'.
    my $sysname = qx( cat /etc/sysconfig/sysname ) ;
    chomp $sysname ;
    if ($sysname eq 'RS0000000000')
    {
      # Define a new sysname of the form: 'RS1234567890'.
      $sysname = sprintf( 'RS%010d', $newSerialNumber ) ;

      # Put it into the configuration file.
      open SYSNAME, '>/tmp/sysname' ;
      print SYSNAME "$sysname\n" ;
      close SYSNAME ;
      # Copy the new file on top of the old one using sudo.  This
      # gives us permission to do the copy, and preserves all of the 
      # original file characteristics.
      system( 'sudo cp /tmp/sysname /etc/sysconfig/sysname' ) ;

      # Since the system name has changed, make the hostname follow it.
      system( "sudo /bin/hostname $sysname" ) ;
    }
  }

  if ( my $warrantyCode = param( "warrantyCode" ) )
  {
    if ($warrantyCode !~ /^\w{16}$/ )
    {
      $error = "Warranty code must have 16 digits" ;
      goto paramDone ;
    }
    my $result = system( "/usr/bin/sudo /usr/local/fireball/warranty -f"
                         . $warrantyCode ) ;
    if ($result == 0)
    {
      $action .= "Warranty Code accepted.<br />" ;
    }
    else
    {
      $error = "Invalid Warranty code." ;
    }
  }
} 
paramDone:  # This is the exit point if parameter processing was
            # aborted in the preceding block.


########################################################################
# When we get to here, either this is a first invocation of the script,
# or something was wrong with the user's input.  Redraw the forms
# (perhaps with an error message alert) and wait for further input.
########################################################################



# warranty -s returns:
#   Serial_number: 5432198765
# Extract the numeric string.
my $serialNumber = qx(/usr/bin/sudo /usr/local/fireball/warranty -s) ;
chomp $serialNumber ; # Drop linefeed
$serialNumber = (split("[ :]+",$serialNumber))[1] ;


# Emit some javascript to update the title block of the GUI 
# that shows the serial number.
print <<EOF ;
  <script>
  parent.updateSerialNumber( "$serialNumber" ) ;
  </script>
EOF


# warranty -w returns:
#   war date :03312003
# (Yes, I know that's incredibly inconsistent with the serial number,
# but don't blame me!  I didn't write it.)
my $warrantyDate = qx(/usr/bin/sudo /usr/local/fireball/warranty -w) ;
chomp $warrantyDate ; # Drop linefeed
$warrantyDate = (split("[ :]+",$warrantyDate))[2] ;

if ($warrantyDate != 0)
{
  my ($month,$day,$year) = ($warrantyDate =~ /(..)(..)(....)/) ;
  $month = $monthStrings[ $month-1 ] ;

  $warrantyDate = "$day $month $year" ;
}
else
{
  $warrantyDate = "none" ;
}



# Create the form with fields for account data entry.
# A table is used to align labels, text entry fields, and the button on
# a central line.
print startform( -name=>"theForm" ) ;

print '<table border="0">' ;

print Tr( td( {align=>'right'}, "Serial Number:" ),
          th( $serialNumber )
        ) ;
print Tr( td( {align=>'right'}, "Current Warranty Date:" ),
          th( $warrantyDate )
        ) ;

# If there is no serial number already, then allow the user to specify
# one.
if ( $serialNumber == 0 )
{
  print Tr( td( {-align=>'right'}, 'New Serial Number:' ),
            td( {-align=>'left'},
                textfield( 'serialNumber', '', 16, 10 ) 
              )
          ) ; 
}

print Tr( td( {-align=>'right'}, 'New Warranty Code:' ),
          td( {-align=>'left'},
              textfield( 'warrantyCode', '', 16, 16 ) 
            )
        ) ; 
print Tr( td( '' ),
          td( 
              button( -name=>'OK', 
                      -value=>'OK',
                      -onClick=> 'parent.submitForm( document.theForm,
                                         "/perl-scripts/warranty.cgi" )'
                    ),

              button( -name=>'Cancel', 
                      -value=>'Cancel',
                      -onClick=>
                  'parent.loadDataFrame( "/perl-scripts/firmware.cgi" )'
                    )
            ) 
        ) ;
      
print "</table>" ;
         

print endform() ;

# If an error message was found earlier during CGI processing,
# post an alert message to the user.  Also display the message on the
# bottom of the page, in case the javascript alert system doesn't work.
if ( $error )
{
  print << "EOF" ;
  <Script> 
  alert( "$error" ) ;
  </Script>
EOF

  print "$error" ;
}

if ( $action )
{
  print $action ;
}

print end_html() ;

########################################################################
# end of file warranty.cgi
########################################################################
