########################################################################
# Copyright (c) 2003 Trimble Navigation Limited
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/sysadmin/Commands.pm,v $
# $Id: Commands.pm,v 1.102 2007/03/28 23:34:34 tom Exp $
########################################################################

########################################################################
# This file contains the Commands perl module used by the Fireball
# CGI/programmatic control system.  It contains a large number of
# subroutines that are used to implement the commands that are
# interpretted by the programmatic system.
#
# To use this library, a CGI script must include the library with the 
# command :
#
#   use Commands ;
#
# These subroutines could just as well exist in the commandHandler.cgi
# script, but that script would get over-whelmingly large and hard to
# maintain.  So stuff that is stable gets moved to here.  For
# development purposes you may want to consider the following process:
#   - Add a new command to the parser in prog/commandhandler.cgi.
#   - Add the matching subroutine in that file, and get it working.
#   - Once it is debugged, move the subroutine into this file.
#     Add it to the EXPORT list below.
#   - Restart the web server on your Fireball. 
#       This causes the library to be re-read when the
#
########################################################################

package Commands ;
require Exporter ;

use strict ;
use IO::Socket;
use lib '..' ;
use Fireball ;
use Time::Local ;

our @ISA = qw( Exporter ) ;
# These subroutines are all exported by default.
our @EXPORT = qw( badParmCount
                  plainTextHeader
                  ShowPps
                  SetPps
                  ShowElevationMask
                  SetElevationMask
                  ShowClockSteering
                  SetClockSteering
                  ShowMultipathReject
                  SetMultipathReject
                  ShowPdopMask
                  SetPdopMask
                  ShowSerialNumber
                  ShowSystemName
                  SetSystemName
                  ShowTemperature
                  ShowVoltage
                  ShowFirmwareVersion
                  ShowFirmwareWarranty
                  ShowFirmwareFiles
                  ShowFirmwareFile
                  UploadFirmwareFile
                  VerifyFirmwareFile
                  InstallFirmwareFile
                  DeleteFirmwareFile
                  ShowPowerSaveMode
                  SetPowerSaveMode
                  ShowFailsafeWakeup
                  SetFailsafeWakeup
                  ShowGpsHealth
                  ShowGpsSatControls
                  SetGpsSatControls
                  ShowTrackingStatus
                  ShowWaasSatControls
                  SetWaasSatControls
                  ShowWaasTracking
                  SetWaasTracking
                  ShowReferenceFrequency
                  SetReferenceFrequency
                  ShowLoggedFiles
                  ShowLoggedFile
                  DeleteLoggedFiles
                  DeleteLoggedFile
                  CreateConfigFile
                  DeleteConfigFile
                  UploadConfigFile
                  ShowConfigFile
                  ShowConfigFiles
                  InstallConfigFile
                  ResetSystem
                  ResetGpsData
                  ShowFtpSetup
                  SetFtpSetup
                  ShowGuiSecurity
                  SetGuiSecurity
                  ShowGuiAccount
                  ShowGuiAccounts
                  SetGuiAccount
                  DeleteGuiAccount
                  ShowAntennaTypes
                  ShowAntenna
                  SetAntenna
                  ShowEthernet
                  SetEthernet
                  EnableSession
                  DisableSession
                  ShowSession
                  ShowSessions
                  SetSession
                  DeleteSession
                  ShowIOport
                  DeleteIOport
                  SetIOport
                  ShowIOports
                  ShowUtcTime
                  ShowGpsTime
                  ShowLoggedFileStats
                  ShowLoggedFilePools
                  SetFirmwareWarranty
                  ShowPosition
                  ShowLoggingStatus
                  DownloadLoggedFile
                  ShowGpsEphemeris
                  ShowGpsAlmanac
                  ShowGpsUtcData
                  ShowGpsIonoData
                  ShowRefStation
                  SetRefStation
                  ShowNtpClient
                  SetNtpClient
                  ShowIoStatus
                  ShowL2Tracking
                  SetL2Tracking
                  ShowL2C
                  SetL2C
                  ShowAutodelete
                  SetAutodelete
                  ShowClientAuthKey
                  SetClientAuthKey
                  ShowIpFiltering
                  SetIpFiltering
                  ShowHttpPorts
                  SetHttpPorts
                  ShowPortAdvertising
                  SetPortAdvertising
                  ShowPppDialout
                  SetPppDialout
                ) ;

use CGI qw/:standard/ ;

########################################################################
sub plainTextHeader
########################################################################
# Generate the http response header describing the return document
# as a simple plain text response.  This is sent in all programmatic
# responses, except those that return a binary document.
########################################################################
{
  # Send the HTTP header information, followed by a single blank line.
  # This defines the content of the file as plain ascii text, so that
  # any browser who happens to read it will just display the contents
  # without interpretation.
  #
  # Also we'd like to have a expiration date on the files, set way in
  # the past, so that they will always be updated when a new version is
  # requested.  The responses contain dynamic data that might change.
  # Don't want any proxy server's caching the data, etc.  Currently it
  # is untested whether I have the Expires field working right.
  #
  # And we would like to force the HTTP status code that is returned to
  # a 200 = Everything is okay.  
  print CGI::header( -type=>'text/plain',
                     -expires=>'+0s',
                     -status=>'200 Okay',
                   ) ;
} # end of plainTextHeader()



########################################################################
sub badParmCount
########################################################################
# Used by a command subroutine to verify the number of
# parameters passed in against a minimum and maximum.
# $_[0] = the count of parameters
# $_[1] = minimum count of parameters
# $_[2] = maximum count of parameters
# Returns 0:false if paramcount is in range.
# Otherwise, prints an error message and returns true.
# Designd to be used in an idiom:
#
#   return if ( badParmCount( scalar( @_ ), 0, 1 ) ) ;
#
{
  if ($_[0] < $_[1]) 
  {
    print "ERROR: Too Few Parameters.\n" ;
    return 1 ;
  }
  elsif ($_[0] > $_[2]) 
  {
    print "ERROR: Too Many Parameters\n" ;
    return 1 ;
  }

  return 0 ;
} # end of checkParmCount



########################################################################
sub ShowFailsafeWakeup
########################################################################
{
  #print "Showing FailsafeWakeup\n" ;
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  # Obtain the current setting from the config file.
  my ($Enable,$Periodic,$Hours,$Minutes) ;
  if ( open( SLEEP, '/etc/sysconfig/sleepmode' ) )
  {
    my $line=<SLEEP> ;
    close SLEEP ;
    chomp $line ;
    ($Enable,$Periodic,$Hours,$Minutes) = split( ' ', $line ) ;
  }

  print "FailsafeWakeup enable=$Periodic "
        . "interval=$Hours duration=$Minutes\n" ;

} # end of ShowFailsafeWakeup



########################################################################
sub SetFailsafeWakeup
########################################################################
{
  #print "Setting FailsafeWakeup\n" ;
  return if ( badParmCount( scalar( @_ ), 3, 3 ) ) ;

  my %parms ;
  for my $p (@_)
  {
    if ($p !~ /=/)
    {
      print "ERROR: Must fully specify id=value for parameter $p\n" ;
      return ;
    }
    my ($id, $value) = split( '=', $p ) ;
    $parms{$id} = $value ;
    #print "$id  =  $value\n" ; 
  }

  my $newEnable = $parms{'enable'} ;
  my $newInterval = $parms{'interval'} ;
  my $newDuration = $parms{'duration'} ;


  if ( !$newEnable 
    || !$newInterval 
    || !$newDuration 
    || $newEnable !~ /^(yes|no)$/
    || $newInterval !~ /^\d+$/
    || $newDuration !~ /^\d+$/
    || $newInterval > 24
    || $newDuration > 60
     )
  {
    print "ERROR: Invalid parameters\n" ;
    return ;
  }

  # Obtain the current settings from the config file.
  # First  enable is for Power Saving mode.
  # Second enable is for FailSafe
  my ($PSenable,$FSenable,$hours,$minutes) = ('no','yes',3, 5 ) ;
  if ( open( SLEEP, '/etc/sysconfig/sleepmode' ) )
  {
    my $line=<SLEEP> ;
    close SLEEP ;
    chomp $line ;
    ($PSenable,$FSenable,$hours,$minutes) = split( ' ', $line ) ;
  }

  # Regenerate the config file.
  open( SLEEP, '>/tmp/sleepmode' ) ;
  print SLEEP join( ' ', 
                    $PSenable,
                    $newEnable, 
                    $newInterval,
                    $newDuration,
                  ), 
              "\n" ;
  close SLEEP ;
  system( 'sudo cp /tmp/sleepmode /etc/sysconfig' ) ;

  print "OK: " ;
  ShowFailsafeWakeup() ;

} # end of ShowPowerSaveMode




########################################################################
sub ShowPowerSaveMode
########################################################################
{
  #print "Showing PowerSaveMode\n" ;
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  # Obtain the current setting from the config file.
  my ($Enable,$Periodic,$Hours,$Minutes) ;
  if ( open( SLEEP, '/etc/sysconfig/sleepmode' ) )
  {
    my $line=<SLEEP> ;
    close SLEEP ;
    chomp $line ;
    ($Enable,$Periodic,$Hours,$Minutes) = split( ' ', $line ) ;
  }

  print "PowerSaveMode enable=$Enable\n" ;

} # end of ShowPowerSaveMode



########################################################################
sub SetPowerSaveMode
########################################################################
{
  #print "Showing PowerSaveMode\n" ;
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $newstate = lc( $_[0] ) ;
  $newstate =~ s/^enable=// ;
  #print "newstate=$newstate\n" ;
  if ($newstate !~ m/^(yes|no)$/)
  {
    print "ERROR: Invalid parameter $_[0]\n" ;
    return ;
  }

  # Obtain the current settings from the config file.
  my ($enable,$periodic,$hours,$minutes) = ('no','yes',3, 5 ) ;
  if ( open( SLEEP, '/etc/sysconfig/sleepmode' ) )
  {
    my $line=<SLEEP> ;
    close SLEEP ;
    chomp $line ;
    ($enable,$periodic,$hours,$minutes) = split( ' ', $line ) ;
  }

  $enable = $newstate ;

  open( SLEEP, '>/tmp/sleepmode' ) ;
  print SLEEP join( ' ', 
                    $enable,
                    $periodic, 
                    $hours,
                    $minutes,
                  ), 
              "\n" ;
  close SLEEP ;
  system( 'sudo cp /tmp/sleepmode /etc/sysconfig' ) ;


  print "OK: " ;
  ShowPowerSaveMode() ;

} # end of SetPowerSaveMode




########################################################################
sub ShowPps
########################################################################
{
  #print "Executing ShowPps\n" ;
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;
  my (undef, undef, $pps_enable, $pps_slope) 
     = split( " ",  
            qx(  /usr/bin/sudo /usr/local/fireball/sysinfo -p pps_mode )
            ) ;

    print "PPS enable=",
          $pps_enable eq 'on' ? 'yes' : 'no',
          " slope=",
          ($pps_slope eq 'pos' ? 'positive' : 'negative' ),
          "\n" ;

} # end of ShowPps




########################################################################
sub SetPps
########################################################################
# setPps enable=yes slope=positive
#
# enable = 'yes' or 'no'.  Must be provided.
#
# 'Slope=' is currently only accepted as 'positive' (due to a hardware 
# limitation in NetRS which won't generate a negative pulse).  The 
# parameter is optional.
#
{
  #print "Executing SetPps\n" ;
  return if ( badParmCount( scalar( @_ ), 1, 2 ) ) ;

  my $newstate = '' ;

  foreach my $p (@_)
  {
    $p = lc($p) ; # Make everything lower case.

    if ($p =~ m/^enable=(yes|no)$/ )
    {
      $p =~ s/^enable=// ;
      $newstate = $p ;
    }
    elsif ($p eq 'slope=positive' )
    {
      # Currently only accept a positive slope, and we hard-code
      # that below.  No need to preserve this information.
    }
    else
    {
      print "ERROR: Bad Parameter: $p\n"  ;
      return ;
    }
  }

  if (!$newstate)
  {
    print "ERROR: Missing 'enable=' Parameter\n"  ;
    return ;
  }

  #print "Setting PPS to $newstate,$newslope\n" ;
  newStingerConfig( 'put pps_mode '
                    . (($newstate eq 'yes') ? 'on' : 'off')
                    ." pos\n"
                  ) ;


  print "OK: " ;
  ShowPps() ;
} # end of SetPps



########################################################################
sub ShowElevationMask
########################################################################
{
  #print "Executing ShowElevationMask\n" ;
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  # Use sysinfo -p elev_mask to get the current setting.
  # Response is of the form
  #   put elev_mask 12.345
  (undef, undef, my $elevMask, undef) 
    = split( ' ',
             qx( /usr/bin/sudo /usr/local/fireball/sysinfo -p elev_mask)
           );

  print "ElevationMask mask=", $elevMask, "\n" ;

} # end of ShowElevationMask




########################################################################
sub SetElevationMask
########################################################################
{
  #print "Executing SetElevationMask\n" ;
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $newmask = lc( $_[0] ) ;
  $newmask =~ s/^mask=// ;
  #print "newmask=$newmask\n" ;
  if ($newmask !~ m/^-?\d+$/)
  {
    print "ERROR: Invalid parameter $_[0]\n" ;
    return ;
  }

  if ($newmask > 90 || $newmask < -10)
  {
    print "ERROR: Parameter Out of Range: $newmask\n"  ;
    return ;
  }

  #print "Setting mask to $newmask\n" ;

  newStingerConfig( "put elev_mask $newmask\n" ) ;

  print "OK: " ;
  ShowElevationMask() ;

} # end of SetElevationMask



########################################################################
sub ShowClockSteering
########################################################################
{
  #print "Executing ShowClockSteering\n" ;
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;


  # Obtain current setting from gpsinfo system, using sysinfo.
  # Response will be of the forms:
  #  put clock_steering on 
  #  put clock_steering off
  $_ = qx( /usr/local/fireball/sysinfo -p clock_steering ) ;
  chomp ;

  my (undef, undef, $CSstate) = split( " ", $_ ) ;

  print "ClockSteering enable=",
        ( ($CSstate eq 'on') ? "yes\n" : "no\n" ) ;

} # end of ShowClockSteering



########################################################################
sub SetClockSteering
########################################################################
{
  #print "Executing SetClockSteering\n" ;
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $newstate = lc( $_[0] ) ;
  $newstate =~ s/^enable=// ;
  #print "newstate=$newstate\n" ;
  if ($newstate !~ m/^(yes|no)$/)
  {
    print "ERROR: Invalid parameter $_[0]\n" ;
    return ;
  }

  newStingerConfig( 'put clock_steering '
                     .( ($newstate eq 'yes') ? "on\n" : "off\n")
                  ) ;

  print "OK: " ;
  ShowClockSteering() ;

} # end of SetClockSteering



########################################################################
sub ShowMultipathReject
########################################################################
{
  #print "Executing ShowMultipathReject\n" ;
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  # Obtain current setting from gpsinfo system, using sysinfo.
  # Response will be of the forms:
  #  put everest on 
  #  put everest off
  $_ = qx( /usr/local/fireball/sysinfo -p everest ) ;
  chomp ;

  my (undef, undef, $MPstate) = split( " ", $_ ) ;
  print "MultipathReject enable=",
        ( ($MPstate eq 'on') ? "yes\n" : "no\n" ) ;

} # end of ShowMultipathReject



########################################################################
sub SetMultipathReject
########################################################################
{
  #print "Executing SetMultipathReject\n" ;
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $newstate = lc( $_[0] ) ;
  $newstate =~ s/^enable=// ;
  #print "newstate=$newstate\n" ;
  if ($newstate !~ m/^(yes|no)$/)
  {
    print "ERROR: Invalid parameter $_[0]\n" ;
    return ;
  }

  newStingerConfig( 'put everest '
                   .( ($newstate eq 'yes') ? "on\n" : "off\n")
                  ) ;

  print "OK: " ;
  ShowMultipathReject() ;

} # end of SetMultipathReject



########################################################################
sub ShowPdopMask
########################################################################
{
  #print "Executing ShowPdopMask\n" ;
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  # Use sysinfo -p pdop_mask to get the current setting.
  # Response is of the form
  #   put elev_mask 12.345
  (undef, undef, my $pdopMask, undef) 
    = split( ' ',
             qx( /usr/bin/sudo /usr/local/fireball/sysinfo -p pdop_mask)
           );

  print "PdopMask mask=", $pdopMask, "\n" ;

} # end of ShowPdopMask




########################################################################
sub SetPdopMask
########################################################################
{
  #print "Executing SetPdopMask\n" ;
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $newmask = lc( $_[0] ) ;
  $newmask =~ s/^mask=// ;
  #print "newmask=$newmask\n" ;
  if ($newmask !~ m/^\d+$/)
  {
    print "ERROR: Invalid parameter $_[0]\n" ;
    return ;
  }

  if ($newmask > 99)
  {
    print "ERROR: Parameter Out of Range: $newmask\n"  ;
    return
  }

  #print "Setting mask to $newmask\n" ;

  newStingerConfig( "put pdop_mask $newmask\n" ) ;

  print "OK: " ;
  ShowPdopMask() ;

} # end of SetPdopMask




########################################################################
sub ShowSerialNumber
########################################################################
{
  #print "Executing ShowSerialNumber\n" ;
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  # Determine a serial number by querying the "warranty" command.
  # warranty -s returns:
  #   Serial_number: 5432198765
  # Extract the numeric string.
  my $serialNumber = qx(/usr/bin/sudo /usr/local/fireball/warranty -s) ;
  chomp $serialNumber ;   # Drop linefeed
  $serialNumber = (split("[ :]+",$serialNumber))[1] ;

  print "SerialNumber sn=$serialNumber\n" ;

} # end of ShowSerialNumber




########################################################################
sub ShowSystemName
########################################################################
{
  #print "Executing ShowSystemName\n" ;
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  # Get the system identifier string from a config file.
  my $identityString= qx|cat /etc/sysconfig/sysname| ;
  chomp $identityString ;

  print "SystemName name=$identityString\n" ;

} # end of ShowSystemName



########################################################################
sub SetSystemName
########################################################################
{
  #print "Executing SetSystemName\n" ;
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $name = $_[0] ;

  if ( $name =~ /=/
    && $name !~ /^name=/
     )
  {
    print "ERROR: Illegal parameter $name\n" ;
    return ;
  }
  $name =~ s/^name=//i ;

  #print "NewName = $name\n" ;

  if ( $name =~ /[^A-Za-z0-9_-]/ )
  {
    # In the error message, we put a specific \n sequence.  This
    # has to be in single quotes to avoid translation to a true
    # newline character.  Otherwise, the call to Javascript alert
    # later on can't handle it.
    print "ERROR: Invalid Characters in System Name!\n" ;
    return ;
  }

  # New name is okay to use.  Put it into the configuration file.
  open SYSNAME, '>/tmp/sysname' ;
  print SYSNAME "$name\n" ;
  close SYSNAME ;

  # Copy the new file on top of the old one using sudo.  This
  # gives us permission to do the copy, and preserves all of the 
  # original file characteristics.
  system( 'sudo cp /tmp/sysname /etc/sysconfig/sysname' ) ;

  system( "sudo /bin/hostname $name" ) ;

  print "OK: " ;
  ShowSystemName() ;

} # end of SetSystemName




########################################################################
sub ShowTemperature
########################################################################
# Show the internal temperature of the NetRS.
{
  #print "Executing ShowTemperature\n" ;
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  my ( $primaryV, $secondaryV, $temperature ) = getVoltages() ;

  print "Temperature temp=$temperature\n" ;

} # end of ShowTemperature



########################################################################
sub ShowVoltage
########################################################################
# Accepts a single parameter specifying the voltage input port:
#   Show Voltage input=1 
#   Show Voltage input=2 
# For grins we also allow 
#   Show Voltage 1 
#   Show Voltage 2.
# We get the voltage from the fireball/voltage program, via the
# utility subroutine getVoltages().
{
  #print "Executing ShowVoltage\n" ;
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $input = lc( $_[0] ) ;
  $input =~ s/^input=// ;
  if ($input !~ m/^\d+$/
    || ($input != 1 && $input != 2)
     )
  {
    print "ERROR: Invalid parameter $_[0]\n" ;
    return ;
  }

  my ( $primaryV, $secondaryV, $temperature ) = getVoltages() ;


  printf "Voltage input=$input volts=%.2f\n", 
         ($input == 1) 
           ? $primaryV 
           : $secondaryV ;

} # end of ShowVoltage



########################################################################
sub ShowFirmwareVersion
########################################################################
{
  #print "Executing ShowFirmwareVersion\n" ;
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  my $fwVersion = 'Unknown' ;
  my $fwDate    = '01-Jan-2000' ;

  if ( -r '/usr/local/fireball/defaults/firmwareVersion' ) 
  {
    ($fwVersion, $fwDate) = 
           split( "\n", 
                  qx( cat /usr/local/fireball/defaults/firmwareVersion )
                ) ;
    
    my ($year,$month,$day) = split( '/', $fwDate ) ;
    $fwDate = sprintf( "%04d/%02d/%02d", $year, $month, $day ) ;
  }
  print "FirmwareVersion version=$fwVersion date=$fwDate\n" ;

} # end of ShowFirmwareVersion




########################################################################
sub ShowFirmwareWarranty
########################################################################
{
  #print "Executing ShowFirmwareWarranty\n" ;
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  # warranty -w returns:
  #   war date :03312003
  # (Yes, I know that's incredibly inconsistent with the serial number,
  # but don't blame me!  I didn't write it.)
  my $warrantyDate = qx(/usr/bin/sudo /usr/local/fireball/warranty -w) ;
  chomp $warrantyDate ; # Drop linefeed
  $warrantyDate = (split("[ :]+",$warrantyDate))[2] ;

  if ($warrantyDate != 0)
  {
    my ($month,$day,$year) = ($warrantyDate =~ /(..)(..)(....)/) ;
    $warrantyDate = sprintf( "%04d/%02d/%02d", $year, $month, $day ) ;
  }
  else
  {
    $warrantyDate = "none" ;
  }

  print "FirmwareWarranty date=$warrantyDate\n" ;

} # end of ShowFirmwareWarranty




########################################################################
sub ShowFirmwareFiles
########################################################################
{
  #print "Executing ShowFirmwareFiles\n" ;
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  my @files = sort( glob( '/var/gps/incoming/*' ) ) ;
  my $count = scalar( @files ) ;
  print "<Show FirmwareFiles>\n" ;
  #print $count, 
  #      " Firmware File", 
  #      (($count != 1) ? 's' : ''),
  #      "\n" ;

  my $maxStringLength = 0 ;

  for my $f (@files)
  {
    my $name = $f ;
    $name =~ s|^.*/|| ;
    if (length( $name ) > $maxStringLength)
    {
      $maxStringLength = length( $name ) 
    }
  }

  for my $f (@files)
  {
    my $size = -s $f ;
    $f =~ s|^.*/|| ;
    printf( "%-${maxStringLength}s  %8d bytes\n", $f, $size ) ;
  }
  print "<end of Show FirmwareFiles>\n" ;

} # end of ShowFirmwareFiles



########################################################################
sub ShowFirmwareFile
########################################################################
{
  #print "Executing ShowFirmwareFile\n" ;
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $filename = $_[0] ;
  $filename =~ s/^name=//i ;


  chdir '/var/gps/incoming' ;

  if ( ! -r $filename ) 
  {
    print "ERROR: can't read $filename\n" ;
    return ;
  }

  # Check if this file is really an RPM file.  It's first four
  # characters should contain the magic number sequence
  #     0xed 0xab 0xee 0xdb
  my @c ;
  if (open( RPMFILE, "$filename" ) )
  {
    $c[0] = ord( getc( RPMFILE ) ) ;
    $c[1] = ord( getc( RPMFILE ) ) ;
    $c[2] = ord( getc( RPMFILE ) ) ;
    $c[3] = ord( getc( RPMFILE ) ) ;
    close( RPMFILE ) ;
  }

  if ( $c[0] != 0xed
    || $c[1] != 0xab
    || $c[2] != 0xee
    || $c[3] != 0xdb
     )
  {
    print  "ERROR: File '$filename' does not appear to be an RPM file "
          ."so it cannot be installed.\n" 
          ;
    return ;
  }


  print "<Show FirmwareFile name=$filename>\n" ;
  # Get the descriptive paragraph info from the RPM package.
  my $description = 
          qx( /usr/lib/rpm/rpmq -q -p $filename --qf '%{DESCRIPTION}') ;

  # Extract the firmware date from the description data.  It will be in
  # a line looking like: "Firmware Date 2003/06/20".  We just want the
  # '2003/06/20' section.
  my $firmwareDate = ($description =~ m/.*Firmware Date\s+(\S+)/ )[0] ;

  if (! $firmwareDate)
  {
    print "ERROR: There is no firmware date in this file " 
         ."so it cannot be installed.\n" ;
    return ;
  }

  my ($year,$month,$day) = ($firmwareDate =~ m|(.+)/(.+)/(.+)| ) ;
  $firmwareDate = sprintf( "%04d/%02d/%02d", $year,$month,$day ) ;

  # warranty -w returns:
  #   war date :03312003
  my $warrantyDate = qx(/usr/bin/sudo /usr/local/fireball/warranty -w) ;
  $warrantyDate = (split("[ :]+",$warrantyDate))[2] ;

  # WarrantyDate is converted to format 2003/06/20 for comparison to the
  # firmware date from an RPM package.
  if ($warrantyDate != 0)
  {
    ($month,$day,$year) = ($warrantyDate =~ /(..)(..)(....)/) ;
    $warrantyDate = sprintf( "%04d/%02d/%02d", $year,$month,$day ) ;
  }
  else
  {
    $warrantyDate = '2000/01/01' ;
  }


  # Based on a string compare, we decide if this firmware is
  # installable.  This works because we have identical formats and 
  # they have the numeric data with most significant stuff first.
  #print $warrantyDate,"\n" ;
  #print $firmwareDate,"\n" ;
  if ( $warrantyDate ge $firmwareDate 
    || $description =~ m/Expected Installation Time.*:.*\./ 
     )
  {
    print "This file is installable under the terms of "
         ."your warranty.\n" ;
  }
  else
  {
    print 'This file is dated after your Warranty Expiration '
         ."date and cannot be installed.\n" ; 
  }

  print "Details:\n" ;

  print qx( /usr/lib/rpm/rpmq -q -p $filename --qf '\
Name        : %{NAME}
Summary     : %{SUMMARY}
Version     : %{VERSION}-%{RELEASE}
Build Date  : %{BUILDTIME:date}
License     : %{LICENSE}
'
          ) ;

  printf( "Size        : %d bytes\n\n", -s $filename ) ;
  print $description, "\n" ;

  print "<end of Show FirmwareFile name=$filename>\n" ;

} # end of ShowFirmwareFile




########################################################################
sub ShowConfigFiles
########################################################################
# Command handler to deal with showing the list of available
# configuration files.
# Syntax:
#
#       Show ConfigFiles
#
# In response a list of the files is sent back, bracketed with the 
# usual multi-line delimiters.
#   <Show ConfigFiles>
#   ConfigFile name=abc size=1234
#   ConfigFile name=def size=5678
#   ...
#   <end of Show ConfigFiles>
#
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;
  print "<Show ConfigFiles>\n" ;

  chdir '/etc/sysconfig/configFiles' ;
  my @allfiles = glob '*' ;

  # Determine the string length of the longest file name.
  # This is used below to control the field width in the 
  # printing operations.
  my $maxLength = 0 ;
  for my $f (@allfiles)
  {
    my $length = length( $f ) ;
    $maxLength = $length if ($length > $maxLength ) ;
  }

  for my $f (@allfiles)
  {
    printf "ConfigFile name=%-${maxLength}s size=%d\n", $f, (-s $f) ;
  }
  print "<end of Show ConfigFiles>\n" ;

} # end of ShowConfigFiles



########################################################################
sub ShowConfigFile
########################################################################
# Command handler to deal with showing the contents of a single
# configuration file.  
# Syntax:
#
#       Show ConfigFile name=abc
#
# In response to this the entire ASCII-encoded configuration file is 
# sent back, bracketed with the usual multi-line delimiters.
#   <Show ConfigFile name=abc>
#   lines from the config file
#   ...
#   ...
#   ...
#   <end of Show ConfigFile name=abc>
#
# An Error is flagged if the file doesn't exist.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $fname = $_[0] ;
  # Drop name= if it exists
  $fname =~ s/^name=//i ;

  chdir '/etc/sysconfig/configFiles' ;
  if ( ! -f $fname )
  {
    print "ERROR: Can't find config file $fname\n" ;
    return ;
  }


  print "<Show ConfigFile name=$fname>\n" ;
  
  open FILE, $fname ;
  print <FILE> ;
  close FILE ;

  print "<end of Show ConfigFile name=$fname>\n" ;

} # end of ShowConfigFile



########################################################################
sub CreateConfigFile
########################################################################
# Handle command that creates a configuration file, based on the 
# functions requested.  
#
# Syntax:
#
#       Create ConfigFile name=abc functions=option[, ...]
#
# Name is the name of the configuration file to create.
#
# The functions value is a comma separated list of words from the
# set: 
#       IOconfig
#       RefStation
#       Sessions 
#       Miscellaneous 
#       ExtReference
#       Ethernet 
#       Ftp 
#       Ntp 
#       Security
#       Everything 
#
# The functions list is case-insensitive.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 2, 2 ) ) ;

  my $fname = '' ;
  my @functions = () ;

  for my $p (@_)
  {
    if ($p !~ /=/)
    {
      print "ERROR: Must fully specify id=value for parameter $p\n" ;
      return ;
    }

    my ($id, $value) = split( '=', $p ) ;
    $id = lc($id) ;
    if ( $id eq 'name' )
    {
      $fname = $value ;
    }
    elsif ( $id eq 'functions' )
    {
      @functions = split( ',', lc($value) ) ;
    }
    else
    {
      print "ERROR: Invalid parameter $p\n" ;
      return ;
    }
  }

  if (length($fname) == 0)
  {
    print "ERROR: Must specify 'fname='.\n" ;
    return ;
  }

  if ( ! @functions )
  {
    print "ERROR: Must specify 'functions=' list.\n" ;
    return ;
  }

  # Check all of the functions that were specified to see if they
  # are from the expected list.
  for my $f ( @functions ) 
  {
    #print "$f\n" ;
    if ( $f !~ m/^( IOconfig
                  | RefStation
                  | Sessions
                  | miscellaneous
                  | ExtReference
                  | Ethernet
                  | Ftp
                  | Ntp
                  | HTTP
                  | IPfiltering
                  | Security
                  | Everything
                  )$/ix
       )
    {
      print "ERROR: Unknown configuration function $f.\n" ;
      return ;
    }
  }
  #print "\n" ;

  if (MakeConfigFile( $fname, @functions ) )
  {
    print "OK: ConfigFile name=$fname created.\n" ;
  }
  else
  {
    print "ERROR: Problem creating config file $fname.\n" ;
  }

} # end of CreateConfigFile



########################################################################
sub DeleteConfigFile
########################################################################
# Deletes an existing configuration file. 
# Syntax:
#       Delete ConfigFile name=abc
#       Delete ConfigFile abc
#
# If the ConfigFile exists, it is deleted.
#
#   OK: ConfigFile name=abc is deleted.
#
# An error will be flagged if the file does not exist.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $fname = $_[0] ;
  # Drop name= if it exists
  $fname =~ s/^name=//i ;

  chdir '/etc/sysconfig/configFiles' ;
  if ( ! -f $fname )
  {
    print "ERROR: Can't find config file $fname\n" ;
    return ;
  }

  system( "sudo /bin/rm -f '/etc/sysconfig/configFiles/$fname'" ) ;

  print "OK: ConfigFile name=$fname is deleted.\n" ;

} # end of DeleteConfigFile




########################################################################
sub InstallConfigFile
########################################################################
# Triggers the installation (application) of an existing configuration
# file. 
# Syntax:
#       Install ConfigFile name=abc
#       Install ConfigFile abc
#
# If the ConfigFile exists, it is verified and the installation process
# triggered.  Note that this requires a receiver reboot.  The
# response will be:
#
#   OK: Installing ConfigFile name=abc
#
# The response will come back about 5 seconds in advance of the reboot,
# to give plenty of time for the message to get out of the system.
# Errors will be indicated if the file doesn't exists, or if it doesn't
# pass some fundamental sanity checks done by the "clone" program.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $fname = $_[0] ;
  # Drop name= if it exists
  $fname =~ s/^name=//i ;

  chdir '/etc/sysconfig/configFiles' ;
  if ( ! -f $fname )
  {
    print "ERROR: Can't find config file $fname\n" ;
    return ;
  }

  my $pathname = "/etc/sysconfig/configFiles/$fname" ;

  # Verify that the chosen configfile is okay to install.
  my $status = system( "/usr/local/fireball/clone -t $fname " ) >> 8;

  if ($status != 0)
  {
    print "ERROR: Configuration File $fname is corrupted!\n" ;
    return ;
  }

  # At this point, the selected file has been verified and we are 
  # allowed to apply it.  We need a method of passing the file to
  # the ramdisk installer.  The method must survive a reboot into
  # a different operating system and file system layout.  So we put 
  # a symbolic link into /etc/sysconfig/configFiles and point it at the
  # desired file.
  my $linkName = 'InstallThisConfig' ;

  # Get rid of this link if it already exists.
  if ( -r $linkName ) 
  {
    system( "sudo rm $linkName" ) ;
  }

  # Create the desired hard link.
  system( "/usr/bin/sudo /bin/ln -s $fname $linkName" ) ;

  # Trigger the actual reconfiguration 
  #   - set the bootstatus flags
  #     We set flag zero to 252 as a signal to the boot system to
  #     load in the ramdisk system, and perform a clone operation.
  #   - do a sleep and reboot
  #     The sleep gives the GUI displays time to get out the chute
  #     before the reboot.  Reboot is implemented by sending a SIGUSR2
  #     signal to the frontpanel task.

  system( "/usr/bin/sudo /sbin/boot_status 0 252 f" ) ;

  # Obtain the process id of the frontpanel_task.  Launch a background
  # process that sends a signal to frontpanel_task.
  # Don't want to just do this directly from here, since frontpanel
  # might send a kill to the web server before we come back to here.
  # We want this script to have a decent chance to completely exit.
  my $spid = qx( cat /var/run/frontpanel_task.pid ) ;
  system( "(sleep 5 ; /usr/bin/sudo /bin/kill -SIGUSR2 $spid)&" ) ;

  print "OK: Installing ConfigFile name=$fname\n" ;
  
} # end of InstallConfigFile



########################################################################
sub UploadConfigFile
########################################################################
# Command handler that accepts and saves configuration files.
# Syntax:
#       Upload ConfigFile name=abc
#       Uplaod ConfigFile abc
#       Upload ConfigFile
# This script is designed to be called as a multi-part-document Post.
# This allows a document to be sent from the client into the NetRS.
# The name of the section containing the data must be "configFile".
# If the "name=abc" or the naked "abc" pararmeters are sent, then
# the value is the desired name for the new configuration file.
# If neither of those are supplied then the name of the file will be
# extracted from the default name in the posted document.
# Any leading path information or invalid characters will be stripped
# out of the name.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 1 ) ) ;

  my $pname = '' ;

  if (scalar( @_ ))
  {
    # If a file was uploaded, the parameter "name=" will have been sent.
    $pname = $_[0] ;
    # Strip off "name=" from beginning of the parameter.
    $pname =~ s/name=//i ;
    #print "Parameter name= $pname\n" ;
  }

  my $uname ;
  $uname = upload( 'configFile' ) ;
  #print "Upload name= $uname\n" ;

  if ( !$uname )
  {
    print "ERROR: no document attached to upload.\n" ;
    return ;
  }

  # if the parameter pname was supplied, then use that.  Otherwise, 
  # use what came in with the multi-part document.
  my $newFname = $pname || $uname ;

  # Strip any path information off of the filename.
  # Substitute underscore for any spaces or illegal characters 
  # in the name.
  $newFname =~ s/^.*\\// ; # Remove path that might use DOS backslashes.
  $newFname =~ s/^.*\/// ; # Remove path that might use Unix slashes.
  $newFname =~ s/[ =]/_/g   ;
  
  # The file has already been uploaded and is in a temporary location,
  # whose path is accessed here:
  my $tmpfilename = tmpFileName($uname) ;

  # If the file successfully uploaded, it will have a non-zero size.
  my $size=(-s $tmpfilename) ;
  #print "$tmpfilename size=", (-s $tmpfilename), "\n";

  if ($size)
  { 
    # Move the file to the final resting place.
    system( "sudo cp $tmpfilename /etc/sysconfig/configFiles/$newFname" ) ;
    system( "sudo chmod 444 /etc/sysconfig/configFiles/$newFname" ) ;

    print "OK: ConfigFile name=$newFname was successfully uploaded.\n" ;
  }
  else
  {
    print "ERROR: There was a problem uploading file '$newFname'.\n" ;
    return ;
  }
  close( $uname ) ;

} # end of UploadConfigFile



########################################################################
sub UploadFirmwareFile
########################################################################
# Command handler that accepts and saves firmware files.
# Syntax:
#       Upload FirmwareFile name=abc
#       Uplaod FirmwareFile abc
#       Upload FirmwareFile
# This script is designed to be called as a multi-part-document Post.
# This allows a document to be sent from the client into the NetRS.
# The name of the section containing the data must be "firmwareFile".
# If the "name=abc" or the naked "abc" pararmeters are sent, then
# the value is the desired name for the new configuration file.
# If neither of those are supplied then the name of the file will be
# extracted from the default name in the posted document.
# Any leading path information or invalid characters will be stripped
# out of the name.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 1 ) ) ;

  my $pname = '' ;

  if (scalar( @_ ))
  {
    # If a file was uploaded, the parameter "name=" will have been sent.
    $pname = $_[0] ;
    # Strip off "name=" from beginning of the parameter.
    $pname =~ s/name=//i ;
    #print "Parameter name= $pname\n" ;
  }

  my $uname ;
  $uname = upload( 'firmwareFile' ) ;
  #print "Upload name= $uname\n" ;

  if ( !$uname )
  {
    print "ERROR: no document attached to upload.\n" ;
    return ;
  }

  # if the parameter pname was supplied, then use that.  Otherwise, 
  # use what came in with the multi-part document.
  my $newFname = $pname || $uname ;

  # Strip any path information off of the filename.
  # Substitute underscore for any spaces or illegal characters 
  # in the name.
  $newFname =~ s/^.*\\// ; # Remove path that might use DOS backslashes.
  $newFname =~ s/^.*\/// ; # Remove path that might use Unix slashes.
  $newFname =~ s/[ =]/_/g   ;
  
  # The file has already been uploaded and is in a temporary location,
  # whose path is accessed here:
  my $tmpfilename = tmpFileName($uname) ;

  # If the file successfully uploaded, it will have a non-zero size.
  my $size=(-s $tmpfilename) ;
  #print "$tmpfilename size=", (-s $tmpfilename), "\n";

  if ($size)
  { 
    # prefix the raw name with the directory we use for firmware file
    # uploads.
    my $newPath = '/var/gps/incoming/'.$newFname ;
  
    if ( ! -d '/var/gps/incoming' )
    {
      mkdir '/var/gps/incoming' ;
    }

    if ( ! -w '/var/gps/incoming' )
    {
      system( '/usr/bin/sudo chown netrx:netrx /var/gps/incoming' ) ;
      system( '/usr/bin/sudo chmod a+rw /var/gps/incoming' ) ;
    }

    # Move the file to the final resting place.
    rename $tmpfilename, $newPath ;
    system( "chmod a+rx $newPath" ) ;
    close( $uname ) ;

    print "OK: FirmwareFile $newFname was successfully uploaded.\n" ;
  }
  else
  {
    print "ERROR: There was a problem uploading file '$newFname'.\n" ;
    return ;
  }

} # end of UploadFirmwareFile



########################################################################
sub DeleteFirmwareFile
########################################################################
# Implements delete firmwareFile request.  Removes one (or all)
# previously uploaded firmware files.
# One parameter expected in one of two forms:
#   delete firmwareFile name=fname.rpm
#   delete firmwareFile all=yes
#
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  chdir '/var/gps/incoming' ;

  my $fname = $_[0] ;

  if ($fname =~ /^name=.+/)
  {
    $fname =~ s/^name=//i ;

    if ( ! -e $fname ) 
    {
      print "ERROR: can't find FirmwareFile $fname\n" ;
      return ;
    }

    system( "sudo /bin/rm -f '/var/gps/incoming/$fname'" ) ;

    print "OK: FirmwareFile $fname is deleted.\n" ;
  }
  elsif ($fname =~ /^all=yes$/i)
  {
    for $fname (glob('*'))
    {
      system( "sudo /bin/rm -f '/var/gps/incoming/$fname'" ) ;
    }
    print "OK: All firmware files deleted.\n" ;
  }

} # end of DeleteFirmwareFile




########################################################################
sub badPatch
########################################################################
# Checks that a firmware file is either a full installation, or if it is
# a Patch upgrade, that the current version matches the required patch
# base.
# $_[0] is the RPM description information.
#
# Returns an empty string if okay to install.  Otherwise returns an
# error message like:
#   Patch requires 1.0-0 but installed is 1.0-2.
#
########################################################################
{

  # Extract some information from the rpm file description.
  # We are trying to extract values from these lines:
  #
  #   Patch Base   0.1-0
  # This value is put into the RPM from the .spec file used in
  # the RPM generation process.
  # PatchBase may not exist, and defaults to blank in that case.
  # For patch installations it says what version the current firemware
  # must be before we'll allow the installation.

  my $description = $_[0] ;
  my $patchBase = '' ;

  $patchBase = $1 if ($description =~ m/Patch Base\s*(\S*)/ );

  if ($patchBase)
  {
    my( $thisVersion, $thisDate ) = 
          split( "\n", 
                  qx( cat /usr/local/fireball/defaults/firmwareVersion )
               ) ;
    # Compare the patch base to the existing installed version.
    if ( $patchBase ne $thisVersion )
    {
      return "Patch requires $patchBase but installed is $thisVersion."
    }
  }

  return '' ;   # No problem found.

} # end of badPatch()



########################################################################
sub triggerInstall
########################################################################
# Trigger the installation of a firmware file.
# The filepath comes in as $_[0].
# The required links are created and a signal is sent to the
# frontpanel task to force a reboot.
########################################################################
{
  my $filepath = $_[0] ;

  # At this point, the selected file has been verified and we are 
  # allowed to install it.  We need a method of passing the file to
  # the ramdisk installer.  The method must survive a reboot into
  # a different operating system and file system layout.  The only
  # thing we are guaranteed is that we can see the files in 
  # /var/incoming.  We could put the name of the selected file into
  # another file with a well defined name.  Or we can just make a 
  # link to that file with a well defined name.  We'll try the latter.
  # Link the the selected file to /var/gps/incoming/TheRPMtoInstall.
  # Use a hard link to make sure that there are no symlink oddities
  my $linkpath = '/var/gps/incoming/TheRPMtoInstall' ;

  # Get rid of this link if it already exists.
  if ( -r $linkpath ) 
  {
    system( "sudo rm $linkpath" ) ;
  }

  # Create the desired hard link.
  link( $filepath, $linkpath ) ;


  # Trigger the actual installation 
  # - set the bootstatus flags
  #   We set flag zero to 255 as a signal to the boot system to
  #   load in the ramdisk system, and perform an installation.
  # - do a sleep and reboot
  #   The sleep gives the GUI script time to get started before the
  #   reboot.  Reboot is implemented by sending a SIGUSR2 signal to the
  #   frontpanel task.

  system( "/usr/bin/sudo /sbin/boot_status 0 255 f" ) ;

  # Obtain the process id of the frontpanel_task.  Launch a background
  # process that waits a bit then sends a signal to frontpanel_task.
  # Do this after a little bit of delay to let any messages get back
  # to the user over http links.
  my $spid = qx( cat /var/run/frontpanel_task.pid ) ;
  system( "(sleep 5 ; /usr/bin/sudo /bin/kill -SIGUSR2 $spid)&" ) ;

} # end of triggerInstall()



########################################################################
sub VerifyOrInstallFirmwareFile
########################################################################
# Verify or Install the contents of an already-uploaded firmware file.
# This is not a direct command handler.  Rather, it is used by
# both VerifyFirmwareFile and InstallFirmwareFile to implement the
# two similar processes.  Both actions first verify the contents of the
# package, producing error messages if there are problems.  For the
# Verify operation, an OK: message is presented and we are done.  The
# install option presents a different message (about to install) and
# then triggers the actual installation.
#
# $_[0] is 0=verify 1=install.
# $_[1] is the filename to verify/install.
#
# Possible responses will be:
#   OK: Installing FirmwareFile abc.rpm.  Expect at least 400 seconds to complete.
#   OK: FirmwareFile abc.rpm can be installed.
#
#   ERROR: Can't install abc.rpm.  Patch requires 1.0-0 but installed is 1.0-2.
#   ERROR: FirmwareFile abc.rpm is dated past your warranty.
#   ERROR: FirmwareFile abc.rpm is corrupted.
#   ERROR: FirmwareFile abc.rpm is not a valid RPM file.
#   ERROR: Can't find FirmwareFile abc.rpm.
#
########################################################################
{
  my $install  = $_[0] ;
  my $filename = $_[1] ;

  my $filepath = '/var/gps/incoming/' . $filename ;

  if ( ! -e $filepath ) 
  {
    print "ERROR: Can't find FirmwareFile $filename.\n" ;
    return ;
  }

  if ( ! IsRpmFile( $filepath ) )
  {
    print "ERROR: FirmwareFile $filename is not a valid RPM file.\n" ;
    return ;
  }


  # Call the RpmVerify routine to check if the file is uncorrupted.
  # Second parameter of zero means non-verbose, so the subroutine
  # will only return a string, and will print nothing to stdout.
  # Returns a blank string if file is okay.  Otherwise returns
  # an error message.
  my $result = Fireball::RpmVerify( $filepath, 0 ) ;
  if ( $result )
  {
    # We might want to check here for the various types of error 
    # message the RPM verify could produce - like improper length,
    # etc.  But for the moment, just indicate the file is bad.
    print "ERROR: FirmwareFile $filename is corrupted.\n" ;
    return ;
  }


  my $description ;
  # Extract the firmware date and compare to the warranty date on
  # this system.  underWarranty also returns the RPM description
  # information.
  if ( ! underWarranty( $filepath, 
                        0,  # Not verbose
                        \$description   
                      ) 
     )
  {
    print "ERROR: FirmwareFile $filename is dated past your warranty.\n" ;
    return ;
  }

  # Check if this is a patch file whose required base version doesn't
  # match the installed version.  
  $result = badPatch( $description ) ;
  if ($result)
  {
    print "ERROR: Can't install $filename.  $result\n" ;
    return ;
  }

  if (!$install)
  {
    print "OK: FirmwareFile $filename can be installed.\n" ;
    return ;
  }

  my $installationTime = '200' ;

  $installationTime = $1 
        if ($description =~ m/Expected Installation Time\s*:\s*(\d+)/ );


  print "OK: Installing FirmwareFile $filename.  ",
        "Expect at least $installationTime seconds to complete.\n" ;

  triggerInstall( $filepath ) ;

} # end of VerifyOrInstallFirmwareFile



########################################################################
sub VerifyFirmwareFile
########################################################################
# Verify the contents of an already-uploaded firmware file.
# Syntax:
#       Verify FirmwareFile name=abc.rpm
#       Verify FirmwareFile abc.rpm
#
# Possible responses will be:
#   OK: FirmwareFile name=abc.rpm can be installed.
#   ERROR: FirmwareFile name=abc.rpm is okay, but dated past your warranty.
#   ERROR: FirmwareFile name=abc.rpm is corrupted.
#   ERROR: FirmwareFile name=abc.rpm is not a valid RPM file.
#   ERROR: Can't find FirmwareFile name=abc.rpm
#
# Most or the functionality of this procedure is actually in
# VerifyOrInstallFirmwareFile().
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $filename = $_[0] ;
  $filename =~ s/^name=//i ;

  VerifyOrInstallFirmwareFile( 0, $filename ) ;

} # end of VerifyFirmwareFile



########################################################################
sub InstallFirmwareFile
########################################################################
{
  #print "Executing InstallFirmwareFile\n" ;
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $filename = $_[0] ;
  $filename =~ s/^name=//i ;
  
  VerifyOrInstallFirmwareFile( 1, $filename ) ;

} # end of InstallFirmwareFile




########################################################################
sub ShowWaasSatControls
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

$_ = qx( /usr/bin/sudo /usr/local/fireball/sysinfo -p waas_sv_masks ) ;
# Return string should be of the form
#   put waas_sv_masks D D D D D D D D D D D D D D E D D D D

  my @state = split( " ", $_ ) ;
  # Drop the "put sv_masks" portion.
  splice( @state, 0, 2 ) ;

  my $waasCount = scalar( @state ) ; # Should be 19!
  #print join( ',', @state), $waasCount, br() ;

  my @enables = () ; 
  my @disables = () ;
  my @ignores = () ;

  for (my $sv = 0 ; $sv < 19 ; ++$sv ) 
  {
    my $state = $state[$sv] ;
    if ($state eq 'I')
    {
      push( @ignores, sprintf( '%d', $sv+120 ) ) ;
    }
    elsif ($state eq 'D')
    {
      push( @disables, sprintf( '%d', $sv+120 ) ) ;
    }
    else 
    {
      push( @enables, sprintf( '%d', $sv+120 ) ) ;
    }
  }


  print "WaasSatControls " ;

  if ( scalar(@enables) >= 9 )
  {
    print "enable=all " ;
  }
  elsif ( scalar(@disables) >= 9 )
  {
    print "disable=all " ;
  }
  elsif ( scalar(@ignores) >= 9 )
  {
    print "ignoreHealth=all " ;
  }


  if ( scalar(@enables) >= 9 )
  {
  }
  elsif ( scalar(@enables) == 0 )
  {
    print "enable=none " ;
  }
  else
  {
    print "enable=", join( ',', @enables ), " " ;
  }


  if ( scalar(@disables) >= 9 )
  {
  }
  elsif ( scalar(@disables) == 0 )
  {
    print "disable=none " ;
  }
  else
  {
    print "disable=", join( ',', @disables ), " " ;
  }

  if ( scalar(@ignores) >= 9 )
  {
  }
  elsif ( scalar(@ignores) == 0 )
  {
    print "ignoreHealth=none" ;
  }
  else
  {
    print "ignoreHealth=",join( ',', @ignores ) ;
  }

  print "\n" ;
} # end of ShowWaasSatControls



########################################################################
sub SetWaasSatControls
########################################################################
# Modify the existing Waas satellite enables according to the parameters
# received.
# Sent parameters in @_ of multiple forms
#   Enable=all
#   Disable=all
#   IgnoreHealth=all
#       Only one of these is allowed.  If received, then
#       apply first.
{
  return if ( badParmCount( scalar( @_ ), 1, 3 ) ) ;

  #Convert the parameter array into an associative hash.
  my %parms = ( 'enable'=>'', 'disable'=>'', 'ignorehealth'=>'' ) ;
  my $parmcount = 0 ;
  my $allFound = 0 ;


  for my $p (@_)
  {
    $p = lc( $p ) ;
    if ($p !~ /=/)
    {
      print "ERROR: Must fully specify id=value for parameter $p\n" ;
      return ;
    }
    my ($id, $value) = split( '=', $p ) ;

    if ( $parms{$id} )
    {
      print "ERROR: Duplicate parameter $p\n" ;
      return ; 
    }

    # Parameter value can be 'all', 'none', ''
    # or a string of comma separated positive integers in the range of
    # 120 to 138.  Otherwise don't accept it.
    if ($value eq 'all')
    {
      if ($allFound)
      {
        print "ERROR: Only one 'all' parameter allowed.\n" ;
        return ; 
      }

      ++$allFound ;
    }
    elsif ( $value =~ /^\d+(,\d+)*$/ )
    {
      # Parameter is a list of comma separated digit strings.
      # Check them al for valid range.
      for my $sv ( split( ',',  $value ) )
      {
        if ( ($sv < 120) || ($sv >138) )
        {
          print "ERROR: Sv number out of range 120 to 138.\n" ;
          return ;
        }
      }
    }
    elsif ( $value ne 'none' 
         && $value ne ''
          )
    {
        print "ERROR: Invalid parameter $p\n" ;
        return ; 
    }

    $parms{$id} = $value ;
    ++$parmcount ;
    #print "$id  =  $value\n" ; 
  }

  # Ask Sysinfo system what the current SV masks are...
  $_ = qx( /usr/bin/sudo /usr/local/fireball/sysinfo -p waas_sv_masks );
  # Return string should be of the form
  #  put waas_sv_masks E I E D E .... 19 of them.
  my @state = split( " ", $_ ) ;
  # Drop the "put waas_sv_masks" portion.
  splice( @state, 0, 2 ) ;

  # Use this array to detect if multiple settings are made on the
  # same satellite.
  my @svSet = (0) x 19 ;

  if ($parms{'enable'} eq 'all')
  {
    #print "Enabling ALL\n" ;
    @state = ('E') x 19 ;
  }

  if ($parms{'disable'} eq 'all')
  {
    #print "Disabling ALL\n" ;
    @state = ('D') x 19 ;
  }

  if ($parms{'ignorehealth'} eq 'all')
  {
    #print "IgnoreHealth on ALL\n" ;
    @state = ('I') x 19 ;
  }


  if ($parms{'enable'} =~ /^\d/ )
  {
    for my $sv ( split( ',',  $parms{'enable'} ) )
    {
      #print "Enabling $sv\n" ;
      $state[$sv-120] = 'E' ;
      $svSet[$sv-120] = 1 ;
    }
  }


  if ($parms{'disable'} =~ /^\d/ )
  {
    for my $sv (split( ',',  $parms{'disable'}))
    {
      if ($svSet[$sv-120])
      {
        print "ERROR: Can't specify satellite $sv twice.\n" ;
        return ; 
      }
      #print "Disabling $sv\n" ;
      $state[$sv-120] = 'D' ;
      $svSet[$sv-120] = 1 ;
    }
  }

  if ($parms{'ignorehealth'} =~ /^\d/ )
  {
    for my $sv (split( ',',  $parms{'ignorehealth'}))
    {
      if ($svSet[$sv-120])
      {
        print "ERROR: Can't specify satellite $sv twice.\n" ;
        return ; 
      }
      #print "Setting Ignore Health for $sv\n" ;
      $state[$sv-120] = 'I' ;
      $svSet[$sv-120] = 1 ;
    }
  }

  my $commandLine = 'put waas_sv_masks' ;

  for (my $i = 0 ; $i < 19 ; ++$i )
  {
    $commandLine .= " $state[$i]" ;
  }

  $commandLine .= "\n" ;
  #print $commandLine ;
  newStingerConfig( $commandLine ) ;

  print "OK: " ;
  ShowWaasSatControls() ;

} # end of SetWaasSatControls




########################################################################
sub ShowGpsSatControls
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  # Ask Sysinfo system what the current SV masks are...
  $_ = qx( /usr/bin/sudo /usr/local/fireball/sysinfo -p sv_masks ) ;
  # Return string should be of the form
  #  put sv_masks E I E D E .... 32 of them.
  my @state = split( " ", $_ ) ;
  # Drop the "put sv_masks" portion.
  splice( @state, 0, 2 ) ;

  my @enables = () ; 
  my @disables = () ;
  my @ignores = () ;

  for (my $sv = 1 ; $sv <= 32 ; ++$sv ) 
  {
    my $state = $state[$sv-1] ;
    if ($state eq 'I')
    {
      push( @ignores, sprintf( '%d', $sv ) ) ;
    }
    elsif ($state eq 'D')
    {
      push( @disables, sprintf( '%d', $sv ) ) ;
    }
    else 
    {
      push( @enables, sprintf( '%d', $sv ) ) ;
    }
  }


  print "GpsSatControls " ;

  if ( scalar(@enables) >= 16 )
  {
    print "enable=all " ;
  }
  elsif ( scalar(@disables) >= 16 )
  {
    print "disable=all " ;
  }
  elsif ( scalar(@ignores) >= 16 )
  {
    print "ignoreHealth=all " ;
  }


  if ( scalar(@enables) >= 16 )
  {
  }
  elsif ( scalar(@enables) == 0 )
  {
    print "enable=none " ;
  }
  else
  {
    print "enable=", join( ',', @enables ), " " ;
  }


  if ( scalar(@disables) >= 16 )
  {
  }
  elsif ( scalar(@disables) == 0 )
  {
    print "disable=none " ;
  }
  else
  {
    print "disable=", join( ',', @disables ), " " ;
  }

  if ( scalar(@ignores) >= 16 )
  {
  }
  elsif ( scalar(@ignores) == 0 )
  {
    print "ignoreHealth=none" ;
  }
  else
  {
    print "ignoreHealth=",join( ',', @ignores ) ;
  }

  print "\n" ;
} # end of ShowGpsSatControls



########################################################################
sub SetGpsSatControls
########################################################################
# Modify the existing GPS satellite enables according to the parameters
# received.
# Sent parameters in @_ of multiple forms
#   Enable=all
#   Disable=all
#   IgnoreHealth=all
#       Only one of these is allowed.  If received, then
#       apply first.
{
  return if ( badParmCount( scalar( @_ ), 1, 3 ) ) ;

  #Convert the parameter array into an associative hash.
  my %parms = ( 'enable'=>'', 'disable'=>'', 'ignorehealth'=>'' ) ;
  my $parmcount = 0 ;
  my $allFound = 0 ;


  for my $p (@_)
  {
    $p = lc( $p ) ;
    if ($p !~ /=/)
    {
      print "ERROR: Must fully specify id=value for parameter $p\n" ;
      return ;
    }
    my ($id, $value) = split( '=', $p ) ;

    if ( $parms{$id} )
    {
      print "ERROR: Duplicate parameter $p\n" ;
      return ; 
    }

    # Parameter value can be 'all', 'none', ''
    # or a string of comma separated positive integers in the range of
    # 1 to 32.  Otherwise don't accept it.
    if ($value eq 'all')
    {
      if ($allFound)
      {
        print "ERROR: Only one 'all' parameter allowed.\n" ;
        return ; 
      }

      ++$allFound ;
    }
    elsif ( $value =~ /^\d+(,\d+)*$/ )
    {
      # Parameter is a list of comma separated digit strings.
      # Check them al for valid range.
      for my $sv ( split( ',',  $value ) )
      {
        if ( ($sv < 1) || ($sv > 32) )
        {
          print "ERROR: Sv number out of range 1 to 32.\n" ;
          return ;
        }
      }
    }
    elsif ( $value ne 'none' 
         && $value ne ''
          )
    {
        print "ERROR: Invalid parameter $p\n" ;
        return ; 
    }

    $parms{$id} = $value ;
    ++$parmcount ;
    #print "$id  =  $value\n" ; 
  }

  # Ask Sysinfo system what the current SV masks are...
  $_ = qx( /usr/bin/sudo /usr/local/fireball/sysinfo -p sv_masks ) ;
  # Return string should be of the form
  #  put sv_masks E I E D E .... 32 of them.
  my @state = split( " ", $_ ) ;
  # Drop the "put sv_masks" portion.
  splice( @state, 0, 2 ) ;


  # Use this array to detect if multiple settings are made on the
  # same satellite.
  my @svSet = (0) x 32 ;

  if ($parms{'enable'} eq 'all')
  {
    #print "Enabling ALL\n" ;
    @state = ('E') x 32 ;
  }

  if ($parms{'disable'} eq 'all')
  {
    #print "Disabling ALL\n" ;
    @state = ('D') x 32 ;
  }

  if ($parms{'ignorehealth'} eq 'all')
  {
    #print "IgnoreHealth on ALL\n" ;
    @state = ('I') x 32 ;
  }


  if ($parms{'enable'} =~ /^\d/ )
  {
    for my $sv ( split( ',',  $parms{'enable'} ) )
    {
      #print "Enabling $sv\n" ;
      $state[$sv-1] = 'E' ;
      $svSet[$sv] = 1 ;
    }
  }


  if ($parms{'disable'} =~ /^\d/ )
  {
    for my $sv (split( ',',  $parms{'disable'}))
    {
      if ($svSet[$sv])
      {
        print "ERROR: Can't specify satellite $sv twice.\n" ;
        return ; 
      }
      #print "Disabling $sv\n" ;
      $state[$sv-1] = 'D' ;
      $svSet[$sv] = 1 ;
    }
  }

  if ($parms{'ignorehealth'} =~ /^\d/ )
  {
    for my $sv (split( ',',  $parms{'ignorehealth'}))
    {
      if ($svSet[$sv])
      {
        print "ERROR: Can't specify satellite $sv twice.\n" ;
        return ; 
      }
      #print "Setting Ignore Health for $sv\n" ;
      $state[$sv-1] = 'I' ;
      $svSet[$sv] = 1 ;
    }
  }


  # Create a command line of the form
  #   put sv_masks E E E D E E E E I E .....
  # with an E/D/I character for all 32 satellites.
  # this line will be sent to stinger and written into Stinger's
  # configuration file.
  my $commandLine = 'put sv_masks' ;

  for (my $i = 0 ; $i < 32 ; ++$i )
  {
    $commandLine .= " $state[$i]" ;
  }

  $commandLine .= "\n" ;
  #print $commandLine ;

  newStingerConfig( $commandLine ) ;

  print "OK: " ;
  ShowGpsSatControls() ;

} # end of SetGpsSatControls



########################################################################
sub ShowTrackingStatus
########################################################################
{
  #print "Executing ShowTrackingStatus\n" ;
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  print "<ShowTrackingStatus>\n" ;

  open( SYSINFO, 
      '/usr/bin/sudo /usr/local/fireball/sysinfo -p tracking_status |');
  # Response is twelve lines, one per channel of the form
  #   0 :  15 51.7 93.3 D2FF   8001 50.8  41.8  35   2.8
  #  Chn  PRN Elev Azm  L1Stat L2St L1SNR L2SNR IODE URA  
  # Or (if channel is unused)
  #   0 : idle

  # Scan the list of Sv information, and extract a list
  # of satellite's being tracked.  These are the [2] element on
  # each of the non-idle tracking lines.
  for ( my $i = 0 ; $i < 12 ; $i++ )
  {
    if ( ($_ = <SYSINFO>) !~ /idle/ )
    {
      #print $_ ;
      my ( $chan, undef, $prn, $elev, $azm, 
           undef, undef, $L1SNR, $L2SNR, $IODE, $URA, $L2CSNR)
           = split( ' ' ) ;
      printf( "Chan=%-2d PRN=%-3d Elv=%-4d Azm=%-3d L1snr=%-2d",
              $chan, 
              $prn,
              $elev,
              $azm,
              $L1SNR
            ) ;
      if ($prn <= 32) 
      {
        printf( " L2snr=%-2d L2Csnr=%-2d IODE=%-3d URA=%-s", 
                $L2SNR,
                $L2CSNR,
                $IODE,
                $URA
              ) ;
      }
      print "\n" ;
    }
  }

  $_ = <SYSINFO> ; # drop OK
  close( SYSINFO ) ;

  print "<end of ShowTrackingStatus>\n" ;

} # end of ShowTrackingStatus



########################################################################
sub ShowWaasTracking
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  $_ = qx( /usr/bin/sudo /usr/local/fireball/sysinfo -p waas_mode ) ;
  # Response will be
  #  put waas_mode on         (or off)

  my $WaasEnabled = ( (split)[2] eq 'on') ? 'yes' : 'no' ;

  print "WaasTracking enable=$WaasEnabled\n" ;

} # end of ShowWaasTracking



########################################################################
sub SetWaasTracking
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $newstate = lc( $_[0] ) ;
  $newstate =~ s/^enable=// ;
  #print "newstate=$newstate\n" ;
  if ($newstate !~ m/^(yes|no)$/)
  {
    print "ERROR: Invalid parameter $_[0]\n" ;
    return ;
  }

  newStingerConfig( 'put waas_mode '
                    .( ( $newstate eq 'yes') ? "on\n" : "off\n" )
                   ) ;

  print "OK: " ;
  ShowWaasTracking() ;

} # end of ShowWaasTracking



########################################################################
sub ShowReferenceFrequency
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  # Get the state of the control by reading the config file.
  my $setting = qx( cat /etc/sysconfig/extRefFreq ) ;
  $setting = ( $setting =~ m/External/ )
             ? 'external'
             : 'internal'
             ;

  # Get the state of the extref signal detector from program extfreq.
  my (undef, $detected) = split( ',', 
                                 qx(/usr/local/fireball/extfreq -p)
                               ) ;
  $detected = ($detected =~ m/present/)
              ? 'yes'
              : 'no'
              ;

  print "ReferenceFrequency source=$setting",
        " externalSignalDetected=$detected\n" ;

} # end of ShowReferenceFrequency



########################################################################
sub SetReferenceFrequency
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $newSource = lc( $_[0] ) ;
  $newSource =~ s/^source=// ;
  if ($newSource eq 'external')
  {
    $newSource = 'External' ;
  }
  elsif ($newSource eq 'internal')
  {
    $newSource = 'Internal' ;
  }
  else
  {
    print "ERROR: Invalid parameter $_[0]\n" ;
    return ;
  }

  # Get the state of the control by reading the config file.
  my $setting = qx( cat /etc/sysconfig/extRefFreq ) ;

  # If the current setting of the External Ref control doesn't match
  # the desired newsource setting, then change change the setting.
  # Otherwise, don't do anything since no change is requested.
  if ( $setting !~ m/$newSource/i )
  {
    #print "Changing to $newSource \n" ;
    # Update the extref configuration file to make this change
    # persist across power cycles.
    open( NEWCONFIG, '>/tmp/extRefFreq' ) ;

    print NEWCONFIG "REFSOURCE=$newSource\n" ;

    close NEWCONFIG ;

    system( 'sudo cp /tmp/extRefFreq /etc/sysconfig/extRefFreq' ) ;
    system( 'sudo /usr/local/fireball/sysinfo -m'  ) ;
  }

  print "OK: " ;
  ShowReferenceFrequency() ;

} # end of SetReferenceFrequency



########################################################################
sub ShowLoggedFiles
########################################################################
# Show LoggedFiles directory=/abc/def
# Okay to leave out 'directory=' and just give the path name.
# If path doesn't start with '/', put one in.
{
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $path = $_[0] ;

  # Remove a possible "directory=" on the file name.
  $path =~ s/^directory=//i ;

  # Insert a leading / if it isn't there already.
  $path =~ s/^([^\/])/\/$1/ ;

  # Give an error message if the directory doesn't exist,
  # or in several special cases.
  # Don't allow doubled-slashes.  Probably an error.
  # Don't allow '..' syntax to move up a level.  Could allow user
  # to view items outside of /var/gps.
  # Don't allow viewing /incoming or /tmp.  Not used for logged data.
  if ( ! -d "/var/gps$path"
    || $path =~ m|//|
    || $path =~ m/\.\./
    || $path =~ m|/incoming|
    || $path =~ m|/tmp|
     )
  {
    print "ERROR: Can't find directory '$path'\n" ;
    return ;
  }

  print "<Show LoggedFiles directory=$path>\n" ;

  chdir "/var/gps$path" ;

  my @files = glob( "*" ) ;

  for my $filename (  sort @files )
  {
    if (-f $filename)
    {
      next if ( $filename =~ m/\.A$/ ) ;

      print "LoggedFile name=$filename size=",
            (-s $filename),
            "\n" ;
    }
    else
    {
      # Don't show any non-data file directories.
      # No CVS stuff (development systems only)
      # No /tmp or /incoming or /lost+found.
      next if ( $filename eq 'CVS'
             || ( ($path eq '/')
               && ( $filename eq 'tmp' 
                 || $filename eq 'incoming'
                 || $filename eq 'lost+found'
                  )
                )
              ) ;

      print "Directory name=$filename\n" ;
    }
  }


  print "<end of Show LoggedFiles directory=$path>\n" ;

} # end of ShowLoggedFiles


########################################################################
sub ShowLoggedFile
########################################################################
# Show the information on a single LoggedFile specified by full path.
#
# Okay to leave out 'path=' and just give the path name.
# If path doesn't start with '/', put one in.
{
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $path = $_[0] ;

  # Remove a possible "path=" on the file name.
  $path =~ s/^path=//i ;

  # Insert a leading / if it isn't there already.
  $path =~ s/^([^\/])/\/$1/ ;

  if ( ! -f "/var/gps$path" 
    || $path =~ m|//|
    || $path =~ m/\.\./
    || $path =~ m|/incoming|
    || $path =~ m|/tmp|
     )
  {
    print "ERROR: Can't find file $path.\n" ;
    return ;
  }

  print "LoggedFile path=$path size=",
            (-s "/var/gps$path"),
            "\n" ;

} # end of ShowLoggedFile


########################################################################
sub DeleteLoggedFile
########################################################################
# Show LoggedFiles directory=/abc/def
# Okay to leave out 'directory=' and just give the path name.
# If path doesn't start with '/', put one in.
{
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $path = $_[0] ;

  # Remove a possible "directory=" on the file name.
  $path =~ s/^path=//i ;

  # Insert a leading / if it isn't there already.
  $path =~ s/^([^\/])/\/$1/ ;

  if ( ! -f "/var/gps$path" 
    || $path =~ m|//|
    || $path =~ m/\.\./
    || $path =~ m|/incoming|
    || $path =~ m|/tmp|
     )
  {
    print "ERROR: Can't find file $path.\n" ;
    return ;
  }

  unlink "/var/gps$path" ;

  print "OK: LoggedFile $path is deleted.\n" ;

} # end of DeleteLoggedFile


########################################################################
sub DeleteLoggedFiles
########################################################################
# Deletes an entire directory tree in the logged data files area.
#
# Syntax: Delete LoggedFiles directory=/abc/def
# Okay to leave out 'directory=' and just give the path name.
# If path doesn't start with '/', put one in.
# Okay for path to end in '/'. 
{
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $path = $_[0] ;

  # Remove a possible "directory=" on the file name.
  $path =~ s/^directory=//i ;

  # Insert a leading / if it isn't there already.
  $path =~ s/^([^\/])/\/$1/ ;

  # Give an error message if the directory doesn't exist,
  # or in several special cases.
  # Don't allow doubled-slashes.  Probably an error.
  # Don't allow '..' syntax to move up a level.  Could allow user
  # to delete items outside of /var/gps.
  # Don't allow deleting /incoming or /tmp.  Not used for logged data.
  if ( ! -d "/var/gps$path"
    || $path =~ m|//|
    || $path =~ m/\.\./
    || $path =~ m|/incoming|
    || $path =~ m|/tmp|
     )
  {
    print "ERROR: Can't find directory '$path'\n" ;
    return ;
  }

  if ($path eq '/')
  {
    print "ERROR: Can't delete '/'\n" ;
    return ;
  }

  system( "rm -fr /var/gps$path" ) ;
  

  print "OK: LoggedFile Directory $path is deleted.\n" ;

} # end of DeleteLoggedFiles



########################################################################
sub ShowGpsHealth
########################################################################
# Display the health status of all GPS satellites.
# Return line is of the form:
#
#   GpsHealth healthy=1-4,6-11,13-18,20,21,23-31
#             unhealthy=5:63 
#             unknown=12,19,22,32
#
# All of that is emitted in a single line.
# the healthy list is usually quite full, so we attempt to compress it
# with a run-length encoding: 1-5 equals 1,2,3,4,5.
# Unhealthy indications show as PRN:healthCode.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  $_ = qx( /usr/local/fireball/sysinfo -p gps_hlth ) ;
  # Sends back a string like:
  # 0 0 0 0 63 0 0 0 0 0 0 -1 0 0 0 0 0 0 -1 0 0 -1 0 0 0 0 0 0 0 0 0 -1
  # zero is healthy.  -1 is not-exist, otherwise value is an unhealthy 
  # status code.
  chomp ;
  # Assign each value to an array, shifting by one, so PRN.1 is at
  # index=[1].
  my @healthCodes = ( 0, split() ) ;


  # For testing, you can enable combinations of these lines:
  #@healthCodes = (0) x 33 ; # default everything to healthy.
  #@healthCodes = (-1) x 33 ; # default everything to unknown.
  #@healthCodes = (63) x 33 ; # default everything to unhealthy.
  #$healthCodes[1] = -1 ;
  #$healthCodes[32] = -1 ;
  #$healthCodes[1] = 1 ;
  #$healthCodes[32] = 32 ;


  my $healthInfo = '' ;
  my $unhealthInfo = '' ;
  my $unknownInfo = '' ;

  my $prevHealthy = -1 ;
  for (my $sv = 1 ; $sv <=32 ; ++$sv)
  {
    my $h = $healthCodes[$sv] ;
    if ( $h == 0 )
    {
      $healthInfo .= ',' if ($healthInfo) ;
      $healthInfo .= $sv ;
    }
    elsif ($h == -1)
    {
      $unknownInfo .= ',' if ($unknownInfo) ;
      $unknownInfo .= $sv ;
    }
    else
    {
      $unhealthInfo .= ',' if ($unhealthInfo) ;
      $unhealthInfo .= "$sv:$h" ;
    }
  }

  # Compact the $healthInfo string by looking for patterns in the
  # numbers.  Admittedly this is obscure, but it is algorithmically
  # obscure to begin with.  We start with a comma separated list of
  # numbers.  When we find and existing chain '...-N,N+1' we extend 
  # the chain to '...-N+2'.
  # Chains are started when we find a pattern 'N,N+1,N+2', which is
  # reduced to 'N-N+2'.
  for (my $sv = 1 ; $sv <=31 ; ++$sv)
  {
    my $s1 = sprintf( "%d", $sv) ;
    my $s2 = sprintf( "%d", $sv+1) ;
    my $s3 = sprintf( "%d", $sv+2) ;

    $healthInfo =~ s/-$s1,$s2/-$s2/ ;
    $healthInfo =~ s/$s1,$s2,$s3/$s1-$s3/ ;
  }


  # Replace empty lists with 'none'.
  $healthInfo = 'none' if (!$healthInfo ) ;
  $unhealthInfo = 'none' if (!$unhealthInfo ) ;
  $unknownInfo = 'none' if (!$unknownInfo ) ;

  print "GpsHealth",
        " healthy=$healthInfo" ,
        " unhealthy=$unhealthInfo" ,
        " unknown=$unknownInfo", 
        "\n" ;

} # end of ShowGpsHealth



########################################################################
sub ResetSystem
########################################################################
# Reboots the NetRS.
#
# Accepts a single optional parameter "defaults=[yes|no]".  By default
# this is "no".  If 'defaults=yes' then we set the flags that will
# trigger resetting the system to factory defaults on ALL controls.
# - THIS IS DANGEROUS.  
# - You can lose communications with the system if, for example, the
#   default DHCP network setup doesn't match your current setup.  
# - This feature is only implemented for Trimble production testing.  
# - This feature should not be advertised to customers, nor documented
#   in the online help.  
{
  my $defaults = 0 ;

  return if ( badParmCount( scalar( @_ ), 0, 1 ) ) ;

  # If a parameter was given, then it has to be 'defaults=[yes|no]'
  if (scalar( @_ ) != 0)
  {
    if ( @_[0] =~ /^defaults=(yes|no)$/i)
    {
      $defaults = 1 if ($1 =~ /yes/i) ;
    }
    else
    {
      print "ERROR: Invalid parameter $_[0]\n" ;
      return ;
    }
  }

  if ($defaults)
  {
    print "OK: Restarting with Factory Defaults.\n" ;

    # Set boot startup flags to trigger defaulting controls.
    system( "/usr/bin/sudo /sbin/boot_status 0 254 f" ) ;
  }
  else
  {
    print "OK: Restarting this NetRS.\n" ;
  }

  # Send the SIGUSR2 to the frontpanel task to trigger a restart.
  # We do this by launching a background process that is delayed by a
  # few seconds.  That lets the response message have a chance to get
  # out to the user who sent the command.
  # In addition, we launch a separate background command, with a much
  # larger delay that directly resets the system with "init".  This
  # covers the case where something goes wrong with signaling
  # frontpanel_task.
  my $spid = qx( cat /var/run/frontpanel_task.pid ) ;
  system( "(sleep 5 ; /usr/bin/sudo /bin/kill -SIGUSR2 $spid)&" ) ;

  system( "(sleep 30 ; /usr/bin/sudo /sbin/init 6)&" ) ;

} # end of ResetSystem



########################################################################
sub ResetGpsData
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  print "OK: Clearing GpsData and restarting satellite tracking operations.\n" ;


  # Send the desired signal to the frontpanel task
  my $spid = qx( cat /var/run/frontpanel_task.pid ) ;

  system( "(sleep 5 ; /usr/bin/sudo /bin/kill -SIGHUP $spid)&" ) ;
} # end of ResetGpsData



########################################################################
sub ShowFtpSetup
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  # Read the current FTP controls into a hash.
  # There will be elements for 
  #   standardPort = [ yes, no ]
  #   alternatePort = [1024...65535]
  #   anon  = ['disabled' 'enabled' 'enabledDelete']
  #   named = ['disabled' 'enabled' 'enabledDelete']
  #   admin = ['disabled' 'enabled']
  #   namedPassword = Encrypted password for user netrsFTP
  #   adminPassword = Encrypted password for user adminFTP
  my %ftpSetup ;
  getFtpSetup( \%ftpSetup ) ;

  if ( ! $ftpSetup{'namedPassword'} )
  {
    $ftpSetup{'namedPassword'} = 'NotSet' ;
  }
  if ( ! $ftpSetup{'adminPassword'} )
  {
    $ftpSetup{'adminPassword'} = 'NotSet' ;
  }

  print "FtpSetup anon=",
        ($ftpSetup{'anon'} eq 'enabledDelete')
        ? 'enabledWithDelete'
        : $ftpSetup{'anon'},

        " named=",
        ($ftpSetup{'named'} eq 'enabledDeleting')
        ? 'enabledWithDelete'
        : $ftpSetup{'named'},

        " admin=",
        $ftpSetup{'admin'},

        " namedPassword=$ftpSetup{'namedPassword'}",
        " adminPassword=$ftpSetup{'adminPassword'}",

        " port=",
        ($ftpSetup{'ftpEnabled'} eq 'no')
        ? 'disabled'
        : ($ftpSetup{'standardPort'} eq 'yes')
          ? 21
          : $ftpSetup{'alternatePort'},
        "\n"
        ;

} # end of ShowFtpSetup



########################################################################
sub SetFtpSetup
########################################################################
# Processes the command Set FtpSetup, which is used to control
# the capabilities of the FTP subsystem.  In particular, the user
# can control which of three accounts are enabled, whether those
# accounts have delete privileges, and what passwords are allowed on
# those accounts.
# Valid parameter options are:
#   anon=(enabled|enabledWithDelete|disabled)
#       Controls the anonymous ftp account.
#
#  named=(enabled|enabledWithDelete|disabled)
#       Controls the "netrsFTP" account.
#
#  admin=(enabled|disabled)
#       Controls the "adminFTP" account.
#
#  namedPassword=(xxxxxxxxxxxxx|NotSet)
#  namedPassword=
#       Allows user to enter an encrypted password for netrsFTP.
#       This lets you transfer a password between systems.
#       Encrypted value must be 13 characters, and is case sensitive.
#       NotSet or a blank password removes that password, and
#       effectively disables the netrsFTP account.
#
#  adminPassword=(xxxxxxxxxxxxx|NotSet)
#  adminPassword=
#       Allows user to enter an encrypted password for adminFTP.
#       This lets you transfer a password between systems.
#       Encrypted value must be 13 characters, and is case sensitive.
#       NotSet or a blank password removes that password, and
#       effectively disables the adminFTP account.
#
#  namedClearPW=cleartext
#       Allows user to enter an UNencrypted password for netrsFTP.
#       This value is encrypted and saved internally.
#
#  adminClearPW=cleartext
#       Allows user to enter an UNencrypted password for adminFTP.
#       This value is encrypted and saved internally.
#
#  port=21       : Use standard FTP port 21
#  port=disabled : Disable FTP entirely
#  port=ddddd    : Use alternate FTP port as specified.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 5 ) ) ;

  # Read the current FTP controls into a hash.
  # There will be elements for 
  #   standardPort = [ yes, no ]
  #   alternatePort = [1024...65535]
  #   anon  = ['disabled' 'enabled' 'enabledDelete']
  #   named = ['disabled' 'enabled' 'enabledDelete']
  #   admin = ['disabled' 'enabled']
  #   namedPassword = Encrypted password for user netrsFTP
  #   adminPassword = Encrypted password for user adminFTP
  my %ftpSetup ;
  getFtpSetup( \%ftpSetup ) ;


  # Run through the parameters, checking for validity,
  # and adjusting the current setup values to match any new
  # parameters.
  for $_ (@_)
  {
    #print $_,"\n" ;
    if ( m/^anon=(disabled|enabled|enabledWithDelete)$/i )
    {
      #print "Setting anon to $1\n" ;
      $ftpSetup{'anon'} = m/delete/i 
                          ? 'enabledDelete'
                          : ( m/disabled/i
                              ? 'disabled'
                              : 'enabled'
                            ) ;
    }
    elsif ( m/^named=(disabled|enabled|enabledWithDelete)$/i )
    {
      #print "Setting named to $1\n" ;
      $ftpSetup{'named'} = m/delete/i 
                           ? 'enabledDelete'
                           : ( m/disabled/i
                               ? 'disabled'
                               : 'enabled'
                             ) ;
    }
    elsif ( m/^admin=(disabled|enabled)$/i )
    {
      #print "Setting admin to $1\n" ;
      $ftpSetup{'admin'} = m/enabled/i
                           ? 'enabled'
                           : 'disabled' ;
    }
    elsif ( m/^namedPassword=(.*)$/i )
    {
      my $pw = $1 ;
      #print "Setting named password to $pw\n" ;
      if ($pw =~ m/^(NotSet|)$/i)
      {
        $ftpSetup{'namedPassword'} = '' ;
      }
      elsif ($pw =~ m/^.{13}$/)
      { 
        $ftpSetup{'namedPassword'} = $pw ;
      }
      else
      {
        print "ERROR: Invalid password $_\n" ;
        return ;
      }
    }
    elsif ( m/^adminPassword=(.*)$/i )
    {
      #print "Setting admin password to $1\n" ;
      my $pw = $1 ;
      if ($pw =~ m/^(NotSet|)$/i)
      {
        $ftpSetup{'adminPassword'} = '' ;
      }
      elsif ($pw =~ m/^.{13}$/)
      { 
        $ftpSetup{'adminPassword'} = $pw ;
      }
      else
      {
        print "ERROR: Invalid password $_\n" ;
        return ;
      }
    }
    elsif ( m/^adminClearPw=(.*)$/i )
    {
      $ftpSetup{'adminPassword'} = makepw( $1 ) ;
    }
    elsif ( m/^namedClearPw=(.*)$/i )
    {
      $ftpSetup{'namedPassword'} = makepw( $1 ) ;
    }
    elsif ( m/^port=disabled$/i )
    {
      $ftpSetup{'ftpEnabled'} = 'no' ;
      $ftpSetup{'standardPort'} = 'yes' ;
    }
    elsif ( m/^port=standard$/i )
    {
      $ftpSetup{'ftpEnabled'} = 'yes' ;
      $ftpSetup{'standardPort'} = 'yes' ;
    }
    elsif ( m/^port=(\d*)$/i )
    {
      if ($1 == 21)
      {
        $ftpSetup{'ftpEnabled'} = 'yes' ;
        $ftpSetup{'standardPort'} = 'yes' ;
      }
      elsif ($1 >= 1024 and $1 <= 65535)
      {
        $ftpSetup{'ftpEnabled'} = 'yes' ;
        $ftpSetup{'standardPort'} = 'no' ;
        $ftpSetup{'alternatePort'} = $1 ;
      }
      else
      {
        print "ERROR: Port $1 out of range 21, 1024..65535.\n" ;
        return ;
      }
    }
    else
    {
      print "ERROR: Invalid parameter $_\n" ;
      return ;
    }
  }

  # Have now determined the new setup requested, and it's all okay.
  # Save that in /etc/sysconfig/ftp.conf, and restart the daemon.
  saveFtpSetup( \%ftpSetup ) ;

  # Done, send back the OK response.
  print "OK: " ;
  ShowFtpSetup() ;

} # end of SetFtpSetup()



########################################################################
sub ShowGuiSecurity
########################################################################
# Shows the current settings of the GUI access security system. 
# Returns:
#   GuiSecurity system=restricted gps=unrestricted files=unrestricted
# Each of the security levels is either 'restricted' or 'unrestricted'.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  my %restrictions ;
  getGuiSecurity( \%restrictions ) ;
  print "GuiSecurity",
        " system=",
        $restrictions{ 'sysadmin' } ? 'restricted' : 'unrestricted',
        " gps=",
        $restrictions{ 'gpsadmin' } ? 'restricted' : 'unrestricted',
        " files=",
        $restrictions{ 'fileaccess' } ? 'restricted' : 'unrestricted',
        "\n" ;

} # end of ShowGuiSecurity



########################################################################
sub SetGuiSecurity
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 3 ) ) ;

  # Get the current access restrictions.
  my %restrictions ;
  getGuiSecurity( \%restrictions ) ;

  for $_ (@_)
  {
    #print $_,"\n" ;
    if ( m/^(system|gps|files)=(restricted|unrestricted)$/i )
    {
      #print "setting $1 to $2\n" ;
      my $level = lc($1) ;
      my $value = lc($2) ;

      $restrictions{ $level eq 'system'
                     ? 'sysadmin'
                     : $level eq 'gps'
                       ? 'gpsadmin'
                       : 'fileaccess' } = ($value eq 'restricted') ;
    }
    else
    {
      print "ERROR: Invalid parameter $_\n" ;
      return ;
    }
  }
 
  #print join( ' ', %restrictions ), "\n" ;
  updateGuiSecurity( \%restrictions ) ;

  print "OK: " ;
  ShowGuiSecurity() ;

} # end of SetGuiSecurity




########################################################################
sub ShowGuiAccount
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $name = $_[0] ;
  $name =~ s/^name=//i ;

  my %accounts ;
  getGuiAccounts( \%accounts ) ;

  if ( !exists( $accounts{$name} ) )
  {
    print "ERROR: GuiAccount $name does not exist.\n" ;
    return ;
  }

  printf "GuiAccount name=%-8s", $name ;
  print ' password=',
        $accounts{$name}{'password'},
        ' system=',
        $accounts{$name}{'sysadmin'} ? 'yes' : 'no ',
        ' gps=',
        $accounts{$name}{'gpsadmin'} ? 'yes' : 'no ',
        ' files=',
        $accounts{$name}{'fileaccess'} ? 'yes' : 'no ',
        "\n" ;

} # end of ShowGuiAccount



########################################################################
sub ShowGuiAccounts
########################################################################
# Show a listing of ALL defined GuiAccounts.
# Response is a Typical multi-line listing, bracketed by 
# <Show GuiAccounts> and <end of ShowGuiAccounts>.
# The process just grabs the GuiAccount information and then calls
# ShowGuiAccount() to display each of the accounts in the hash.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  my %accounts ;

  print "<Show GuiAccounts>\n" ;

  getGuiAccounts( \%accounts ) ;
  for my $name (sort keys %accounts )
  {
    ShowGuiAccount( $name ) ;
  }

  print "<end of Show GuiAccounts>\n" ;

} # end of ShowGuiAccount



########################################################################
sub SetGuiAccount
########################################################################
# Creates or modified a GuiAccount using the supplied parameters.
#   name=user              # Must be specified.
#
#   password=Zjjiiwel1k79d # Encrypted password
#   clearPassword=SomeText # Clear-text password
#   system=(yes|no)        # User will/won't have sysadmin privileges
#      gps=(yes|no)        # User will/won't have gpsadmin privileges
#    files=(yes|no)        # User will/won't have fileaccess privileges
#
# On new accounts, then a password must be specified in one of the two
# forms.  Unspecified privileges default to "no".
#
# On existing accounts, unspecified parameters are left at their current
# values.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 5 ) ) ;

  # Get current GuiAccount data
  my %accounts ;
  getGuiAccounts( \%accounts ) ;

  my %parms ;
  # Go through all of the parameters and 
  for my $p (@_)
  {
    if ($p !~ /=/)
    {
      print "ERROR: Must fully specify id=value for parameter $p\n" ;
      return ;
    }

    my ($id, $value) = split( '=', $p ) ;

    if ( $id =~ m/^(system|gps|files)$/i )
    {
      $value = lc( $value ) ;
      if ( ($value ne 'yes') and ($value ne 'no') ) 
      {
        print "ERROR: Invalid parameter $p\n" ;
        return ;
      }
    }
    elsif ( $id =~ m/^password$/i )
    {
      # Check for 13 character encrpted password
      if ( $value !~ m/^.{13}$/ )
      {
        print "ERROR: Invalid encrypted password.\n" ;
        return ;
      }
    }
    elsif ( $id =~ m/^clearpassword$/i )
    {
      # Check for valid password, at least 6 characters
      if ( $value !~ m/^.{6,}$/ )
      {
        print "ERROR: Password is too short.\n" ;
        return ;
      }
    }
    elsif ( $id =~ m/^name$/i )
    {
      # Don't allow non-printing characters in account names.
      if ($value =~ m/[\x00-\x20\x7f-\xff]/)
      {
        print "ERROR: Invalid account name.\n" ;
        return ;
      }
    }
    else
    {
      print "ERROR: Invalid parameter $p.\n" ;
      return ;
    }

    $parms{lc($id)} = $value ;

    #print "$id  =  $value\n" ; 
  }

  # To make things easier in the next section, copy all of the
  # parameter values into simple variables.  Note that some
  # of these may be undefined, which is okay.
  my $name = $parms{'name'} ;
  my $password = $parms{'password'} ;
  my $clearPassword = $parms{'clearpassword'} ;
  my $system = $parms{'system'} ;
  my $gps = $parms{'gps'} ;
  my $files = $parms{'files'} ;

  if (! $name )
  {
    print "ERROR: Must specify name for GuiAccount.\n" ;
    return ;
  }

  if ($clearPassword)
  {
    # Encrypt the cleartext password into $password.
    $password = makepw( $clearPassword ) ;
  }

  if ( ! exists( $accounts{$name} ) )
  {
    # This is a new account.
    # Check that a password has been specified, and set up its defaults.
    if ( !$password )
    {
      print "ERROR: Must specify password for new account.\n" ;
      return ;
    }

    $accounts{$name}{'sysadmin'} = 0 ;
    $accounts{$name}{'gpsadmin'} = 0 ;
    $accounts{$name}{'fileaccess'} = 0 ;
  }

  if ($password) 
  {
    $accounts{$name}{'password'} = $password ;
  }

  if ($system) 
  {
    $accounts{$name}{'sysadmin'} = ($system eq 'yes' ? 1 : 0 ) ;
  }

  if ($gps) 
  {
    $accounts{$name}{'gpsadmin'} = ($gps eq 'yes' ? 1 : 0 ) ;
  }

  if ($files) 
  {
    $accounts{$name}{'fileaccess'} = ($files eq 'yes' ? 1 : 0 ) ;
  }

  if ( $name eq 'sysadmin'
    && ( !$accounts{'sysadmin'}{'sysadmin'}
      || !$accounts{'sysadmin'}{'gpsadmin'}
      || !$accounts{'sysadmin'}{'fileaccess'}
       )
     )
  {
    print "ERROR: Can't modify permissions on sysadmin account.\n" ;
    return ;
  }

  updateGuiAccounts( \%accounts ) ;

  print "OK: " ;
  ShowGuiAccount( $parms{'name'} ) ;

} # end of SetGuiAccount



########################################################################
sub DeleteGuiAccount
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  # Get current GuiAccount data
  my %accounts ;
  getGuiAccounts( \%accounts ) ;

  my $name = $_[0] ;
  $name =~ s/^name=//i ;

  if ( $name eq 'sysadmin' ) 
  {
    print "ERROR: Can't delete sysadmin account.\n" ;
    return ;
  }

  if ( !exists( $accounts{$name} ) )
  {
    print "ERROR: GuiAccount $name does not exist.\n" ;
    return ;
  }

  delete( $accounts{$name} ) ;
  updateGuiAccounts( \%accounts ) ;

  print "OK: GuiAccount $name is deleted.\n" ;

} # end of DeleteGuiAccount



########################################################################
sub ShowAntennaTypes
########################################################################
# Emits a list of all supported antenna types.
# Usual multi-line response bracketted with <Show AntennaTypes>
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  print "<Show AntennaTypes>\n" ;

  my %antInfo ;
  getAntennaInfo( \%antInfo ) ;

  foreach (sort( keys %antInfo ))
  {
    my $ngsName = $antInfo{$_}{'ngs'} ;
    $ngsName =~ s/ /_/g ;

    my $name = $antInfo{$_}{'name'} ;
    $name =~ s/ /_/g ;

    printf "type=%-3d code=%-2s NgsIgs=%-20s name=%s\n", 
            $_, 
            $antInfo{$_}{'code'},
            $ngsName, 
            $name ;
  }

  print "<end of Show AntennaTypes>\n" ;

} # end of ShowAntennaTypes



########################################################################
sub ShowAntenna
########################################################################
# Shows the characteristics of the current antenna setup.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  my %antInfo ;
  getAntennaInfo( \%antInfo ) ;

  my %antSetup ;
  getAntennaSetup( \%antSetup ) ;

  my $type = $antSetup{'type'} ;

  print "Antenna serialNumber=$antSetup{'serial'}" ;

  my $ts = $antSetup{'textSerial'} ;
  $ts =~ s/'/\\'/g ;
  print " textSerialNumber='$ts'" ;

  print " type=$type" ;

  my $method = $antInfo{$type}{'methods'}[$antSetup{'method'}] ;
  $method =~ s/ /_/g ;
  print " refPoint=$method" ;

  print " height=$antSetup{'height'}" ;

  my $name = $antInfo{$type}{'name'} ;
  $name =~ s/ /_/g ;
  print " name=$name" ;

  my $code = $antInfo{$type}{'code'} ;
  print " code=$code" ;

  my $ngs = $antInfo{$type}{'ngs'} ;
  $ngs =~ s/ /_/g ;
  print " NgsIgs=$ngs" ;
  print "\n";

} # end of ShowAntenna



########################################################################
sub SetAntenna
########################################################################
# Sets the characteristics of the current antenna setup.
# Several parameters can be set, either individually or in
# combination.  
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 8 ) ) ;

  my %antInfo ;
  getAntennaInfo( \%antInfo ) ;

  # Get the curent antenna setup values into a hash that defines:
  #  'type' = Trimble numeric identifier of antenna type.
  #  'serial'
  #  'textSerial'
  #  'height'
  #  'method' : Index into the methods list for this antenna type.
  #             The index to string mapping is not the same for all
  #             antennas.
  my %antSetup ;
  getAntennaSetup( \%antSetup ) ;


  # Below we may be changing antenna type.  The new type may not be
  # compatible with the old refpoint method.  Or we might be specifying
  # a new refpoint method.  Regardless, we eventually have to check that
  # the method (new or old) is compatible with the antenna type (new or
  # old).  And this has to be done with string descriptions rather than
  # method-indices, because the indices mean different things to
  # different antenna types.  So... We have to save the string form of
  # the old antenna type here, in case we need it later, after the
  # antSetup{'type'} is modified.  This value will be overwritten with a
  # new string if a "refpoint=" parameter is received.
  my $newRefPoint =
           $antInfo{$antSetup{'type'}}{'methods'}[$antSetup{'method'}] ;


  # Parse the parameters that were fed into this subroutine.
  for my $p (@_)
  {
    if ($p !~ /=/)
    {
      print "ERROR: Must fully specify id=value for parameter $p\n" ;
      return ;
    }

    # Split the parameter around the equal sign.  The parameter
    # identifier is case insensitive, so we standardize on lower-case
    # for it in all tests below.
    my ($id, $value) = split( '=', $p ) ;
    $id = lc($id) ;
    #print "$id = $value \n" ;

    if ($id eq 'serialnumber')
    {
      if ($value !~ /^\d{1,8}$/)
      {
        print "ERROR: SerialNumber must an integer < 100000000.\n" ;
        return ;
      }
      $antSetup{'serial'} = $value + 0 ;
    }
    elsif ($id eq 'textserialnumber')
    {
      $antSetup{'textSerial'} = $value ;
    }
    elsif ($id eq 'height')
    {
      # Check height for valid value
      if ( $value !~ m/^\d*\.\d+$/ # .333 or 333.333
       and $value !~ m/^\d+\.\d*$/ # 333.
       and $value !~ m/^\d+$/     # 333
         )
      {
        print "ERROR: Illegal height value '$value'\n" ;
        return ;
      }

      $antSetup{'height'} = $value ;
    }
    elsif ($id eq 'type')
    {
      # Verify that the type specified is numeric, and that a 
      # matching antenna type exists in our 'supported' data.
      # Note that the $value is a string, and must be converted to
      # numeric to be consistent with other usages.
      if ( $value !~ /\d+/
        or !exists( $antInfo{$value+0} ) 
         )
      {
        print "ERROR: Unsupported antenna type '$value'\n" ;
        return ;
      }
      $antSetup{'type'} = $value+0 ;
    }
    elsif ($id eq 'refpoint')
    {
      # The antenna height RefPoint is specified by a string such as
      # 'Bottom_of_antenna_mount' or 'APC'.  Only specific combinations
      # of antenna type and refpoint are supported.  Since we don't
      # necessarily know at this point what the antenna type will be,
      # we can only record the requested refpoint and then check it 
      # versus the type later.
      $newRefPoint = $value ;
    }
    elsif ($id eq 'name')
    {
      # Compare the given name to those that are supported.
      # To make this as forgiving as possible, we make all comparisons
      # in upper-case text, and drop all spaces and underscores.
      my $ucvalue = uc($value) ; 
      $ucvalue =~ s/[ _]//g ;
      my $notfound = 1 ;
      for my $type ( keys %antInfo )
      {
        my $supported = uc( $antInfo{$type}{'name'} ) ;
        $supported =~ s/[ _]//g ;
        if ( $ucvalue eq $supported )
        {
          $antSetup{'type'} = $type ;
          $notfound = 0 ;
          last ;
        }
      }
      if ($notfound)
      {
        print "ERROR: Invalid antenna name '$value'.\n" ;
        return ;
      }
    }
    elsif ($id eq 'code')
    {
      # Compare the given code to those that are supported.
      # To make this as forgiving as possible, we make all comparisons
      # in upper-case text.  
      my $ucvalue = uc($value) ; 
      my $notfound = 1 ;
      for my $type ( keys %antInfo )
      {
        if ( $ucvalue eq uc( $antInfo{$type}{'code'} ) )
        {
          $antSetup{'type'} = $type ;
          $notfound = 0 ;
          last ;
        }
      }
      if ($notfound)
      {
        print "ERROR: Invalid antenna code '$value'.\n" ;
        return ;
      }
    }
    elsif ($id eq 'ngsigs')
    {
      # Compare the given name to those that are supported.
      # To make this as forgiving as possible, we make all comparisons
      # in upper-case text, and drop all spaces and underscores.
      my $ucvalue = uc($value) ; 
      $ucvalue =~ s/[ _]//g ;
      my $notfound = 1 ;
      for my $type ( keys %antInfo )
      {
        my $supported = uc( $antInfo{$type}{'ngs'} ) ;
        $supported =~ s/[ _]//g ;
        if ( $ucvalue eq $supported )
        {
          $antSetup{'type'} = $type ;
          $notfound = 0 ;
          last ;
        }
      }
      if ($notfound)
      {
        print "ERROR: Invalid antenna NgsIgs name '$value'.\n" ;
        return ;
      }
    }
    else
    {
      print "ERROR: Invalid parameter $p.\n" ;
      return ;
    }
  }

  # Have a new set of antenna parameters in %antSetup.
  # Do a little more verification and then save those settings.
  my $type = $antSetup{'type'} ;

  # We may have a new antenna type OR a new refpoint method string 
  # (or neither, in which case we are about to waste some time.)
  # Regardless, we need to verify that the requested method is supported
  # for the requested antenna type.
  # Compare the method to those that are supported for the specified
  # antenna type.  To make this as forgiving as possible, we make all
  # comparisons in upper-case text, and drop all spaces and underscores.
  # The result is a refpoint method integer index (or an error message.)
  my $ucvalue = uc($newRefPoint) ; 
  $ucvalue =~ s/[ _]//g ;
  my $notfound = 1 ;
  for ( my $method = 0 ; 
        $method < $antInfo{$type}{'methodCount'} ;
        ++$method
      )
  {
    my $methodString = uc( $antInfo{$type}{'methods'}[$method] ) ;
    $methodString =~ s/[ _]//g ;
    if ( $ucvalue eq $methodString )
    {
      $antSetup{'method'} = $method ;
      $notfound = 0 ;
      last ;
    }
  }
  if ($notfound)
  {
    print "ERROR: Unsupported antenna refpoint method '$newRefPoint'.\n" ;
    return ;
  }

  #for my $k (keys %antSetup)
  #{
  #  print "$k = $antSetup{$k}\n" ;
  #}


  updateAntennaSetup( $antSetup{'serial'},
                      $antSetup{'type'},
                      $antSetup{'height'},
                      $antSetup{'method'},
                      $antInfo{$type}{'name'},
                      $antSetup{'textSerial'},
                    ) ;

  print "OK: " ;
  ShowAntenna ;

} # end of ShowAntenna



########################################################################
sub ShowEthernet
########################################################################
# Shows the current Ethernet port configuration.
# We obtain the setup from getEthernetSetup() and then format and
# display the individual values.
# The exact format is pretty easy to discern from the code below.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  my %ethernetSetup ;
  getEthernetSetup( \%ethernetSetup ) ;

  print "Ethernet",
         " ipmode=$ethernetSetup{'ipmode'}",
         " ip=$ethernetSetup{'ip'}",
         " netmask=", $ethernetSetup{'netmask'} || 'NotSet',
         " gateway=", $ethernetSetup{'gateway'} || 'NotSet',
         " nameServers=", $ethernetSetup{'nameServers'} || 'NotSet',
         " dnsDomain=", $ethernetSetup{'dnsDomain'} || 'NotSet',
         " dnsSearch=", $ethernetSetup{'dnsSearch'} || 'NotSet',
         " mac=$ethernetSetup{'mac'}",
         " mtu=$ethernetSetup{'mtu'}",
         "\n" ;

} # end of ShowEthernet



########################################################################
sub validIP
########################################################################
# Determines if an IP address is a valid xxx.xxx.xxx.xxx format.
# Returns the string, perhaps slightly modified, if okay.  Otherwise
# returns an empty string to indicate failure.
# The octets are checked for range 0..255.
# We modify the string to remove any leading zeroes in the octets.
########################################################################
{
  my $ip = $_[0] ;

  return '' if ( $ip !~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/ ) ;

  return '' if ($1 > 255 || $2 > 255 || $3 > 255 || $4 > 255) ;

  return sprintf( '%d.%d.%d.%d', $1, $2, $3, $4 ) ;

} # end of validIP()



########################################################################
sub SetEthernet
########################################################################
# Allows the user to specify several different parameters that configure
# the Ethernet port.
# Parameters are:
#   ipmode=(static|dhcp)    : Mode to assign IPs
#   ip=xxx.xxx.xxx.xxx      : IP address for this receiver.
#   netmask=xxx.xxx.xxx.xxx : Network mask 
#   gateway=xxx.xxx.xxx.xxx : Network gateway
#   namservers=xxx.xxx.xxx.xxx,xxx.xxx.xxx.xxx,xxx.xxx.xxx.xxx
#                           : DNS name servers in a comma separated
#                             list.  
#   dnsDomain               : DNS domain for this box.
#   dnsSearch               : DNS search path, in a comma separated
#                             list of domains.
#   mac                     : MAC (low-level ethernet hardware address)
#                             This parameter is ignored.
#   mtu                     ; Integer value for Maximum Transfer Unit.
#
# All parameters must be specified with the 'id=value' syntax.
# Parameters that aren't specified are defaulted to their current
# values.  
#
# Some values can be specified as blank, using 
#       dnsSearch=
#    or dnsSearch=NotSet
#
# Note that if ipmode is dhcp, then all of the other values are
# ignored.  However, since those values are emitted in a Show Ethernet
# command, even in dhcp mode, we want to accept them without any warning
# message.
#
# Note that the 'mac=' parameter will be accepted without error, but is
# ignored, since it is an unchangeable characteristic of the hardware.
#
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 10 ) ) ;

  # Get current setup.  These provide the defaults for all parameters
  # that aren't explicitly set below.
  my %ethernetSetup ;
  getEthernetSetup( \%ethernetSetup ) ;

  # Parse the parameters that were fed into this subroutine.
  for my $p (@_)
  {
    if ($p !~ /=/)
    {
      print "ERROR: Must fully specify id=value for parameter $p\n" ;
      return ;
    }

    # Split the parameter around the equal sign.  The parameter
    # identifier is case insensitive, so we standardize on lower-case
    # for it in all tests below.
    my ($id, $value) = split( '=', $p ) ;
    my $lcvalue = lc( $value ) ;
    $id = lc($id) ;
    #print "$id = $value \n" ;

    if ($id eq 'ipmode')
    {
      if ($lcvalue ne 'static' && $lcvalue ne 'dhcp')
      {
        print "ERROR: ipmode must be 'static' or 'dhcp'\n" ;
        return ;
      }
      $ethernetSetup{'ipmode'} = $lcvalue ;
    }
    elsif ($id eq 'ip')
    {
      if ( ! ($ethernetSetup{'ip'} = validIP( $value )) )
      {
        print "ERROR: Invalid IP address, '$value'\n" ;
        return ;
      }
    }
    elsif ($id eq 'mtu')
    {
      if ( $value !~ /^\d{3,4}$/
        || $value > 1500
        || $value < 600
         )
      {
        print "ERROR: Invalid MTU value, '$value'\n" ;
        return ;
      }
      $ethernetSetup{'mtu'} = $value ;
    }
    elsif ($id eq 'netmask')
    {
      if ($lcvalue eq 'notset' or $lcvalue eq '')
      {
        $ethernetSetup{'netmask'} = '' ;
      }
      elsif ( ! ($ethernetSetup{'netmask'} = validIP( $value )) )
      {
        print "ERROR: Invalid Netmask '$value'\n" ;
        return ;
      }
    }
    elsif ($id eq 'gateway')
    {
      if ($lcvalue eq 'notset' or $lcvalue eq '')
      {
        $ethernetSetup{'gateway'} = '' ;
      }
      elsif ( ! ($ethernetSetup{'gateway'} = validIP( $value )) )
      {
        print "ERROR: Invalid Gateway IP '$value'\n" ;
        return ;
      }
    }
    elsif ($id eq 'nameservers')
    {
      $value = '' if ($lcvalue eq 'notset') ;
      my @nameservers = split( ',', $value ) ;
      if (@nameservers > 3)
      {
        print "ERROR: Too many Nameservers Specified\n" ;
        return ;
      }

      foreach my $ns (@nameservers)
      {
        my $ns = validIP( $ns ) ;
        if (!$ns)
        {
          print "ERROR: Invalid Nameserver IP\n" ;
          return ;
        }
      }
      # Format the nameservers into a comma separated list.
      $ethernetSetup{'nameServers'} = join( ',', @nameservers ) ;
    }
    elsif ($id eq 'dnsdomain')
    {
      $value = '' if ($lcvalue eq 'notset') ;
      $ethernetSetup{'dnsDomain'} = $value ;
    }
    elsif ($id eq 'dnssearch')
    {
      $value = '' if ($lcvalue eq 'notset') ;
      $ethernetSetup{'dnsSearch'} = $value ;
    }
    elsif ($id eq 'mac')
    {
      # 'mac=' parameter is ignored.  Can't be changed.
    }
    else
    {
      print "ERROR: Invalid parameter $p.\n" ;
      return ;
    }
  }

  # Now that the %ethernetSetup structure is updated, we need to
  # update the Ethernet configuration files to match and then 
  # restart networking with the new values.
  updateEthernetSetup( \%ethernetSetup ) ;

  # Note that the previous call to update the networking has an 
  # intentional 5 second delay before anything happens and that the
  # network restart will happen in background.  So when we reach
  # this point, the changes haven't occurred yet.  Thus, we can'
  # display the new settings to the user.  In fact, all we want to do is
  # say that we are installing the new data.  User has to be aware that
  # they may soon lose communications, etc.  
  # Ideally we'd give a longer response here, but that's best left to a
  # manual page.
  print "OK: Installing new ethernet configurations.  ",
        "Wait 15 seconds for that to take effect.\n" ;

} # end of SetEthernet



########################################################################
sub EnableSession
########################################################################
# Alternative to using Set Session name=abc enable=yes
# Instead you can use  Enable Session name=abc
#                  or  Enable Session abc
# $_[0] will be the session name.
########################################################################
{
  #print "Enable Session\n" ;

  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $sessname = $_[0] ;

  # The parameter must have NO '=' or be of the form 'name='
  if ( $sessname =~ /=/
    && $sessname !~ /^name=/i
     )
  {
    print "ERROR: Invalid parameter $sessname.\n";
    return ;
  }

  # Drop name= if it exists
  $sessname =~ s/^name=//i ;

  if ( EnaDisSession( $sessname, 1) )
  {
    print "OK: Session name=$sessname enable=yes\n" ;
  }
  else
  {
    print "ERROR: No Session '$sessname' exists.\n";
  }

} # end of EnableSession()



########################################################################
sub DisableSession
########################################################################
# Alternative to using Set Session name=abc enable=no
# Instead you can use  Disable Session name=abc
#                  or  Disable Session abc
# $_[0] will be the session name.
########################################################################
{
  #print "Disable Session\n" ;

  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $sessname = $_[0] ;

  # The parameter must have NO '=' or be of the form 'name='
  if ( $sessname =~ /=/
    && $sessname !~ /^name=/i
     )
  {
    print "ERROR: Invalid parameter $sessname.\n";
    return ;
  }

  # Drop name= if it exists
  $sessname =~ s/name=//i ;

  if ( EnaDisSession( $sessname, 0) )
  {
    print "OK: Session name=$sessname enable=no\n" ;
  }
  else
  {
    print "ERROR: No Session '$sessname' exists.\n";
  }

} # end of DisableSession()



########################################################################
sub formatSchedString
########################################################################
# Takes two parameters, 
#   $_[0] : schedule type string
#   $_[0] : time value
# Returns a string formatted tho show the schedule in human readable
# form.
########################################################################
{
  my $sched = $_[0] ;
  my $time  = $_[1] ;

  if ($sched eq 'manual')
  {
    return 'Manual' ;
  }
  elsif ($sched eq 'cont')
  {
    return 'Continuous' ;
  }
  elsif ($sched eq 'daily')
  {
    return sprintf( 'Daily@%02d:%02d', $time/3600, $time/60%60 ) ;
  }
  else # Once-Only Session  
  {
    my (undef,$min,$hour,$day,$month,$year,undef)
      = gmtime( $time + 315964800 ) ;

    return sprintf( 'Once@%4d/%02d/%02d/%02d:%02d', 
                    $year+1900,
                    $month+1,
                    $day,
                    $time/3600%24, # Hours
                    $time/60%60 ,  # Minutes
                  ) ;
  }

} # end of formatSchedString()



########################################################################
sub formatSession
########################################################################
# $_[0] = session's name
# $_[1] = Reference to a hash containing the session data.
# Returns a string containing the formatted description of the
# session.  Returns empty string (false) if no such session.
{
  my $String = 'Session ' ;

  my $name = $_[0] ;
  my %sessData = %{$_[1]} ;

  return '' if (! exists( $sessData{$name} ) ) ;

  $String .= "name=$name" ;

  my $sched = $sessData{$name}{'sched'} ;
  my $enable = $sessData{$name}{'enable'} ;
  my $time = $sessData{$name}{'time'} ;
  my $duration = $sessData{$name}{'duration'} ;

  my $sessid = $sessData{$name}{'sessid'} ;
  my $perDay = $sessData{$name}{'perday'} ;
  my $perSessid = $sessData{$name}{'persessid'} ;
  # If enabled-manual session, determine if its time has elapsed and
  # force it to disabled in this listing.

  $String .= ' enable=' . ( ($enable==1)?'yes':'no ') ; 

  $String .= ' schedule=' ;
  $String .= formatSchedString( $sched, $time ) ;

  $String .= " duration=$duration" ;

  $String .= " sessid=$sessid" ;
  $String .= ' daySubdir=' . ($perDay ? 'yes' : 'no') ;
  $String .= ' sessidSubdir=' . ($perSessid ? 'yes' : 'no') ;

  my $ftype = $sessData{$name}{'ftype'} ;
  my $measInterval = $sessData{$name}{'measInterval'} ;

  if ($ftype eq 'T00')
  {
    my $posInterval = $sessData{$name}{'posInterval'} ;
    my $option1 = $sessData{$name}{'option1'} ;
    my $option2 = $sessData{$name}{'option2'} ;
    my $rt27    = $sessData{$name}{'RT27'} ;
    my $rawWaas = $sessData{$name}{'rawWaas'} ;

    $String .= " format=T00" ;
    $String .= " measInterval=$measInterval posInterval=$posInterval" ;
    $String .= " smoothCode=" . ($option1==1 ? 'yes' : 'no') ; 
    $String .= " smoothCarrier=" . ($option2==1 ? 'yes' : 'no') ; 
    $String .= " record27=" . ($rt27==1 ? 'yes' : 'no') ; 
    $String .= " rawWaas=" . ($rawWaas==1 ? 'yes' : 'no') ; 
  }
  elsif ($ftype eq 'Binex')
  {
    $String .= " format=Binex" ;
    $String .= " measInterval="
               . ($measInterval || 'off') ;
    $String .= " smoothCarrier=" 
               . ($sessData{$name}{'smoothing'} ? 'yes' : 'no') ; 
    $String .= " 7f02=" . ($sessData{$name}{'7f02'} ? 'yes' : 'no') ; 
    $String .= " 7f03=" . ($sessData{$name}{'7f03'} ? 'yes' : 'no') ; 
    $String .= " 7f03Dopplers=" 
               . ($sessData{$name}{'7f03Doppler'} ? 'yes' : 'no') ; 
    $String .= " 7f03Slips=" 
               . ($sessData{$name}{'7f03Slips'} ? 'yes' : 'no') ; 
    $String .= " 7f03ClockOffsets=" ;
    if ( $sessData{$name}{'7f03RxClkOfsAlways'} )
    {
      $String .= "always" ;
    }
    elsif ( $sessData{$name}{'7f03RxClkOfsRollover'} )
    {
      $String .= "onRollover" ;
    }
    else
    {
      $String .= "never" ;
    }

    $String .= " metaInterval=" 
             . ($sessData{$name}{'metaInterval'} || 'off') ;
    $String .= " metaSiteDesc="
             . ($sessData{$name}{'siteDesc'} ? 'yes' : 'no') ;
    $String .= " metaStnID="
             . ($sessData{$name}{'stationID'} ? 'yes' : 'no') ;
    $String .= " metaRcvrType="
             . ($sessData{$name}{'rcvrType'} ? 'yes' : 'no') ;
    $String .= " metaRinexRcvr="
             . ($sessData{$name}{'rcvrNumber'} ? 'yes' : 'no') ;
    $String .= " metaFirmware="
             . ($sessData{$name}{'firmwareVersion'} ? 'yes' : 'no') ;
    $String .= " metaAntXYZ="
             . ($sessData{$name}{'antXYZ'} ? 'yes' : 'no') ;
    $String .= " metaAntOffset="
             . ($sessData{$name}{'antOffset'} ? 'yes' : 'no') ;
    $String .= " metaRinexAnt="
             . ($sessData{$name}{'antNum'} ? 'yes' : 'no') ;
    $String .= " metTilt="
             . ($sessData{$name}{'metTilt'} ? 'yes' : 'no') ;

    $String .= " sysStatusInterval="
             . ($sessData{$name}{'statusInterval'} || 'off') ;
  }

  return $String ;

} # end of formatSession()



########################################################################
sub ShowSession
########################################################################
{
  #print "Executing ShowSession\n" ;
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  # Read in the data from the sessions configuration file.
  my %sessData ;
  getSessionData( \%sessData ) ;

  my $name = $_[0] ;
  $name =~ s/^name=//i ;

  my $result = formatSession( $name, \%sessData ) ;

  if ($result)
  {
    print "$result\n" ;
  }
  else
  {
    print "ERROR: No data for session $name\n" ;
  }
} # end of ShowSession



########################################################################
sub ShowSessions
########################################################################
{
  #print "Executing ShowSessions\n" ;
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  # Read in the data from the sessions configuration file.
  my %sessData ;
  getSessionData( \%sessData ) ;

  print "<Show Sessions>\n" ;

  for my $name (sort keys %sessData)
  {
    print formatSession( $name, \%sessData ), "\n" ;
  }

  print "<end of Show Sessions>\n" ;

} # end of ShowSessions




########################################################################
sub SetSession
########################################################################
# Used to Set the characteristics of a session and update the
# session configuration file.
# Called in one of two ways:
#
#   set session name=abc enable=(yes|no)
#       Just used to enable or disable an existing session.
#
#   set session name=abc enable=(yes|no) ......
#       Defines a new session or redefines an existing session.
#       This one gets complex because of the large number of
#       perameters, which might need to be defaulted.
#
# As with all of these command handlers, the set of parameters
# are passed in the array @_.  All of the parameters MUST have
# explicit identifiers (id=value).  The range of possible values makes
# it impossible to tell what they are just from their values.  We don't
# want to make any assumptions about the order of the parameters, so
# we just force them all to be explicitly identified.
# The explicit parameters are:
#   name           A session name.  Required!
#   enable         yes or no
#   schedule       four different possibilities:
#                  continuous
#                  manual
#                  daily@hh:mm
#                  once@yyyy/mm/dd/hh:mm
#   duration       value in minutes
#   sessid         single lower case letter
#   daySubdir      yes or not
#   sessidSubdir   yes or no
#
#   format         T00 or Binex
#
# For T00 Sessions:
#   measinterval   numeric in range 0.1,0.2.0.5,1,2,5,10,
#                                   15,30,60,300,600
#   posinterval    numeric in range 0.1,0.2.0.5,1,2,5,10,
#                                   15,30,60,300,600
#   smoothcode     yes or no
#   smoothcarrier  yes or no
#   record27       yes or no
#   rawWaas        yes or no
# For Binex Sessions:
#   measInterval   numeric in range 0,0.1,0.2.0.5,1,2,5,
#                                   10,15,30,60,300,600
#                  or 'off'.
#   smoothCarrier  yes or no
#   7f02           yes or no
#   7f03           yes or no
#   7f03Dopplers   yes or no
#   7f03Slips      yes or no
#   7f03ClockOffsets never or always or onRollover
#
#   metaInterval   numeric in range 0 or 'off', 60, 300, 900, 1800, 3600
#   
#   metaSiteDesc   yes or no
#   metaStnID      yes or no
#   metaRcvrType   yes or no
#   metaRinexRcvr  yes or no
#   metaFirmware   yes or no
#   metaAntXYZ     yes or no
#   metaAntOffset  yes or no
#   metaRinexAnt   yes or no
#   includeMet     yes or no
#
#   sysStatusInterval   numeric in range 0 or 'off', 60, 300, 900, 
#                                        1800, 3600
########################################################################
{

  # Convert the parameter array into an associative hash.
  # This allows us to detect the parameters by name, in a random order,
  # and delete them as they are processed.  Once we are supposedly done,
  # any random leftover values can be detected as invalid parameters.
  my %parms ;
  my $parmcount = 0 ;
  for my $p (@_)
  {
    if ($p !~ /=/)
    {
      print "ERROR: Must fully specify id=value for parameter $p\n" ;
      return ;
    }
    my ($id, $value) = split( '=', $p ) ;
    $parms{lc($id)} = $value ;
    ++$parmcount ;
    #print "$id  =  $value\n" ; 
  }


  # If ONLY fed parameters name= and enable=, then this is a request
  # to enabled/disable an existing session.
  if ( ( $parmcount == 2)
    && exists( $parms{'name'} )
    && exists( $parms{'enable'} )
     )
  {
    my $name = $parms{'name'} ;
    my $enable = lc( $parms{'enable'} ) ;
    if ( $enable !~ /^(yes|no)$/ )
    {
      print "ERROR: enable must be yes or no.\n";
      return ;
    }

    if ( EnaDisSession( $name, ($enable eq 'yes') ) )
    {
      print "OK: Session name=$name enable=$enable\n";
    }
    else
    {
      print "ERROR: No Session $name exists.\n";
    }
    return ;
  }

  # Normal session creation or edit.
  # We need to run through the parameters again, to verify that
  # they are all valid in this context, and then adjust the
  # configuration settings for this session to match.

  # First, read in the data from the sessions configuration file.
  # We'll need this to set the unspecified parameters if we are changing
  # things on an existing session.
  my %sessData ;
  getSessionData( \%sessData ) ;
  
  my $name = $parms{'name'} ;
  delete( $parms{'name'} ) ;

  if (! $name )
  {
    print "ERROR: Must specify session 'name='.\n";
    return ;
  }

  # Record the old session information (if it exists) by copying
  # the hash associated with the $name.  This is used later when
  # we want to detect changes when potentially restarting a manual
  # session.  If the old session doesn't exist, we'll give this a value
  # that always conflicts with a valid session.
  my %oldSessData ;

  if (! exists( $sessData{$name} ) )
  {
    # No session exists with this name.  Create a new one with all
    # defaulted parameters.
    $sessData{$name}{'enable'} = 1 ;
    $sessData{$name}{'sched'} = 'cont' ;
    $sessData{$name}{'time'} = 0 ;
    $sessData{$name}{'duration'} = 60 ;
    $sessData{$name}{'sessid'} = 'a' ;
    $sessData{$name}{'perday'} = 0 ;
    $sessData{$name}{'persessid'} = 0 ;
    $sessData{$name}{'ftype'} =  'T00' ;

    # T00 parameters
    $sessData{$name}{'measInterval'} =  15 ;
    $sessData{$name}{'posInterval'} =  60 ;
    $sessData{$name}{'option1'} = 0 ;
    $sessData{$name}{'option2'} = 0 ;
    $sessData{$name}{'RT27'} = 0 ;

    # Binex stuff
    $sessData{$name}{'measInterval'} =  1 ;
    $sessData{$name}{'smoothing'} = 0 ;
    $sessData{$name}{'7f02'} = 0 ;
    $sessData{$name}{'7f03'} = 1 ;
    $sessData{$name}{'7f03RxClkOfsRollover'} = 0 ;
    $sessData{$name}{'7f03RxClkOfsAlways'} = 0 ;
    $sessData{$name}{'7f03Slips'} = 0 ;
    $sessData{$name}{'7f03Doppler'} = 0 ;

    $sessData{$name}{'metaInterval'} = 0 ;
    $sessData{$name}{'siteDesc'} = 0 ;
    $sessData{$name}{'stationID'} = 0 ;
    $sessData{$name}{'antNum'} = 0 ;
    $sessData{$name}{'rcvrType'} = 0 ;
    $sessData{$name}{'rcvrNumber'} = 0 ;
    $sessData{$name}{'firmwareVersion'} = 0 ;
    $sessData{$name}{'antXYZ'} = 0 ;
    $sessData{$name}{'antOffset'} = 0 ;
    $sessData{$name}{'metTilt'} = 1 ; # for backward compatability.

    $sessData{$name}{'statusInterval'} = 0 ;

    # 'enable' will either be 1 or 0, so 2 will never match.  This
    # will trigger a restart if the new session is manual.
    $oldSessData{'enable'} = 2 ;
  }
  else
  {
    # Otherwise the session we are modifying already exists.
    # Copy all of its settings into %oldSessData.
    %oldSessData = %{$sessData{$name}} ;
  }

  my $pname ;
  my $value ;

  if (exists( $parms{'format'} ))
  {
    # Format is either T00 or Binex.  Others may be added later.
    # Note that if we end up defining new values, and they have
    # different possible option settings, then this parameter must
    # be detected FIRST before processing the others, much as $name
    # is.  So this section may have to be moved.
    $pname = 'format' ;
    $value = lc($parms{'format'}) ;
    delete( $parms{'format'} ) ;

    if ( $value ne 't00' 
     and $value ne 'binex'
       )
    {
      BADPARM:
      print "ERROR: Invalid parameter '$pname=$value'\n" ;
      return ;
    }
    $value = 'T00' if ($value eq 't00') ;
    $value = 'Binex' if ($value eq 'binex') ;
    $sessData{$name}{'ftype'} = $value ;
  }

  if (exists( $parms{'schedule'} ) )
  {
    $pname = 'schedule' ;
    $value = lc( $parms{'schedule'} ) ;
    delete( $parms{'schedule'} ) ;

    if ($value eq 'continuous')
    {
      $sessData{$name}{'sched'} = 'cont' ;
      $sessData{$name}{'time'} = 0 ;
    }
    elsif ($value eq 'manual')
    {
      $sessData{$name}{'sched'} = 'manual' ;
      # Note that we don't set the time on a manual session here.
      # This will only be done if the session is truly modified.
    }
    elsif ($value =~ /^daily@(\d{1,2}):(\d{1,2})$/)
    {
      # Daily sessions should look like: daily@14:32
      goto BADPARM if ( $1 > 23  or $2 > 59 ) ;
      $sessData{$name}{'sched'} = 'daily' ;
      $sessData{$name}{'time'} = ($1*60+$2)*60 ;
    }
    elsif ( $value =~ 
           /^once@(\d{4})\/(\d{1,2})\/(\d{1,2})\/(\d{1,2}):(\d{1,2})$/
          )
    {
      $sessData{$name}{'sched'} = 'once' ;

      # Once only sessions should look like: 
      #
      #    once@2003/10/16/15:24
      #
      # Extract all the fields and do range checking, especially
      # on the calendar data.  The timegm() function below has a fit
      # if you feed it bad data.
      my ($year,$month,$day,$hour,$min) = ($1, $2, $3, $4, $5 ) ;
      goto BADPARM if ( $year < 2000 or $year > 2100 
                     or $month > 12 or $month < 1
                     or $day > 31 or $day < 1  
                     or $hour > 23  
                     or $min > 59 
                     or ( $day == 31 
                      and ( $month == 4 
                         or $month == 6 
                         or $month == 9 
                         or $month == 11 
                          )
                        ) 
                     or ( $month == 2
                      and ($day > 29)
                        )
                     or ( $month == 2 
                      and ($year % 4 != 0)
                      and ($day > 28)
                        )
                      ) ;

      # Convert the sec..year values into a UTC time since 01Jan1970.
      # Then add the offset between that and the start of GPS time
      # at 06Jan1980.  This gives us the absolute GPS time for the
      # start of the session.
      $sessData{$name}{'time'} = timegm( 0, # Seconds
                                         $min,
                                         $hour,
                                         $day,
                                         $month-1,
                                         $year-1900,
                                       ) - 315964800 ;
    }
    else
    {
      goto BADPARM ;
    }
  } # schedule parameter

  if ( exists( $parms{'duration'} ) )
  {
    $pname = 'duration' ;
    $value = lc( $parms{'duration'} ) ;
    delete( $parms{'duration'} ) ;
    # Duration is positive numeric limited to 4 digits.
    # If schedule mode is daily, duration is limited to just less
    # than one day (1440).
    goto BADPARM if ( $value !~ /^\d+$/
                   or $value > 9999
                   or ( $sessData{$name}{'sched'} eq 'daily' 
                    and $value >= 1440 
                      )
                   or $value == 0
                    ) ;
    $sessData{$name}{'duration'} = $value ;
  }


  if ( exists( $parms{'measinterval'} ) )
  {
    $pname = 'measinterval' ;
    $value = lc( $parms{'measinterval'} ) ;
    delete( $parms{'measinterval'} ) ;
    # measintervals are limited to 0.1, 0.2, 0.5, 1,2,5,10,15,30,60,
    # 300, 600 and 'off'.  The numerics present a problem since we want
    # to be tolerant of various formats but comparisions to fractions
    # like 0.1 are a bit dicey in floating point.  So we'll accept any
    # %f style format, '1.0' '0.5' '5.' '5' with leading and trailing
    # zeroes, etc.  Note that the comparisons to 0.1, 0.2 are a bit
    # suspect, since those values are not expressable exactly in binary.
    # But Perl is interpreting our strings and what the user types in
    # through the same conversion routines, so the comparions work.
    $value = 0 if ($value eq 'off') ;
    goto BADPARM if ( $value !~ /^\d+$/
                  and $value !~/^\d*\.\d+$/
                  and $value !~/^\d+\.\d*$/
                    ) ;
    $value += 0 ; # Convert to pure numeric                      
    if ($value == 0 or $value >= 1.0)
    {
      goto BADPARM if ( $value != 0
                    and $value != 1
                    and $value != 2
                    and $value != 5
                    and $value != 10
                    and $value != 15
                    and $value != 30
                    and $value != 60
                    and $value != 300
                    and $value != 600
                      ) ;
      $value = sprintf( '%.0f', $value ) ;
    }  
    else
    {
      goto BADPARM if ( $value != 0.1
                    and $value != 0.2
                    and $value != 0.5
                      ) ;
      $value = sprintf( '%.1f', $value ) ;
    }
    $sessData{$name}{'measInterval'} = $value ;
  }

  if ( exists( $parms{'sessid'} ) )
  {
    $pname = 'sessid' ;
    $value = lc( $parms{'sessid'} ) ;
    delete( $parms{'sessid'} ) ;
    # sessid's are limited to single, lower-case alphabetic 
    # characters.
    goto BADPARM if ($value !~ /^[a-z]$/) ;
    $sessData{$name}{'sessid'} = $value ;
  }

  # Below we'll check all the 'yes|no' parameters.
  # The hash %yesNoOptions defines all of the options that can be "yes"
  # or "no" and which are stored as boolean values in the %allPortData
  # hash, as indexed by the name in the second half of each element.
  # For example, the parameter "dayssubdir", if present, must be either
  # 'yes' or 'no', and will be saved in $sessData{$name}{'perday'}
  my %yesNoOptions ;

  if ( $sessData{$name}{'ftype'} eq 'T00' )
  {
    # Special processing for T00 files
    %yesNoOptions =  ( 
        'enable'           => 'enable',
        'smoothcode'       => 'option1',
        'smoothcarrier'    => 'option2',
        'daysubdir'        => 'perday',
        'sessidsubdir'     => 'persessid',
        'record27'         => 'RT27',
        'rawwaas'          => 'rawWaas'
        ) ;

    if ( exists( $parms{'posinterval'} ) )
    {
      $pname = 'posinterval' ;
      $value = lc( $parms{'posinterval'} ) ;
      delete( $parms{'posinterval'} ) ;
      # posintervals are limited to 0.1, 0.2, 0.5, 1,2,5,10,15,30,60,
      # 300, 600 and 'off'.  The numerics present a problem since we
      # want to be tolerant of various formats but comparisions to
      # fractions like 0.1 are a bit dicey in floating point.  So we'll
      # accept any %f style format, '1.0' '0.5' '5.' '5' with leading
      # and trailing zeroes, etc.  Note that the comparisons to 0.1, 0.2
      # are a bit suspect, since those values are not expressable
      # exactly in binary.  But Perl is interpreting our strings and
      # what the user types in through the same conversion routines, so
      # the comparions work.
      $value = 0 if ($value eq 'off') ;
      goto BADPARM if ( $value !~ /^\d+$/
                    and $value !~/^\d*\.\d+$/
                    and $value !~/^\d+\.\d*$/
                      ) ;
      $value += 0 ; # Convert to pure numeric                      
      if ($value == 0 or $value >= 1.0)
      {
        goto BADPARM if ( $value != 0
                      and $value != 1
                      and $value != 2
                      and $value != 5
                      and $value != 10
                      and $value != 15
                      and $value != 30
                      and $value != 60
                      and $value != 300
                      and $value != 600
                        ) ;
        $value = sprintf( '%.0f', $value ) ;
      }  
      else
      {
        goto BADPARM if ( $value != 0.1
                      and $value != 0.2
                      and $value != 0.5
                        ) ;
        $value = sprintf( '%.1f', $value ) ;
      }
      $sessData{$name}{'posInterval'} = $value ;
    }

  }
  else
  {
    # Special processing for Binex files

    %yesNoOptions =  ( 
        'enable'           => 'enable',
        'smoothcarrier'    => 'smoothing',
        'daysubdir'        => 'perday',
        'sessidsubdir'     => 'persessid',
        '7f02'             => '7f02',
        '7f03'             => '7f03',
        'smoothcarrier'    => 'smoothing',
        '7f03dopplers'     => '7f03Doppler',
        '7f03slips'        => '7f03Slips',
        'metasitedesc'     => 'siteDesc',
        'metastnid'        => 'stationID',
        'metarcvrtype'     => 'rcvrType',
        'metarinexrcvr'    => 'rcvrNumber',
        'metafirmware'     => 'firmwareVersion',
        'metaantxyz'       => 'antXYZ',
        'metaantoffset'    => 'antOffset',
        'metarinexant'     => 'antNum',
        'mettilt'          => 'metTilt',
        ) ;

    # Also have to handle the following non-yes/no Binex options 

    if ( exists( $parms{'7f03clockoffsets'} ) )
    {
      $pname = '7f03clockoffsets' ;
      $value = lc( $parms{'7f03clockoffsets'} ) ;
      delete( $parms{'7f03clockoffsets'} ) ;

      if ($value eq 'always')
      {
        $sessData{$name}{'7f03RxClkOfsAlways'} = 1 ;
        $sessData{$name}{'7f03RxClkOfsRollover'} = 0 ;
      }
      elsif ($value eq 'onrollover')
      {
        $sessData{$name}{'7f03RxClkOfsAlways'} = 0 ;
        $sessData{$name}{'7f03RxClkOfsRollover'} = 1 ;
      }
      elsif ($value eq 'never')
      {
        $sessData{$name}{'7f03RxClkOfsAlways'} = 0 ;
        $sessData{$name}{'7f03RxClkOfsRollover'} = 0 ;
      }
      else
      {
        print "ERROR: Bad 7f03ClockOffsets=$value.\n" ;
        return ;
      }
    }

    # Determine the value for metaInterval, if specified.
    if ( exists( $parms{'metainterval'} ) )
    {
      $pname = 'metainterval' ;
      $value = lc( $parms{'metainterval'} ) ;
      delete( $parms{'metainterval'} ) ;

      $value = '0' if ($value eq 'off') ;
      if ( $value != 0
       and $value != 60
       and $value != 300
       and $value != 900
       and $value != 1800
       and $value != 3600
         )
      {
        print "ERROR: Bad metaInterval=$value.\n" ;
        return ;
      }
      $sessData{$name}{'metaInterval'} = $value ;
    }

    # Determine the value for sysStatusInterval, if specified.
    if ( exists( $parms{'sysstatusinterval'} ) )
    {
      $pname = 'sysstatusinterval' ;
      $value = lc( $parms{'sysstatusinterval'} ) ;
      delete( $parms{'sysstatusinterval'} ) ;

      $value = '0' if ($value eq 'off') ;
      if ( $value != 0
       and $value != 60
       and $value != 300
       and $value != 900
       and $value != 1800
       and $value != 3600
         )
      {
        print "ERROR: Bad sysstatusInterval=$value.\n" ;
        return ;
      }
      $sessData{$name}{'statusInterval'} = $value ;
    }
  }

  # Run through the previously defined list of yesNoOptions and
  # determine which were supplied.  Check for yes or no and
  # set the proper element in the sessData
  #
  for my $pname ( keys %yesNoOptions )
  {
    if (exists $parms{$pname})
    {
      $value = $parms{$pname} ;
      delete $parms{$pname} ;

      if ($value ne 'yes' and $value ne 'no')
      {
        print "ERROR: Bad yes/no value in $pname=$value.\n" ;
        return ;
      }


      # Set the matching option to 1 or 0  based on yes or no.
      $sessData{$name}{ $yesNoOptions{$pname} } = ($value eq 'yes') ;

    }
  }

  if ( (scalar(keys %parms)) != 0 )
  {
    print "ERROR: Excess parameters:",
          join( ',', keys(%parms) ),
          ".\n" ;
    return ;
  }

  # Don't allow carrier phase smoothing for high rate measurements.
  if ( ( $sessData{$name}{'ftype'} eq 'T00'
      && $sessData{$name}{'measInterval'} < 1.0
      && $sessData{$name}{'option2'}  # T00 smoothing
       )
    or ( $sessData{$name}{'ftype'} eq 'Binex'
      && $sessData{$name}{'measInterval'} < 1.0
      && $sessData{$name}{'smoothing'}
       )
     )
  {
    print "ERROR: Can't smooth carrier on high rate measurements.\n" ;
    return ;
  }


  # Check if a manual session restart is needed.
  if (  $sessData{$name}{'sched'} eq 'manual'
    and $sessData{$name}{'enable'}
     )
  {
    for my $k (keys %oldSessData)
    {
      if ($oldSessData{$k} ne $sessData{$name}{$k})
      {
        # This is an enabled manual start session and the session is 
        # being modified in some way.  This requires us to restart
        # the session, and hence we have to reset the start time to
        # now.  A little margin is added so that the session can start
        # cleanly and get its full duration.

        #print "Restarting manual session\n" ;

        # Get the time of day in GPS seconds.  Includes a 13 second
        # offset for the UTC to GPS leapseconds.  It would be nice to
        # have the 13 second value be obtained from a system utility,
        # but that's not available yet.  But even if the value changes
        # by a few seconds, it won't matter.  All that we want is for
        # the time to start the session to be in the future a little
        # bit - about 10 seconds.  Won't matter if that's 9 or 8 or 7.
        my $gpsTime = time() - 315964800 + 13 ;

        $sessData{$name}{'time'} = $gpsTime + 10 ;
        last ;
      }
    }
  }

  # Update the session configuration file with the new data, and
  # trigger the log scheduler to act on the change, if any.
  updateSessionData( \%sessData ) ;

  print "OK: " ;
  ShowSession( $name ) ;
    
} # end of SetSession()



########################################################################
sub DeleteSession
########################################################################
# Remove an existing Session definition.
# $_[0] will be the session name, or 'name=sessname'.
# or "all=yes"
########################################################################
{
  #print "Enable Session\n" ;

  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $sessname = $_[0] ;

  # The parameter must have NO '=' or be of the form 'name='
  if ( $sessname =~ /=/
    && $sessname !~ /^name=/i
    && $sessname !~ /^all=yes/i
     )
  {
    print "ERROR: Invalid parameter $sessname.\n";
    return ;
  }

  # Read in the data from the sessions configuration file.
  my %sessData ;
  getSessionData( \%sessData ) ;

  if ( $sessname !~ /^all=yes/i )
  {
    # Drop name= if it exists
    $sessname =~ s/^name=//i ;

    if (! exists( $sessData{$sessname} ) ) 
    {
      print "ERROR: No session '$sessname' to delete.\n";
      return ; 
    }

    # Remove this element from the hash.
    delete( $sessData{$sessname} ) ;
    print "OK: Session '$sessname' is deleted.\n" ;
  }
  else
  {
    for my $s (keys %sessData)
    {
      delete( $sessData{$s} ) ;
    }
    print "OK: All sessions are deleted.\n" ;
  }

  # Save the new data structure back in the config file.
  updateSessionData( \%sessData ) ;

} # end of DeleteSession()



########################################################################
sub formatPort
########################################################################
#
# Format and emit a data string, to stdout, describing a particular Port
# setup.
#
# $_[0] is the name of the port.
# $_[1] is a reference to the hash containing the complete %portData 
#       structure.
#
# Returns true if the port definition was found and false if it doesn't
# exist.
#
# Need some heavy documentation here to describe the highly-variable
# format for this function.
#
# IOport port=SerialPort1 ...
#
# IOport port=TcpPort9999 ...
#
# IOport port=UdpPort9999 ...
#
# Here are some samples:
#
# IOport port=SerialPort1 service=ppp baud=115200 flow=None \
#        localIP=192.168.0.1 remoteIP=192.168.0.10 connection=Modem \
#        reset=RST init1=IN1 init2=IN2 init3=IN3
#        enable=yes
#
# IOport port=SerialPort2 service=ppp baud=38400 flow=Hardware \
#        localIP=192.168.0.1 remoteIP=192.168.0.20 connection=Direct
#        enable=yes
#
# IOport port=SerialPort3 service=Met cmd1=1:c1 cmd2=5:c2 cmd3=once:c3 \
#        cmd4=off cmd5=off cmd6=off baud=115200 parity=Odd flow=XonXoff
#        enable=yes
#
# IOport port=SerialPort4 service=ppp baud=38400 flow=None \
#        localIP=192.168.0.1 remoteIP=192.168.0.40 connection=Direct
#        enable=yes
#
# IOport port=TcpPort5017 service=RT17 interval=15 measurements=yes 
#        positions=yes concise=yes iodeSlips=yes gpsdata=no rawgps=no \
#        rawWaas=yes smoothcode=no smoothcarrier=yes
#        enable=yes clientAuth=no
#
# IOport port=TcpPort5018 service=Binex measInterval=1 smoothCarrier=no
#        7f02=no 7f03=yes 7f03Dopplers=no 7f03Slips=no 
#        7f03ClockOffsets=never metTilt=yes
#        metaInterval=off metaSiteDesc=no metaStnID=no metaRcvrType=no
#        metaRinexRcvr=no metaFirmware=no metaAntXYZ=no metaAntOffset=no
#        metaRinexAnt=no sysStatusInterval=off 
#        enable=yes clientAuth=no
#
#
# IOport port=TcpPort5020 service=Ephemerides
#        enable=yes clientAuth=no
#
# IOport port=TcpPort5021 service=Almanacs
#        enable=yes clientAuth=no
#
# IOport port=TcpPort5022 service=SvStatus
#        enable=yes clientAuth=no
#
# IOport port=TcpPort5025 service=Trimcom
#        enable=yes clientAuth=no
#
# IOport port=TcpPort5026 service=Rtcm version=2.3 style=DGPS
#        enable=yes clientAuth=no
#
# IOport port=UdpPort5024 service=Rtcm version=2.3 style=Custom:59.3 \
#        udpTimeout=10 enable=yes clientAuth=no
########################################################################
{
  my $name = $_[0] ;
  my %allPortData = %{$_[1]} ;

  $name =~ s/tcpport/TcpPort/i ;
  $name =~ s/udpport/UdpPort/i ;
  $name =~ s/serialport/SerialPort/i ;

  return 0 if ( !exists $allPortData{$name} ) ;

  # This hash contains the data for this particular port.
  my %portData = %{$allPortData{$name}} ;

  print "IOport port=$name" ;

  my $protocol = $portData{'protocol'} ;
  my $service = $portData{'service'} ;

  print " service=", $service ;
  $service=lc($service) ;

  if ($service eq 'rtcm')
  {
    print " version=", $portData{'version'} ;
    if ($portData{'version'} ne '3.0')
    {
      print " style=", $portData{'rtcmStyle'} ;
    }
  }
  elsif ($service eq 'cmr')
  {
    print " style=", $portData{'cmrStyle'} ;
  }
  elsif ($service eq 'binex')
  {
    print " measInterval=", 
          ($portData{'interval'} != 0.0)
          ? $portData{'interval'}
          : 'off' ;
    print " smoothCarrier=", $portData{'smoothing'} ? 'yes' : 'no' ;
    print " 7f02=", $portData{'7f02'} ? 'yes' : 'no' ;
    print " 7f03=", $portData{'7f03'} ? 'yes' : 'no' ;
    print " 7f03Dopplers=", $portData{'7f03Doppler'} ? 'yes' : 'no' ;
    print " 7f03Slips=", $portData{'7f03Slips'} ? 'yes' : 'no' ;
    print " 7f03ClockOffsets=" ;
    if    ( $portData{'7f03RxClkOfsAlways'} )   {print "always" ;}
    elsif ( $portData{'7f03RxClkOfsRollover'} ) {print "onRollover" ;}
    else                                        {print( "never" ) ;}
    print " metTilt=", $portData{'metTilt'} ? 'yes' : 'no' ;

    print " metaInterval=", $portData{'metaInterval'} || 'off' ;
    print " metaSiteDesc=", $portData{'siteDesc'} ? 'yes' : 'no' ;
    print " metaStnID=", $portData{'stationID'} ? 'yes' : 'no' ;
    print " metaRcvrType=", $portData{'rcvrType'} ? 'yes' : 'no' ;
    print " metaRinexRcvr=", $portData{'rcvrNumber'} ? 'yes' : 'no' ;
    print " metaFirmware=", $portData{'firmwareVersion'} ? 'yes' : 'no';
    print " metaAntXYZ=", $portData{'antXYZ'} ? 'yes' : 'no' ;
    print " metaAntOffset=", $portData{'antOffset'} ? 'yes' : 'no' ;
    print " metaRinexAnt=", $portData{'antNum'} ? 'yes' : 'no' ;

    print " sysStatusInterval=", $portData{'statusInterval'} || 'off' ;
  }
  elsif ($service eq 'rt17')
  {
    print " interval=", $portData{'interval'} ;
    print " measurements=", $portData{'measurements'} ? 'yes' : 'no' ;
    print " positions=", $portData{'positions'} ? 'yes' : 'no' ;
    print " concise=", $portData{'concise'} ? 'yes' : 'no' ;
    print " iodeslips=", $portData{'iodeslips'} ? 'yes' : 'no' ;
    print " GPSdata=", $portData{'gpsdata'} ? 'yes' : 'no' ;
    print " rawGPS=", $portData{'rawgps'} ? 'yes' : 'no' ;
    print " rawWAAS=", $portData{'rawwaas'} ? 'yes' : 'no' ;
    print " smoothCode=", $portData{'smoothcode'} ? 'yes' : 'no' ;
    print " smoothCarrier=", $portData{'smoothcarrier'} ? 'yes' : 'no' ;
    print " record27=", $portData{'useRt27'} ? 'yes' : 'no' ;
  }
  elsif ($service eq 'ppp')
  {
    print " baud=",$portData{'pppbaud'} ;
    print " flow=",$portData{'pppflow'} ;

    print " localIP=", $portData{'localip'} ;
    print " remoteIP=", $portData{'remoteip'} ;

    my $connection = $portData{'connection'} ;
    print " connection=", $connection ;

    if ($connection =~ /modem/i)
    {
      print " reset=", $portData{'reset'} ;
      print " init1=", $portData{'init1'} ;
      print " init2=", $portData{'init2'} ;
      print " init3=", $portData{'init3'} ;
    }
  }
  elsif ($service eq 'met')
  {
    for (my $i = 1 ; $i <= 6 ; ++$i)
    {
      print " cmd$i=", $allPortData{'MetControls'}{$i} ;
    }
  }

  if ($protocol eq 'serial' and $service ne 'ppp')
  {
    print " baud=", $portData{'baud'} ;
    print " parity=", $portData{'parity'} ;
    print " flow=", $portData{'flow'} ;
  }

  if ($protocol eq 'udp')
  {
    print " udpTimeout=",$portData{'udptimeout'} ;
  }

  print " enable=",
          $portData{'enable'}
          ? 'yes'
          : 'no' ;

  if ( $protocol eq 'udp' 
    or $protocol eq 'tcp' 
     )
  {
    print " clientAuth=",
          $portData{'authenticate'} 
          ? "yes"
          : "no" ;
  }

  print "\n" ;
  return 1 ;

} # end of formatPort()



########################################################################
sub ShowIOports
########################################################################
# SHows a listing of ALL Serial/TCP/UDP port specifications.
# Uses same format as 'show ioport'.  Combines the listings for
# all ports into a single report, bracketed with:
#
#   <Show IOports>
#   IOport port=SerialPort1 ....
#   IOport port=SerialPort2 ....
#   ....
#   <end of Show IOports>
#
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;
  print "<Show IOports>\n" ;

  my %allPortData ;
  getPortSetup( \%allPortData ) ;

  for my $p (sort keys %allPortData)
  {
    # Don't list certain ports that are for internal use only.
    # The need for this will probably go away as these will no longer
    # be real TCP ports and won't be listed.
    next if ( $p eq 'MetControls' ) ;
    next if ( $p eq 'AuthKey' ) ;
    next if ( $allPortData{$p}{'service'} =~ /^(internal.|metdata)$/ ) ;

    formatPort( $p, \%allPortData ) ;
  }

  print "<end of Show IOports>\n" ;

} # end of ShowIOports



########################################################################
sub ShowIOport
########################################################################
# Shows the configuration for a particular Serial, TCP, or UDP port.
#
#   show IOport [port=]TcpPort1234
#
# Emits a strings showing all pertinent parameters for the specified
# port.  Too numerous to include a full description here, and this is
# highly subject to change.  Unclear how best to document this.
# See the function formatPort() for details.
#
# Emits an error message if the port doesn't exist.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $name = $_[0] ;
  $name =~ s/^port=//i ;
  if ($name =~ /=/)
  {
    print "ERROR: Invalid parameter $_[0].\n" ;
    return ;
  }

  my %allPortData ;
  getPortSetup( \%allPortData ) ;


  if ( ! formatPort( $name, \%allPortData ) )
  {
    print "ERROR: IOport '$name' is not defined.\n" ;
    return ;
  }

} # end of ShowIOport



########################################################################
sub DeleteIOport
########################################################################
# Deletes the configuration setup for a particular TCP or UDP port.
#   Delete IOport port=TcpPort1234
#   Delete IOport port=UdpPort1234
#   Delete IOport all=yes
#
# Everything is case insensitive.  'port=' is not required.
# Can't delete a serial port setup.
# Error if the requested port doesn't exist.
# On success, prints
#
#   OK: IOport xxxx is deleted.
# or
#   OK: All TCP and UDP IOports have been deleted.
#
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $name = lc($_[0]) ;
  $name =~ s/tcpport/TcpPort/i ;
  $name =~ s/udpport/UdpPort/i ;

  if ( $name !~ /^port=(Tcp|Udp)Port\d+/
   and $name ne 'all=yes'
     )
  {
    print "ERROR: Invalid parameter $_[0].\n" ;
    return ;
  }

  my %allPortData ;
  getPortSetup( \%allPortData ) ;

  if ($name ne 'all=yes')
  {
    $name =~ s/^port=//i ;

    if ( ! exists( $allPortData{$name} ) )
    {
      print "ERROR: IOport '$name' is not defined.\n" ;
      return ;
    }

    delete( $allPortData{$name} ) ;

    print "OK: IOport $name is deleted.\n" ;
  }
  else
  {
    # Delete IOport All
    # Remove all TCP and UDP ports that the user has set up.

    for my $p (keys %allPortData)
    {
      # Delete this port unless it's an internally generated one.
      next if ($p !~ /^(Tcp|Udp)Port/) ;
      next if ( $p eq 'TcpPort4001' # Mfg Test
             or $p eq 'TcpPort8099' # Internal metdata stream
             or $p eq 'TcpPort8086' # IoPort status reports
              ) ;
      #print "Deleting $p\n" ;
      delete( $allPortData{$p} ) ;
    }
    print "OK: All TCP and UDP IOports have been deleted.\n" ;
  }

  # Save the updated configuration data.
  saveGpssdConfig( \%allPortData ) ;

} # end of DeleteIOport



########################################################################
sub SetIOport
########################################################################
# 
# Accepts and applies a description of new or modified IOport controls.
#   Set IOport port=SerialPort1 service=XXXX [parameters]
#   Set IOport port=TcpPort9999 service=XXXX [parameters]
#   Set IOport port=UdpPort9999 service=XXXX [parameters]
#   Set IOport port=RepPort9999 service=XXXX [parameters]
#
# The acceptable parameters are quite lengthy and variable depending on
# the port type, service requested, etc.  But the port and service MUST
# be given.
# 
########################################################################
{

  # Convert the parameter array into an associative hash, %parms.
  # As we process parameters from this list, we'll delete them from the
  # hash.  When we are done, they should all be gone.  Otherwise we
  # have been given too many parameters.
  my %parms ;
  for my $p (@_)
  {
    if ($p !~ /=/)
    {
      print "ERROR: Must fully specify id=value for parameter $p\n" ;
      return ;
    }
    my ($id, $value) = split( '=', $p ) ;

    # The parameter names are all case insensitive.  Below we assume
    # they were entered in lower case.
    $id = lc($id) ;

    # Most of the parameters are case-insensitive, except for the
    # modem init strings and the Met command strings.  So we want to 
    # lower-case them (as used in the code tests below) unless they
    # fall into those categories.
    $value = lc( $value ) if ( $id ne 'reset' 
                           and $id !~ /^init\d$/ 
                           and $id !~ /^cmd\d$/ 
                             ) ;

    $parms{$id} = $value ;
  }

  # Port should be "TcpPortXXXX" or "UdpPortXXXX" or "SerialPortX"
  # or "RepPortXXXX"
  my $port = $parms{'port'} ;
  delete $parms{'port'} ;

  if ( ! $port )
  {
    print "ERROR: Must specify a 'port=' parameter.\n" ;
    return ;
  }

  $port =~ s/tcpport/TcpPort/i ;
  $port =~ s/repport/RepPort/i ;
  $port =~ s/udpport/UdpPort/i ;
  $port =~ s/serialport/SerialPort/i ;

  if ( $port !~ /^(Tcp|Udp|Rep|Serial)Port(\d+)/ )
  {
    print "ERROR: Bad Port Name '$port'.\n" ;
    return ;
  }

  my $protocol = lc( $1 ) ; # Either 'udp' 'tcp' or 'serial' or 'rep'

  my $portnum = $2 ;

  # Check port number ranges...
  if ( ($protocol eq 'serial') 
       ? ( $portnum < 1 
        or $portnum > 4
         )
       : ( $portnum > 65535 
        or $portnum < 1024 
        or $portnum == 8099 
        or $portnum == 8086
         )
     )
  {
    print "ERROR: Bad Port Number '$portnum'.\n" ;
    return ;
  }

  # Read the existing configuration data from multiple sources.
  # Put this into the hash %allPortData, indexed by port name.
  my %allPortData ;
  getPortSetup( \%allPortData ) ;


  # Save the protocol
  $allPortData{$port}{'protocol'} = $protocol ;


  if ($protocol eq 'rep')
  {
    # We can only have a single active REP protocol.  Since we
    # are activating this one, scan the others looking for REP
    # protocols.  Delete any other REP ports.
    for my $p ( keys %allPortData )
    {
      next if ($p eq $port) ;
      next if ($p eq 'MetControls') ;
      next if ($p eq 'AuthKey') ;
      if ($allPortData{$p}{'protocol'} eq 'rep')
      {
        delete( $allPortData{$p} ) ;
      }
    }
  }

  # Required parameter 'service=' 
  my $service = $parms{'service'} ;
  delete $parms{'service'} ;
  if (!$service)
  {
    print "ERROR: Must specify 'service='.\n" ;
    return ;
  }
  $allPortData{$port}{'service'} = $service ;


  # Check for UDP timeout.  Default it to 10 seconds if not specified.
  if ($protocol eq 'udp')
  {
    my $udpTimeout = $parms{'udptimeout'} || '10' ;
    delete $parms{'udptimeout'} ; # consume it

    if ( $udpTimeout !~ /^(\d+)$/ )
    {
      print "ERROR: Bad UdpTimeout.\n" ;
      return ;
    }
    $allPortData{$port}{'udptimeout'} = $udpTimeout ;
  }


  # Check for client authentication on UDP or TCP or REP streams.  
  # Can be either 'clientAuth=yes' or 'no'.
  # Default it to no if not specified.
  if ( $protocol eq 'udp'
    or $protocol eq 'tcp'
    or $protocol eq 'rep'
     )
  {
    my $auth = $parms{'clientauth'} || 'no' ;
    delete $parms{'clientauth'} ; # consume it

    if ( $auth !~ /^(yes|no)$/ )
    {
      print "ERROR: Bad ClientAuth value.\n" ;
      return ;
    }
    $allPortData{$port}{'authenticate'} = ($auth eq 'yes') ;
  }

  # For all ports, there is an "enable" control.  The parameter to set
  # it is optional, defaulting to "enable=yes".
  my $enable = $parms{'enable'} || 'yes' ;
  delete $parms{'enable'} ; # consume it

  if ( $enable !~ /^(yes|no)$/ )
  {
    print "ERROR: Bad Enable value.\n" ;
    return ;
  }
  $allPortData{$port}{'enable'} = ($enable eq 'yes') ;

  # REP protocol only supports binex service.
  if ( $protocol eq 'rep' and $service ne 'binex' )
  {
    print "ERROR: REP ports only support Binex service.\n" ;
    return ;
  }

  ######################################################################
  # For each of the possible service types, handle the parameters that
  # are appropriate to that type.  Consume each parameter (with delete)
  # as it is used.  
  ######################################################################
  if ($service =~ /^(ephemerides|almanacs|svstatus|trimcom)$/)
  {
    # Trivial services.  No other parameters required.
    # Port must use TCP protocol.
    if ($protocol ne 'tcp')
    {
      print "ERROR: Service $service only allowed on TCP ports.\n" ;
      return ;
    }

    if ($service eq 'trimcom')
    {
      # We can only have a single trimcom TCPport active.  Since we
      # are activating this one, scan the others looking for duplicates.
      # Delete any other trimcom ports.
      for my $p ( keys %allPortData )
      {
        next if ($p eq $port) ;
        next if ($p eq 'MetControls') ;
        if ($allPortData{$p}{'service'} =~ /trimcom/i)
        {
          delete( $allPortData{$p} ) ;
        }
      }
    }
  }
  ######################################################################
  elsif ($service eq 'rtcm')
  {
    # RTCM ports.  Allowable parameters are:
    # version=(2.1|2.2|2.3|3.0)
    # style=(rtk|dgps|dgps_rtk|custom...)

    my $version = $parms{'version'} ;
    delete $parms{'version'} ;
    if ( !$version
      or $version !~ /^(2\.1|2\.2|2\.3|3\.0)$/
       )
    {
      print "ERROR: Bad or missing RTCM version.\n" ;
      return ;
    }
    $allPortData{$port}{'version'} = $version ;

    if ( $version ne '3.0' )
    {
      my $style = $parms{'style'} ;
      delete $parms{'style'} ;
      if ( !$style
        or $style !~ /^(dgps|rtk|dgps_rtk|custom:.*)$/
         )
      {
        print "ERROR: Bad or missing RTCM style.\n" ;
        return ;
      }

      if ($style =~ /^Custom:(.*)$/i)
      {
        # Custom: style has comma separate list of options after the
        # colon.  This string goes directly into the structure.  Put
        # it there and then verify the individual entries on the list.
        $allPortData{$port}{'rtcmStyle'} = 'Custom:'.$1 ;
        for my $type ( split( ',',$1 ) )
        {
          if ($type !~ /^(1|2|3|9|14|18\+19|22|23\+24|59\.3|59\.13)$/)
          {
            print "ERROR: Bad RTCM style Custom option '$type'.\n" ;
            return ;
          }
        }
      }
      else
      {
        # Non-custom types, DGPS,RTK,DGPS_RTK.  Put them into the 
        # structure in upper case.
        $allPortData{$port}{'rtcmStyle'} = uc($style) ;
      }
    }
    else
    {
      # Variable Style parameter not allowed for v3.0.
      # Force it to DGPS.
      $allPortData{$port}{'rtcmStyle'} = 'DGPS' ;
    }

  } # end of rtcm section
  ######################################################################
  elsif ($service eq 'cmr')
  {
    # CMR ports.  
    # style=(Standard|CmrPlus)

    my $style = $parms{'style'} ;
    delete $parms{'style'} ;
    if ( !$style
      or $style !~ /^(standard|cmrplus)$/
       )
    {
      print "ERROR: Bad or missing CMR style.\n" ;
      return ;
    }

    $allPortData{$port}{'cmrStyle'} = $style ;

  } # end of cmr section
  ######################################################################
  elsif ($service eq 'binex')
  {
    ####################################################################
    # Binex port: 
    #
    # Allowable parameters are:
    #
    # measInterval = number 
    #                Measurement rate in seconds. 
    #                Required value.
    #                Allowable values are:
    #                0, off, 0.1 0.2 0.5 1 2 5 10 15 30 60 300 600
    #
    # carrierSmoothing    = (yes|no)   Defaults to no.
    #
    # 7f02         = (yes|no)   Defaults to no.
    #                Include measurements in 7f02 format.
    #
    # 7f03         = (yes|no)   Defaults to no.
    #                Include measurements in 7f03 format.
    #
    # 7f03Dopplers = (yes|no)   Defaults to no.
    #                Include Doppler in 7f03.
    # 7f03Slips    = (yes|no)   Defaults to no.
    #                Include Cycle Slip Count in 7f03.
    # 7f03ClockOffsets = (never|always|onRollover)   Defaults to never
    #                When to include clock offsets in 7f03 records.
    #
    # metTilt      = (yes|no) Defaults to yes.
    #                Include Met/Tilt data in the Binex stream.
    # metaInterval = number
    #                Meta data record interval, in seconds. 
    #                Allowable values are: off 0 60 300 900 1800 3600
    #                Default value is 'off' == 0.
    # metaSiteDesc = (yes|no) Defaults to no.
    #                Include Site Description in meta data.
    # metaStnID    = (yes|no) Defaults to no.
    #                Include Binex Station ID in meta data.
    # metaRcvrType = (yes|no) Defaults to no.
    #                Include Receiver Type in meta data.
    # metaRinexRcvr= (yes|no) Defaults to no.
    #                Include Rinex Receiver Type in meta data.
    # metaFirmware = (yes|no) Defaults to no.
    #                Include Receiver Firmware Version in meta data.
    # metaAntXYZ   = (yes|no) Defaults to no.
    #                Include Antenna XYZ coordinate in meta data.
    # metaAntOffset= (yes|no) Defaults to no.
    #                Include Antenna Offset in meta data.
    # metaRinexAnt = (yes|no) Defaults to no.
    #                Include Rinex Antenna Number in meta data.
    #
    # sysStatusInterval = number
    #                System Status Record Interval, in seconds. 
    #                Allowable values are: off 0 60 300 900 1800 3600
    #                Default value is 'off' == 0.
    ####################################################################

    # Determine the required value for measInterval.
    # This goes into the port data as hash index 'interval'.
    my $interval = $parms{'measinterval'} ;
    delete $parms{'measinterval'} ;
    $interval = '0' if ( defined( $interval ) and $interval eq 'off' ) ;
    if ( (! defined( $interval ))
      or $interval !~ /^(\d+|\d+\.\d*|\d*\.\d+)$/
       )
    {
      BadMeasInterval:
      print "ERROR: Bad or missing measInterval.\n" ;
      return ;
    }
    $interval += 0 ; # Convert to pure numeric                      
    if ($interval == 0 or $interval >= 1.0)
    {
      goto BadMeasInterval if ( $interval != 0
                        and $interval != 1
                        and $interval != 2
                        and $interval != 5
                        and $interval != 10
                        and $interval != 15
                        and $interval != 30
                        and $interval != 60
                        and $interval != 300
                        and $interval != 600
                          ) ;
      $interval = sprintf( '%.0f', $interval ) ;
    }  
    else
    {
      goto BadMeasInterval if ( $interval != 0.1
                        and $interval != 0.2
                        and $interval != 0.5
                          ) ;
      $interval = sprintf( '%.1f', $interval ) ;
    }
    $allPortData{$port}{'interval'} = $interval ;

    # Determine the value for metaInterval, if specified.
    # Default to off==0.
    $interval = 0 ;
    if (exists $parms{'metainterval'})
    {
      $interval = $parms{'metainterval'} ;
      delete $parms{'metainterval'} ;
    }
    $interval = '0' if ($interval eq 'off') ;
    if ( $interval != 0
     and $interval != 60
     and $interval != 300
     and $interval != 900
     and $interval != 1800
     and $interval != 3600
       )
    {
      print "ERROR: Bad metaInterval=$interval.\n" ;
      return ;
    }
    $allPortData{$port}{'metaInterval'} = $interval ;


    # Determine the value for sysStatusInterval, if specified.
    # Default to off==0.
    $interval = 0 ;
    if (exists $parms{'sysstatusinterval'})
    {
      $interval = $parms{'sysstatusinterval'} ;
      delete $parms{'sysstatusinterval'} ;
    }
    $interval = '0' if ($interval eq 'off') ;
    if ( $interval != 0
     and $interval != 60
     and $interval != 300
     and $interval != 900
     and $interval != 1800
     and $interval != 3600
       )
    {
      print "ERROR: Bad sysStatusInterval=$interval.\n" ;
      return ;
    }
    $allPortData{$port}{'statusInterval'} = $interval ;


    # Determine the value for the 7f03ClockOffsets control.
    # Default to 'never'.
    my $value = 'never' ;
    if (exists $parms{'7f03clockoffsets'})
    {
      $value = $parms{'7f03clockoffsets'} ;
      delete $parms{'7f03clockoffsets'} ;
    }

    if ($value eq 'always')
    {
      $allPortData{$port}{'7f03RxClkOfsAlways'} = 1 ;
      $allPortData{$port}{'7f03RxClkOfsRollover'} = 0 ;
    }
    elsif ($value eq 'onrollover')
    {
      $allPortData{$port}{'7f03RxClkOfsAlways'} = 0 ;
      $allPortData{$port}{'7f03RxClkOfsRollover'} = 1 ;
    }
    elsif ($value eq 'never')
    {
      $allPortData{$port}{'7f03RxClkOfsAlways'} = 0 ;
      $allPortData{$port}{'7f03RxClkOfsRollover'} = 0 ;
    }
    else
    {
      print "ERROR: Bad 7f03ClockOffsets=$value.\n" ;
      return ;
    }


    # Check all the 'yes|no' parameters.
    # The hash below defines all of the options that can be "yes" or
    # "no" and which are stored as boolean values in the %allPortData
    # hash, as indexed by the name in the second half of each element.
    # For example, the parameter "carriersmoothing", if present, must be
    # either 'yes' or 'no', defaulting to 'no', and will be saved in
    # $allPortData{$port}{'smoothing'}
    # For almost all of these, the default value is 'no'.
    # The default for metTilt is 'yes', in order to maintain backward 
    # compatibility.
    my %yesNoOptions =  ( 
        '7f02'             => '7f02',
        '7f03'             => '7f03',
        'mettilt'          => 'metTilt',
        'smoothcarrier'    => 'smoothing',
        '7f03dopplers'     => '7f03Doppler',
        '7f03slips'        => '7f03Slips',
        'metasitedesc'     => 'siteDesc',
        'metastnid'        => 'stationID',
        'metarcvrtype'     => 'rcvrType',
        'metarinexrcvr'    => 'rcvrNumber',
        'metafirmware'     => 'firmwareVersion',
        'metaantxyz'       => 'antXYZ',
        'metaantoffset'    => 'antOffset',
        'metarinexant'     => 'antNum',
        ) ;

    for my $pname ( keys %yesNoOptions )
    {
      $value = ( $pname ne 'mettilt' ? 'no' : 'yes') ;
      if (exists $parms{$pname})
      {
        $value = $parms{$pname} ;
        delete $parms{$pname} ;
      }

      if ($value ne 'yes' and $value ne 'no')
      {
        print "ERROR: Bad value in $pname=$value.\n" ;
        return ;
      }

      $allPortData{$port}{ $yesNoOptions{$pname} } = ($value eq 'yes') ;
    }

  } # end of Binex section
  ######################################################################
  elsif ($service eq 'rt17')
  {
    # RT17 port: 
    #
    # Allowable parameters are:
    #
    # interval     = number     Measurement rate in seconds. 
    #                           Allowable values are:
    #                           0.1 0.2 0.5 1 2 5 10 15 30 60 300 600
    #                           Required value
    #
    # measurements = (yes|no)   Defaults to yes
    # positions    = (yes|no)   Defaults to no
    #                           Cannot have both meas=no and pos=no
    #
    # concise      = (yes|no)   Defaults to no
    # iodeslips    = (yes|no)   Defaults to no
    # gpsdata      = (yes|no)   Defaults to no
    # rawgps       = (yes|no)   Defaults to no
    # rawwaas      = (yes|no)   Defaults to no
    # smoothcarrier= (yes|no)   Defaults to no
    # smoothcode   = (yes|no)   Defaults to no
    # record27     = (yes|no)   Defaults to no

    my $interval = $parms{'interval'} ;
    delete $parms{'interval'} ;
    if ( !$interval
      or $interval !~ /^(\d+|\d+\.\d*|\d*\.\d+)$/
       )
    {
      BADRT17INTERVAL:
      print "ERROR: Bad or missing RT17 interval.\n" ;
      return ;
    }
    $interval += 0 ; # Convert to pure numeric                      
    if ($interval == 0 or $interval >= 1.0)
    {
      goto BADRT17INTERVAL if ( $interval != 1
                        and $interval != 2
                        and $interval != 5
                        and $interval != 10
                        and $interval != 15
                        and $interval != 30
                        and $interval != 60
                        and $interval != 300
                        and $interval != 600
                          ) ;
      $interval = sprintf( '%.0f', $interval ) ;
    }  
    else
    {
      goto BADRT17INTERVAL if ( $interval != 0.1
                        and $interval != 0.2
                        and $interval != 0.5
                          ) ;
      $interval = sprintf( '%.1f', $interval ) ;
    }
    $allPortData{$port}{'interval'} = $interval ;

    # Check the remaining parameters.  They all  have mostly the
    # same processing characteristics.  
    # Accept 'yes|no' setting.  Hash entry name matches the name of the
    # parameter.  Set a default value (mostly 'no') if no explicit
    # parameter is given.
    for my $p ( qw( measurements positions concise iodeslips 
                 gpsdata rawgps rawwaas smoothcarrier smoothcode
                 record27 ) 
              )
    {
      my $value = $parms{$p} ;
      if ( $value ) 
      {
        # Parameter was supplied.  Consume it.
        delete $parms{$p} ;
      }
      else
      {
        # Not supplied.  Default to 'yes' or 'no' depending on the
        # parameter.
        if ( $p eq 'measurements' 
          or $p eq 'concise' 
          or $p eq 'iodeslips'
          or $p eq 'gpsdata'
           )
        {
          $value = 'yes'
        }
        else
        {
          $value = 'no' ;
        }
      }

      if ($value ne 'yes' and $value ne 'no')
      {
        print "ERROR: Parameter $p must be yes or no..\n" ;
        return ;
      }

      my $index = $p ;
      $index = "useRt27" if ($index eq 'record27') ;

      $allPortData{$port}{$index} = ($value eq 'yes') ? 1 : 0 ;
    }

    # Don't allow both measurements and positions to be off.
    if ( ! $allPortData{$port}{'measurements'} 
     and ! $allPortData{$port}{'positions'} 
       )
    {
      print "ERROR: RT17 Must enable measurements or positions.\n" ;
      return ;
    }

    # Don't allow carrier smoothing on high rate measurements.
    if ( $allPortData{$port}{'smoothcarrier'} 
     and ($allPortData{$port}{'interval'} < 1.0)
       )
    {
      print "ERROR: Can't smooth carrier on high rate data.\n" ;
      return ;
    }

  } # end of rt17 section
  ######################################################################
  elsif ($service eq 'met')
  {
    # Met Service.  Optional parameters are cmd1, cmd2, ... cmd6
    # These are formatted as:
    #    5:commandString
    # The digit is the periodicity in minutes (zero = once only).
    # Period could be in seconds as with "5S".
    # Period of "once" is equivalent to zero.
    # The command string is after the colon.
    # If NO command strings are supplied, then the existing met.conf
    # file is left alone.  Otherwise it is completely rewritten using
    # the supplied commands.
    if ($protocol ne 'serial')
    {
      print "ERROR: Met service only allowed on serial ports.\n" ;
      return ;
    }

    my $commandFound = 0 ;
    for my $n ( 1..6 )
    {
      my $cmd = $parms{ "cmd$n" } ;

      if (defined $cmd)
      {
        delete $parms{"cmd$n"} ; # Consume this parameter
        
        # Strip leading zeroes if there is a starting number.
        $cmd =~ s/^0+(\d)/$1/ ;

        # Check validity of the command and adjust if needed.
        if ( $cmd =~ /^off$/i )
        {
          $cmd = "off" ; # force lower case.
        }
        elsif ( $cmd =~
                  /^(once|0|1|5|10|15|30|60|120|240):.*$/i
              )
        {
        }
        elsif ( $cmd =~ /^(1S|5S|15S|30S):.*$/i )
        {
          $cmd =~ s/^(\d+)s:/$1S:/ ;
        }
        else
        {
          printf( "ERROR: Invalid Met command string, %s.\n", $cmd ) ;
          return ;
        }

        if (!$commandFound)
        {
          # Found the first cmdX parameter.  That triggers
          # erasure of all existing commands.  Only do this once.
          $commandFound = 1 ;
          $allPortData{'MetControls'}{1} = 'off' ;
          $allPortData{'MetControls'}{2} = 'off' ;
          $allPortData{'MetControls'}{3} = 'off' ;
          $allPortData{'MetControls'}{4} = 'off' ;
          $allPortData{'MetControls'}{5} = 'off' ;
          $allPortData{'MetControls'}{6} = 'off' ;
        }

        $allPortData{'MetControls'}{$n} = $cmd ;
      }
    }

    # Since we are setting up a Met port on this port we have to
    # be sure there isn't one already running on another port, and
    # cancel that.
    for my $p ( qw( SerialPort1 SerialPort2 SerialPort3 SerialPort4 ) )
    {
      next if ($p eq $port) ;
      if ($allPortData{$p}{'service'} =~ /met/i)
      {
        # Switch that port back to ppp default.
        $allPortData{$p}{'service'} = 'ppp' ;
      }
    }
  } # end of Met section
  ######################################################################
  elsif ($service eq 'ppp')
  {
    if ($protocol ne 'serial')
    {
      print "ERROR: PPP Service only allowed on serial ports.\n" ;
      return ;
    }

    for my $p ( qw( localip remoteip ) )
    {
      my $value = $parms{$p} ;
      delete $parms{$p} ;
      if ( !$value
        or $value !~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/
        or $1 > 255
        or $2 > 255
        or $3 > 255
        or $4 > 255
         )
      {
        print "ERROR: Bad or missing $p parameter.\n" ;
        return ;
      }
      $allPortData{$port}{$p} = $value ;
    }

    my $connection = $parms{'connection'} ;
    delete $parms{'connection'} ;
    if (!$connection
      or ( $connection ne 'modem'
       and $connection ne 'direct'
         )
       )
    {
      print "ERROR: Bad or missing 'connection' parameter.\n" ;
      return ;
    }
    $allPortData{$port}{'connection'} = $connection ;

    # If this is a modem connection, then we'll accept the
    # optional modem setup string parameters.  These are defaulted to
    # the empty string if not specified.
    if ($connection eq 'modem')
    {
      for my $p ( qw( init1 init2 init3 reset ) )
      {
        my $value = $parms{$p} || '' ;
        delete $parms{$p} ;
        $allPortData{$port}{$p} = $value ;
      }
    }

    # Since we are updating the ppp configuration, we have to force the
    # mgetty stuff to restart.  This is handled by gpssd, which is fed a
    # special parameter in field6 in gpssd.config.  This value toggles
    # between 1 and 10, and is solely used to determine if something has
    # changed which requires an mgetty restart.
    my $toggle = $allPortData{$port}{'ppptoggle'} || 1 ;
    $allPortData{$port}{'ppptoggle'} = ( ($toggle == 1) ? 10 : 1 ) ;

  } # end of ppp section
  ######################################################################
  else
  {
    print "ERROR: Unknown service type '$service'.\n" ;
    return ;
  }


  ######################################################################
  # Check for serial port baud, parity, and flow control.
  # For PPP ports, parity is forced to none and xon/xoff flow control
  # is not allowed.
  if ($protocol eq 'serial')
  {
    # Baud= is required.
    my $baud = $parms{'baud'} || '' ;
    delete $parms{'baud'} ;
    unless ( $baud == 2400
      or $baud == 4800
      or $baud == 9600
      or $baud == 19200
      or $baud == 38400
      or $baud == 57600
      or $baud == 115200
       )
    {
      print "ERROR: Bad or missing baud parameter.\n" ;
      return ;
    }

    # Parity is required except on ppp connections.
    my $parity ;
    if ($service ne 'ppp')
    {
      $parity = $parms{'parity'} || '' ;
      delete $parms{'parity'} ;
      unless ( $parity eq 'none'
        or $parity eq 'even'
        or $parity eq 'odd'
         )
      {
        print "ERROR: Bad or missing parity parameter.\n" ;
        return ;
      }
    }

    # Flow control is required except on SerialPort1, where it defaults
    # to 'none'.  If specified on SerialPort1 it MUST be 'none'.
    my $flow = $parms{'flow'} || '';
    delete $parms{'flow'} ;

    $flow = 'none' if ( $port eq 'SerialPort1' and $flow eq '' ) ;

    unless ( $flow eq 'none' 
      or $flow eq 'hardware'
      or $flow eq 'xonxoff'
       )
    {
      print "ERROR: Bad or missing flow parameter.\n" ;
      return ;
    }

    if ( $port eq 'SerialPort1' and $flow ne 'none' )
    {
      print 
          "ERROR: Can't specify $flow flow control for SerialPort1.\n" ;
      return ;
    }

    if ($service eq 'ppp' and $flow eq 'xonxoff')
    {
      print "ERROR: No Xon/Xoff flow control on PPP ports.n" ;
      return ;
    }

    # Non-ppp serial port settings are saved in the normal gpssd.config
    # file.  These are encoded in 'baud','parity', and 'flow'.
    # But for PPP serial ports, these are kept in a separate location
    # in 'pppbaud' and 'pppflow'.
    if ($service ne 'ppp')
    {
      $allPortData{$port}{'baud'} = $baud ;
      $allPortData{$port}{'parity'} = $parity ;
      $allPortData{$port}{'flow'} = $flow ;
    }
    else
    {
      # PPP ports get different settings.
      $allPortData{$port}{'pppbaud'} = $baud ;
      $allPortData{$port}{'pppflow'} = $flow ;
    }

  } # Non-PPP serial port parameters
  ######################################################################



  # At this point, we should have consumed all of the given parameters.
  # Anything left over is an error.
  if ( (scalar(keys %parms)) != 0 )
  {
    print "ERROR: Excess parameters:",
          join( ',', keys(%parms) ),
          ".\n" ;
    return ;
  }


  ######################################################################
  # Now that the configuration structure has been updated, save
  # the results back in the configuration file.  
  ######################################################################

  # Diagnostic: Emit the new port characteristics
  # formatPort( $port, \%allPortData ) ;

  # If this is a PPP port specfication, we'll need to update the
  # ppp configuration files.  Must be done prior to updating the
  # gpssd.config, since that operation is what restarts the mgetty
  # on the port.
  if ($service eq 'ppp')
  {
    savePppConfig( \%allPortData ) ;
  }

  # If this is a met port, then update the Met configuration file.
  if ($service eq 'met')
  {
    saveMetConfig( \%allPortData ) ;
  }

  # Finally, regenerate gpssd.config and inform the streaming daemon
  # to examine the configuration for changes.
  saveGpssdConfig( \%allPortData ) ;

  # Emit the result string, showing the complete new port setup.
  # Note that this could be done by "formatPort()" using the existing
  # %allPortData hash, but that would merely confirm what we think
  # we've set in the file.  Using ShowIOport grabs completely new data
  # from the config files, making it a more robust confirmation.
  print "OK: " ;
  ShowIOport( "port=$port" ) ;

} # end of SetIoPort



########################################################################
sub DownloadLoggedFile
########################################################################
# Download LoggedFile [path=]/200310/.....
#
# Handles a request to send a logged data file over http protocol.
# This subroutine expects a single parameter specifying the path
# name of the file.  If there are any problems with the parameter or
# the file it references, then a plaintext ERROR response is sent.
# Otherwise the file is sent as a binary attachment.  Note that this
# makes this script different from most other command handlers, in that
# the http header isn't already sent when the script is called.  You
# must explicitly call plainTextHeader() on all error messages here.
########################################################################
{
  # Don't use badParmCount() to handle parameter counts (as in other
  # command handlers).  That function assumes the plainTextHeader() has
  # already been called.
  if ( scalar( @_ ) == 0 ) 
  {
    plainTextHeader() ;
    print "ERROR: Too Few Parameters.\n" ;
    return ;
  }
  if ( scalar( @_ ) > 1 ) 
  {
    plainTextHeader() ;
    print "ERROR: Too Many Parameters.\n" ;
    return ;
  }

  my $path = $_[0] ;
  $path =~ s/^path=//i ;
  # Insert a leading / if it isn't there already.
  $path =~ s/^([^\/])/\/$1/ ;

  my $fullPath = "/var/gps$path" ;

  if ( ! -f $fullPath ) 
  {
    plainTextHeader() ;
    print "ERROR: File $path does not exist.\n" ;
    return ;
  }

  sendBinaryFile( $fullPath ) ;

} # end of DownloadLoggedFile()



########################################################################
sub ShowLoggingStatus
########################################################################
# Shows the runtime status of each defined data logging session.
# Roughly equivalent to the GUI Data Logging Status page.
# The response is a multi-line request of the form:
#
#   <Show LoggingStatus>
#   Session name=CORSA   status=Logging file=/200310/28/TomsPro...
#   Session name=CORSB   status=Disabled
#   Session name=daily   status=Pending
#   Session name=Once    status=Completed
#   Session name=man     status=Completed
#   <end of Show LoggingStatus>
#
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  # Get the time of day in GPS seconds.  Includes a 13 second offset for
  # the UTC to GPS leapseconds.  315964800 converts from Unix time to
  # gps time (01Jan1970 to 06Jan1980).
  my $gpsTime = time() - 315964800 + 13 ;

  # Read in the data from the sessions configuration file.
  my %sessData ;
  getSessionData( \%sessData ) ;

  # Read in the descriptions of any active sessions, as defined by
  # files of the form /var/run/gpslog.*.  @ActiveSessions will contain
  # an array of the contents of these files, which can be searched for
  # specific session names, and the parameters of the active session(s)
  # associated with those names.
  my @ActiveSessions = () ;
  for my $f (glob '/var/run/gpslog.*')
  {
    # Open and read that file.
    # Note that though the file $f existed above (glob found it!) it
    # may have gone away before we got around to reading it.  So test
    # if the open succeeds here:
    if ( open( LOG,  $f ) )
    {
      my $line = <LOG> ; # Get first line

      # The file should always have a line, but sometimes it doesn't
      # during testing.  If not, it will be empty and we'll ignore it.
      if ($line)
      {
        # Add to list of sessions if the file contained anything
        push @ActiveSessions, $line ;
      }
      close LOG ;
    }
  }

  print "<Show LoggingStatus>\n" ;

  my $maxSessionName = 0 ;
  for my $name (keys %sessData)
  {
    $maxSessionName = length($name) 
                if length( $name ) > $maxSessionName ;
  }

  for my $name (sort keys %sessData)
  {
    # Determine the status
    my $enable = $sessData{$name}{'enable'} ;
    my $sched = $sessData{$name}{'sched'} ;
    my $time = $sessData{$name}{'time'} ;
    my $duration = $sessData{$name}{'duration'} ;

    my $status = "" ;

    # This session is enabled, and the scheduler may be running it.
    # Search for it in the active sessions stuff.
    for my $s (@ActiveSessions)
    {
      my (undef, undef, $sessname, undef, undef, undef, 
          undef, undef, $fname) = split( ' ', $s ) ;
      if ($sessname eq $name)
      {
        # Strip off a little bit of the file name to keep the length
        # to a minimum.
        $fname =~ s|^/var/gps|| ; 

        $status = "Logging file=$fname" ;
        last ;
      }
    }

    if ( ! $status )
    {
      # Session not actively logging (perhaps, not yet.)
      # Session is either Disabled, Pending or Completed.
      # Continuous sessions are either disabled or always running, But
      #   if they are JUST starting up, we'll indicate they are pending.
      # Daily sessions are disabled or always either running or pending,
      #   so they are always pending if we get to here.  
      # Once-only sessions have a single time window with a
      #   well defined stop-time.  If we are before that stop time and
      #   since we aren't running, we'll display pending.  Otherwise
      #   we'll display 'completed'.
      # Manual sessions also have a well defined time window, and should
      #   be similar to Once-only sessions.  However, their 'enable'
      #   state is artificially made false if the window has passed, so
      #   that the user can re-trigger the session.  So we infer this
      #   condition to show the 'completed' state.  Note that if a
      #   manual session is actually disabled, using the checkboxes,
      #   it's start time is adjusted to decades in the future, so it
      #   will show Disabled.

      if ( $sched eq 'cont' 
        or $sched eq 'daily'
         )
      {
        $status = $enable ? 'Pending' : 'Disabled' ;
      }
      elsif ( $sched eq 'once' )
      {
        $status = (! $enable) 
                  ? 'Disabled'
                  : ( $gpsTime >= ($time + $duration*60) )
                    ? 'Completed'
                    : 'Pending' ;
      }
      else
      {
        # manual schedule
        $status = ( $gpsTime >= ($time + $duration*60) ) 
                  ? 'completed'
                  : ($enable)
                    ? 'pending'
                    : 'disabled' ;
      }
    }

    printf "Session name=%-${maxSessionName}s status=%s\n", 
           $name,
           $status ;
  }

  print "<end of Show LoggingStatus>\n" ;

} # end of ShowLoggingStatus()



########################################################################
sub ShowPosition
########################################################################
# Displays a complete position record, including timetags, lat/lon/alt,
# velocity, clock values, DOPs and satellites used.  This is formatted
# into a multi-line response due to the large number of values that
# must be included.  Format looks like:
#
#    <Show Position>
#    GpsWeek     1242
#    WeekSeconds 123415.4   
#    Latitude    37.12345678956 deg
#    Longitude   122.05212341234 deg
#    Altitude    27.125 meters
#    Qualifiers  WGS84,3D,WAAS-DGPS
#    Satellites  1,2,3,8,10,13,27
#    ClockOffset -0.395894 msec
#    ClockDrift  -0.103 ppm
#    VelNorth    -0.01 m/sec
#    VelEast     +0.02 m/sec
#    VelUp       +0.02 m/sec
#    PDOP        2.6
#    HDOP        1.4
#    VDOP        2.2
#    TDOP        1.6
#    <end of Show Position>
# 
########################################################################
{
  print "<Show Position>\n" ;

  # Open a channel to the sysinfo program so we can read the
  # pos_info information.
  open( SYSINFO, '/usr/local/fireball/sysinfo -p pos_info |' ) ;

  # Response is of the form
  #   167368.0, 1166:  37.3891508323 -122.034234235 -3.456
  #    [timetag   week#  Lat-degrees   Lon-degrees    Height-meters]
  #   3 14 9 11 18 25 23 31
  #    [Satellites used]
  #   1.5 0.9 1.3 0.8
  #    [Pdop,hdop,vdop,tdop]
  #   0.03 -0.05 0.02
  #    [Velocities E,N,Up]
  #   0.000000e+00 -2.054596e-02
  #    [clock offset, clock drift]
  #   3D autonomous
  #    [postype (2D 3D or OLD) and corrections]
  #   OK

  my ($seconds, $week) = (0,0) ;
  my ($latitude, $longitude, $altitude ) = (0,0,0) ;
  my @svsUsed = "None" ;
  my ($pdop, $hdop, $vdop, $tdop) = (0,0,0,0) ;
  my ($velE, $velN, $velUp) = (0,0,0) ;
  my ($clockOffset, $clockDrift) = (0,0) ;
  my ($posType, $posCorrections) = ("old", "Autonomous") ;
  my $warning = '' ;

  my $line = <SYSINFO> ;
  chomp $line ;
  if ($line !~ /^OK/)
  {
    $line =~ s/[,:]//g ;
     
    ($seconds, $week, $latitude, $longitude, $altitude) 
                                            = split(' ',$line) ;


    @svsUsed = split( ' ', <SYSINFO> ) ;

    ($pdop, $hdop, $vdop, $tdop) = split( ' ', <SYSINFO> ) ;

    ($velE, $velN, $velUp) = split( ' ', <SYSINFO> ) ;

    ($clockOffset, $clockDrift) = split( ' ', <SYSINFO> ) ;

    # Convert clockOffset in meters to milliseconds, by dividing by
    # the speed of light and then multiplying by 1000.
    $clockOffset /= 2.99792458e5 ; 

    # Convert drift (in meters per second) to parts-per-million by 
    # dividing by speed of light (2.99e8, and then multiplying by a 
    # million.
    $clockDrift /= 2.99792458e2 ;

    ($posType, $posCorrections) = split( ' ', <SYSINFO> ) ;
    if ($posType eq 'old' ) 
    {
      $posType = 'OLD' ;
      @svsUsed = () ;
    }

    $posCorrections = 'Autonomous' 
            if ($posCorrections eq 'autonomous') ;

    $posCorrections =~ s/_/-/ ;
  }
  close(SYSINFO) ;

  my $format = "%-11s %s\n" ;
  printf $format, 'GpsWeek', $week ;
  printf $format, 'WeekSeconds', $seconds ;
  printf $format, 'Latitude', $latitude.' deg' ;
  printf $format, 'Longitude', $longitude.' deg' ;
  printf $format, 'Altitude', $altitude.' meters' ;
  printf $format, 'Qualifiers', "WGS84,$posType,$posCorrections" ;
  printf $format, 'Satellites', 
                  join( ',', sort( {$a <=> $b} @svsUsed ) ) ;
  printf $format, 'ClockOffset', sprintf( "%.6f msec", $clockOffset ) ;
  printf $format, 'ClockDrift', sprintf( "%.6f ppm", $clockDrift ) ;
  printf $format, 'VelNorth', sprintf( "%5.2f m/sec", $velN ) ;
  printf $format, 'VelEast', sprintf( "%5.2f m/sec", $velE ) ;
  printf $format, 'VelUp', sprintf( "%5.2f m/sec", $velUp ) ;
  printf $format, 'PDOP', $pdop ;
  printf $format, 'HDOP', $hdop ;
  printf $format, 'VDOP', $vdop ;
  printf $format, 'TDOP', $tdop ;

  print "<end of Show Position>\n" ;

} # end of ShowPosition()




########################################################################
sub SetFirmwareWarranty
########################################################################
# Set Warranty code=dd7c6798271f69f0
# Used to feed a warranty enable code into the system.  The code has to
# be a 16 character string.  If it is accepted as legal by the system,
# then the warranty date is changed, and the OK: response below will be
# sent.  Otherwise an error message will be produced.
#
# Response:
#   OK: FirmwareWarranty date=2003/09/04
#
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $code = $_[0] ;
  $code =~ s/^code=//i ;

  if ($code !~ /^\w{16}$/ )
  {
    print "ERROR: Warranty code must have 16 characters.\n" ;
    return ;
  }
  my $result = system( "/usr/bin/sudo /usr/local/fireball/warranty -f"
                      . $code ) ;
  if ($result != 0)
  {
    print "ERROR: Invalid warranty code.\n" ;
    return ;
  }

  print "OK: " ;
  ShowFirmwareWarranty() ;

} # end of SetFirmwareWarranty()



########################################################################
sub ShowLoggedFileStats
########################################################################
# ShowLoggedFileStats [directory=/abc]
# Shows the logging usage for a specified directory.  If no parameter
# is given, then usage for '/' is shown.
# Displays the number of regular files (not directories) in the path.
# Displays the total of 1024 byte blocks in those files.
# Displays the amount of space available for further logging.
#   Note that this is independent of the path.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 1 ) ) ;

  my $path= $_[0] ;
  if ($path)
  {
    $path =~ s/^directory=//i ;
    # Insert a leading / if it isn't there already.
    $path =~ s/^([^\/])/\/$1/ ;

    # Give an error message if the directory doesn't exist,
    # or in several special cases.
    # Don't allow doubled-slashes.  Probably an error.
    # Don't allow '..' syntax to move up a level.  Could allow user
    # to view items outside of /var/gps.
    # Don't allow viewing /incoming or /tmp.  Not used for logged data.
    if ( ! -d "/var/gps$path"
      || $path =~ m|//|
      || $path =~ m/\.\./
      || $path =~ m|/incoming|
      || $path =~ m|/tmp|
       )
    {
      print "ERROR: Can't find directory '$path'\n" ;
      return ;
    }
  }
  else
  {
    $path = '/' ;
  }

  # Get the diskspace available data.  Sadly, Perl doesn't have an easy
  # way to get this via a system call.  We just run the "df" command and
  # parse the output, which will look something like this:
  #
  #   Filesystem           1k-blocks      Used Available Use% Mounted on
  #  /dev/discs/disc0/part4    182447    171879      1135  99% /var/gps
  #
  # This is just split into a list, and the appropriate fields lifted
  # from that.  'Filesystem' is field[0], 'on' is [6], etc.
  my $availSpace = ( split " ", qx( df /var/gps ) )[10] ;
  $availSpace /= 1000.0 ;

  # Determine the number of files in the selected directory tree.
  # If the directory tree is '/' then we'll search all of the
  # subdirectories that contain data files, which all start with
  # '2'.  That is, they are all YYYYMM names.  This lets us skip
  # the /var/gps/incoming directory which contains FTP uploaded
  # firmware files.
  my @dataFiles ;
  if ($path ne '/')
  {
    @dataFiles=qx( find /var/gps$path -type f ) ;
  }
  else
  {
    @dataFiles=qx( find /var/gps/2* -type f ) ;
  }

  my $fileCount = scalar( @dataFiles ) ;

  # Run through all of those files and add up their total space used.
  # Correct each length for the quantization to 1024 byte blocks.
  # Note that this does NOT account for any pointer-blocks that are
  # part of the directory structure, so it won't exactly match
  # what you'd get from the df command, etc.  
  my $usedSpace = 0 ;
  for my $f (sort @dataFiles)
  {
    chomp $f ;
    my $size = int( ((-s $f)+1023)/1024 ) ;
    $usedSpace += $size ;
    #print $size, $f, "\n" ;
  }
  $usedSpace /= 1024 ;

  # Likewise we have to obtain the number of datafiles used using
  # a call to 'find' which searches for regular files.

  printf "LoggedFileStats directory=%s "
        ."fileCount=%d "
        ."MbytesUsed=%.3f MbytesAvailable=%.3f\n",
         $path,
         $fileCount,
         $usedSpace,
         $availSpace
        ;

} # end of ShowLoggedFileStats() 



########################################################################
sub ShowUtcTime
########################################################################
# Emits the current Unix date and time in a parsed form with labeled
# fields.
#
#   UtcTime year=2003 month=10 day=24 \
#           hour=17 minute=4 second=6 \
#           julianDay=247
#
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  my ($sec, $min,$hour, $mday, $month, $year, $wday, $yday, undef)
    = gmtime() ;
  $year += 1900 ;
  ++$month ; # 0..11 to 1..12
  ++$yday ;  # 0..365 to 1..366
  print "UtcTime year=$year month=$month day=$mday",
        " hour=$hour minute=$min second=$sec",
        " julianDay=$yday",
        "\n" ;

} # end of ShowUtcTime()



########################################################################
sub ShowGpsTime
########################################################################
# Emits the current GPS weeknumber and week seconds.
#
#   GpsTime gpsweek=1247 weekseconds=123456
#
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;


  # Get the time of day in GPS seconds.  Includes a 13 second offset for
  # the UTC to GPS leapseconds.  315964800 converts from Unix time to
  # gps time (01Jan1970 to 06Jan1980).
  my $gpsTime = time() - 315964800 + 13 ;

  my $weeks = int( $gpsTime / 604800.0 ) ;
  my $weekseconds = int( $gpsTime ) % 604800 ;

  print "GpsTime gpsweek=$weeks weekseconds=$weekseconds\n" ;

} # end of ShowGpsTime() 





########################################################################
sub ShowGpsUtcData
########################################################################
# Send a nicely formatted ASCII listing of a GPS UTC/Clock data.
# No parameters are required or allowed.
# The response looks like:
#
#   <Show GpsUtcData>
#   A_0 (s):          0.000000000E+00
#   A_1 (s/s):       -1.865174681E-14
#   DELTA_T_LS (s):         13
#   ...
#   <end of Show GpsUtcData>
#
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  print "<Show GpsUtcData>\n" ;

  open( SYSINFO, 
       "/usr/bin/sudo /usr/local/fireball/sysinfo -p utc_data |"
      ) ;
  # Response is of the form
  #   **** UTC data ****
  #   ...
  #   OK
  # Print the returned data  until a line with "OK" comes back.

  # But skip the first line...
  $_ = <SYSINFO> ;

  while ( ($_ = <SYSINFO>) !~ '^OK' )
  {
    print $_ ;
  } 
  close( SYSINFO ) ;

  print "<end of Show GpsUtcData>\n" ;

} # end of ShowGpsUtcData()



########################################################################
sub ShowGpsIonoData
########################################################################
# Send a nicely formatted ASCII listing of a GPS Ionospheric data.
# No parameters are required or allowed.
# The response looks like:
#
#   <Show GpsIonoData>
#   ALPHA_0 (sec):                 4.004687071E-08
#   ALPHA_1 (sec/semicircle):      1.490116119E-08
#   ...
#   <end of Show GpsIonoData>
#
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  print "<Show GpsIonoData>\n" ;

  open( SYSINFO, 
       "/usr/bin/sudo /usr/local/fireball/sysinfo -p iono_data |"
      ) ;
  # Response is of the form
  #   **** IONO data ****
  #   ...
  #   OK
  # Print the returned data  until a line with "OK" comes back.

  # But skip the first line...
  $_ = <SYSINFO> ;

  while ( ($_ = <SYSINFO>) !~ '^OK' )
  {
    print $_ ;
  } 
  close( SYSINFO ) ;

  print "<end of Show GpsIonoData>\n" ;

} # end of ShowGpsIonoData()



########################################################################
sub ShowGpsEphemeris
########################################################################
# Send a nicely formatted ASCII listing of a GPS satellite's ephemeris
# data.  The required parameter is [prn=]27, specifying the desired
# satellite number.  The multiline response is:
#
#   <Show GpsEphemeris prn=12>
#   ID:                         01
#   IODC:                       0221
#   Eccentricity:               5.235160817E-03
#   ...
#   <end of Show GpsEphemeris prn=12>
#
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $prn = $_[0] ;
  $prn =~ s/^prn=//i ;

  if ( $prn !~ /^\d{1,2}$/
    || $prn == 0
    || $prn > 32
     )
  {
    print "ERROR: Invalid PRN number\n" ;
    return ;
  }

  print "<Show GpsEphemeris prn=$prn>\n" ;

  open( SYSINFO, 
       "/usr/bin/sudo /usr/local/fireball/sysinfo -p eph_data $prn |"
      ) ;
  # Response is of the form
  #   **** Week 1242 ephemeris for SV-01 ************
  #   ...
  #   OK
  # Print the returned data  until a line with "OK" comes back.
  # But skip the first line...
  $_ = <SYSINFO> ;

  while ( ($_ = <SYSINFO>) !~ '^OK' )
  {
    print $_ ;
  } 
  close( SYSINFO ) ;

  print "<end of Show GpsEphemeris prn=$prn>\n" ;

} # end of ShowGpsEphemeris



########################################################################
sub ShowGpsAlmanac
########################################################################
# Send a nicely formatted ASCII listing of a GPS satellite's Almaanc
# data.  The required parameter is [prn=]27, specifying the desired
# satellite number.  The multiline response is:
#
#   <Show GpsAlmanac prn=12>
#   ID:                         1
#   Health:                     000
#   Eccentricity:               5.239009857E-03
#   ...
#   <end of Show GpsAlmanac prn=12>
#
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $prn = $_[0] ;
  $prn =~ s/^prn=//i ;

  if ( $prn !~ /^\d{1,2}$/
    || $prn == 0
    || $prn > 32
     )
  {
    print "ERROR: Invalid PRN number\n" ;
    return ;
  }

  print "<Show GpsAlmanac prn=$prn>\n" ;

  open( SYSINFO, 
       "/usr/bin/sudo /usr/local/fireball/sysinfo -p alm_data $prn |"
      ) ;
  # Response is of the form
  #   **** Week 1242 almanac for SV-01 ************
  #   ...
  #   OK
  # Print the returned data  until a line with "OK" comes back.
  # But skip the first line...
  $_ = <SYSINFO> ;

  while ( ($_ = <SYSINFO>) !~ '^OK' )
  {
    print $_ ;
  } 
  close( SYSINFO ) ;

  print "<end of Show GpsAlmanac prn=$prn>\n" ;

} # end of ShowGpsAlmanac




########################################################################
sub ShowRefStation
########################################################################
# Shows the reference station configuration in one of two forms.
#   Show RefStation [[style=]llh]
#   Show RefStation [[style=]xyz]
#
# Default, if no parameter is specified is "llh".
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 1 ) ) ;


  my $style = lc($_[0] || 'llh') ;
  $style =~ s/^style=//i ;
  if ($style ne 'llh' and $style ne 'xyz')
  {
    print "ERROR: Style must be 'llh' or 'xyz'.\n" ;
    return ;
  }

  my %refStationData ;
  getRefStationData( \%refStationData ) ;

  print "RefStation " ;
  if ($style eq 'llh')
  {
    print "lat=$refStationData{'lat'} ",
          "lon=$refStationData{'lon'} ",
          "height=$refStationData{'height'}" ;
  }
  else
  {
    printf "x=%.4f y=%.4f z=%.4f", 
           $refStationData{'x'}, 
           $refStationData{'y'}, 
           $refStationData{'z'} ; 
  }

  my $name=$refStationData{'cmrStationName'} ;
  $name =~ s/'/\\'/g ;

  my $desc = $refStationData{'rtcmdesc'} ;
  $desc =~ s/'/\\'/g ;

  print " RtcmId=",$refStationData{'rtcmid'},
        " BinexId=",$refStationData{'binexid'},
        " CmrId=",$refStationData{'cmrid'},
        " CmrStationName='$name'",
        " Description='$desc'\n" ;

} # end of ShowRefStation



########################################################################
sub SetRefStation
########################################################################
# Sets the reference station configuration to match the given
# parameters.
########################################################################
{

  # Get current settings into a local hash.
  my %refStationData ;
  getRefStationData( \%refStationData ) ;

  my $style = '' ;
  my %parms ;

  for my $p (@_)
  {
    if ($p !~ /=/)
    {
      print "ERROR: Must fully specify id=value for parameter $p\n" ;
      return ;
    }

    my ($id, $value) = split( '=', $p ) ;
    $id = lc($id) ;
    $parms{$id} = $value ;

    if ($id =~ /^(lat|lon|height)$/)
    {
      if ($style eq 'xyz')
      {
        NOMIX:
        print "ERROR: can't mix llh and xyz.\n" ;
        return ;
      }
      if ( $value !~ /^[+-]?\d+\.?\d*$/ )
      {
        BADNUM:
        print "ERROR: Bad numeric value '$value'.\n" ;
        return ;
      }
      $style = 'llh' ;

      $refStationData{'lat'} = $value if ($id eq 'lat') ;
      $refStationData{'lon'} = $value if ($id eq 'lon') ;
      $refStationData{'height'} = $value if ($id eq 'height') ;
    }
    elsif ($id =~ /^(x|y|z)$/)
    {
      goto NOMIX if ($style eq 'llh') ;
      goto BADNUM if ( $value !~ /^[+-]?\d+\.?\d*$/ ) ;
      $style = 'xyz' ;
      $refStationData{$id} = $value ;
    }
    elsif ($id eq 'rtcmid')
    {
      if ( $value !~ /^\d{1,4}$/
        or $value > 1023
         )
      {
        print "ERROR: RtcmID must be numeric < 1024.\n" ;
        return ;
      }
      $refStationData{'rtcmid'} = $value ;
    }
    elsif ($id eq 'cmrid')
    {
      if ( $value !~ /^\d{1,4}$/
        or $value > 31
         )
      {
        print "ERROR: CmrID must be numeric < 32.\n" ;
        return ;
      }
      $refStationData{'cmrid'} = $value ;
    }
    elsif ( $id eq 'name'
         or $id eq 'cmrstationname'
          )
    {
      $refStationData{'cmrStationName'} = $value ;
    }
    elsif ( $id eq 'binexid')
    {
      $refStationData{'binexid'} = $value ;
    }
    elsif ($id eq 'description')
    {
      $refStationData{'rtcmdesc'} = $value ;
    }
    else
    {
      print "ERROR: Invalid Parameter type '$id'.\n" ;
      return ;
    }
  }

  if ( $style eq 'llh'
   and ( !$parms{'lat'}
      or !$parms{'lon'}
      or !$parms{'height'}
       )
   or $style eq 'xyz'
   and ( !$parms{'x'}
      or !$parms{'y'}
      or !$parms{'z'}
       )
     )
  {
    print "ERROR: Missing position coordinate.\n" ;
    return ;
  }

  $style = 'llh' if !$style ;


  updateRefStationData( \%refStationData, ($style eq 'xyz') ) ;

  print "OK: " ;
  ShowRefStation( "style=$style" ) ;

} # end of SetRefStation



########################################################################
sub ShowNtpClient
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  my @servers = getNtpServers() ;

  print "NTPclient " ;
  if (@servers)
  {
    print "enable=yes servers=", 
          join( ',', @servers ),
          "\n" ; 
  }
  else
  {
    print "enable=no\n" ;
  }

} # end of ShowNtpClient()


########################################################################
sub SetNtpClient
########################################################################
# Set NtpClient enable=no
# Set NtpClient [enable=yes] servers=server1[,server2[,server3]]]
#
# Used to control the Ntp Client setup (setting time via external
# servers).  This can be disabled with 'enable=no'.  Or it can be 
# enabled by specifying a list of one-to-three servers.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 2 ) ) ;

  my @servers = () ;
  my $enable = '' ;

  # Check the parameters that were fed to this handler.
  for my $p (@_)
  {
    if ($p !~ /=/)
    {
      print "ERROR: Must fully specify id=value for parameter '$p'\n" ;
      return ;
    }

    my ($id, $value) = split( '=', $p ) ;
    $id = lc($id) ;

    if ($id eq 'enable')
    {
      $enable = lc($value) ;
      if ($enable !~ /^(yes|no)$/)
      {
        BADPARM:
        print "ERROR: bad parameter '$p'\n" ;
        return ;
      }
    }
    elsif ($id eq 'servers')
    {
      @servers = split( ',', $value ) ;
      goto BADPARM if ($value =~ / / ) ;
    }
    else
    {
      goto BADPARM ;
    }
  }

  if (@servers and $enable eq 'no')
  {
    print "ERROR: Can't disable and specify servers.\n" ;
    return ;
  }

  if ( $enable eq 'yes' and !@servers)
  {
    print "ERROR: Must specify servers to enable.\n" ;
    return ;
  }

  # Test for max of three servers.
  if (scalar(@servers) > 3)
  {
    print "ERROR: More than three servers.\n" ;
    return ;
  }

  saveNtpServers( @servers ) ;

  print "OK: " ;
  ShowNtpClient() ;

} # end of SetNtpClient()


########################################################################
sub ShowIoStatus
########################################################################
# Show IOstatus
#    Lists currently active IOports and some information about them.
# 
#    Response:
#     <Show IOstatus>
#     IOport name=SerialPort3 service=CMR 
#     IOport name=TcpPort1234 service=RT17 remoteIP=192.168.142.7
#     <end of Show IOstatus>
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;
  my @clients = getIoStatus() ;

  print "<Show IOstatus>\n" ;

  for (my $c = 0 ; $c < @clients ; ++$c )
  {
    my $name = $clients[$c]{'name'} ;
    my $service = $clients[$c]{'service'} ;

    # Skip any internal services.
    next if ( $service eq 'metdata' ) ;

    print "IOport name=$name service=$service" ;

    print " remoteIP=$clients[$c]{'ip'}" if ($name !~ /Serial/) ;

    print "\n" ;
  }

  print "<end of Show IOstatus>\n" ;
} # end of ShowIoStatus() 


########################################################################
sub ShowL2Tracking
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;


  # Obtain current setting from gpsinfo system, using sysinfo.
  # Response will be of the forms:
  #  put l2_mode [off | enigma | l2c_or_enigma | l2c_and_enigma] 
  $_ = qx( /usr/local/fireball/sysinfo -p l2_mode ) ;
  chomp ;

  my (undef, undef, $L2state) = split( " ", $_ ) ;

  print "L2Tracking mode=",
        ( ($L2state eq 'enigma') 
          ? "Y\n" 
          : ($L2state eq 'l2c_or_enigma') 
            ? "CorY\n" 
            : ($L2state eq 'l2c_and_enigma') 
              ? "CandY\n" 
              : "Off\n"
        ) ;

} # end of ShowL2Tracking



########################################################################
sub SetL2Tracking
########################################################################
# set l2Tracking mode=[ off | Y | CorY | CandY ]
{
  #print "Executing SetClockSteering\n" ;
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $newstate = lc( $_[0] ) ;
  $newstate =~ s/^mode=// ;
  #print "newstate=$newstate\n" ;
  if ($newstate !~ m/^(off|y|cory|candy)$/)
  {
    print "ERROR: invalid parameter $_[0]\n" ;
    return ;
  }

  newStingerConfig( 'put l2_mode '
                    . ( ($newstate eq 'y') 
                        ? "enigma\n" 
                        : ($newstate eq 'cory') 
                          ? "l2c_or_enigma\n" 
                          : ($newstate eq 'candy') 
                            ? "l2c_and_enigma\n" 
                            : "off\n"
                      ) 
                  ) ;


  print "OK: " ;
  ShowL2Tracking() ;

} # end of SetL2Tracking




########################################################################
sub ShowL2C
########################################################################
# This command has been superceded by ShowL2Tracking.
# It formerly showed "L2C enable=[yes|no]".
# But now we show the results of the ShowL2Tracking command.
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  ShowL2Tracking() ;

} # end of ShowL2C



########################################################################
sub SetL2C
########################################################################
{
  #print "Executing SetClockSteering\n" ;
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  my $newstate = lc( $_[0] ) ;
  $newstate =~ s/^enable=// ;
  #print "newstate=$newstate\n" ;
  if ($newstate !~ m/^(yes|no)$/)
  {
    print "ERROR: invalid parameter $_[0]\n" ;
    return ;
  }

  SetL2Tracking( ($newstate eq 'yes') 
                 ? "CorY" 
                 : "Y"
               )  ;


} # end of SetL2C



########################################################################
sub ShowAutodelete
########################################################################
# Responds to command Show AutoDelete
# Emits a response showing the current state of the AutoDelete and
# Pool-Management controls.  Response is of the form:
#
#   AutoDelete Reserved=1,yes a=50,yes default=10,no
#
# Reserved will always be shown.  The presence of any other values is
# dependent on which pools are actually defined.  Every pool shows the
# size in Megabytes, and a 'yes' or 'no' indication of whether
# autoDelete is enabled for that section.
########################################################################
{
  #print "Executing ShowAutodelete\n" ;
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  my %autoDeleteControls ;

  getAutoDeleteControls( \%autoDeleteControls ) ;

  print "AutoDelete" ;

  for my $pool ('Reserved','a'..'z','Default')
  {
    if (exists $autoDeleteControls{$pool})
    {
      printf " %s=%d,%s",
             $pool, 
             $autoDeleteControls{$pool}{'size'},
             $autoDeleteControls{$pool}{'enable'} ? 'yes' : 'no'
             ;
    }
  }

  print "\n" ;

} # end of ShowAutodelete



########################################################################
sub SetAutodelete
########################################################################
# Deal with Set AutoDelete command
#
#     Set AutoDelete Reserved=1,[yes|no]  \
#                    a-1,[yes,no]  \
#                    ... \
#                    Default=1,[yes,no]
# Must have at least one setting defined, even if it's just 
# 'Reserved=1,no'.
#
# For backward compatibility, thisis also accepted:
#     Set AutoDelete enable=[yes|no]
# which is equivalent to
#     Set AutoDelete Reserved:1,[yes|no]
# In this case, no pool definitions are allowed.
#
# On succes, the new controls are installed and their new
# state is printed as with 'OK: AutoDelete ....'
########################################################################
{
  my $parmCount = scalar( @_ ) ;
  return if ( badParmCount( $parmCount, 1, 100 ) ) ;

  my %autoDeleteControls ;

  # The Reserved setting is always defined with a default of
  # 1 Mbyte and disabled.
  $autoDeleteControls{'Reserved'}{'size'} = 1 ;
  $autoDeleteControls{'Reserved'}{'enable'} = 0 ;

  # Go through each of the parameters.
  # Check for proper format and then
  # encode the values into %autoDeleteControls
  for my $p (@_)
  {
    $p = lc($p) ; # Everything in lower case.

    # Handle backward compatible format.
    # Single parameter allowed of the form
    #   Enable=[yes|no], all case insensitive.
    if ($p =~ /^enable=(yes|no)$/ )
    {
      if ( $parmCount > 1)
      {
        print "ERROR: Bad parameter combination.\n" ;
        return ;
      } ;
      # Use the default 1MB setting and set the enable if 'yes'.
      $autoDeleteControls{'Reserved'}{'enable'} = ($1 eq 'yes') ;
      goto DONE ;
    }

    # New style parameters.  Expecting:
    #   pool=size,[yes|no]
    # where pool is 'reserved', 'default', 'a'-'z'.
    #       size is integer > 0
    if ( $p !~ /^(reserved|default|[a-z])=(\d+),(yes|no)$/ )
    {
      print "ERROR: Bad parameter format: $p" ;
      return ;
    }
    my( $pool, $size, $enable) = ($1, $2, $3) ;
    $pool = 'Default' if ($pool eq 'default') ;
    $pool = 'Reserved' if ($pool eq 'reserved') ;
    if ($size == 0) 
    {
      print "ERROR: AutoDelete threshold must be > 0." ;
      return ;
    }
    $autoDeleteControls{$pool}{'enable'} = ($enable eq 'yes') ;
    $autoDeleteControls{$pool}{'size'} = $size ;
  }

  DONE:
  saveAutoDeleteControls( \%autoDeleteControls ) ;

  print "OK: " ;
  ShowAutodelete() ;

} # end of setAutodelete




########################################################################
sub ShowLoggedFilePools
########################################################################
# ShowLoggedFilePools
# Shows the logging usage for all data logging Pools.
# Produces a multi-line response showing all defined pools,
# their definitions and file space usage.
# Pool status is obtained from the file /var/run/poolStatus.
# The accessor function getPoolStatus() parses this into a hash
# indexed by the pool name, with :
#        'ad' : Boolean, true if autodelete is enabled on his pool.
#      'size' : Current size in bytes.
#   'maxsize' : Pool's allocated space in bytes.
#     'files' : Files in the pool.
########################################################################
{
  # No parameters allowed.
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;


  # get the current pool status from the 
  # runtime file /var/run/poolStatus.
  my %poolStatus ;
  getPoolStatus( \%poolStatus ) ;


  print "<Show LoggedFilePools>\n" ;

  for my $pool ('a'..'z','Default')
  {
    if (exists( $poolStatus{$pool} ) ) 
    {
      my $ad = $poolStatus{$pool}{'ad'} eq 'yes' ; 
      my $size = $poolStatus{$pool}{'size'}/1000000 ;
      my $maxSize = $poolStatus{$pool}{'maxsize'}/1000000 ;

      my $full = ( ( ! $ad ) && ($size >= $maxSize) ) ;

      printf "pool=%-7s files=%-3d size=%-5.1f maxSize=%-3.0f "
            ."autoDelete=%-3s full=%s\n", 
             $pool,
             $poolStatus{$pool}{'files'},
             $size,
             $maxSize,
             ($ad ? 'Yes' : 'No'),
             ($full ? 'Yes' : 'No'),
             ;
    }
  }

  print "<end of Show LoggedFilePools>\n" ;

} # end of ShowLoggedFilePools() 



########################################################################
sub ShowClientAuthKey
########################################################################
# Prints the current client authentication key used for authenticating
# I/O stream requesters.  Only the secured (encrypted) key is visible
# using this method.  As such, it is only useful for saving and
# restoring the key and perhaps for transfering the key from one system
# to another.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  my %allPortData ;
  getPortSetup( \%allPortData ) ;

  print "ClientAuthKey key=$allPortData{'AuthKey'}\n" ;

} # end of ShowClientAuthKey



########################################################################
sub SetClientAuthKey
########################################################################
# Used to assign a value to the key used for I/O stream client 
# authentication.  A single parameter is required, which can be
# either
#   key=abcdef1234567890
# or
#   clearkey=1234567
#
# "key" is the encoded key, as saved in gpssd.config.  This encoded
# value cannot easily be used to determine the actual shared key.
# Thus, it can be used to archive the key or transfer the key from 
# one system to another.  "key" must be a 16-digit hexadecimal number.
# If this parameter is supplied, it will be placed directly into the
# gpssd.config file.
#
# "clearkey" is the true clear-text authentication key, which must be
# known to the NetRS and the connecting clients.  It is a 7-character 
# Ascii string.  This value will be encoded prior to being placed
# into the gpssd.config file.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 1 ) ) ;

  $_ = $_[0] ;
  my $key ;

  if ( m/^key=([0-9a-f]{16})$/i )
  {
    $key = lc($1) ;

  }
  elsif ( m/^clearkey=(.{7})$/i )
  {
    $key = qx( /usr/local/fireball/makeKey $1 ) ;
  }
  else
  {
    print "ERROR: Invalid parameter $_\n" ;
    return ;
  }

  # Get the current port configuration data from gpssd.config
  my %allPortData ;
  getPortSetup( \%allPortData ) ;

  # Insert the new key
  $allPortData{'AuthKey'} = $key ;

  # Save the updated configuration data.
  saveGpssdConfig( \%allPortData ) ;
  
  # Format the response showing the new encrypted key.
  print "OK: " ;
  ShowClientAuthKey() ;

} # end of SetClientAuthKey



########################################################################
sub ShowIpFiltering
########################################################################
# Responds to command Show IpFiltering
# Emits a response showing the current state of the IP Filtering
# (iptables) system.  Response is of the form:
#
#   IpFiltering enable=[yes|no] range1=192.168.142.7/32 range2=...
#
########################################################################
{
  #print "Executing ShowAutodelete\n" ;
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  my %iptablesSetup ;

  getIptablesSetup( \%iptablesSetup ) ;
  my $rangeCount = $iptablesSetup{'rangeCount'} ;

  print "IpFiltering enable=",
        $iptablesSetup{'enable'} ? 'yes' : 'no' ;

  for ( my $i = 0 ; $i < $rangeCount ; ++$i )
  {
    print " range",
          ($i+1),
          '=',
          $iptablesSetup{'rangeBases'}[$i],
          '/',
          $iptablesSetup{'rangeBits'}[$i]
          ;
  }

  print "\n" ;

} # end of ShowIpFiltering




########################################################################
sub SetIpFiltering
########################################################################
# Responds to command Set IpFiltering
# On success, emits "OK:" followed by the ShowIpFiltering response.
# Parameters must be one or more of the following:
#   enable=[yes|no]
#   range[n]=192.168.142.20
#   range[n]=192.168.142.20/bb
#
# Ranges parameter names can be specified with a numeric suffix 
# (range0, range1, etc.) or as just multiple 'range=' values,
# at the whim of the programmer.  The suffixes are ignored.
# IP addresses can be specified as xxx.xxx.xxx.xxx (a single address)
# or with a /bb suffix indicating a netmask bit count from 1 to 32.
# A single address has a netmask bitcount of 32.
#
########################################################################
{
  #print "Executing ShowAutodelete\n" ;
  return if ( badParmCount( scalar( @_ ), 1, 20 ) ) ;

  # Read in current settings.
  my %iptablesSetup ;

  my $rangeCount = 0 ;
  my @rangeBases ;
  my @rangeBits ;

  # Check the parameters that were fed to this handler.
  for my $p (@_)
  {
    if ($p !~ /=/)
    {
      print "ERROR: Must fully specify id=value for parameter '$p'\n" ;
      return ;
    }

    my ($id, $value) = split( '=', $p ) ;
    $id = lc($id) ;
    $value = lc($value) ;

    if ($id eq 'enable')
    {
      if ($value !~ /^(yes|no)$/)
      {
        BADPARM:
        print "ERROR: bad parameter '$p'\n" ;
        return ;
      }
      $iptablesSetup{'enable'} = ($value eq 'yes' ? 1 : 0) ;
    }
    elsif ($id =~ '^range\d*$')
    {
      goto BADPARM if ( !$value ) ;

      # Default these assusming this is a simple address without
      # an explicit netmask.
      my $ip = $value ;
      my $netmask = 32 ;

      if ($value =~ /\/\d{1,2}$/)
      {
        # Ends in /n or /nn, so there is an explicit netmask.
        # Extract the two pieces.
        ($ip, $netmask) = $value =~ /^(.*)\/(\d{1,2})$/ ;
      }

      # Verify the IP format, octet ranges, and netmask range.
      if ( $ip !~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/
        or $1 > 255
        or $2 > 255
        or $3 > 255
        or $4 > 255
        or $netmask < 8
        or $netmask > 32
         ) 
      {
        print "ERROR: Bad Ip address '$value'\n" ;
        return ;
      }

      # A good address range.  Add to the end of the arrays.
      push( @rangeBases, $ip ) ;
      push( @rangeBits, $netmask ) ;
      ++$rangeCount ;
    }
    else
    {
      goto BADPARM ;
    }
  }

  # Install the address arrays, and the count into the hash.
  $iptablesSetup{'rangeCount'} = $rangeCount ;
  $iptablesSetup{'rangeBases'} = \@rangeBases ;
  $iptablesSetup{'rangeBits' } = \@rangeBits ;

  if ( $iptablesSetup{'enable'}
    && $rangeCount == 0
     )
  {
    print "ERROR: IpFiltering must have at least one filter range.\n" ;
    return ;
  }

  # Install the new settings.
  # Note that this has a delayed effect to allow us time to
  # send the response, assuming we are about to be cut off.
  saveIptablesSetup( \%iptablesSetup ) ;

  print "OK: " ;
  ShowIpFiltering() ;

} # end of SetIpFiltering



########################################################################
sub ShowHttpPorts
########################################################################
# Responds to command Show HttpPorts
# Emits a response showing the current state of the Http and Https 
# port controls.  Response is of the form:
#
#   HttpPorts httpPort80=yes httpAltPort=no,8080 \
#             httpsPort443=yes httpsAltPort=yes,8443
#
########################################################################
{
  #print "Executing ShowAutodelete\n" ;
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  my %httpSetup ;

  getHttpSetup( \%httpSetup ) ;

  print "HttpPorts httpPort80=",
         $httpSetup{'httpPort80'} ? 'yes' : 'no',
        " httpAltPort=",
         $httpSetup{'httpAltEnable'} ? 'yes,' : 'no,',
         $httpSetup{'httpAltPort'},
        " httpsPort443=",
         $httpSetup{'httpsPort443'} ? 'yes' : 'no',
        " httpsAltPort=",
         $httpSetup{'httpsAltEnable'} ? 'yes,' : 'no,',
         $httpSetup{'httpsAltPort'},
        "\n" ;
        ;

} # end of ShowHttpPorts




########################################################################
sub SetHttpPorts
########################################################################
# Responds to command Set HttpPorts
# On success, emits "OK:" followed by the ShowHttpPorts response.
# Parameters must be one or more of the following:
#
#      httpPort80=[yes|no]
#   httpsPort4430=[yes|no]
#     httpAltPort=[yes|no],NNNNN
#     httpAltPort=no
#    httpsAltPort=[yes|no],NNNNN
#    httpsAltPort=no
#
########################################################################
{
  #print "Executing ShowAutodelete\n" ;
  return if ( badParmCount( scalar( @_ ), 1, 4 ) ) ;

  # Read in current settings.
  my %httpSetup ;
  getHttpSetup( \%httpSetup ) ;

  # Check the parameters that were fed to this handler.
  for my $p (@_)
  {
    if ($p !~ /=/)
    {
      print "ERROR: Must fully specify id=value for parameter '$p'\n" ;
      return ;
    }

    my ($id, $value) = split( '=', $p ) ;
    $id = lc($id) ;
    $value = lc($value) ;

    if ($id eq 'httpport80')
    {
      if ($value !~ /^(yes|no)$/)
      {
        BADPARM:
        print "ERROR: bad parameter '$p'\n" ;
        return ;
      }
      $httpSetup{'httpPort80'} = ($value eq 'yes' ? 1 : 0) ;
    }
    elsif ($id eq 'httpsport443')
    {
      if ($value !~ /^(yes|no)$/)
      {
        goto BADPARM ;
      }
      $httpSetup{'httpsPort443'} = ($value eq 'yes' ? 1 : 0) ;
    }
    elsif ($id eq 'httpaltport')
    {
      if ($value eq 'no')
      {
        $httpSetup{'httpAltEnable'} = 0 ;
      }
      elsif ($value =~ /^(yes|no),\d{4,5}$/)
      {
        my( $enable, $port ) = split( ',', $value ) ;
        goto BADPARM if ($port >65535 || $port <1024) ;
        $httpSetup{'httpAltEnable'} =  ($enable eq 'yes' ? 1 : 0) ;
        $httpSetup{'httpAltPort'} = $port ; 
      }
      else
      {
        goto BADPARM ;
      }
    }
    elsif ($id eq 'httpsaltport')
    {
      if ($value eq 'no')
      {
        $httpSetup{'httpsAltEnable'} = 0 ;
      }
      elsif ($value =~ /^(yes|no),\d{4,5}$/)
      {
        my( $enable, $port ) = split( ',', $value ) ;
        goto BADPARM if ($port >65535 || $port <1024) ;
        $httpSetup{'httpsAltEnable'} =  ($enable eq 'yes' ? 1 : 0) ;
        $httpSetup{'httpsAltPort'} = $port ; 
      }
      else
      {
        goto BADPARM ;
      }
    }
    else
    {
      goto BADPARM ;
    }
  }

  if ( ! $httpSetup{'httpPort80'} 
    && ! $httpSetup{'httpsPort443'} 
    && ! $httpSetup{'httpAltEnable'} 
    && ! $httpSetup{'httpsAltEnable'} 
     )
  {
    print "ERROR: Must have at least one HTTP or HTTPS port enabled.\n";
    return ;
  }

  # Install the new settings
  saveHttpSetup( \%httpSetup ) ;

  print "OK: " ;
  ShowHttpPorts() ;

} # end of SetHttpPorts



########################################################################
sub ShowPortAdvertising
########################################################################
# Return a string showing the current controls over the Port Advertising
# function.  Something like this will be produced:
#   PortAdvertising enable=yes 
#                   interval=300 
#                   target0=123.123.123.123:5678
#                   target1=123.123.123.234:6789
#                   ...
#
# Multiple targets may exist and these are listed as target0, target1,
# etc.  Each target is an "IPaddress:PortNumber"
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  # A Hash that will be filled with data from the configuration file.
  my %advertiseControls ;

  # Read from the configuration file 
  getAdvertiseControls( \%advertiseControls ) ;



  print "PortAdvertising ",
        "enable=",
        $advertiseControls{'enable'} ? 'yes' : 'no',
        " interval=",
        $advertiseControls{'interval'}
        ;

  
  my @ports = @{$advertiseControls{'ports'}} ;
  my @IPs = @{$advertiseControls{'IPs'}} ;

  for my $i (0..$#ports)
  {
    printf " target%d=%s:%d", $i+1, $IPs[$i], $ports[$i] ;
  }

  print "\n" ;

} # end of ShowPortAdvertising



########################################################################
sub SetPortAdvertising
########################################################################
# Set the controls used by the PortAdvertising function.
# 
# Set PortAdvertising   enable=[yes|no]
#                       interval=DDD
#                       target[DD]=IP:port
#
# Enable and Interval must be specified.  Any number (including zero)
# of targets may be specified.
#
# Target parameter names can be either 'target' or 'targetDD' where DD
# is an integer.  The integer DD is ignored and no pariticular order
# is expected for the integers.  In fact, the name 'target' can be used
# for multiple targets with no problems.  We just parse the targets
# one-by-one, regardless of their numbering.
#
# IP address must be of the form 123.123.123.123 and ports are limited
# to positive values from 0 and 65535.
#
# The response to SetPortAdvertising is either an error message or
# "OK: " followed by the results from ShowPortAdvertising.  This always
# shows the complete new state of the controls.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 20 ) ) ;

  # A Hash that will be filled with data from the configuration file.
  # This will contain:
  #   $enable   : 1 or 0
  #   $interval : integer seconds
  #   @IPs      : array of target IP strings.
  #   @ports    : matching array of target port numbers.
  my %advertiseControls ;

  # Erase the two arrays.  We can check the size of these to
  # detectif any targets were specified.
  $advertiseControls{'ports'} = [] ;
  $advertiseControls{'IPs'} = [] ;
  # We'll check for the existence of enable and interval to determine
  # if those were specified.

  # Check the parameters that were fed to this handler.
  for my $p (@_)
  {
    if ($p !~ /=/)
    {
      print "ERROR: Must fully specify id=value for parameter '$p'\n" ;
      return ;
    }

    my ($id, $value) = split( '=', $p ) ;
    $id = lc($id) ;
    $value = lc($value) ;

    if ($id eq 'enable')
    {
      if ($value !~ /^(yes|no)$/)
      {
        BADPARM:
        print "ERROR: bad parameter '$p'\n" ;
        return ;
      }
      $advertiseControls{'enable'} = ($value eq 'yes' ? 1 : 0) ;
    }
    elsif ($id eq 'interval')
    {
      if ( $value !~ /^\d{1,5}$/
        || $value > 86400
         )
      {
        goto BADPARM ;
      }
      $advertiseControls{'interval'} = $value ;
    }
    elsif ($id =~ '^target\d*$')
    {
      goto BADPARM if ( !$value ) ;

      # Split ddd.ddd.ddd.ddd:ppppp into IP and Port around the colon.
      my ($ip, $port) = $value =~ /^(.*):(\d{1,5})$/ ;

      # Verify the IP format, octet ranges, and port range.
      if ( $ip !~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/
        or $1 > 255
        or $2 > 255
        or $3 > 255
        or $4 > 255
        or $port > 65535
         ) 
      {
        print "ERROR: Bad target address or port '$value'\n" ;
        return ;
      }

      # Add the new IP & Port to the arrays.
      push( @{$advertiseControls{'IPs'}}, $ip ) ;
      push( @{$advertiseControls{'ports'}}, $port ) ;
    }
    else
    {
      goto BADPARM ;
    }
  }

  if ( ! defined( $advertiseControls{'enable'} ) )
  {
    print "ERROR: Must define 'enable' parameter.\n" ;
    return ;
  }

  if ( ! defined( $advertiseControls{'interval'} ) )
  {
    print "ERROR: Must define 'interval' parameter.\n" ;
    return ;
  }

  # Encode the new configuration setup into the configuration file,
  # and activate those settings.
  saveAdvertiseControls( \%advertiseControls ) ;

  print "OK: " ;
  ShowPortAdvertising() ;

} # end of SetPortAdvertising



########################################################################
sub ShowPppDialout
########################################################################
# Shows the state of the controls used for PPP Dialout.
# Parameters indicated are:
#   papPassword=ea91eb77fb60ac4a
#           The encoded PAP password.  The value is always one or
#           more blocks of 16-character hex data.
#   chapPassword=ea91eb77fb60ac4a
#           The encoded CHAP password.  The value is always one or
#           more blocks of 16-character hex data.
#   optN='option'
#           Multiple option lines.  Always in single quotes.
#           Actual single-quote characters in the values are
#           back-slash escaped.  N starts a 1 and goes upward.
#   chatN='chat line data'
#           Multiple chat script lines.  Always in single quotes.
#           Actual single-quote characters in the values are
#           back-slash escaped.  N starts a 1 and goes upward.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 0, 0 ) ) ;

  my %pppDialoutControls ;
  getPppDialoutControls( \%pppDialoutControls ) ;
  # hash contains:
  #   papPw      : encrypted password to use with PAP authentication
  #   chapPw     : encrypted password to use with CHAP authentication
  #   chatScript : potentially multi-line chat script
  #   options    : potentially multi-line set of options.

  print "PppDialout",
        " papPassword=$pppDialoutControls{'papPw'}" ,
        " chapPassword=$pppDialoutControls{'chapPw'}" ;

  # Emit the option lines, one-by-one.
  my $count = 1 ;
  my @lines = split( /\n/, $pppDialoutControls{'options'} ) ;
  foreach (@lines)
  {
    chomp ;
    print " opt$count='$_'" ;
    ++$count ;
  }

  # Emit the chatscript lines, one-by-one.
  $count = 1 ;
  @lines = split( /\n/, $pppDialoutControls{'chatScript'} ) ;
  foreach (@lines)
  {
    chomp ;
    s/'/\\'/g ;
    print " chat$count='$_'" ;
    ++$count ;
  }

  print "\n" ;

} # end of ShowPppDialout



########################################################################
sub SetPppDialout
########################################################################
# Accept new controls for the PPP Dialout function.
#
# Set PppDialout    papPassword=ea91eb77fb60ac4a
#                   chapPassword=ea91eb77fb60ac4a
#                   clearPapPassword=cleartext
#                   clearChapPassword=cleartext
#                   opt[dd]='option line'
#                   chat[dd]='chat line'
#
# Any combination of these parameters will be accepted.  
# Only those parameters explicitly mentioned will be updated in
# the configuration file.
# 
# If any 'opt' parameters are given, then the entire options file will
# be overwritten, regardless of the number of the 'optdd' names or how
# many of them are present.  That is, the entire option set must be
# updated at once.  Same comment for the chat options: any chat option
# implies that the entire chat script is being updated.
# For optdd and chatdd, the numeric suffixes are ignored.  
#
# The response to Set PppDialout is either an error message or
# "OK: " followed by the results from ShowPppDialout.  This always
# shows the complete new state of the controls.
########################################################################
{
  return if ( badParmCount( scalar( @_ ), 1, 1000 ) ) ;

  # Get the current control settings.
  my %pppDialoutControls ;
  getPppDialoutControls( \%pppDialoutControls ) ;
  # hash contains:
  #   papPw      : encrypted password to use with PAP authentication
  #   chapPw     : encrypted password to use with CHAP authentication
  #   chatScript : potentially multi-line chat script
  #   options    : potentially multi-line set of options.

  # If we find any opt or chat parameters, append their lines to
  # these strings, separated by newlines.  We detect the new
  # options or chatScript by non-zero length in these strings.
  my $newOptions = '' ;
  my $newChat = '' ;

  # Check the parameters that were fed to this handler.
  for my $p (@_)
  {
    if ($p !~ /=/)
    {
      print "ERROR: Must fully specify id=value for parameter '$p'\n" ;
      return ;
    }

    my ($id, $value) = split( '=', $p ) ;
    $id = lc($id) ;
    my $lcvalue = lc($value) ;

    if ($id eq 'pappassword')
    {
      $pppDialoutControls{'papPw'} = $lcvalue ;
    }
    elsif ($id eq 'chappassword')
    {
      $pppDialoutControls{'chapPw'} = $lcvalue ;
    }
    elsif ($id eq 'clearpappassword')
    {
      $value = qx( /usr/local/fireball/makeKey $value ) ;
      chomp $value ;
      $pppDialoutControls{'papPw'} = $value ;
    }
    elsif ($id eq 'clearchappassword')
    {
      $value = qx( /usr/local/fireball/makeKey $value ) ;
      chomp $value ;
      $pppDialoutControls{'chapPw'} = $value ;
    }
    elsif ($id =~ /^opt\d*$/)
    {
      $newOptions .= "$value\n" ;
    }
    elsif ($id =~ /^chat\d*$/)
    {
      $newChat .= "$value\n" ;
    }
    else
    {
      print "ERROR: invalid parameter '$p' \n" ;
      return ;
    }

  }

  # If we found any opt or chat parameters, then update the 
  # associated control.
  $pppDialoutControls{'options'} = $newOptions if ( $newOptions ) ;
  $pppDialoutControls{'chatScript'} = $newChat if ( $newChat ) ;

  # Encode the new configuration setup into the configuration file,
  # and activate those settings.
  savePppDialoutControls( \%pppDialoutControls ) ;

  print "OK: " ;
  ShowPppDialout() ;

} # end of SetPppDialout





# Return a '1' as the final operation in this library, to be sure
# that whoever imports it sees success.
1 ;
########################################################################
# End of file Commands.pm
########################################################################
